// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/AeWSDLDefHelper.java,v 1.15 2005/01/19 22:36:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel;

import java.util.Iterator;
import java.util.Map;

import javax.wsdl.Fault;
import javax.wsdl.Operation;
import javax.xml.namespace.QName;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.def.validation.IAeValidationDefs;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;
import org.activebpel.rt.wsdl.def.IAeProperty;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;

/**
 * Utility helper class to do various WSDL definition lookups.
 */
public class AeWSDLDefHelper
{
   /**
    * unreachable ctor 
    */
   private AeWSDLDefHelper()
   {
   }
   
   /**
    * Returns the WSDL definition for the given service or null if none
    * is found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aService The QName of the PLT to find.
    * @return AeBPELExtendedWSDLDef
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForService( IAeWSDLProvider aProvider, QName aService )
   {
      for ( Iterator iter = aProvider.getWSDLIterator( aService.getNamespaceURI() ) ; iter.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
         Map services = def.getServices();
         if( services.get(aService) != null )
         {
            return def;
         }
      }
      return null;
   }
   
   /**
    * Returns the WSDL definition containing the given partner link type or null if
    * not found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aPartnerLinkType The QName of the PLT to find.
    * @return AeBPELExtendedWSDLDef
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForPLT( IAeWSDLProvider aProvider, QName aPartnerLinkType )
   {
      for ( Iterator iter = aProvider.getWSDLIterator( aPartnerLinkType.getNamespaceURI() ) ; iter.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
         // check that we have a def with the right namespace or we may end up getting the def from imports by accident
         if ( AeUtil.compareObjects(def.getTargetNamespace(), aPartnerLinkType.getNamespaceURI()) )
            if ( def.getPartnerLinkType( aPartnerLinkType.getLocalPart()) != null )
               return def ;
      }

      return null;
   }
   
   /**
    * Returns the WSDL definition containing the given portType or null if
    * not found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aPortType The QName of the portType to find.
    * @return AeBPELExtendedWSDLDef
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForPortType( IAeWSDLProvider aProvider, QName aPortType )
   {
      if (aPortType != null && aProvider != null )
      {
         for ( Iterator iter = aProvider.getWSDLIterator( aPortType.getNamespaceURI() ) ; iter.hasNext() ; )
         {
            AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
            if ( def.getPortType( aPortType ) != null )
               return def ;
         }
      }

      return null;
   }

   /**
    * Returns the WSDL definition containing the given message or null if not found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aMsgName the message we are interested in.
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForMsg(IAeWSDLProvider aProvider, QName aMsgName)
   {
      if(aMsgName == null)
         return null;
         
      for ( Iterator iter = aProvider.getWSDLIterator( aMsgName.getNamespaceURI() ) ; iter.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
         // check that we have a def with the right namespace or we may end up getting the def from imports by accident
         if ( AeUtil.compareObjects(def.getTargetNamespace(), aMsgName.getNamespaceURI()) )
            if( def.definesMessage( aMsgName ) )
               return def ;
      }

      return null;
   }

   /**
    * Returns the WSDL definition containing the given element or null if not found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aElementName the element we are interested in.
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForElement(IAeContextWSDLProvider aProvider, QName aElementName)
   {
      for ( Iterator iter = aProvider.getWSDLIterator() ; iter.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
         try
         {
            if ( def.findElement( aElementName ) != null )
               return def ;
         }
         catch (AeException e)
         {
            // ignore error as if we couldn't find the element
         }
      }

      return null;
   }

   /**
    * Returns the WSDL definition containing the given Type or null if not found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aTypeName the Type we are interested in.
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForType(IAeContextWSDLProvider aProvider, QName aTypeName)
   {
      for ( Iterator iter = aProvider.getWSDLIterator() ; iter.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
         try
         {
            if ( def.findType( aTypeName ) != null )
               return def ;
         }
         catch (AeException e)
         {
            // ignore error as if we couldn't find the Type
         }
      }

      return null;
   }

   /**
    * Returns the WSDL definition containing the given property or null if not found.
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aPropName the property we are interested in.
    */
   public static AeBPELExtendedWSDLDef getWSDLDefinitionForProp(IAeWSDLProvider aProvider, QName aPropName)
   {
      for ( Iterator iter = aProvider.getWSDLIterator( aPropName.getNamespaceURI() ) ; iter.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iter.next());
         for ( Iterator iterProps = def.getPropExtElements().iterator() ; iterProps.hasNext() ; )
         {
            IAeProperty prop = (IAeProperty)iterProps.next(); 
            if ( prop.getQName().equals( aPropName ) )
               return def ;
         }
      }

      return null;
   }

   /**
    * Returns the WSDL definition containing the given portType/operation combination with
    * the specified input/output variables, or null if not found.
    * 
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aPortType The portType QName.
    * @param aOperation The operation name.
    * @param aInput The name of the input variable, or null.
    * @param aOutput The name of the output variable, or null.
    * 
    * @return AeBPELExtendedWSDLDef
    */   
   public static AeBPELExtendedWSDLDef getWSDLDefinition(IAeWSDLProvider aProvider, 
                                                   QName aPortType, String aOperation, 
                                                   QName aInput, QName aOutput )
   {
      AeBPELExtendedWSDLDef def = getWSDLDefinitionForPortType( aProvider, aPortType );
      if ( def != null )
      {
         for ( Iterator iter = def.getOperations( aPortType ) ; iter.hasNext() ; )
         {
            // Check for multiple operation matches, since the operations can 
            //  be overloaded with different input and/or output variables.
            //
            Operation op = (Operation)iter.next(); 
            if ( op.getName().equals( aOperation ))
            {
               if ( aInput != null )
               {
                  if ( op.getInput() != null && op.getInput().getMessage() != null &&
                        !AeUtil.compareObjects( aInput, op.getInput().getMessage().getQName()))
                     // no input match - keep looking
                     continue ;
               }
               
               if ( aOutput != null )
               {
                  if ( op.getOutput() != null && op.getOutput().getMessage() != null && 
                       !AeUtil.compareObjects( aOutput, op.getOutput().getMessage().getQName()))
                     // no output match - keep looking
                     continue ;
               }

               return def;
            }
         }
      }

      return null;
   }

   /**
    * Returns the WSDL definition containing the given portType/operation/fault combination with
    * the specified fault variable, or null if not found.
    * 
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aOperation The operation name.
    * @param aPortType The portType QName.
    * @param aFault The name of the fault.
    * 
    * @return QName The fault variable's message type or null if FAULT not found.
    */
   public static QName getFaultVarType(IAeWSDLProvider aProvider, 
                                                   String aOperation, QName aPortType,  
                                                   QName aFault)
   {
      AeBPELExtendedWSDLDef def = getWSDLDefinitionForPortType( aProvider, aPortType );
      if ( def != null )
      {
         for ( Iterator iter = def.getOperations( aPortType ) ; iter.hasNext() ; )
         {
            // Check for multiple operation matches, since the operations can 
            //  be overloaded with different input and/or output variables.
            //
            Operation op = (Operation)iter.next(); 
            if ( op.getName().equals( aOperation ))
            {
               // Find associated fault.
               //
               Fault fault = op.getFault( aFault.getLocalPart() );
               if ( fault != null )
               {
                  // Find associated msg type.
                  //
                  if ( fault.getMessage().getQName() == null )
                     return IAeValidationDefs.EMPTY_QNAME ;
                  else
                     return fault.getMessage().getQName();
               }
            }
         }
      }

      return null;
   }
   
   /**
    * Get the WSDL PropertyAlias object for a given WSDL Message type and Property name.
    *   
    * @param aProvider The IAeWSDLProvider instance to use for lookup.
    * @param aMsgType the WSDL Message name.
    * @param aPropName the name of the WSDL Property we are searching for
    * 
    * @return PropertyAlias object or null if not found.
    */
   public static IAePropertyAlias getWSDLMessagePropertyAlias(IAeContextWSDLProvider aProvider, 
                                                         QName aMsgType, QName aPropName)
   {
      // Search all known WSDL documents.
      //
      for ( Iterator iterW = aProvider.getWSDLIterator() ; iterW.hasNext() ; )
      {
         AeBPELExtendedWSDLDef def = aProvider.dereferenceIteration( iterW.next());
         for ( Iterator iter = def.getPropAliasExtElements().iterator() ; iter.hasNext() ; )
         {
            IAePropertyAlias alias = (IAePropertyAlias)iter.next();
            if ( aMsgType.equals( alias.getMessageType()) && aPropName.equals( alias.getPropertyName()))
               return alias ;
         }
      }

      return null;
   }

   /**
    * Gets an <code>IAeProperty</code> given the property name and a WSDL provider.
    * 
    * @param aWSDLProvider
    * @param aPropertyName
    * @return The <code>IAeProperty</code> with the given name.
    */
   public static IAeProperty getProperty(IAeWSDLProvider aWSDLProvider, QName aPropertyName)
   {
      IAeProperty prop = null;
      AeBPELExtendedWSDLDef def = AeWSDLDefHelper.getWSDLDefinitionForProp(aWSDLProvider, aPropertyName);
      if (def != null)
      {
         for (Iterator iter = def.getPropExtElements().iterator(); iter.hasNext() && prop == null;)
         {
            IAeProperty property = (IAeProperty) iter.next();
            if (property.getQName().equals(aPropertyName))
            {
               prop = property;
            }
         }
      }

      return prop;
   }
}
