// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/IAeBusinessProcessEngine.java,v 1.49 2006/02/10 17:02:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel;

import java.util.Date;
import java.util.Map;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.config.IAeEngineConfiguration;
import org.activebpel.rt.bpel.impl.AeConflictingRequestException;
import org.activebpel.rt.bpel.impl.AeCorrelationViolationException;
import org.activebpel.rt.bpel.impl.AeTypeMapping;
import org.activebpel.rt.bpel.impl.IAeMessageReceiver;
import org.activebpel.rt.bpel.impl.IAeProcessPlan;
import org.activebpel.rt.bpel.impl.IAeReplyReceiver;
import org.activebpel.rt.bpel.impl.queue.AeInboundReceive;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.wsio.IAeWebServiceMessageData;
import org.activebpel.wsio.IAeWebServiceResponse;
import org.activebpel.wsio.receive.AeTimeoutException;
import org.activebpel.wsio.receive.IAeMessageContext;
import org.w3c.dom.Document;

/** Describes the interface used for interacting with a business process engine */
public interface IAeBusinessProcessEngine
{
   
   /**
    * Return the partner role endpoint reference for the partnerLink 
    * identified by the path or null if none is found.
    * @param aPid
    * @param aPartnerLinkPath
    * @throws AeBusinessProcessException
    */
   public IAeEndpointReference getPartnerRoleEndpointReference( long aPid, String aPartnerLinkPath ) 
   throws AeBusinessProcessException;
   
   /**
    * Set the variable data for the given suspended process and variable path.
    * @param aPid
    * @param aVariablePath
    * @param aVariableData
    * @throws AeBusinessProcessException
    */
   public void setVariableData( long aPid, String aVariablePath, Document aVariableData )
   throws AeBusinessProcessException;

   
   /**
    * Returns the correlation data for the given process and correlation path.
    * @param aPid
    * @param aCorrsetPath
    * @throws AeBusinessProcessException
    */
   public Map getCorrelationData( long aPid, String aCorrsetPath)
   throws AeBusinessProcessException;
   
   /**
    * Set the correlation data for the given suspended process and correlation path.
    * @param aPid
    * @param aCorrsetPath
    * @param aCorrelationData
    * @throws AeBusinessProcessException
    */
   public void setCorrelationData( long aPid, String aCorrsetPath, Map aCorrelationData )
   throws AeBusinessProcessException;
   
   /**
    * Set the endpoint ref data for a partnerRole partner link.
    * @param aPid
    * @param aIsPartnerRole
    * @param aPartnerLinkPath
    * @param aPartnerEndpointRef
    * @throws AeBusinessProcessException
    */
   public void setPartnerLinkData( long aPid, boolean aIsPartnerRole, String aPartnerLinkPath, Document aPartnerEndpointRef )
   throws AeBusinessProcessException;
   
   /**
    * Suspends the business process identified by the passed pid.
    * @param aPid The process id of the process to suspend.
    * @throws AeBusinessProcessException
    */
   public void suspendProcess(long aPid) throws AeBusinessProcessException;

   /**
    * Resumes the business process identified by the passed pid.
    * @param aPid The process id of the process to resume.
    * @throws AeBusinessProcessException
    */
   public void resumeProcess(long aPid) throws AeBusinessProcessException;

   /**
    * Resumes the business process identified by the passed pid for the passed
    * suspended location.
    * @param aPid The process id of the process to resume.
    * @param aLocation A location path for a suspended bpel object.
    * @throws AeBusinessProcessException
    */
   public void resumeProcessObject(long aPid, String aLocation) throws AeBusinessProcessException;
   
   /**
    * Step resumes the business process identified by the passed pid. The activity
    * identified by the location arg (or its enclosing scope) will be re-executed.
    * @param aPid The process id of the process to resume.
    * @param aLocation A location path for a suspended bpel object.
    * @throws AeBusinessProcessException
    */
   public void retryActivity( long aPid, String aLocation, boolean aAtScope )
   throws AeBusinessProcessException;
   
   /**
    * Step resumes the business process identified by the passed pid. The activity
    * identified by the location arg (or its enclosing scope) will be treated as
    * if it had completed normally.  
    * It is up to the use to ensure that the process is a "good state" before
    * executing this method. 
    * @param aPid
    * @param aLocation
    * @throws AeBusinessProcessException
    */
   public void completeActivity( long aPid, String aLocation )
   throws AeBusinessProcessException;
   

   /**
    * Terminates the business process identified by the passed pid.
    * @param aPid The process id of the process to terminate.
    * @throws AeBusinessProcessException
    */
   public void terminateProcess(long aPid) throws AeBusinessProcessException;

   /**
    * Add a listener for engine notification events.
    * @param aListener The listener to be added
    */
   public void addEngineListener(IAeEngineListener aListener);

   /**
    * Remove the given listener from receiving engine notification events.
    * @param aListener The listener to be added
    */
   public void removeEngineListener(IAeEngineListener aListener);

   /**
    * Add a listener to those notified of process events for all executing processes.
    * @param aListener The listener instance to add.
    */
   public void addProcessListener(IAeProcessListener aListener);

   /**
    * Add a listener to those notified of process events for the given process ID.
    * @param aListener The listener instance to add.
    * @param aPid the process id we are being installed for
    */
   public void addProcessListener(IAeProcessListener aListener, long aPid);

   /**
    * Removes the passed listener from list of those notified of process events
    * for all executing processes.
    * @param aListener The listener instance to remove.
    */
   public void removeProcessListener(IAeProcessListener aListener);

   /**
    * Removes the passed listener from list of those notified of process events
    * for the given process ID.
    * @param aListener The listener instance to remove.
    * @param aPid the process id we are being removed for
    */
   public void removeProcessListener(IAeProcessListener aListener, long aPid);

   /**
    * Allows an externally received message to be dispatched to a queued receive.
    * @param aProcessQName QName of the process being called
    * @param aMessageData The data we have received.
    * @param aPartnerLinkName The name of the partner link which is allowed to execute the opertaion (null if any).
    * @param aPortType The portType the opertaion is associated with.
    * @param aOperation The operation to be performed.
    * @param aReply Non null if the receive being queued is part of a request/response operation.
    * @param aPrincipal The principal that was authenticated or null if there was no authentication information.
    * @param aEmbeddedEndpointReference An endpoint reference that was extracted from the transport layer headers.
    *        This is only set if the partnerlink is configured to look for an endpoint reference within the message context.
    * @throws AeBusinessProcessException Thrown if error occurs setting the receiver.
    * @deprecated Use the queue method that takes the plan instead.
    */
   public void queueReceiveData( QName aProcessQName,
                                 IAeMessageData aMessageData,
                                 String aPartnerLinkName,
                                 QName aPortType,
                                 String aOperation,
                                 IAeMessageReceiver aReply,
                                 String aPrincipal,
                                 IAeEndpointReference aEmbeddedEndpointReference)
         throws AeBusinessProcessException;

   /**
    * Allows an externally received message to be dispatched to a queued receive.
    * @param aProcessPlan Plan used to dispatch the message.
    * @param aMessageData The data we have received.
    * @param aPartnerLinkName The name of the partner link which is allowed to execute the opertaion (null if any).
    * @param aPortType The portType the opertaion is associated with.
    * @param aOperation The operation to be performed.
    * @param aReply Non null if the receive being queued is part of a request/response operation.
    * @param aPrincipal The principal that was authenticated or null if there was no authentication information.
    * @param aEmbeddedEndpointReference An endpoint reference that was extracted from the transport layer headers.
    *        This is only set if the partnerlink is configured to look for an endpoint reference within the message context.
    * @throws AeBusinessProcessException Thrown if error occurs setting the receiver.
    * @deprecated Use the queue method that takes the IAeReplyReceiver interface instead
    */
   public void queueReceiveData( IAeProcessPlan aProcessPlan,
                                 IAeMessageData aMessageData,
                                 String aPartnerLinkName,
                                 QName aPortType,
                                 String aOperation,
                                 IAeMessageReceiver aReply,
                                 String aPrincipal,
                                 IAeEndpointReference aEmbeddedEndpointReference)
         throws AeBusinessProcessException;
   
   /**
    * Accepts an inbound message which will either create a new process instance or get dispatched to
    * an existing message receiver from an already executing process.
    *  
    * @param aProcessPlan
    * @param aMessageData
    * @param aReply
    * @param aContext
    * @throws AeBusinessProcessException
    * @deprecated use the queue method that takes the IAeReplyReceiver instead
    */
   public void queueReceiveData(IAeProcessPlan aProcessPlan, IAeMessageData aMessageData, IAeMessageReceiver aReply, IAeMessageContext aContext)
      throws AeBusinessProcessException;

   /**
    * Accepts an inbound message which will either create a new process instance or get dispatched to
    * an existing message receiver from an already executing process.
    *  
    * @param aProcessPlan
    * @param aMessageData
    * @param aReply
    * @param aContext
    * @throws AeBusinessProcessException
    */
   public void queueReceiveData(IAeProcessPlan aProcessPlan, IAeMessageData aMessageData, IAeReplyReceiver aReply, IAeMessageContext aContext)
      throws AeBusinessProcessException;
   
   /**
    * Accepts an inbound message which will either create a new process instance or get dispatched to
    * an existing message receiver from an already executing process.  This version simply takes an
    * AeInboundReceive and a IAeReplyReceiver.
    *  
    * @param aInboundReceive
    * @param aReply
    * @throws AeCorrelationViolationException
    * @throws AeConflictingRequestException
    * @throws AeBusinessProcessException
    */
   public void queueReceiveData(AeInboundReceive aInboundReceive, IAeReplyReceiver aReply)
         throws AeCorrelationViolationException, AeConflictingRequestException, AeBusinessProcessException;

   /**
    * Primary interface for disptaching messages from web services.
    * 
    * @param aContext Provides contextual information about what process and operation are being targeted.
    * @param aData The data for the operation being invoked
    * @throws AeBusinessProcessException
    * @throws AeTimeoutException
    */
   public IAeWebServiceResponse queueReceiveData(IAeMessageContext aContext, IAeWebServiceMessageData aData) 
      throws AeBusinessProcessException, AeTimeoutException;
   
   /**
    * Specialized doc-literal invocation method for the engine. At a minimum, the user must pass
    * a context that contains the service name for the invoke and the engine will determine the
    * process, partnerlink, and operation by using the data to find the matching signature
    * for the port type.
    * 
    * @param aContext Provides contextual information about what process and operation are being targeted.
    * @param aDataArray The data for the operation being invoked
    * @throws AeTimeoutException
    * @throws AeBusinessProcessException
    */
   public IAeWebServiceResponse queueReceiveData(IAeMessageContext aContext, Document[] aDataArray) throws AeBusinessProcessException, AeTimeoutException;

   /**
    * Allows an externally invoked operation data to dispatch to a queued invoke.
    * @param aProcessId The id of the process expecting the response from the invoke
    * @param aLocationPath The path to the location awaiting the response
    * @param aMessageData The data we have received from invoke.
    * @param aProcessProperties Any string name/value pairs we received back from the invoke.
    * @throws AeBusinessProcessException Thrown if error occurs setting the receiver.
    */
   public void queueInvokeData(long aProcessId, String aLocationPath, IAeMessageData aMessageData, Map aProcessProperties )
         throws AeBusinessProcessException;

   /**
    * Allows an externally invoked operation fault to dispatch to a queued invoke.
    * @param aProcessId The process that's expecting the invoke response
    * @param aLocationPath The path to the location awaiting the response
    * @param aFault The fault we received from invoke.
    * @param aProcessProperties Any string name/value pairs we received back from the invoke.
    * @throws AeBusinessProcessException Thrown if error occurs setting the receiver.
    */
   public void queueInvokeFault(long aProcessId, String aLocationPath, IAeFault aFault, Map aProcessProperties)
         throws AeBusinessProcessException;

   /**
    * The engine configuration.
    * @return IAeEngineConfiguration The engine configuration for this engine instance.
    */
   public IAeEngineConfiguration getEngineConfiguration();
   
   /**
    * Gets the date/time that the engine started.
    */
   public Date getStartDate();

   /**
    * Gets the serialized state of a process, by ID.
    * @param aProcessId ID of the process.
    * @return Document containing the XML serialized state of the process.
    * @throws AeBusinessProcessException
    */
   public Document getProcessState( long aProcessId ) throws AeBusinessProcessException;

   /**
    * Gets the serialized variable for a process, by ID and location.
    * @param aProcessId ID of the process.
    * @param aLocationPath The location XPath for the variable's enclosing scope (?)
    * @return Document containing the XML serialized variable data.
    * @throws AeBusinessProcessException
    */
   public Document getProcessVariable( long aProcessId, String aLocationPath ) throws AeBusinessProcessException;

   /**
    * Returns the type mapping helper for the engine for schema to java and back.
    */
   public AeTypeMapping getTypeMapping();
}
