//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/IAeExpressionLanguageFactory.java,v 1.1 2005/06/08 12:50:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel;

import java.util.List;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.def.expr.IAeExpressionAnalyzer;
import org.activebpel.rt.bpel.def.validation.expr.IAeExpressionValidator;
import org.activebpel.rt.bpel.impl.expr.IAeExpressionRunner;

/**
 * This interface must be implemented by any classes desiring to function as an expression
 * language factory.  An expression language factory is responsible for create expression
 * validators and expression runners for different types of expression languages.
 */
public interface IAeExpressionLanguageFactory
{
   /**
    * Given a language identifier found in the 'expressionLanguage' attribute of the BPEL 'process'
    * element, returns an expression validator capable of validating expressions written in that 
    * language.
    * 
    * @param aLanguage
    * @throws AeException
    */
   public IAeExpressionValidator createExpressionValidator(String aLanguage) throws AeException;

   /**
    * Given a language identifier found in the 'expressionLanguage' attribute of the BPEL 'process'
    * element, returns an expression runner capable of running expressions written in that 
    * language.
    * 
    * @param aLanguage
    * @throws AeException
    */
   public IAeExpressionRunner createExpressionRunner(String aLanguage) throws AeException;

   /**
    * Given a language identifier found in the 'expressionLanguage' attribute of the BPEL 'process'
    * element, returns an expression analyzer object capable of analyzing and manipulating expressions
    * written in that language.
    * 
    * @param aLanguage
    * @throws AeException
    */
   public IAeExpressionAnalyzer createExpressionAnalyzer(String aLanguage) throws AeException;

   /**
    * Returns true if the given language is supported by this expression language factory.  If
    * null or empty string is passed in, then this method should return true if the BPEL default
    * language is supported (xpath 1.0).
    * 
    * @param aLanguage
    */
   public boolean supportsLanguage(String aLanguage);
   
   /**
    * Returns the list of languages that this factory knows how to create.  The return value is a 
    * list of expression language URIs.
    */
   public List getSupportedLanguages();
}
