// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/IAeVariable.java,v 1.19 2006/02/07 14:44:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.message.IAeMessageDataListener;
import org.w3c.dom.Element;

/**
 * Describes the interface used for interacting with business process variables
 * these Business Process variables are associated with either messages or
 * standard schema described xml data.
 */
public interface IAeVariable extends IAeMessageDataListener, IAeLocatableObject
{
   /**
    * Get the name of the variable.
    */
   public String getName();

   /**
    * Get the definition of the variable.
    */
   public AeVariableDef getDefinition();

   /**
    * Get the version number.
    */
   public int getVersionNumber();

   /**
    * Get the message data associated with the variable or throws an exception if
    * the variable has not been initialized.
    */
   public IAeMessageData getMessageData() throws AeBusinessProcessException;

   /**
    * Sets the message data associated with the variable.
    */
   public void setMessageData(IAeMessageData aMessage);

   /** 
    * Returns true if the variable has data associated with it. 
    */
   public boolean hasMessageData();

   /**
    * Get the data if the variable is specified as type. Returns null if not type.
    */
   public Object getTypeData();

   /**
    * Set the type data associated with this variable.
    */
   public void setTypeData(Object aTypeData);

   /**
    * Get the data if the variable is specified as element. Returns null if not element.
    */
   public Element getElementData();

   /**
    * Set the element data associated with this variable.
    */
   public void setElementData(Element aElement);

   /**
    * Pass through to definition, declared here as a convenience method. Returns
    * null if not a message.
    */
   public QName getMessageType();

   /**
    * Pass through to definition, declared here as a convenience method. Returns
    * null if not an Element.
    */
   public QName getElement();

   /**
    * Pass through to definition, declared here as a convenience method. Returns
    * null if not a type.
    */
   public QName getType();

   /**
    * Iterates through all of the parts and adds xsi:type information where
    * necessary.
    */
   public void declarePartTypes();

   /**
    * Performs validation against the data set for this variable. An
    * AeBusinessProcessException is thrown if the data is invalid.
    * @throws AeBusinessProcessException
    */
   public void validate() throws AeBusinessProcessException;

   /**
    * Performs validation against the data set for this variable. An
    * AeBusinessProcessException is thrown if the data is invalid.
    * @param aAllowEmptyPartData set to True if an empty message should pass validation False otherwise
    * @throws AeBusinessProcessException
    */
   public void validate(boolean aAllowEmptyPartData) throws AeBusinessProcessException;
   
   /**
    * Clears the value for the variable. Called from the variable's declaring
    * scope each time the scope is going to execute since the variable's state
    * is not preserved across invocations.
    */
   public void clear();

   /**
    * Returns <code>true</code> if and only if the variable has data.
    */
   public boolean hasData();

   /**
    * Sets the version number.
    *
    * @param aVersionNumber
    */
   public void setVersionNumber(int aVersionNumber);

   /**
    * @return Returns the scope which owns this variable.
    */
   public IAeBpelObject getScope();
   
   /**
    * Clones the variable
    */
   public Object clone();
}
