//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/config/IAeUpdatableEngineConfig.java,v 1.6 2006/01/06 23:55:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.config;

/**
 * Interface for the runtime mutable engine settings.
 */
public interface IAeUpdatableEngineConfig extends IAeEngineConfiguration
{
   /**
    * Set the time the engine will hold onto unmatched correlated receives
    * before rejecting them. This is to address the race condition that is possible
    * with one-way invokes and a resulting asynchronous callback. Briefly, it's
    * possible for a response from a one-way invoke to arrive before the intended
    * receive activity has executed. The spec says that the implementation needs
    * to support that receive arriving prior to the execution of the invoke.
    * The details can be found in Issue 33 of the WS-BPEL issues list.
    */
   public void setUnmatchedCorrelatedReceiveTimeout( int aTime );
   
   /**
    * Setter for enabling/disabling process logging.
    * @param aFlag
    */
   public void setLoggingEnabled( boolean aFlag );
   
   /**
    * If true the bpel engine should allow xpath queries which return 0 nodes.
    */
   public void setAllowEmptyQuerySelection( boolean aFlag );

   /**
    * If true the bpel engine should automatically create XPath nodes that don't exist.
    * @param aFlag
    */
   public void setAllowCreateXPath( boolean aFlag );
   
   /**
    * Determine if engine should validate all inbound and outbound messages.
    * These are the messages used in receive, reply, invoke and onMessage.
    */
   public void setValidateServiceMessages( boolean aFlag );

   /**
    * Sets the minimum number of threads to have in the work manager's thread pool.
    */
   public void setWorkManagerThreadPoolMin( int aMin );

   /**
    * Sets the maximum number of threads to have in the work manager's thread pool
    */
   public void setWorkManagerThreadPoolMax( int aMax );
   
   /**
    * Set the maximum number of WSDL Def objects that will be cached
    * in memory.
    * @param aMax
    */
   public void setWsdlCacheMax( int aMax );

   /**
    * Add a listener to be notified after the internal state
    * of the config has been updated.
    * @param aListener
    */
   public void addConfigChangeListener( IAeConfigChangeListener aListener );
   
   /**
    * Remove a listener.
    * @param aListener
    */
   public void removeConfigChangeListener( IAeConfigChangeListener aListener );
   
   /**
    * Adds a listener for storage changes
    * @param aListener
    */
   public void addStorageChangeListener(IAeStorageChangeListener aListener);
   
   /**
    * Removes a listener for storage changes.
    * @param aListener
    */
   public void removeStorageChangeListener(IAeStorageChangeListener aListener);
   
   /**
    * Notify any listeners that they need to update their setting.
    */
   public void update();
   
   /**
    * If set to true then unauthenticated requests to services deployed with 
    * an allowedRoles attribute will fail. 
    * 
    * @param aFlag
    */
   public void setAllowedRolesEnforced(boolean aFlag);
   
   /**
    * If set to true, existing wsdl (mapped to same location hint) will be 
    * replaced.  
    * 
    * @param aFlag
    */
   public void setWsdlReplaceEnabled( boolean aFlag );
   
   /**
    * If set to true, any non-service flow processes will be suspended 
    * if they encounter an uncaught fault.
    * 
    * @param aFlag
    */
   public void setSuspendProcessOnUncaughtFault( boolean aFlag );

   /**
    * Sets the maximum number of work requests to schedule per-process.
    */
   public void setProcessWorkCount(int aProcessWorkCount);
}
