// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/AeCorrelationContainerDef.java,v 1.4 2004/12/16 22:25:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;


/**
 * Container class for <code>correlations</code> that are a part of <code>receive</code>,
 * <code>reply</code>, <code>invoke</code>, and <code>pick</code>.
 */
public class AeCorrelationContainerDef extends AeBaseContainer
{
   /** list of names for the correlation sets used to correlate this activity */
   private Set mCorrelationSetNames;
   
   /**
    * Convenience method that returns the list of correlation set names for this
    * activity.
    */
   public Set getCorrelatonSetNames()
   {
      if (mCorrelationSetNames == null)
      {
         if (getSize() > 0)
         {
            mCorrelationSetNames = new TreeSet();
            for (Iterator iter = getValues(); iter.hasNext();)
            {
               AeCorrelationDef def = (AeCorrelationDef) iter.next();
               mCorrelationSetNames.add(def.getCorrelationSetName());
            }
         }
         else
         {
            mCorrelationSetNames = Collections.EMPTY_SET;
         }
      }
      return mCorrelationSetNames;
   }
   
   
   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }
   
   public void add(AeCorrelationDef aDef)
   {
      super.add(aDef);
   }
}
