// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/AeEventHandlersDef.java,v 1.4 2004/07/08 13:09:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def;

import org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

/**
 * This is the container for event handlers for a scope.  Event handlers
 * include onMessage and onAlarm activity containers.
 */
public class AeEventHandlersDef extends AeBaseDef
{
   /** The list of on message definitions, may be null */
   private ArrayList mOnMessageList;
   /** The list of on alaram definitions, may be null */
   private ArrayList mOnAlarmList;

   /**
    * Default constructor
    */
   public AeEventHandlersDef()
   {
   }
   
   /**
    * Provides the ability to add an onMessage element to the Event Handlers.
    * 
    * @param aMessage message element to be added
    */
   public void addMessage(AeOnMessageDef aMessage)
   {
      if (mOnMessageList == null)
         mOnMessageList = new ArrayList();
         
      mOnMessageList.add(aMessage);
   }

   /**
    * Provides the ability to delete an onMessage element from the Pick activity.
    * 
    * @param aMessage the onMessage element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeMessage(AeOnMessageDef aMessage)
   {
      boolean found = false;
      for (Iterator iter = getMessageList(); iter.hasNext() && !found;)
      {
         if (aMessage == iter.next())
         {
            iter.remove();
            found = true;
         }
      }
      
      return found;
   }

   /**
    * Provide a list of the message elements for the user to iterate .
    * 
    * @return iterator of AeOnMessageDef objects
    */
   public Iterator getMessageList()
   {
      if (mOnMessageList == null)
         return Collections.EMPTY_LIST.iterator();
      else
         return mOnMessageList.iterator();
   }

   /**
    * Provides the ability to add an Alarm element to the Pick activity.
    * 
    * @param aAlarm catch element to be added
    */
   public void addAlarm(AeOnAlarmDef aAlarm)
   {
      if (mOnAlarmList == null)
         mOnAlarmList = new ArrayList();
         
      mOnAlarmList.add(aAlarm);
   }

   /**
    * Provides the ability to delete an Alarm element from a Pick activity.
    * 
    * @param aAlarm the Alarm element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeAlarm(AeOnAlarmDef aAlarm)
   {
      boolean found = false;
      for (Iterator iter = getAlarmList(); iter.hasNext() && !found;)
      {
         if (aAlarm == (AeOnAlarmDef)iter.next())
         {
            iter.remove();
            found = true;
         }
      }
      
      return found;
   }

   /**
    * Provide a list of the Alarm objects for the user to iterate .
    * 
    * @return iterator of AeOnAlarmDef objects
    */
   public Iterator getAlarmList()
   {
      if (mOnAlarmList == null)
         return Collections.EMPTY_LIST.iterator();
      else
         return mOnAlarmList.iterator();
   }
   
   /** Returns true if has any onAlarm or onMessage defined */
   public boolean hasEventHandler()
   {
      return getMessageList().hasNext() || getAlarmList().hasNext();
   }
   
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

}
