// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/AeFaultHandlerDef.java,v 1.6 2004/07/08 13:09:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def;

import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

import javax.xml.namespace.QName;

/**
 * Definition for bpel fault handler element.
 */
public class AeFaultHandlerDef extends AeBaseDef implements IAeSingleActivityContainerDef
{
   // attributes of the Fault Handler Definition
   private QName mFaultName;
   private String mFaultVariable;

   /** Activity to execute if this fault handler is called */
   private AeActivityDef mActivity;


   /**
    * Default constructor
    */
   public AeFaultHandlerDef()
   {
   }

   /**
    * Accessor method to obtain fault name of this object.
    * 
    * @return the fault name of the object
    */
   public final QName getFaultName()
   {
      return mFaultName;
   }

   /**
    * Mutator method to set the fault name of this object.
    * 
    * @param aFaultName the fault name for the object
    */
   public final void setFaultName(QName aFaultName)
   {
      mFaultName = aFaultName;
   }

   /**
    * Accessor method to obtain fault variable of this object.
    * 
    * @return the fault variable of the object
    */
   public final String getFaultVariable()
   {
      return mFaultVariable;
   }

   /**
    * Mutator method to set the fault variable of this object.
    * 
    * @param aFaultVariable the fault variable for the object
    */
   public final void setFaultVariable(String aFaultVariable)
   {
      mFaultVariable = aFaultVariable;
   }

   /**
    * Accessor method to obtain the activity to be executed upon for
    * the fault condition.
    * 
    * @return the activity associated with the fault condition
    */
   public final AeActivityDef getActivity()
   {
      return mActivity;
   }

   /**
    * Mutator method to set the activity which will be executed for 
    * the fault condition.
    * 
    * @param aActivity the activity to be executed
    */
   public final void setActivity(AeActivityDef aActivity)
   {
      mActivity = aActivity;
   }
   
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

}
