//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/AeMessageExchangesDef.java,v 1.1 2006/01/11 21:01:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def; 

import java.util.HashSet;
import java.util.Set;

import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;
import org.activebpel.rt.util.AeUtil;

/**
 * Simple container for string values used for binding receives and replies
 */
public class AeMessageExchangesDef extends AeBaseDef
{
   /** container for the message exchanges */
   private Set mMessageExchanges = new HashSet();
   /** true if the messageExchanges implicitly declares the "default" messageExchange value */
   private boolean mDefaultDeclared = false;

   /**
    * Returns true if the message exchange value is contained within the set of declared message exchanges
    * or if the value is empty and the def implicitly declares a default. 
    * @param aValue
    */
   public boolean declaresMessageExchange(String aValue)
   {
      return getMessageExchanges().contains(aValue) || (isDefaultDeclared() && AeUtil.isNullOrEmpty(aValue));
   }

   /**
    * @return Returns the messageExchanges.
    */
   public Set getMessageExchanges()
   {
      return mMessageExchanges;
   }
   
   /**
    * @param aMessageExchanges The messageExchanges to set.
    */
   public void setMessageExchanges(Set aMessageExchanges)
   {
      mMessageExchanges = aMessageExchanges;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

   /**
    * @return Returns the defaultDeclared.
    */
   public boolean isDefaultDeclared()
   {
      return mDefaultDeclared;
   }

   /**
    * @param aDefaultDeclared The defaultDeclared to set.
    */
   public void setDefaultDeclared(boolean aDefaultDeclared)
   {
      mDefaultDeclared = aDefaultDeclared;
   }
}
 
