//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/AePartnerLinkDelegate.java,v 1.1 2005/12/08 23:33:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def;

import java.util.Collections;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;

/**
 * Class to hold partner link related information.
 */
public class AePartnerLinkDelegate
{
   /** partner link */
   private String mPartnerLink;
   /** port type.*/
   private QName mPortType;
   /** operation */
   private String mOperation;
   /** correlation container */
   private AeCorrelationContainerDef mCorrelations;   

   /**
    * Default ctor.
    */
   public AePartnerLinkDelegate()
   {
   }

   /**
    * Getter for the container. Will create instance if it's null.
    */
   public AeCorrelationContainerDef getCorrelationContainer()
   {
      if (mCorrelations == null)
      {
         mCorrelations = new AeCorrelationContainerDef();
      }
      return mCorrelations;
   }

   /**
    * Provides the ability to add a correlation element to the correlation list.
    * 
    * @param aCorrelation the correlation to be added
    */
   public void addCorrelation(AeCorrelationDef aCorrelation)
   {
      getCorrelationContainer().add(aCorrelation);
   }
   
   /**
    * Provides the ability to delete a correlation element from a Message.
    * 
    * @param aCorrelation the Correlation element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeCorrelation(AeCorrelationDef aCorrelation)
   {
      boolean found = false;
      for (Iterator iter = getCorrelationList(); iter.hasNext() && !found;)
      {
         if (aCorrelation == (AeCorrelationDef)iter.next())
         {
            iter.remove();
            found = true;
         }
      }      
      return found;
   }
   
   /**
    * Provide a list of the Correlation objects for the user to iterate .
    * 
    * @return iterator of AeCorrelationDef object
    */
   public Iterator getCorrelationList()
   {
      if (mCorrelations == null)
      {
         return Collections.EMPTY_LIST.iterator();
      }
      else
      {
         return mCorrelations.getValues();
      }
   }
   
   /**
    * Utility method to determine if a message has a correlation list 
    * @return true if there are elements in the correlation list.
    */
   public boolean hasCorrelationList()
   {
      if (mCorrelations == null)
      {
         return false;
      }
      else
      {        
         return getCorrelationContainer().getSize() > 0;
      }
   }
   
   /**
    * Returns the name of the partner link associated with this activity.
    */
   public String getPartnerLink()
   {
      return mPartnerLink;
   }

   /**
    * Set the name of the partner link associated with this activity.
    */
   public void setPartnerLink(String aPartnerLink)
   {
      mPartnerLink = aPartnerLink;
   }

   /**
    * Accessor method to obtain the port type for the object.
    * 
    * @return QName of the port type for the object
    */
   public QName getPortType()
   {
      return mPortType;
   }

   /**
    * Mutator method to set the port type for the object.
    * 
    * @param aPortType the port type value to be set
    */
   public void setPortType(QName aPortType)
   {
      mPortType = aPortType;
   }

   /**
    * Accessor method to obtain the operation for the object.
    * 
    * @return name of the operation for the object
    */
   public String getOperation()
   {
      return mOperation;
   }

   /**
    * Mutator method to set the operation for the object.
    * 
    * @param aOperation the operation value to be set
    */
   public void setOperation(String aOperation)
   {
      mOperation = aOperation;
   }
   
}
