// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/AeVariableDef.java,v 1.13 2005/08/04 20:01:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def;

import java.util.HashMap;
import java.util.Iterator;

import javax.wsdl.Part;
import javax.xml.namespace.QName;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;
import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;
import org.exolab.castor.xml.schema.ElementDecl;
import org.exolab.castor.xml.schema.XMLType;

/**
 * Definition for bpel Variable construct.
 */
public class AeVariableDef extends AeNamedDef
{
   /** Holds a map of part names to part type information. */
   private HashMap mPartMap = new HashMap();

   /** The message type of the variable, null if not a message type */
   private QName mMessageType;
   
   /** The XML type of the variable, null if not a type */
   private QName mType;
   
   /** The element type of the variable, null if not a type */
   private QName mElement;
   
   /** Flag that indicates the variable definition is implicit as in the case of the counter var in forEach */
   private boolean mImplicit = false;

   /**
    * Default constructor
    */
   public AeVariableDef()
   {
   }

   /**
    * Accessor method to obtain messageType of this object.
    * 
    * @return messageType of object
    */
   public final QName getMessageType()
   {
      return mMessageType;
   }

   /**
    * Mutator method to set messageType of this object.  
    * @param messageType of object
    */
   public final void setMessageType(QName aMessageType)
   {
      if (aMessageType != null && (getType() != null || getElement() != null))
      {
         throw new IllegalStateException(AeMessages.getString("AeVariableDef.ERROR_0")); //$NON-NLS-1$
      }
      mMessageType = aMessageType;
   }
   
   /**
    * Returns the element decaration associated with this variable, null if
    * not an element.
    */
   public QName getElement()
   {
      return mElement;
   }

   /**
    * Sets the element decaration associated with this variable, set to null if
    * not an element.
    */
   public void setElement(QName aElementName)
   {
      if (aElementName != null && (getType() != null || getMessageType() != null))
      {
         throw new IllegalStateException(AeMessages.getString("AeVariableDef.ERROR_0")); //$NON-NLS-1$
      }
      mElement = aElementName;
   }

   /**
    * Returns the type decaration associated with this variable, null if
    * not an type.
    */
   public QName getType()
   {
      return mType;
   }

   /**
    * Sets the type decaration associated with this variable, set to null if
    * not an type.  
    */
   public void setType(QName aSchemaType)
   {
      if (aSchemaType != null && (getMessageType() != null || getElement() != null))
      {
         throw new IllegalStateException(AeMessages.getString("AeVariableDef.ERROR_0")); //$NON-NLS-1$
      }
      mType = aSchemaType;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);      
   }
   
   /**
    * Returns an iterator over the part names for this message.
    */
   public Iterator getPartNames()
   {
      return mPartMap.keySet().iterator();
   }
   
   /**
    * Returns true if this variable has parts.
    */
   public boolean hasParts()
   {
      return !mPartMap.isEmpty();
   }

   /**
    * Adds the part type information for this Part to the variable definition. 
    * @param aPart the part to be added.
    * @param aDef
    */
   public void addPartTypeInfo(Part aPart, AeBPELExtendedWSDLDef aDef) throws AeException
   {
      boolean isElement = false;
      XMLType type = null;
      
      if(aPart.getTypeName() != null)
         type = aDef.findType(aPart.getTypeName());
      else if(aPart.getElementName() != null)
      {
         isElement = true;
         ElementDecl element = aDef.findElement(aPart.getElementName());
         if(element != null)
            type = element.getType();
      }
      
      if (type != null)
         mPartMap.put(aPart.getName(), new AePartTypeInfo(aPart, type, isElement));
      else
         throw new AeException(AeMessages.getString("AeVariableDef.ERROR_3") + aPart.getName());          //$NON-NLS-1$
   }
   
   /**
    * Returns the XML type for the given message part. 
    * @param aPartName the part name we are looking for
    * @throws AeBusinessProcessException when unknown part is requested
    */
   public XMLType getPartType(String aPartName) throws AeBusinessProcessException
   {
      AePartTypeInfo partType = (AePartTypeInfo)mPartMap.get(aPartName);
      if (partType == null)
         throw new AeBusinessProcessException(AeMessages.getString("AeVariableDef.ERROR_4") + aPartName); //$NON-NLS-1$
      
      return partType.getType();
   }
   
   /**
    * Returns the WSDL Part for the given message part. 
    * @param aPartName the part name we are looking for
    * @throws AeBusinessProcessException when unknown part is requested
    */
   public Part getPart(String aPartName) throws AeBusinessProcessException
   {
      AePartTypeInfo partType = (AePartTypeInfo)mPartMap.get(aPartName);
      if (partType == null)
         throw new AeBusinessProcessException(AeMessages.getString("AeVariableDef.ERROR_4") + aPartName); //$NON-NLS-1$

      return partType.getPart();
   }
   
   /**
    * Returns a flag indicating if the given message part is an element. 
    * @param aPartName the part name we are looking for
    * @throws AeBusinessProcessException when unknown part is requested
    */
   public boolean isPartElement(String aPartName) throws AeBusinessProcessException
   {
      AePartTypeInfo partType = (AePartTypeInfo)mPartMap.get(aPartName);
      if (partType == null)
         throw new AeBusinessProcessException(AeMessages.getString("AeVariableDef.ERROR_4") + aPartName); //$NON-NLS-1$

      return partType.isElement();
   }
   
   /**
    * Getter for the implicit flag
    */
   public boolean isImplicit()
   {
      return mImplicit;
   }
   
   /**
    * Setter for the implicit flag
    * 
    * @param aImplicit
    */
   public void setImplicit(boolean aImplicit)
   {
      mImplicit = aImplicit;
   }
}
   
/**
 * Inner helper class to manage the Message Part type definition data.
 */
class AePartTypeInfo
{
   /** The WSDL Part which represents this message part.*/
   private Part mPart;
   /** The XML data type which represents the message part*/
   private XMLType mType;
   /** Flag indicating if message part is an element or type */
   private boolean mIsElement;
      
   /**
    * Constructor used to create and initialize the part type information
    * of a message part.  
    * @param aPart the WSDL Part which defines the object
    * @param aType the XML data type which defines the object
    * @param aIsElement flag indicating if part is a type of element.
    */
   public AePartTypeInfo(Part aPart, XMLType aType, boolean aIsElement)
   {
      mPart = aPart;
      mType = aType;
      mIsElement = aIsElement;        
   }

   /**
    * Returns the WSDL Part object for the message part.
    */
   public Part getPart()
   {
      return mPart;
   }

   /**
    * Returns the XML data type for the message part.
    */
   public XMLType getType()
   {
      return mType;
   }

   /**
    * Returns True if this part is an element and False if it is a type. 
    */
   public boolean isElement()
   {
      return mIsElement;
   }
}
