// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivityAssignDef.java,v 1.4 2004/07/08 13:09:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Definition for bpel assign activity.
 */
public class AeActivityAssignDef extends AeActivityDef
{
   // standard elements of the activity definition
   private ArrayList mCopies = new ArrayList();

   /**
    * Default constructor
    */
   public AeActivityAssignDef()
   {
   }

   /**
    * Provides the ability to add a copy element to the assign element.
    * 
    * @param aAssignCopyDef copy element to be added
    */
   public void addCopy(AeAssignCopyDef aCopy)
   {
      mCopies.add(aCopy);
   }

   /**
    * Provides the ability to delete a copy element from the assign element.
    * 
    * @param aCopy the copy element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeCopy(AeAssignCopyDef aCopy)
   {
      boolean found = false;
      
      Iterator iter = getCopyList();
      while (iter.hasNext() && !found)
      {
         if (aCopy == (AeAssignCopyDef)iter.next())
         {
            iter.remove();
            found = true;
         }
      }
      
      return found;
   }

   /**
    * Provide a list of the Copy objects for the user to iterate .
    * 
    * @return iterator of AeAssignCopyDef objects
    */
   public Iterator getCopyList()
   {
      return mCopies.iterator();
   }

   /**
    * @see org.activebpel.rt.bpel.def.AeActivityDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }
}
