// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivityFlowDef.java,v 1.4 2004/07/08 13:09:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.IAeMultipleActivityContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

/**
 * Definition for bpel flow activity.
 */
public class AeActivityFlowDef extends AeActivityDef implements IAeMultipleActivityContainerDef
{
   // standard elements of the flow activity definition
   private ArrayList mActivities = new ArrayList();  // Will always be at least one

   /** Container used to store links for Flow. */
   private AeLinkContainerDef mLinks;

   /**
    * Default constructor
    */
   public AeActivityFlowDef()
   {
   }
   
   /**
    * Getter for the link container. Will create the container if it's null.
    */
   public AeLinkContainerDef getLinkContainer()
   {
      if (mLinks == null)
      {
         mLinks = new AeLinkContainerDef();
      }
      return mLinks;
   }
   
   /**
    * Provides the ability to add a Link to the Flow activity.
    * 
    * @param aLink the link to be added.
    */
   public void addLink(AeLinkDef aLink)
   {
      getLinkContainer().add(aLink);
   }

   /**
    * Provides the ability to remove a Link from a Flow activity.
    * 
    * @param aLink the Link element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeLink(AeLinkDef aLink)
   {
      return getLinkContainer().remove(aLink);
   }

   /**
    * Provide a list of the Link objects for the user to iterate .
    * 
    * @return iterator of AeLinkDef objects 
    */
   public Iterator getLinkList()
   {
      if (mLinks == null)
         return Collections.EMPTY_LIST.iterator();
      else
         return mLinks.getLinks();
   }

   /**
    * Adds an activity definition to the list of activities to execute.
    * @param aActivity the link to be added.
    */
   public void addActivity(AeActivityDef aActivity)
   {
      mActivities.add(aActivity);
   }

   /**
    * Provides the ability to delete an activity.
    * @param aActivity the activity to remove from container
    * @return boolean status indicating if element was removed
    */
   public boolean removeActivity(AeActivityDef aActivity)
   {
      boolean found = false;
      
      Iterator iter = getActivityList();
      while (iter.hasNext() && !found)
      {
         if (aActivity == (AeActivityDef)iter.next())
         {
            iter.remove();
            found = true;
         }
      }
      
      return found;
   }

   /**
    * Provide a list of the activity elements for the user to iterate .
    * @return iterator of AeActivityDef objects
    */
   public Iterator getActivityList()
   {
      return mActivities.iterator();
   }

   /**
    * @see org.activebpel.rt.bpel.def.AeActivityDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }
}
