//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivityForEachDef.java,v 1.9 2006/01/11 21:01:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity; 

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.IAeSingleActivityContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;
import org.activebpel.rt.util.AeUtil;

/**
 * Definition object for the "forEach" activity. This activity provides a way of
 * executing a child scope multiple times in sequence or in parallel.
 */
public class AeActivityForEachDef extends AeActivityDef implements IAeSingleActivityContainerDef, IAeLoopActivityDef
{
   /** optional completion condition for serial and parallel forEach */
   private AeForEachCompletionConditionDef mCompletionCondition;
   /** for each defs can only contain scope activities */
   private AeActivityScopeDef mChildDef;
   /** true if the foreach should execute the scope in parallel or false if sequentially */
   private boolean mParallel;
   
   /** start def contains expression that evaluates to the starting value used for the loop */
   private AeForEachStartDef mStartDef;
   /** final def contains expression that evaluates to the final value used for the loop */
   private AeForEachFinalDef mFinalDef;

   /** name of the counter variable (which is always an xs:unsignedInt */
   private String mCounterName;

   /**
    * @return Returns the completionCondition.
    */
   public AeForEachCompletionConditionDef getCompletionCondition()
   {
      return mCompletionCondition;
   }
   
   /**
    * Return true if the def contains the optional completion def
    */
   public boolean hasCompletionCondition()
   {
      return mCompletionCondition != null;
   }
   
   /**
    * @return Returns the finalDef.
    */
   public AeForEachFinalDef getFinalDef()
   {
      if (mFinalDef == null)
         mFinalDef = new AeForEachFinalDef();
      return mFinalDef;
   }

   /**
    * @param aFinalDef The finalDef to set.
    */
   public void setFinalDef(AeForEachFinalDef aFinalDef)
   {
      mFinalDef = aFinalDef;
   }
   
   /**
    * @return Returns the startDef.
    */
   public AeForEachStartDef getStartDef()
   {
      if (mStartDef == null)
         mStartDef = new AeForEachStartDef();
      return mStartDef;
   }
   
   /**
    * @param aStartDef The startDef to set.
    */
   public void setStartDef(AeForEachStartDef aStartDef)
   {
      mStartDef = aStartDef;
   }
   
   /**
    * @param aCompletionCondition The completionCondition to set.
    */
   public void setCompletionCondition(AeForEachCompletionConditionDef aCompletionCondition)
   {
      mCompletionCondition = aCompletionCondition;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

   /**
    * @see org.activebpel.rt.bpel.def.IAeSingleActivityContainerDef#getActivity()
    */
   public AeActivityDef getActivity()
   {
      return mChildDef;
   }

   /**
    * @see org.activebpel.rt.bpel.def.IAeSingleActivityContainerDef#setActivity(org.activebpel.rt.bpel.def.AeActivityDef)
    */
   public void setActivity(AeActivityDef aActivity)
   {
      mChildDef = (AeActivityScopeDef) aActivity;
   }
   
   /**
    * @return Returns the parallel.
    */
   public boolean isParallel()
   {
      return mParallel;
   }
   
   /**
    * @param aParallel The parallel to set.
    */
   public void setParallel(boolean aParallel)
   {
      mParallel = aParallel;
   }

   /**
    * @return Returns the counterName.
    */
   public String getCounterName()
   {
      return mCounterName;
   }

   /**
    * @param aCounterName The counterName to set.
    */
   public void setCounterName(String aCounterName)
   {
      mCounterName = aCounterName;
   }

   /**
    * Convenience method that returns the child scope for the forEach.
    */
   public AeActivityScopeDef getChildScope()
   {
      return (AeActivityScopeDef) getActivity();
   }
   
   /**
    * Helper method for stripping any and all "[instance()=XXX]" substrings from the given node path. 
    * @param aNodePath
    * @return the node path minus any instance() info.
    */
   public static String removeInstanceInfo(String aNodePath)
   {
      // TODO (MF) store as a pattern and create matcher
      return aNodePath.replaceAll("\\[instance\\( *\\)=[0-9]+\\]", "");  //$NON-NLS-1$//$NON-NLS-2$
   }
   
   /**
    * Helper method. If the given node path is terminated with an instance number, ie "[instance()=XXX]", 
    * then that instance number is return otherwise -1 is returned.
    * @param aNodePath
    * @return int the node path's intance number or -1 if not specified.
    */
   public static int getNodePathInstanceNum(String aNodePath)
   {
      final String INSTANCE_TOKEN = "[instance()="; //$NON-NLS-1$
      
      if ( AeUtil.isNullOrEmpty(aNodePath) )
        return -1;
      
      int start = aNodePath.lastIndexOf(INSTANCE_TOKEN);
      
      if ( start != -1 )
      {
         int end = aNodePath.substring(start).indexOf("]"); //$NON-NLS-1$
         if ( end != -1 )
         {
            String instStr = aNodePath.substring( start + INSTANCE_TOKEN.length(), start+end);
            try 
            {
               Integer instInt = Integer.valueOf(instStr);
               return instInt.intValue();
            }
            catch (NumberFormatException e) 
            {
            }
         }
      }
      return -1;
   }

   /**
    * Converts the paths in the collection to include instance information
    * if the referenced fall within a parallel forEach scope.
    * 
    * Use case:
    * 
    * The variables that need to be locked for a given activity are determined by
    * visiting the process def when the process definition is first loaded from the
    * src xml. This process does not take into account that a variable might be 
    * declared within a parallel forEach. If this variable is declared within a 
    * parallel forEach, then the actual runtime path to the variable will include
    * the instance information. The conversion here is necessary in order to insert
    * the instance info into the path so we can be sure that we're locking the proper
    * variable.
    * 
    * i.e. Consider the following collection of variable paths to lock:
    * 
    *    /process/variables/variable[@name='p1_v1']
    *    /process/forEach/scope[@name='s1']/variables/variable[@name='s1_v1'] 
    *    /process/forEach/scope[@name='s1']/sequence/scope[@name='s2']/variables/variable[@name='s2_v1']
    *    /process/forEach/scope[@name='s1']/sequence/scope[@name='s2']/flow/scope[@name='s3']/variables/variable[@name='s3_v1']
    *
    *    The variable p1_v1 is declared at the process level and therefore does
    *    need to change. We only need to change variables declared within scopes
    *    that are enclosed within a parallel forEach. 
    *    
    *    The paths to s1_v1, s2_v1, s3_v1 all need the instance info for the parallel
    *    forEach instance scope. As such, the collection returned will contain
    *    the following paths:  
    * 
    *    /process/variables/variable[@name='p1_v1']
    *    /process/forEach/scope[@name='s1'][instance()=1]/variables/variable[@name='s1_v1'] 
    *    /process/forEach/scope[@name='s1'][instance()=1]/sequence/scope[@name='s2']/variables/variable[@name='s2_v1'] 
    *    /process/forEach/scope[@name='s1'][instance()=1]/sequence/scope[@name='s2']/flow/scope[@name='s3']/variables/variable[@name='s3_v1']  
    *                                      ^------------^: notice the instance info  
    * @param aSetOfPaths
    * @param aActivityPath - should contain instance information
    */
   public static Set addInstanceInfo(Set aSetOfPaths, String aActivityPath)
   {
      Set set = null;
      if (!AeUtil.isNullOrEmpty(aSetOfPaths))
      {
         set = new HashSet();
         
         // split the acticity path into a list of paths that identify all of the
         // enclosed scopes.
         List scopePaths = splitScopes(aActivityPath);
         
         // TODO (MF) I could do some of this work during the loading of the def, 
         //           but doesn't quite seem worth it. It's simple string pushing
         //           and probably not something that'll bubble up in the profiler.
         //           If it were to, then I could determine which variables in the 
         //           collection were declared w/in parallel forEach's and then 
         //           modify only them.
         
         // create a map of the scope paths to their non-instance paths. The 
         // paths in the collection will NOT include any instance information
         // Using a map to avoid having to run removeInstanceInfo multiple times
         // for the same scope.
         Map scopePathsToNonInstancePaths = new HashMap();
         for(Iterator it = scopePaths.iterator(); it.hasNext();)
         {
            String path = (String) it.next();
            scopePathsToNonInstancePaths.put(path, removeInstanceInfo(path));
         }

         // walk all of the paths in the collection and see if they are declared
         // in any of the nested parallel forEach scope paths
         for (Iterator iter = aSetOfPaths.iterator(); iter.hasNext();)
         {
            String path = (String) iter.next();
            
            // if we don't find a match against the scope paths then this variable
            // is declared outside of the parallel forEach and its current path
            // is correct.
            boolean foundMatch = false;
            for (Iterator iter2 = scopePaths.iterator(); !foundMatch && iter2.hasNext();)
            {
               String scopePath = (String) iter2.next();
               String nonInstanceLastScopePath = (String) scopePathsToNonInstancePaths.get(scopePath);
               
               if (path.startsWith(nonInstanceLastScopePath))
               {
                  // add the instance info to the variable path
                  set.add(scopePath + path.substring(nonInstanceLastScopePath.length()));
                  // record that we've matched this variable to break the loop
                  foundMatch = true;
               }
            }
            
            if (!foundMatch)
            {
               set.add(path);
            }
         }
      }
      else
      {
         set = aSetOfPaths;
      }
      return set;
   }
   
   /**
    * Splits the activity path into a series of paths to each of its scopes - starting
    * with the first parallel forEach scope.
    * 
    * The following path:
    * 
    *    <code>/process/sequence/scope/flow/forEach/scope[@name='s1'][instance()=1]/sequence/scope[@name='s2']/sequence/assign</code>
    * 
    *    would be split into a list like this:
    * 
    *    <code>/process/sequence/scope/flow/forEach/scope[@name='s1'][instance()=1]/sequence/scope[@name='s2']</code>
    *    <code>/process/sequence/scope/flow/forEach/scope[@name='s1'][instance()=1]</code>
    * 
    * @param aActivityPath
    */
   public static List splitScopes(String aActivityPath)
   {
      LinkedList list = new LinkedList();
      int offset = -1;
      
      // We only care about variables declarations nested within parallel forEach's
      // Therefore, start our search from the first occurrence of a parallel forEach scope
      int startFrom = aActivityPath.lastIndexOf('/', aActivityPath.indexOf("[instance()=")); //$NON-NLS-1$
      while((offset = aActivityPath.indexOf("/scope", startFrom)) != -1) //$NON-NLS-1$
      {
         int endOfScopePath = aActivityPath.indexOf('/', offset+1);
         if (endOfScopePath == -1)
         {
            // must be the end of the string
            endOfScopePath = aActivityPath.length();
         }
         String path = aActivityPath.substring(0, endOfScopePath);
         list.addFirst(path);
         startFrom = endOfScopePath;
      }
      return list;
   }
   
   /**
    * Returns the immediate instance scope path of the given node path. 
    * 
    * E.g.
    * 
    * aNodePath     "/process/flow/forEach[@name='ForEach']/scope[instance()=2]/sequence/wait"
    *   will return "/process/flow/forEach[@name='ForEach']/scope[instance()=2]"
    * 
    * @param aNodePath a model node path.
    * @return String the immediate instance scope path of the given node path or null
    *    if the node path doesn't contain a scope instance.
    */
   public static String getScopeNodePath(String aNodePath)
   {
      String scopeNodePath = null;
      
      int start = aNodePath.lastIndexOf( "[instance()=" ); //$NON-NLS-1$
      if ( start != -1 )
      {
         int end = aNodePath.substring(start).indexOf("]"); //$NON-NLS-1$
         if ( end != -1 )
            scopeNodePath = aNodePath.substring( 0, start+end+1); //$NON-NLS-1$
      }
      return scopeNodePath;
   }

   /**
    * Given some root path with instance info and some other path that is nested
    * other that root, return a version of the second path that includes the instance
    * information.
    *
    * e.g. given the following paths
    *    ancestor = /process/forEach/scope[@name='s1']/sequence/forEach/scope[@name='s4'][instance()=1]
    *    path     = /process/forEach/scope[@name='s1']/sequence/forEach/scope[@name='s4']/sequence/forEach/scope[@name='s5']/invoke[@name='I']
    * return the following:
    *               /process/forEach/scope[@name='s1']/sequence/forEach/scope[@name='s4'][instance()=1]/sequence/forEach/scope[@name='s5']/invoke[@name='I']
    *                  ------------* notice that the instance info has been added -----* ^------------^
    *   
    * @param aAncestorNodeWithInstanceInfo
    * @param aPathThatNeedsInstanceInfo
    */
   public static String addInstanceInfo(String aAncestorNodeWithInstanceInfo, String aPathThatNeedsInstanceInfo)
   {
      if (AeUtil.isNullOrEmpty(aAncestorNodeWithInstanceInfo))
      {
         return aPathThatNeedsInstanceInfo;
      }
      else
      {
         Set set = addInstanceInfo(Collections.singleton(aPathThatNeedsInstanceInfo), aAncestorNodeWithInstanceInfo);
         return (String) set.iterator().next();
      }
   }
   
   /**
    * Given the current instance value for a parallel forEach and its start and
    * final values, calculate what the instance value was for the starting instance
    * (if applicable).
    * 
    * i.e. paths for a parallel forEach include instance info. If a parallel forEach
    *      evaluates its start and final expressions to 11 and 13 respectively, then
    *      the paths created for the scopes will be as follows:
    * 
    *      /process/flow/forEach/scope[instance()=1]
    *      /process/flow/forEach/scope[instance()=2]
    *      /process/flow/forEach/scope[instance()=3]
    *
    *      another execution of the same parallel forEach with start and final 
    *      expressions of 11 and 13 will generate the following paths 
    * 
    *      /process/flow/forEach/scope[instance()=4]
    *      /process/flow/forEach/scope[instance()=5]
    *      /process/flow/forEach/scope[instance()=6]
    * 
    * @param aInstanceValue
    * @param aStart
    * @param aFinal
    */
   public static int getStartInstanceValue(int aInstanceValue, int aStart, int aFinal)
   {
      if (aStart <= aFinal && aStart != -1)
      {
         return aInstanceValue - (aFinal - aStart) - 1;
      }
      else
      {
         return -1;
      }
   }
   
   /**
    * Given the current instance value for a parallel forEach and its start and
    * final values, calculate what the instance value was for the final instance
    * (if applicable).
    * 
    * @param aInstanceValue
    * @param aStart
    * @param aFinal
    */
   public static int getFinalInstanceValue(int aInstanceValue, int aStart, int aFinal)
   {
      if (aStart <= aFinal && aStart != -1)
      {
         return getStartInstanceValue(aInstanceValue, aStart, aFinal) + (aFinal - aStart);
      }
      else
      {
         return -1;
      }
   }
   
   /**
    * Helper routine to get the path to the forEach.
    * @param aScopeInstancePath
    */
   public static String getForEachPath(String aScopeInstancePath)
   {
      int idx = aScopeInstancePath.lastIndexOf('/');
      return aScopeInstancePath.substring(0, idx);
   }
}
