// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivityInvokeDef.java,v 1.9 2006/01/03 19:55:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.AeActivityPartnerLinkBaseDef;
import org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef;
import org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.activity.support.AeSourceDef;
import org.activebpel.rt.bpel.def.activity.support.AeTargetDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;
import org.activebpel.rt.util.AeUtil;

/**
 * Definition for bpel invoke activity.
 */
public class AeActivityInvokeDef extends AeActivityPartnerLinkBaseDef
{
   private String mInputVariable;
   private String mOutputVariable;

   /** The scope which holds fault and compensation information if necessary */
   private AeActivityScopeDef mScope;
   
   /**
    * Default constructor
    */
   public AeActivityInvokeDef()
   {
   }
   
   /**
    * Gets the inbound correlation defs needed to correlate this invoke.
    */
   public Iterator getInboundCorrelationDefs()
   {
      List list = null;
      for(Iterator iter = getCorrelationList(); iter.hasNext(); )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         if (corrDef.usesInboundDataForCorrelation())
         {
            list = addToList(corrDef, list);
         }
      }
      
      if (list == null)
         list = Collections.EMPTY_LIST;

      return list.iterator();
   }
   
   
   /**
    * Gets the outbound correlation defs needed to correlate this invoke.
    */
   public Iterator getOutboundCorrelationDefs()
   {
      List list = null;
      for(Iterator iter = getCorrelationList(); iter.hasNext(); )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         if (corrDef.usesOutboundDataForCorrelation())
            list = addToList(corrDef, list);
      }
      
      if (list == null)
         list = Collections.EMPTY_LIST;

      return list.iterator();
   }
   
   /**
    * Adds to the list passed in, returning a new list if the list param was null
    * @param aObject
    * @param aListToAddTo
    */
   protected List addToList(Object aObject, List aListToAddTo)
   {
      if (aListToAddTo == null)
         aListToAddTo = new ArrayList();
      aListToAddTo.add(aObject);
      return aListToAddTo;
   }
   

   /**
    * @see org.activebpel.rt.bpel.def.AeActivityDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

   /**
    * Returns the input Variable of the invoke activity.
    * @return input variable of invoke activity
    */
   public final String getInputVariable()
   {
      return mInputVariable;
   }

   /**
    * Sets the input variable of the invoke activity.
    * @param aInputVariable input variable of the invoke activity 
    */
   public final void setInputVariable(String aInputVariable)
   {
      mInputVariable = aInputVariable;
   }

   /**
    * Returns the output variable of the invoke activity.
    * @return output variable of invoke activity
    */
   public final String getOutputVariable()
   {
      return mOutputVariable;
   }

   /**
    * Sets the output variable of the invoke activity.
    * @param aOutputVariable output variable of the invoke activity 
    */
   public final void setOutputVariable(String aOutputVariable)
   {
      mOutputVariable = aOutputVariable;
   }

   /**
    * Take over base implementation so we may add a scope around the invoke 
    * activity if necessary.
    * @return activity definition object or scope object if necessary
    * @see org.activebpel.rt.bpel.def.AeActivityDef#getScopedActivity()
    */
   public AeActivityDef getScopedActivity()
   {
      // If an implicit scope was never created, no need to wrap the invoke in a scope
      if (getImplicitScope() == null)
         return this;
      else
      {
         // Move any source link references from the invoke to the scope
         for (Iterator iter=getSourceList(); iter.hasNext();)
            getImplicitScope().addSource((AeSourceDef)iter.next());
         
         // Move any target link references from the invoke to the scope
         for (Iterator iter=getTargetList(); iter.hasNext();)
            getImplicitScope().addTarget((AeTargetDef)iter.next());

         // Clear source and target links from the invoke
         clearSourceList();
         clearTargetList();
         
         // Set the invoke as the activity of the scope and return newly created scope
         getImplicitScope().setActivity(this);
         getImplicitScope().setName(getName());
         setName(""); //$NON-NLS-1$
         return getImplicitScope();
      }
   }

   /**
    * Returns the scope activity or null if none was created.  
    */
   public AeActivityScopeDef getImplicitScope()
   {
      return mScope;
   }

   /**
    * Sets the scope which is contains the fault and compensation elements to be
    * used to serialize an implicit scope.
    * @param aScope the scope to be set or null to clear  
    */
   public void setImplicitScope(AeActivityScopeDef aScope)
   {
      mScope = aScope;
   }
   
   /**
    * Creates an implicit scope for use by the invoke if one has not already been created. 
    */
   public AeActivityScopeDef createImplicitScope()
   {
      if (mScope == null)
      {
         mScope = new AeActivityScopeDef();
         mScope.setImplicitScope(true);
         mScope.setName(""); //$NON-NLS-1$
      }
      return mScope;
   }
   
      
   /**
    * Create implicit scope (if one doesn't extist) and return its 
    * compenstator container def.
    * @return implicit scope's compensator container def
    */
   public AeScopeCompensatorContainerDef getCompensatorContainer()
   {
      return createImplicitScope().getScope().getCompensatorContainer();
   }
   
   /**
    * Create implicit scope (if one doesn't extist) and return its 
    * fault handler container def.
    * @return implicit scope's fault handler container def
    */
   public AeFaultHandlersContainerDef getFaultHandlersContainer()
   {
      return createImplicitScope().getScope().getFaultHandlerContainer();
   }

   /**
    * If invoke has implicit scope, add source to it, otherwise
    * add source to invoke directly.
    * @see org.activebpel.rt.bpel.def.AeActivityDef#addSource(org.activebpel.rt.bpel.def.activity.support.AeSourceDef)
    */
   public void addSource(AeSourceDef aSource)
   {
      if( getImplicitScope() != null )
      {
         getImplicitScope().addSource( aSource );
      }
      else
      {
         super.addSource(aSource);
      }
   }

   /**
    * If invoke has implicit scope, add target to it, otherwise
    * add target to invoke directly.
    * @see org.activebpel.rt.bpel.def.AeActivityDef#addTarget(org.activebpel.rt.bpel.def.activity.support.AeTargetDef)
    */
   public void addTarget(AeTargetDef aTarget)
   {
      if( getImplicitScope() != null )
      {
         getImplicitScope().addTarget( aTarget );
      }
      else
      {
         super.addTarget(aTarget);
      }
   }
   
   /**
    * Returns true if this is a one-way invoke.
    */
   public boolean isOneWay()
   {
      return AeUtil.isNullOrEmpty(getOutputVariable());
   }   
}
