// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivityScopeDef.java,v 1.7 2006/01/11 21:01:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef;
import org.activebpel.rt.bpel.def.AeMessageExchangesDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.IAeMessageExchangesParent;
import org.activebpel.rt.bpel.def.IAeSingleActivityContainerDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

/**
 * Definition for bpel scope activity.
 */
public class AeActivityScopeDef extends AeActivityDef 
   implements IAeSingleActivityContainerDef, IAeMessageExchangesParent
{
   // standard attributes of the throw activity definition
   private boolean mVariableAccessSerializable;

   // standard elements of the activity definition
   private AeScopeDef mScope = new AeScopeDef();
   
   /** Flag to indicate if the scope is an implicit scope */
   private boolean mImplicitScope;
   
   /**
    * Default constructor
    */
   public AeActivityScopeDef()
   {
   }

   /**
    * Visitor implementation for scope definition.
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);      
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.IAeMessageExchangesParent#setMessageExchangesDef(org.activebpel.rt.bpel.def.AeMessageExchangesDef)
    */
   public void setMessageExchangesDef(AeMessageExchangesDef aDef)
   {
      getScope().setMessageExchangesDef(aDef);
   }

   /**
    * Returns a flag indicating if this scope activity is implicit and was not
    * defined by the user.
    */
   public boolean isImplicitScope()
   {
      return mImplicitScope;
   }
   
   /**
    * Sets this scope activity as an implicit scope if set to True. This is used
    * by invoke activities which use the scope to hold fault and compensation
    * elements.
    * @param aFlag True to set as implicit False otherwise
    */
   public void setImplicitScope(boolean aFlag)
   {
      mImplicitScope = aFlag;
   }

   /**
    * Accessor method to obtain the flag indicating if variable access is serializable.
    * 
    * @return flag indicating if variable access is serializable
    */
   public final boolean getVariableAccessSerializable()
   {
      return mVariableAccessSerializable;
   }

   /**
    * Mutator method to set the flag indicating if variable access is serializable.
    * 
    * @param aVariableAccessSerializable boolean flag indicating if variable  
    *         access is serializable
    */
   public final void setVariableAccessSerializable(boolean aVariableAccessSerializable)
   {
      mVariableAccessSerializable = aVariableAccessSerializable;
   }

   /**
    * Returns the underlying scope definition.
    * @return AeScopeDef
    */
   public AeScopeDef getScope()
   {
      return mScope;
   }

   /**
    * Obtains the current activity associated with this object.
    * 
    * @return an activity associated with this object
    * @see AeActivityDef
    */
   public final AeActivityDef getActivity()
   {
      return getScope().getActivity();
   }

   /**
    * Set the activity to execute if this scope is called
    * 
    * @param aActivity activity to set
    * @see AeActivityDef
    */
   public final void setActivity(AeActivityDef aActivity)
   {
      getScope().setActivity(aActivity);
   }
   

   /**
    * @return fault handler container
    */
   public AeFaultHandlersContainerDef getFaultHandlerContainer()
   {
      return getScope().getFaultHandlerContainer();
   }
}
