// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivitySequenceDef.java,v 1.5 2004/08/10 19:09:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import java.util.ArrayList;
import java.util.Iterator;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.IAeMultipleActivityContainerDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

/**
 * Definition for bpel sequence activity.
 */
public class AeActivitySequenceDef extends AeActivityDef implements IAeMultipleActivityContainerDef
{
   // standard elements of the activity definition
   private ArrayList mActivities = new ArrayList();

   /**
    * Default constructor
    */
   public AeActivitySequenceDef()
   {
   }

   /**
    * Adds an activity definition to the list of activities to execute.
    * 
    * @param aActivity activity to be added
    */
   public void addActivity(AeActivityDef aActivity)
   {
      mActivities.add(aActivity);
   }

   /**
    * Provides the ability to delete an activity from the sequence.
    * 
    * @param aActivity the activity to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeActivity(AeActivityDef aActivity)
   {
      boolean found = false;
      
      Iterator iter = getActivityList();
      while (iter.hasNext() && !found)
      {
         if (aActivity == (AeActivityDef)iter.next())
         {
            iter.remove();
            found = true;
         }
      }
      
      return found;
   }

   /**
    * Returns an iterator of activity definitions to iterate over.
    * 
    * @return iterator over activity list
    */
   public Iterator getActivityList()
   {
      return mActivities.iterator();
   }

   /**
    * @see org.activebpel.rt.bpel.def.AeActivityDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }
}
