// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivitySwitchDef.java,v 1.4 2004/07/08 13:09:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

import java.util.Iterator;

/**
 * Definition for bpel switch activity.
 */
public class AeActivitySwitchDef extends AeActivityDef
{
   // standard elements of the activity definition
   private AeSwitchCaseContainerDef mCases;
   
   /** Otherwise condition (akin to default: in a switch/case)*/
   private AeOtherwiseDef mOtherwise;

   /**
    * Default constructor
    */
   public AeActivitySwitchDef()
   {
   }
   
   /**
    * Getter for switch case container. Creates an instance if it's null.
    */
   public AeSwitchCaseContainerDef getSwitchCaseContainer()
   {
      if (mCases == null)
      {
         mCases = new AeSwitchCaseContainerDef();
      }
      return mCases;
   }

   /**
    * Adds a case to the switch activity.
    * @param aCase case object to be added
    */
   public void addCase(AeSwitchCaseDef aCase)
   {
      getSwitchCaseContainer().add(aCase);
   }

   /**
    * Provides the ability to delete a case element from the Switch activity.
    * 
    * @param aCase the Case element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeCase(AeSwitchCaseDef aCase)
   {
      return getSwitchCaseContainer().remove(aCase);
   }

   /**
    * Provide a list of the Case objects for the user to iterate .
    * 
    * @return iterator of AeSwitchCaseDef objects
    */
   public Iterator getCaseList()
   {
      return getSwitchCaseContainer().getValues();
   }

   /**
    * Gets the otherwise clause activity.
    * 
    * @return otherwise clause object associated with switch activity
    */
   public final AeOtherwiseDef getOtherwise()
   {
      return mOtherwise;
   }

   /**
    * Sets the otherwise clause activity.
    * 
    * @param aOtherwise otherwise clause to be associated with the switch activity
    */
   public final void setOtherwise(AeOtherwiseDef aOtherwise)
   {
      mOtherwise = aOtherwise;
   }

   /**
    * @see org.activebpel.rt.bpel.def.AeActivityDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }
}

