// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/AeActivityWaitDef.java,v 1.6 2005/02/01 19:53:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

/**
 * Definition for bpel wait activity.
 */
public class AeActivityWaitDef extends AeActivityDef implements IAeTimedDef
{
   // standard attributes of the wait activity definition
   private String mFor;
   private String mUntil;

   // standard element names of the activity definition
   public static final String TAG_WAIT = "wait"; //$NON-NLS-1$

   /**
    * Default constructor
    */
   public AeActivityWaitDef()
   {
   }

   /**
    * Accessor method to obtain the For attribute.
    * 
    * @return name of For attribute
    */
   public final String getFor()
   {
      return mFor;
   }

   /**
    * Mutator method to set the For attribute.
    * 
    * @param aFor the For attribute to be set
    */ 
   public final void setFor(String aFor)
   {
      // Only one can be active at a time, so zap the other
      if (aFor != null && aFor.length() > 0)
      {
	      mFor   = aFor;
	      mUntil = null;
      }
   }

   /**
    * Accessor method to obtain the Until attribute.
    * 
    * @return name of Until attribute
    */
   public final String getUntil()
   {
      return mUntil;
   }

   /**
    * Mutator method to set the Until attribute.
    * 
    * @param aFor the Until attribute to be set
    */ 
   public final void setUntil(String aUntil)
   {
      // Only one can be active at a time, so zap the other
      if (aUntil != null && aUntil.length() > 0)
      {
	      mUntil = aUntil;
	      mFor   = null;
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.AeActivityDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }
}
