// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/support/AeAssignFromDef.java,v 1.4 2005/10/12 19:26:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity.support;

import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;
import org.activebpel.rt.util.AeUtil;
import org.w3c.dom.Node;

/**
 * Models the <code>from</code> element in a copy operation. Broke this out as
 * a separate class so it can be explicitly visited.
 */
public class AeAssignFromDef extends AeAssignVarDef
{
   /** The value of the expression attribute (if any). */
   private String mExpression;
   /** The value of the opaque attribute (if any). */
   private boolean mOpaque;
   /** The value of the literal (if any). */
   private Node mLiteral;
   /** The value of the endpoint reference (if any). */
   private String mEndpointReference;

   /**
    * Default ctor
    */
   public AeAssignFromDef()
   {
   }

   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

   /**
    * Accessor method to obtain the expression for the object.
    * 
    * @return name of the expression for the object
    */
   public final String getExpression()
   {
      return mExpression;
   }

   /**
    * Mutator method to set the expression for the object.
    * 
    * @param aExpression the expression value to be set
    */
   public final void setExpression(String aExpression)
   {
      mExpression = aExpression;
   }

   /**
    * Returns the literal Node for the copy variable.
    * @return Node the literal node
    */
   public final Node getLiteral()
   {
      return mLiteral;
   }

   /**
    * Sets the literal node for the for the copy variable.
    * @param aNode the node for the literal.
    */
   public final void setLiteral(Node aNode)
   {
      mLiteral = aNode;
   }

   /**
    * Accessor method to obtain the opaque flag.
    * 
    * @return flag indicating if assignment variable is opaque
    */
   public final boolean getOpaque()
   {
      return mOpaque;
   }

   /**
    * Mutator method to set the opaque flag.
    * 
    * @param aOpaque boolean flag indicating if assignment variable is opaque
    */
   public final void setOpaque(boolean aOpaque)
   {
      mOpaque = aOpaque;
   }
   

   /**
    * Accessor method to obtain the endpoint reference for the object.
    * 
    * @return name of the endpoint reference for the object
    */
   public final String getEndpointReference()
   {
      return mEndpointReference;
   }

   /**
    * Mutator method to set the endpoint reference for the object.
    * 
    * @param aEndpointReference the endpoint reference value to be set
    */
   public final void setEndpointReference(String aEndpointReference)
   {
      mEndpointReference = aEndpointReference;
   }

   /**
    * Returns true if this from is empty.  This is used during validation, since the from portion
    * of an assign's copy child should never be empty.
    */
   public boolean isEmpty()
   {
      return AeUtil.isNullOrEmpty(getExpression()) && getLiteral() == null && !getOpaque()
            && AeUtil.isNullOrEmpty(getPartnerLink()) && AeUtil.isNullOrEmpty(getVariable());
   }
}
