// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/support/AeCorrelationDef.java,v 1.12 2005/02/01 19:53:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity.support;

import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

/** Holds individual correlation elements */
public class AeCorrelationDef extends AeBaseDef
{
   // Pattern attribute options
   public static final String PATTERN_IN     = "in"; //$NON-NLS-1$
   public static final String PATTERN_OUT    = "out"; //$NON-NLS-1$
   public static final String PATTERN_OUT_IN = "out-in"; //$NON-NLS-1$

   /** Correlation set name from set attribute. */
   private String mCorrelationSetName;
   /** Correlation patter (out-in, in-out, both) */
   private String mPattern;
   /** true if user should initiate the corresponding correlation set */
   private boolean mInitiate;

   /**
    * Default constructor
    */
   public AeCorrelationDef()
   {
   }

   /**
    * Returns the initiate flag value.
    * 
    * @return the initiation flag (true="yes", false="no")
    */
   public final boolean isInitiate()
   {
      return mInitiate;
   }

   /**
    * Sets the initiate flag.
    * 
    * @param aInitiation initiation flag to set
    */
   public final void setInitiate(boolean aInitiation)
   {
      mInitiate = aInitiation;
   }

   /**
    * Returns the correlation pattern.
    * 
    * @return the correlation pattern
    */
   public final String getPattern()
   {
      return mPattern;
   }

   /**
    * Sets the correlation pattern
    * 
    * @param aPattern pattern to set
    */
   public final void setPattern(String aPattern)
   {
      mPattern = aPattern;
   }

   /**
    * Returns the correlation set name, which identifies the associated
    * correlation set.
    * @return the correlation set attribute
    */
   public final String getCorrelationSetName()
   {
      return mCorrelationSetName;
   }

   /**
    * Sets the correlation set attribute.
    * 
    * @param aSet the name of the correlation set
    */
   public final void setCorrelationSetName(String aSet)
   {
      mCorrelationSetName = aSet;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

   /**
    * Convenience method that returns true if the pattern is either 'in' or 'out-in'.
    * Inbound data is what the invoke receives as a response from the
    * outside world. As per the spec, the inbound data is extracted from the
    * invoke's outputVariable.
    */
   public boolean usesInboundDataForCorrelation()
   {
      return PATTERN_IN.equals(getPattern()) ||
               PATTERN_OUT_IN.equals(getPattern());
   }

   /**
    * Convenience method that returns true if the pattern is either 'out' or 'out-in'.
    * Outbound data is what the invoke sends along with its invocation
    * of the web service. As per the spec, the outbound data is extracted from
    * the invoke's inputVariable.
    */
   public boolean usesOutboundDataForCorrelation()
   {
      return PATTERN_OUT.equals(getPattern()) ||
               PATTERN_OUT_IN.equals(getPattern());
   }

   /**
    * Convenience method for determining if a given correlation set should be 
    * initialized by the timed the activity goes to use it. Any usage where the
    * initiate flag is set to no or not set at all assumes that the correlation
    * set has already been initialized through the execution of some other activity.
    */
   public boolean shouldBeInitialized()
   {
      return !isInitiate();
   }

}

