// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/activity/support/AeOnMessageDef.java,v 1.17 2006/01/03 19:55:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.activity.support;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeCorrelationContainerDef;
import org.activebpel.rt.bpel.def.AePartnerLinkDelegate;
import org.activebpel.rt.bpel.def.IAeActivityCreateInstanceDef;
import org.activebpel.rt.bpel.def.IAeSingleActivityContainerDef;
import org.activebpel.rt.bpel.def.activity.IAeReceiveActivityDef;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;


/** The onMessage element used within the Pick activity */
public class AeOnMessageDef extends AeBaseDef 
   implements IAeSingleActivityContainerDef, IAeReceiveActivityDef, IAeActivityCreateInstanceDef

{
   /** message variable */
   private String mVariable;
   /** on message one-way flag. */
   private boolean mOneWay = false;
   // standard elements of the activity definition
   private AeActivityDef mActivity;   
   /** Message exchange value */
   private String mMessageExchange;
   /** delegate which handle the partner link and correlation information. */
   private AePartnerLinkDelegate mDelegate = new AePartnerLinkDelegate();
   
   /**
    * Default constructor
    */
   public AeOnMessageDef()
   {
   }
   
   /**
    * Returns the delegate.
    */
   protected AePartnerLinkDelegate getDelegate()
   {
      return mDelegate;
   }   

   /**
    * Accessor method to obtain the name of the partner link.
    * 
    * @return name of partner link
    */
   public final String getPartnerLink()
   {
      return getDelegate().getPartnerLink();
   }

   /**
    * Mutator method to set the name of the partner link.
    * 
    * @param aPartnerLink name of partner link
    */
   public final void setPartnerLink(String aPartnerLink)
   {
      getDelegate().setPartnerLink( aPartnerLink );
   }

   /**
    * Accessor method to obtain the port type for the message.
    * 
    * @return port type of message
    */
   public final QName getPortType()
   {
      return getDelegate().getPortType();
   }

   /**
    * Mutator method to set the port type of the message.
    * 
    * @param aPortType port type of the message 
    */
   public final void setPortType(QName aPortType)
   {
      getDelegate().setPortType(aPortType);
   }

   /**
    * Accessor method to obtain the operation for the message.
    * 
    * @return operation of message
    */
   public final String getOperation()
   {
      return getDelegate().getOperation();
   }

   /**
    * Mutator method to set the operation of the message.
    * 
    * @param aOperation operation of the message 
    */
   public final void setOperation(String aOperation)
   {
      getDelegate().setOperation(aOperation);
   }

   /**
    * Accessor method to obtain the variable for the message.
    * 
    * @return variable for message
    */
   public final String getVariable()
   {
      return mVariable;
   }

   /**
    * Mutator method to set the variable for the message.
    * 
    * @param aVariable the variable for the message 
    */
   public final void setVariable(String aVariable)
   {
      mVariable = aVariable;
   }

   /**
    * Accessor method to get the activity associated with the message.
    * 
    * @return activity associated with the message
    */
   public final AeActivityDef getActivity()
   {
      return mActivity;
   }

   /**
    * Mutator method to set the activity associated with the message.
    * 
    * @param aActivity the activity to be set
    */
   public final void setActivity(AeActivityDef aActivity)
   {
      mActivity = aActivity;
   }

   /**
    * Provides the ability to add a correlation element to the correlation list.
    * 
    * @param aCorrelation the correlation to be added
    */
   public void addCorrelation(AeCorrelationDef aCorrelation)
   {
      getDelegate().addCorrelation(aCorrelation);
   }
   
   /**
    * Getter for the correlation container
    */
   public AeCorrelationContainerDef getCorrelationContainer()
   {
      return getDelegate().getCorrelationContainer();
   }

   /**
    * Provides the ability to delete a correlation element from a Message.
    * 
    * @param aCorrelation the Correlation element to be removed
    * @return boolean status indicating if element was removed
    */
   public boolean removeCorrelation(AeCorrelationDef aCorrelation)
   {
      return getDelegate().removeCorrelation(aCorrelation);
   }

   /**
    * Provide a list of the Correlation objects for the user to iterate .
    * 
    * @return iterator of AeCorrelationDef objects or null if none defined
    */
   public Iterator getCorrelationList()
   {
      return getDelegate().getCorrelationList();
   }
   
   /**
    * Utility method to determine if a message has a correlation list 
    * @return true if there are elements in the correlation list.
    */
   public boolean hasCorrelationList()
   {
      return getDelegate().hasCorrelationList();   
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.AeBaseDef#accept(org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void accept(IAeDefVisitor aVisitor)
   {
      aVisitor.visit(this);
   }

   /**
    * Accessor method to obtain the oneWay flag.
    */
   public final boolean isOneWay()
   {
      return mOneWay;
   }
   
   /**
    * Mutator method to set the one way flag for the activity.
    * 
    * @param aOneWay boolean flag indicating if the instance should be one way.
    */
   public final void setOneWay(boolean aOneWay)
   {
      mOneWay = aOneWay;
   }

   /**
    * @return Returns the messageExchange.
    */
   public String getMessageExchange()
   {
      return mMessageExchange;
   }

   /**
    * @param aMessageExchange The messageExchange to set.
    */
   public void setMessageExchange(String aMessageExchange)
   {
      mMessageExchange = aMessageExchange;
   }
   
   /** 
    * Overrides method to return the state of the parent def (Pick) activity's createInstance attribute. 
    * @see org.activebpel.rt.bpel.def.IAeActivityCreateInstanceDef#isCreateInstance()
    */
   public boolean isCreateInstance()
   {
      if ( getParent() instanceof IAeActivityCreateInstanceDef )
      {
         return ( (IAeActivityCreateInstanceDef) getParent()).isCreateInstance();
      }
      else
      {
         return false;
      }
   }
}

