//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/expr/AeExpressionLanguageUtil.java,v 1.2 2006/01/10 22:18:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.expr;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.def.IAeBPELConstants;

/**
 * This class has some static convenience methods for doing various tasks related to expressions
 * and expression languages.
 */
public class AeExpressionLanguageUtil
{
   /** The BPEL extension function used retrieve a variable property */
   public final static String VAR_PROPERTY_FUNC_NAME = "getVariableProperty"; //$NON-NLS-1$
   /** The BPEL extension function used retrieve variable data */
   public final static String VAR_DATA_FUNC_NAME = "getVariableData"; //$NON-NLS-1$
   /** The BPEL extension function used retrieve variable data */
   public final static String LINK_STATUS_FUNC_NAME = "getLinkStatus"; //$NON-NLS-1$
   /** The BPEL extension function used retrieve myRole properties */
   public final static String MYROLE_PROP_FUNC_NAME = "getMyRoleProperty"; //$NON-NLS-1$
   /** The fully qualified name of the getVariableProperty function. */
   public final static QName VAR_PROPERTY_FUNC = new QName(IAeBPELConstants.BPWS_NAMESPACE_URI, VAR_PROPERTY_FUNC_NAME);
   /** The fully qualified name of the getVariableProperty function. */
   public final static QName MYROLE_PROPERTY_FUNC = new QName(IAeBPELConstants.BPWS_NAMESPACE_URI, MYROLE_PROP_FUNC_NAME);
   /** The fully qualified name of the getVariableData function. */
   public final static QName VAR_DATA_FUNC = new QName(IAeBPELConstants.BPWS_NAMESPACE_URI, VAR_DATA_FUNC_NAME);
   /** The fully qualified name of the getLinkStatus function. */
   public final static QName LINK_STATUS_FUNC = new QName(IAeBPELConstants.BPWS_NAMESPACE_URI, LINK_STATUS_FUNC_NAME);

   /**
    * Returns true if the function is a bpws:getVariableData function.
    * 
    * @param aFunction
    */
   public static boolean isVarDataFunction(AeScriptFuncDef aFunction)
   {
      return VAR_DATA_FUNC.equals(aFunction.getQName());
   }

   /**
    * Returns true if the function is a bpws:getVariableProperty function.
    * 
    * @param aFunction
    */
   public static boolean isVarPropertyFunction(AeScriptFuncDef aFunction)
   {
      return VAR_PROPERTY_FUNC.equals(aFunction.getQName());
   }

   /**
    * Returns true if the function is a abx:getMyRoleProperty function.
    * 
    * @param aFunction
    */
   public static boolean isMyRolePropertyFunction(AeScriptFuncDef aFunction)
   {
      return MYROLE_PROPERTY_FUNC.equals(aFunction.getQName());
   }
   
   
   /**
    * Returns true if the function is a bpws:getLinkStatus function.
    * 
    * @param aFunction
    */
   public static boolean isLinkStatusFunction(AeScriptFuncDef aFunction)
   {
      return LINK_STATUS_FUNC.equals(aFunction.getQName());
   }

}
