//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/expr/IAeExpressionParseResult.java,v 1.3 2006/01/10 22:18:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.expr;

import java.util.List;
import java.util.Set;

/**
 * This interface defines what an expression parse result is.  An expression parse result is the
 * result of parsing an expression (of course).
 */
public interface IAeExpressionParseResult
{
   /**
    * Gets the expression that was parsed to create this result.
    */
   public String getExpression();

   /**
    * Returns true if there were errors during parse.
    */
   public boolean hasErrors();

   /**
    * Gets a list of errors found during parsing.
    */
   public List getParseErrors();
   
   /**
    * Gets a list of all the function calls made in the expression.  The return value is a set of 
    * AeScriptFuncDef objects.
    */
   public Set getFunctions();
   
   /**
    * Gets a list of all the variables used in the expression.  This method returns a list of String
    * objects (the names of the variables).
    */
   public Set getVarNames();
   
   /**
    * Gets a list of all the bpws:getVariableData functions in the expression. The return value is a list of
    * AeScriptFuncDef objects.
    */
   public List getVarDataFunctionList();

   /**
    * Gets a list of all the bpws:getVariableProperty functions in the expression. The return value is a list
    * of AeScriptFuncDef objects.
    */
   public List getVarPropertyFunctionList();

   /**
    * Gets a list of all the abx:getMyRoleProperty functions in the expression. The return value is a list
    * of AeScriptFuncDef objects.
    */
   public List getMyRolePropertyFunctionList();
   
   
   /**
    * Gets a list of all the bpws:getLinkStatus functions in the expression. The return value is a list of
    * AeScriptFuncDef objects.
    */
   public List getLinkStatusFunctionList();
   
   /**
    * Gets a list of all the variable data (AeVariableData) objects.
    */
   public List getVarDataList();
   
   /**
    * Gets a list of all the variable property (AeVariableProperty) objects.
    */
   public List getVarPropertyList();
}
