// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/AeBpelIO.java,v 1.3 2004/11/30 19:26:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 * Wraps the basic serialization/deserialization calls 
 * currently in use. 
 */
public class AeBpelIO
{
   /** AeBpelFactory impl class */
   private static AeBpelFactory mFactory;

   /**
    * private x-tor - no instantiation 
    */
   private AeBpelIO()
   {
   }
   
   /**
    * Deserialize inputstream to AeProcessDef.
    * @param aInputSource input source that contains the bpel xml
    * @return AeProcessDef object model
    * @throws AeBusinessProcessException
    */
   public static AeProcessDef deserialize( InputSource aInputSource )
   throws AeBusinessProcessException
   {
      return getBpelFactory().getBpelReader().readBPEL( aInputSource );
   }
   
   /**
    * Deserialize the BPEL xml to AeProcessDef.
    * @param aDocument
    * @return AeProcessDef object model
    * @throws AeBusinessProcessException
    */
   public static AeProcessDef deserialize( Document aDocument )
   throws AeBusinessProcessException
   {
      return getBpelFactory().getBpelReader().readBPEL(aDocument);
   }
   
   /**
    * Serialize the AeProcessDef to its BPEL xml representation.
    * @param aProcessDef the BPEL process object model
    * @return BPEL xml 
    * @throws AeBusinessProcessException
    */
   public static Document serialize( AeProcessDef aProcessDef )
   throws AeBusinessProcessException
   {
      return getBpelFactory().getBpelWriter().writeBPEL( aProcessDef );
   }
   
   /**
    * Accessor to AeBpelFactory impl.
    * @return AeBpelFactory
    * @throws AeBusinessProcessException
    */
   public static AeBpelFactory getBpelFactory()
   throws AeBusinessProcessException
   {
      if( mFactory == null )
      {
         mFactory = AeBpelFactory.newInstance();
      }
      return mFactory;
   }
}
