// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/readers/AeBpelReaderImpl.java,v 1.7 2005/08/18 21:35:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.readers;

import java.text.MessageFormat;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.io.IAeBpelReader;
import org.activebpel.rt.bpel.def.io.registry.IAeBpelRegistry;
import org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry;
import org.activebpel.rt.bpel.def.visitors.AeImplicitVariableVisitor;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.util.AeXmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 * IAeBpelReader impl. 
 */
public class AeBpelReaderImpl implements IAeBpelReader
{
   /** bpel reader/writer registry */ 
   private IAeBpelRegistry mBpelRegistry;   
   /** extension reader/writer registry */
   private IAeExtensionRegistry mExtRegistry;

   /**
    * @see org.activebpel.rt.bpel.def.io.IAeBpelReader#readBPEL(org.xml.sax.InputSource)
    */
   public AeProcessDef readBPEL(InputSource aInputSource)
      throws AeBusinessProcessException
   {
      try
      {
         DocumentBuilder builder = AeXmlUtil.getDocumentBuilder(); 

         return readBPEL(builder.parse(aInputSource));
      }
      catch (Exception ex)
      {
         throw new AeBusinessProcessException(AeMessages.getString("AeBpelReaderImpl.ERROR_0"), ex); //$NON-NLS-1$
      }
   }


   /**
    * @see org.activebpel.rt.bpel.def.io.IAeBpelReader#readBPEL(org.w3c.dom.Document)
    */
   public AeProcessDef readBPEL(Document aBpelDoc) throws AeBusinessProcessException
   {
      Element processElement = aBpelDoc.getDocumentElement();
      
      // Check top-level element qname for validity.           
      if ( ! AeUtil.compareObjects(processElement.getLocalName(), IAeBPELConstants.TAG_PROCESS) ||
           ! AeUtil.compareObjects(processElement.getNamespaceURI(), IAeBPELConstants.BPWS_NAMESPACE_URI) )
         throw new AeBusinessProcessException(
            MessageFormat.format(AeMessages.getString("AeBpelReaderImpl.ERROR_1"), //$NON-NLS-1$
               new Object[] { (new QName(IAeBPELConstants.BPWS_NAMESPACE_URI, "process")).toString() }) ); //$NON-NLS-1$

      AeBpelDomTraverser traverser = new AeBpelDomTraverser(getBpelRegistry(), getExtensionRegistry());
      traverser.traverseProcess( processElement );
      
      AeProcessDef def = traverser.getProcessDef();
      
      // create any implicit variables in the def layer
      def.accept(new AeImplicitVariableVisitor());
      
      return def;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.io.IAeBpelReader#setExtensionRegistry(org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry)
    */
   public void setExtensionRegistry(IAeExtensionRegistry aRegistry)
   {
      mExtRegistry = aRegistry;
   }

   /**
    * Internal accessor for extension registry
    * @return extension registry
    */
   private IAeExtensionRegistry getExtensionRegistry()
   {
      return mExtRegistry;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.io.IAeBpelReader#setBpelRegistry(org.activebpel.rt.bpel.def.io.registry.IAeBpelRegistry)
    */
   public void setBpelRegistry(IAeBpelRegistry aBpelRegistry)
   {
      mBpelRegistry = aBpelRegistry;
   }

   /**
    * Internal accessor for bpel registry
    * @return bpel registry
    */
   private IAeBpelRegistry getBpelRegistry()
   {
      return mBpelRegistry;
   }
}
