// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/readers/def/AeContainerReaderFactory.java,v 1.2 2004/07/08 13:09:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.readers.def;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeCorrelationContainerDef;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityFlowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReplyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.def.visitors.AeAbstractDefVisitor;

/**
 * Base class for deserializing container element to their
 * corresponding AeDef object.
 * <br />
 * Container readers behave differently in that they are not
 * directly responsible for instantiating the container defs.
 * Their parent def objects use lazy instantiation on the
 * container accessor methods.
 * <br />
 * In addition, the containers may have various parent def
 * objects (with no appropriate interface to identify them
 * generically).
 * <br />
 * In cases where the container has more than two possible
 * parents, the subclass uses double dispatch to extract
 * its container by type. In all other cases, instanceof
 * and casting is used to extract the container from the
 * parent type. 
 */
public class AeContainerReaderFactory
{

   /**
    * Accessor for compensationHandlers element reader.
    * @return container reader
    */
   public static AeDispatchReader getCompensatorReader()
   {
      return new AeDispatchCompensatorReader();
   }
   
   /**
    * Accessor for links element reader.
    * @return container reader
    */
   public static AeDispatchReader getLinksContainerReader()
   {
      return new AeDispatchLinksReader();
   }

   /**
    * Accessor for correlations element reader.
    * @return container reader
    */
   public static AeDispatchReader getCorrelationsReader()
   {
      return new AeDispatchCorrelationsReader();
   }
   
   /**
    * Accessor for variables element reader.
    * @return container reader
    */
   public static AeDispatchReader getVariablesReader()
   {
      return new AeDispatchVariablesReader();
   }

   /**
    * Accessor for partnerLinks element reader.
    * @return container reader
    */
   public static AeDispatchReader getPartnerLinksReader()
   {
      return new AeDispatchPartnerLinksReader();
   }
   
   /**
    * Accessor for partners element reader.
    * @return container reader
    */
   public static AeDispatchReader getPartnersReader()
   {
      return new AeDispatchPartnersReader();
   }
   
   /**
    * Accessor for correlationSets element reader.
    * @return container reader
    */
   public static AeDispatchReader getCorrelationSetsReader()
   {
       return new AeDispatchCorrelationSetsReader();
   }
   
   /**
    * Accessor for faultHandlers element reader.
    * @return container reader
    */
   public static AeDispatchReader getFaultHandlersReader()
   {
      return new AeDispatchFaultHandlerReader();
   }
}

/**
 * Deserialize links element.
 */
class AeDispatchLinksReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchLinksReader()
   {
      super(null);
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      return ((AeActivityFlowDef)aParent).getLinkContainer();
   }
}

/**
 * Deserialize compensationHandlers element.
 */
class AeDispatchCompensatorReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchCompensatorReader()
   {
      super(null);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
       AeCompenstatorVisitor visitor = new AeCompenstatorVisitor();
       aParent.accept( visitor );
       return visitor.getContainer();
   }

    /**
     * Returns the AeScopeCompensatorContainer from the
     * specified AeBaseDef object.
     *
     */
    private class AeCompenstatorVisitor extends AeAbstractDefVisitor
    {
       /** container */
       private AeScopeCompensatorContainerDef mContainer;
      
       /**
        * Acessor method for the container.
        * @return scope container def 
        */
       public AeScopeCompensatorContainerDef getContainer()
       {
          return mContainer;
       }
      
       /**
        * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef)
        */
       public void visit(AeActivityInvokeDef aDef)
       {
          mContainer = aDef.getCompensatorContainer();
       }

       /**
        * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityScopeDef)
        */
       public void visit(AeActivityScopeDef aDef)
       {
          mContainer = aDef.getScope().getCompensatorContainer();
       }

       /**
        * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeProcessDef)
        */
       public void visit(AeProcessDef aDef)
       {
          mContainer = aDef.getCompensatorContainer();
       }

       /**
        * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeScopeDef)
        */
       public void visit(AeScopeDef aDef)
       {
          mContainer = aDef.getCompensatorContainer();
       }
    }
}

/**
 * Deserialize correlations element.
 */
class AeDispatchCorrelationsReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchCorrelationsReader()
   {
      super(null);
   }

   
   /**
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      AeCorrelationsVisitor visitor = new AeCorrelationsVisitor();
      aParent.accept(visitor);
      return visitor.getContainer();
   }
   
   
   /**
    * Use double dispatch to extract the AeCorrelationContainerDef
    * object from the AeBaseDef object based on its type.
    */   
   private class AeCorrelationsVisitor extends AeAbstractDefVisitor
   {
      /** container member data */
      private AeCorrelationContainerDef mContainer;
      
      /**
       * Accessor for the container.
       * @return the container
       */
      public AeCorrelationContainerDef getContainer()
      {
         return mContainer;
      }
      
      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef)
       */
      public void visit(AeOnMessageDef aDef)
      {
         mContainer = aDef.getCorrelationContainer();
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef)
       */
      public void visit(AeActivityInvokeDef aDef)
      {
         mContainer = aDef.getCorrelationContainer();
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef)
       */
      public void visit(AeActivityReceiveDef aDef)
      {
         mContainer = aDef.getCorrelationContainer();
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReplyDef)
       */
      public void visit(AeActivityReplyDef aDef)
      {
         mContainer = aDef.getCorrelationContainer();
      }
   }
}

/**
 * Deserialize variables element.
 */
class AeDispatchVariablesReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchVariablesReader()
   {
      super(null);
   }

   /**
    * Parent can only be AeProcessDef or AeActivityScopeDef.
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      if( aParent instanceof AeProcessDef )
      {
         return ((AeProcessDef)aParent).getVariablesContainer();
      }
      return ((AeActivityScopeDef)aParent).getScope().getVariablesContainer();
   }
}

/**
 * Deserialize partnerLinks element.
 */
class AeDispatchPartnerLinksReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchPartnerLinksReader()
   {
      super(null);
   }

   /**
    * Parent can only be AePartnerLinksContainerDef.
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      return ((AeProcessDef)aParent).getPartnerLinksContainer();
   }
}


/**
 * Deserialize partners element.
 */
class AeDispatchPartnersReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchPartnersReader()
   {
      super(null);
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      return ((AeProcessDef)aParent).getPartnersContainer();
   }
}

/**
 * Deserialize the correlationSets child element of the
 * process or scope (activity) elements.
 */
class AeDispatchCorrelationSetsReader extends AeDispatchReader
{
   /**
    * Constructor
    */
   public AeDispatchCorrelationSetsReader()
   {
      super(null);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      if( aParent instanceof AeProcessDef )
      {
         return ((AeProcessDef)aParent).getCorrelationSetContainer();
      }
      return ((AeActivityScopeDef)aParent).getScope().getCorrelationSetContainer();
   }
}

/**
 * Deserializes the faultHandlers child element for either a
 * process or scope (activity) element.
 */
class AeDispatchFaultHandlerReader extends AeDispatchReader
{
   /**
    * Constructor.
    */
   public AeDispatchFaultHandlerReader()
   {
      super(null);
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      if( aParent instanceof AeProcessDef )
      {
         return ((AeProcessDef)aParent).getFaultHandlerContainer();
      }
      return ((AeActivityScopeDef)aParent).getScope().getFaultHandlerContainer();
   }
}


