// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/readers/def/AeDispatchReader.java,v 1.6 2006/02/01 22:44:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.readers.def;

import java.util.Iterator;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.AeBaseDef;

import org.w3c.dom.Element;

/**
 * Generic AeBaseDefReader impl that creates a new instance of
 * its child class and used double dispatch to populate it
 * based on type.
 */
public class AeDispatchReader extends AeBaseDefReader
{
   /** the AeBaseDef class type to create */
   private Class mChildClass;
   
   /**
    * Constructor.
    * @param aChildClass the AeBaseDef type to create
    */
   public AeDispatchReader( Class aChildClass )
   {
      mChildClass = aChildClass;
   }

   /**
    * Dispatch newly created AeBaseDef to an instance of the
    * AeReaderVisitor to set its properties based on the current element.
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#configureChild(org.activebpel.rt.bpel.def.AeBaseDef, org.activebpel.rt.bpel.def.AeBaseDef, org.w3c.dom.Element)
    */
   protected boolean configureChild(AeBaseDef aParentDef, AeBaseDef aNewDef, Element aElement)
      throws AeBusinessProcessException
   {
      AeReaderVisitor visitor = new AeReaderVisitor( aParentDef, aElement );
      aNewDef.accept( visitor );
      if (visitor.hasErrors())
      {
         // report any errors to the console
         for (Iterator iter = visitor.getErrors().iterator(); iter.hasNext();)
         {
            String error = (String) iter.next();
            AeBusinessProcessException.logWarning(error);
         }
      }
      return !visitor.hasErrors();
   }

   /**
    * Returns a mChildClass.newInstance()
    * @see org.activebpel.rt.bpel.def.io.readers.def.AeBaseDefReader#createChild(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected AeBaseDef createChild(AeBaseDef aParent)
      throws AeBusinessProcessException
   {
      try
      {
         return (AeBaseDef)mChildClass.newInstance();
      }
      catch (InstantiationException e)
      {
         throw new AeBusinessProcessException(
                  AeMessages.getString("AeDispatchReader.ERROR_0") + mChildClass.getName(), e); //$NON-NLS-1$
      }
      catch (IllegalAccessException e)
      {
         throw new AeBusinessProcessException(
                  AeMessages.getString("AeDispatchReader.ERROR_0") + mChildClass.getName(), e); //$NON-NLS-1$
      }
   }
}
