// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/registry/AeDefWriterRegistry.java,v 1.8 2006/01/11 21:01:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.registry;

import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.io.IAeBpelClassConstants;
import org.activebpel.rt.bpel.def.io.writers.def.AeDispatchWriter;
import org.activebpel.rt.bpel.def.io.writers.def.AeProcessWriter;
import org.activebpel.rt.bpel.def.io.writers.def.IAeBpelDefWriter;

import java.util.HashMap;
import java.util.Map;

/**
 * This impl simply maps the AeDef class object to its associated writer. 
 */
public class AeDefWriterRegistry implements IAeBPELConstants, IAeBpelClassConstants
{
   /** writer registry */
   private Map mRegistry = new HashMap();
   
   /**
    * Default constructor.
    */
   public AeDefWriterRegistry()
   {
      init();
   }
   
   /**
    * Populates the registry with entries.
    */
   protected void init()
   {
      mRegistry.put( PROCESS_CLASS,              new AeProcessWriter() );

      mRegistry.put( ACTIVITY_ASSIGN_CLASS,      new AeDispatchWriter(TAG_ASSIGN) );   
      mRegistry.put( ACTIVITY_COMPENSATE_CLASS,  new AeDispatchWriter(TAG_COMPENSATE) );
      mRegistry.put( ACTIVITY_EMPTY_CLASS,       new AeDispatchWriter(TAG_EMPTY) );
      mRegistry.put( ACTIVITY_FLOW_CLASS,        new AeDispatchWriter(TAG_FLOW ) );
      mRegistry.put( ACTIVITY_INVOKE_CLASS,      new AeDispatchWriter(TAG_INVOKE) );
      mRegistry.put( ACTIVITY_PICK_CLASS,        new AeDispatchWriter(TAG_PICK) );
      mRegistry.put( ACTIVITY_RECEIVE_CLASS,     new AeDispatchWriter(TAG_RECEIVE) );
      mRegistry.put( ACTIVITY_REPLY_CLASS,       new AeDispatchWriter(TAG_REPLY) );
      mRegistry.put( ACTIVITY_SCOPE_CLASS,       new AeDispatchWriter(TAG_SCOPE) );
      mRegistry.put( ACTIVITY_SEQUENCE_CLASS,    new AeDispatchWriter(TAG_SEQUENCE) );
      mRegistry.put( ACTIVITY_SWITCH_CLASS,      new AeDispatchWriter(TAG_SWITCH) );
      mRegistry.put( ACTIVITY_TERMINATE_CLASS,   new AeDispatchWriter(TAG_TERMINATE) );
      mRegistry.put( ACTIVITY_THROW_CLASS,       new AeDispatchWriter(TAG_THROW) );
      mRegistry.put( ACTIVITY_WAIT_CLASS,        new AeDispatchWriter(TAG_WAIT) );
      mRegistry.put( ACTIVITY_WHILE_CLASS,       new AeDispatchWriter(TAG_WHILE) );
      mRegistry.put( ACTIVITY_CONTINUE_CLASS,    new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_CONTINUE)); 
      mRegistry.put( ACTIVITY_BREAK_CLASS,       new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_BREAK)); 
      mRegistry.put( ACTIVITY_FOREACH_CLASS,     new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_FOREACH)); 
      mRegistry.put( ACTIVITY_SUSPEND_CLASS,     new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_SUSPEND)); 

      mRegistry.put( CASE_CLASS,                 new AeDispatchWriter(TAG_CASE) );
      mRegistry.put( OTHERWISE_CLASS,            new AeDispatchWriter(TAG_OTHERWISE ) );
      mRegistry.put( LINK_CLASS,                 new AeDispatchWriter(TAG_LINK) );
      mRegistry.put( TARGET_CLASS,               new AeDispatchWriter(TAG_TARGET) );
      mRegistry.put( SOURCE_CLASS,               new AeDispatchWriter(TAG_SOURCE) );
      mRegistry.put( MESSAGE_EXCHANGES_CLASS,    new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_SCOPE_MESSAGEEXCHANGES)); 
      mRegistry.put( ACTIVITY_FOREACH_COMPLETION_CONDITION, new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_FOREACH_COMPLETION_CONDITION)); 
      mRegistry.put( ACTIVITY_FOREACH_BRANCH,    new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_FOREACH_BRANCHES)); 
      mRegistry.put( ACTIVITY_FOREACH_START,     new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_FOREACH_STARTCOUNTER)); 
      mRegistry.put( ACTIVITY_FOREACH_FINAL,     new AeDispatchWriter(IAeBPELConstants.ABX_2_0_NAMESPACE_URI, TAG_FOREACH_FINALCOUNTER)); 
      mRegistry.put( ON_ALARM_CLASS,             new AeDispatchWriter(TAG_ON_ALARM) );
      mRegistry.put( ON_MESSAGE_CLASS,           new AeDispatchWriter(TAG_ON_MESSAGE) );
      mRegistry.put( ASSIGN_COPY_CLASS,          new AeDispatchWriter(TAG_COPY) );
      mRegistry.put( CORRELATION_CLASS,          new AeDispatchWriter(TAG_CORRELATION) );
      mRegistry.put( FAULT_HANDLER_CLASS,        new AeDispatchWriter(TAG_CATCH) );
      mRegistry.put( DEFAULT_FAULT_HANDLER_CLASS,new AeDispatchWriter(TAG_CATCH_ALL) );

      mRegistry.put( ASSIGN_FROM_CLASS,          new AeDispatchWriter(TAG_FROM) );
      mRegistry.put( ASSIGN_TO_CLASS,            new AeDispatchWriter(TAG_TO) );

      mRegistry.put( PARTNER_LINK_CLASS,         new AeDispatchWriter(TAG_PARTNER_LINK) );
      mRegistry.put( PARTNER_CLASS,              new AeDispatchWriter(TAG_PARTNER) );
      mRegistry.put( VARIABLE_CLASS,             new AeDispatchWriter(TAG_VARIABLE) );
      mRegistry.put( EVENT_HANDLERS_CLASS,       new AeDispatchWriter(TAG_EVENT_HANDLERS) );
      mRegistry.put( CORRELATION_SET_CLASS,      new AeDispatchWriter(TAG_CORRELATION_SET) );
      mRegistry.put( CORRELATION_CONTAINER_CLASS,new AeDispatchWriter(TAG_CORRELATIONS) );
      mRegistry.put( LINKS_CONTAINER_CLASS,      new AeDispatchWriter(TAG_LINKS) );
      mRegistry.put( PARTNER_CONTAINER_CLASS,    new AeDispatchWriter(TAG_PARTNERS) );
      mRegistry.put( VARIABLE_CONTAINER_CLASS,   new AeDispatchWriter(TAG_VARIABLES) );
      mRegistry.put( CORRELATION_SETS_CLASS,     new AeDispatchWriter(TAG_CORRELATION_SETS) );

      mRegistry.put( PARNTER_LINKS_CONTAINER_CLASS,     new AeDispatchWriter(TAG_PARTNER_LINKS) );
      mRegistry.put( FAULT_HANDLER_CONTAINER_CLASS,     new AeDispatchWriter(TAG_FAULT_HANDLERS) );
      mRegistry.put( SCOPE_COMPENSATOR_CONTAINER_CLASS, new AeDispatchWriter(TAG_COMPENSATION_HANDLER ) );
   }


   /**
    * Retrieve the writer class for the AeDef object.  In this
    * impl, the parent class is ignored as no special mappings
    * were required.
    * @param aParentClass ignored
    * @param aDef the base def object to be serialized
    * @return the appropriate writer
    */
   public IAeBpelDefWriter getWriter(Class aParentClass, AeBaseDef aDef)
   {
      IAeBpelDefWriter writer = (IAeBpelDefWriter)mRegistry.get(aDef.getClass());
      if( writer == null )
      {
         throw new UnsupportedOperationException(AeMessages.format("AeDefWriterRegistry.ERROR_0",  //$NON-NLS-1$
               aDef.getClass().getName() ));   
      }
      return writer;
   }
}
