// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/registry/AeDefaultExtensionRegistry.java,v 1.5 2005/07/22 21:43:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.registry;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.io.ext.IAeExtensionReader;
import org.activebpel.rt.bpel.def.io.ext.IAeExtensionWriter;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

/**
 * Default impl of the IAeExtensionRegistry.
 * <br />
 * Maintains a map of maps for both readers and writers.<br />
 * Top level map: parent class to inner map. <br />
 * Inner map: extension element QNames to reader/writer impls.
 */
public class AeDefaultExtensionRegistry implements IAeExtensionRegistry
{
   /** reader map */
   private Map mReaders;
   /** writer map */
   private Map mWriters;

   /** default exension reader */
   private IAeExtensionReader mDefaultReader;
   /** defaul extension writer */
   private IAeExtensionWriter mDefaultWriter;   

   /**
    * @see org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry#getExtensionReader(java.lang.Class, javax.xml.namespace.QName)
    */
   public IAeExtensionReader getExtensionReader( Class aParentClass, QName aQName)
   throws AeBusinessProcessException
      
   {
      IAeExtensionReader reader = null;
      Map innerMap = (Map)getReaders().get( aParentClass );
      if( innerMap != null )
      {
         reader = (IAeExtensionReader)innerMap.get(aQName);
      }
      
      if( reader == null )
      {
         reader = getDefaultReader();
      }
      
      if( reader == null )
      {
         throw new AeBusinessProcessException(AeMessages.format("AeDefaultExtensionRegistry.ERROR_0", aQName) ); //$NON-NLS-1$
      }
      
      return reader;
   }
   

   /**
    * @see org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry#registerExtensionReader(java.lang.Class, javax.xml.namespace.QName, org.activebpel.rt.bpel.def.io.ext.IAeExtensionReader)
    */
   public void registerExtensionReader(
      Class aParentClass,
      QName aQName,
      IAeExtensionReader aReader)
   {
      Map innerReaders = (Map)getReaders().get( aParentClass );
      if( innerReaders == null )
      {
         innerReaders = new HashMap();
         getReaders().put( aParentClass, innerReaders );
      }
      innerReaders.put( aQName, aReader );
   }


   /**
    * @see org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry#setDefaultExtensionReader(org.activebpel.rt.bpel.def.io.ext.IAeExtensionReader)
    */
   public void setDefaultExtensionReader(IAeExtensionReader aDefault)
   {
      mDefaultReader = aDefault;
   }
   

   /**
    * @see org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry#getExtensionWriter(java.lang.Class, javax.xml.namespace.QName)
    */
   public IAeExtensionWriter getExtensionWriter(
      Class aParentClass,
      QName aQName)
   throws AeBusinessProcessException
   {
      IAeExtensionWriter writer = null;
      Map innerMap = (Map)getWriters().get( aParentClass );
      if( innerMap != null )
      {
         writer = (IAeExtensionWriter)innerMap.get(aQName);
      }
      
      if( writer == null )
      {
         writer = getDefaultWriter();
      }
      
      if( writer == null )
      {
         throw new AeBusinessProcessException(AeMessages.format("AeDefaultExtensionRegistry.ERROR_1",aQName) ); //$NON-NLS-1$
      }
      
      return writer;
   }


   /**
    * @see org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry#registerExtensionWriter(java.lang.Class, javax.xml.namespace.QName, org.activebpel.rt.bpel.def.io.ext.IAeExtensionWriter)
    */
   public void registerExtensionWriter(
      Class aParentClass,
      QName aQName,
      IAeExtensionWriter aWriter)
   {
      Map innerMap = (Map)getWriters().get(aQName);
      if( innerMap == null )
      {
         innerMap = new HashMap();
         getWriters().put( aParentClass, innerMap );
      }
      innerMap.put( aQName, aWriter );
   }


   /**
    * @see org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry#setDefaultExtensionWriter(org.activebpel.rt.bpel.def.io.ext.IAeExtensionWriter)
    */
   public void setDefaultExtensionWriter(IAeExtensionWriter aExtensionWriter)
   {
      mDefaultWriter = aExtensionWriter;
   }
   
   /**
    * Accessor for default reader
    * @return default reader or null if none is set
    */
   private IAeExtensionReader getDefaultReader()
   {
      return mDefaultReader;
   }
   
   /**
    * Accessor for default writer
    * @return default writer or null if none is set
    */
   private IAeExtensionWriter getDefaultWriter()
   {
      return mDefaultWriter;
   }
   
   /**
    * @return registry map for extension readers
    */
   private Map getReaders()
   {
      if( mReaders == null )
      {
         mReaders = new HashMap();
      }
      return mReaders;
   }
   
   /**
    * @return registry map for extension writers.
    */
   private Map getWriters()
   {
      if( mWriters == null )
      {
         mWriters = new HashMap();
      }
      return mWriters;
   }

}
