// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/registry/IAeExtensionRegistry.java,v 1.2 2004/07/08 13:09:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.registry;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.io.ext.IAeExtensionReader;
import org.activebpel.rt.bpel.def.io.ext.IAeExtensionWriter;

import javax.xml.namespace.QName;

/**
 * Interface for accessing BPEL extension element/object serializers
 * and deserializers.
 */
public interface IAeExtensionRegistry
{
   
   /**
    * Accessor for retrieving an IAeExtensionReader for an
    * extension element.
    * <br />
    * If no specific reader is found, the
    * default extension reader will be returned. If the default
    * extension reader is null, an exception will be thrown.
    * @param aParentClass top level key
    * @param aQName specific key for all ext readers under this parent
    * @return IAeExtensionReader impl
    * @throws AeBusinessProcessException
    */
   public IAeExtensionReader getExtensionReader( Class aParentClass, QName aQName )
   throws AeBusinessProcessException;

   /**
    * Register a custom reader for an extension element with the given
    * QName whose parent is an instance of the given class in the AeDef 
    * object model.
    * @param aParentClass parent in the AeDefObject model
    * @param aQName extension element QName
    * @param aReader the reader impl
    */
   public void registerExtensionReader( Class aParentClass, QName aQName, IAeExtensionReader aReader );

   /**
    * Register a default reader for any extension elements that do
    * not have a specific mapping.
    * <br />
    * Setting this value to null will cause an exception to be thrown
    * if an unregistered extension element is encountered.
    * @param aDefault
    */
   public void setDefaultExtensionReader( IAeExtensionReader aDefault );
   
   
   /**
    * Accessor for retrieving an IAeExtensionWriter for an
    * extension element.
    * <br />
    * If no specific writer is found, the
    * default extension writer will be returned. If the default
    * extension writer is null, an exception will be thrown.
    * @param aParentClass
    * @param aQName
    * @return extension element writer
    * @throws AeBusinessProcessException
    */
   public IAeExtensionWriter getExtensionWriter( Class aParentClass, QName aQName )
   throws AeBusinessProcessException;
   
   
   /**
    * Register a custom writer for an extension element with the given
    * QName whose parent is an instance of the given class in the AeDef 
    * object model.
    * @param aParentClass
    * @param aQName
    * @param aWriter
    */
   public void registerExtensionWriter( Class aParentClass, QName aQName, IAeExtensionWriter aWriter );
   

   /**
    * Register a default writer for any extension elements that do
    * not have a specific mapping.
    * <br />
    * Setting this value to null will cause an exception to be thrown
    * if an unregistered extension element is encountered.
    * @param aDefault
    */
   public void setDefaultExtensionWriter( IAeExtensionWriter aExtensionWriter );
}
