// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/writers/AeBpelWriterImpl.java,v 1.2 2004/07/08 13:09:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.writers;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.io.IAeBpelWriter;
import org.activebpel.rt.bpel.def.io.registry.IAeBpelRegistry;
import org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry;

import org.w3c.dom.Document;

/**
 * IAeBpelWriter impl.
 * <br />
 * Delegates the actual work to the <code>AeBpelWriterVisitor</code>
 * class. 
 */
public class AeBpelWriterImpl implements IAeBpelWriter
{
   /** bpel registry */
   private IAeBpelRegistry mBpelRegistry;
   /** extension registry */
   private IAeExtensionRegistry mExtensionRegistry;

   /**
    * Uses the <code>AeBpelWriterVisitor</code> to traverse the
    * process def and convert it and all of its children to a 
    * DOM tree. 
    * @see org.activebpel.rt.bpel.def.io.IAeBpelWriter#writeBPEL(org.activebpel.rt.bpel.def.AeProcessDef)
    */
   public Document writeBPEL( AeProcessDef aProcessDef ) 
                              throws AeBusinessProcessException
   {
      try
      {
         AeBpelDefTraverserVisitor writerVisitor = 
                  new AeBpelDefTraverserVisitor( mBpelRegistry, mExtensionRegistry );
         writerVisitor.visit(aProcessDef);
         return writerVisitor.getProcessDoc();
      }
      catch( AeBpelWriterException e )
      {
         throw new AeBusinessProcessException( e.getMessage(), e.getCause() );
      }
      catch( Throwable t )
      {
         t.printStackTrace();
         throw new AeBusinessProcessException(t.getMessage());
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.io.IAeBpelWriter#setBpelRegistry(org.activebpel.rt.bpel.def.io.registry.IAeBpelRegistry)
    */
   public void setBpelRegistry(IAeBpelRegistry aRegistry)
   {
      mBpelRegistry = aRegistry;
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.IAeBpelWriter#setExtensionRegistry(org.activebpel.rt.bpel.def.io.registry.IAeExtensionRegistry)
    */
   public void setExtensionRegistry(IAeExtensionRegistry aRegistry)
   {
      mExtensionRegistry = aRegistry;
   }
}

