// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/writers/AeInvokeWithScopeTraverser.java,v 1.3 2004/07/08 13:09:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.writers;

import org.activebpel.rt.bpel.def.AeFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.def.visitors.AeDefTraverser;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;

import java.util.Iterator;

/**
 * This class provides special traversal behavior if it encounters
 * a <code>AeActivityInvokeDef</def> object that contains an
 * implicit scope.
  */
public class AeInvokeWithScopeTraverser extends AeDefTraverser
{
   /**
    * Determine which traversal method to call.  If the invoke def
    * does not have an implicit scope, traverse normally.  Otherwise
    * call <code>traverseInvokeWithScope</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityInvokeDef aDef, IAeDefVisitor aVisitor)
   {
      if( isSpecialCaseInvoke(aDef) )
      {
         traverseInvokeWithScope(aDef, aVisitor);
      }
      else
      {
         super.traverse(aDef, aVisitor);
      }
   }
   
   /**
    * @param aDef
    * @return true is invoke def object contains an implicit scope
    */
   private boolean isSpecialCaseInvoke( AeActivityInvokeDef aDef )
   {
      return AeImplicitScopeUtil.isParentInlinedScope(aDef);
   }
   
   /**
    * Overrides the default traversal behavior to account for
    * special "inlining" of BPEL scope xml.
    * @param aDef
    * @param aVisitor
    */
   private void traverseInvokeWithScope(AeActivityInvokeDef aDef, IAeDefVisitor aVisitor)
   {
      AeActivityScopeDef scopeActivity = (AeActivityScopeDef)(aDef.getImplicitScope());
      traverseSourceAndTargetLinks(scopeActivity,aVisitor);
  
      if (aDef.hasCorrelationList())
      {
         callAccept(aDef.getCorrelationContainer(), aVisitor);
      }

      AeScopeDef scope = scopeActivity.getScope();
      if (scope.hasFaultHandlers())
      {
         for( Iterator iter = scope.getFaultHandlerContainer().getFaultHandlers(); iter.hasNext(); )
         {
            callAccept( (AeFaultHandlerDef)iter.next(), aVisitor);
         }
         
         if( scope.getFaultHandlerContainer().getDefaultFaultHandler() != null )
         {
            callAccept( scope.getDefaultFaultHandler(), aVisitor );
         }
      }

      if (scope.hasCompensator())
      {
         callAccept(aDef.getCompensatorContainer(), aVisitor);
      }
   }
   
   
}
