// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/writers/def/AeDispatchWriter.java,v 1.6 2005/08/04 20:01:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.writers.def;

import java.util.Iterator;

import org.activebpel.rt.bpel.def.AeBaseContainer;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeCorrelationContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef;
import org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef;
import org.activebpel.rt.bpel.def.AePartnerLinksContainerDef;
import org.activebpel.rt.bpel.def.AePartnersContainerDef;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.def.AeVariablesContainerDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef;
import org.activebpel.rt.bpel.def.visitors.AeAbstractDefVisitor;
import org.w3c.dom.Element;

/**
 * Generic writer class for serializing AeDef objects.
 * Creates an element with the given tag name
 * and uses an instance of the AeWriterVisitor to 
 * extract specific attributes based on object type.
 */
public class AeDispatchWriter implements IAeBpelDefWriter
{
   /** tag name for element*/
   private String mTagName;
   
   /** namespace for the element, defaults to the bpel namespace */
   private String mNamespace;
   
   /**
    * Constructor.
    * @param aTagName for element
    */
   public AeDispatchWriter( String aTagName )
   {
      this(IAeBPELConstants.BPWS_NAMESPACE_URI, aTagName);
   }
   
   /**
    * Constructor. Creates the writer with the specified namespace uri and tagname.
    * 
    * @param aNamespaceUri
    * @param aTagName
    */
   public AeDispatchWriter(String aNamespaceUri, String aTagName)
   {
      setNamespace(aNamespaceUri);
      setTagName(aTagName);
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.writers.def.IAeBpelDefWriter#createElement(org.activebpel.rt.bpel.def.AeBaseDef, org.w3c.dom.Element)
    */
   public Element createElement(AeBaseDef aBaseDef, Element aParentElement)
   {
      if( isEmptyContainer(aBaseDef) || isImplicitVariable(aBaseDef))
      {
         return null;
      }
      AeWriterVisitor visitor = new AeWriterVisitor( aParentElement, getNamespace(), getTagName() );
      aBaseDef.accept(visitor);
      return visitor.getElement();      
   }
   
   /**
    * Tests to see if the def passed in is an implicit variable. These variables
    * are not written to the xml.
    * 
    * @param aBaseDef
    */
   protected boolean isImplicitVariable(AeBaseDef aBaseDef)
   {
      if (aBaseDef instanceof AeVariableDef)
      {
         return ((AeVariableDef)aBaseDef).isImplicit();
      }
      return false;
   }

   /**
    * Used to determine if the base def is an empty container.
    * Empty container elements are not added to the bpel xml.
    * @param aBaseDef
    * @return true is the base def is an AeBaseContainer with no contents
    */
   private boolean isEmptyContainer(AeBaseDef aBaseDef)
   {
      if( aBaseDef instanceof AeBaseContainer )
      {
         AeEmptyContainerVisitor visitor = new AeEmptyContainerVisitor();
         aBaseDef.accept(visitor);
         return visitor.isEmpty();
      }
      return false;
   }

   /**
    * Setter for tag name
    * @param tagName
    */
   protected void setTagName(String tagName)
   {
      mTagName = tagName;
   }

   /**
    * Getter for tag name
    */
   protected String getTagName()
   {
      return mTagName;
   }

   /**
    * Setter for namespace
    * @param namespace
    */
   protected void setNamespace(String namespace)
   {
      mNamespace = namespace;
   }

   /**
    * Getter for namespace
    */
   protected String getNamespace()
   {
      return mNamespace;
   }

   /**
    * Avoids writing an empty container to the bpel xml by checking to see if 
    * the def being written is empty.
    */
   protected static class AeEmptyContainerVisitor extends AeAbstractDefVisitor
   {
      /** indicates that the container was empty */
      private boolean mEmpty;
      
      /**
       * Getter for the empty flag
       */
      public boolean isEmpty()
      {
         return mEmpty;
      }
      
      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationContainerDef)
       */
      public void visit(AeCorrelationContainerDef aDef)
      {
         checkEmpty(aDef);
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef)
       */
      public void visit(AeCorrelationSetContainerDef aDef)
      {
         checkEmpty(aDef);
      }

      /**
       * Check the container size and look to see if there
       * is a default fault handler installed.
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef)
       */
      public void visit(AeFaultHandlersContainerDef aDef)
      {
         if( aDef.getDefaultFaultHandler() != null )
         {
            mEmpty = false;
         }
         else
         {
            checkEmpty(aDef);
         }
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeVariablesContainerDef)
       */
      public void visit(AeVariablesContainerDef aDef)
      {
         checkEmpty(aDef);
         if (!isEmpty())
         {
            boolean foundExplicit = false;
            // we might still be empty if the container only has dynamic variables
            for (Iterator it=aDef.getValues(); it.hasNext();)
            {
               AeVariableDef def = (AeVariableDef) it.next();
               if (!def.isImplicit())
               {
                  foundExplicit = true;
                  break;
               }
            }
            
            // if the variable container only had implicit variables then we consider
            // it empty in terms of writing xml since these defs will get recreated
            // during the read.
            mEmpty = !foundExplicit;
         }
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef)
       */
      public void visit(AeLinkContainerDef aDef)
      {
         checkEmpty(aDef);
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerLinksContainerDef)
       */
      public void visit(AePartnerLinksContainerDef aDef)
      {
         checkEmpty(aDef);
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnersContainerDef)
       */
      public void visit(AePartnersContainerDef aDef)
      {
         checkEmpty(aDef);
      }

      /**
       * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef)
       */
      public void visit(AeSwitchCaseContainerDef aDef)
      {
         checkEmpty(aDef);
      }
      
      /**
       * Sets the empty flag if the container is empty
       * @param aContainer
       */
      private void checkEmpty( AeBaseContainer aContainer )
      {
         mEmpty = (aContainer.getSize() == 0);
      }
   }
}
