// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/io/writers/def/AeProcessWriter.java,v 1.5 2005/10/17 19:44:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.io.writers.def;

import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.util.AeXmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Serailizes AeProcessDef.
 */
public class AeProcessWriter extends AeDispatchWriter implements IAeBPELConstants
{
   /**
    * Default constructor.
    */
   public AeProcessWriter()
   {
      super(TAG_PROCESS);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.io.writers.def.IAeBpelDefWriter#createElement(org.activebpel.rt.bpel.def.AeBaseDef, org.w3c.dom.Element)
    */
   public Element createElement(AeBaseDef aBaseDef, Element aParentElement)
   {
      AeWriterVisitor visitor = new AeProcessWriterVisitor( null, IAeBPELConstants.BPWS_NAMESPACE_URI, getTagName() );
      aBaseDef.accept(visitor);
      return visitor.getElement();      
   }
}

/**
 * Custom AeWriterVisitor to create document prior to
 * creating process element.
 */
class AeProcessWriterVisitor extends AeWriterVisitor
{
   /**
    * @param aParentElement
    * @param aTagName
    */
   public AeProcessWriterVisitor(Element aParentElement, String aNamespace, String aTagName)
   {
      super(aParentElement, aNamespace, aTagName);
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.writers.def.AeWriterVisitor#addChildToParent(org.w3c.dom.Element)
    */
   protected void addChildToParent(Element aParentElement)
   {
      // no op here
   }

   /**
    * @see org.activebpel.rt.bpel.def.io.writers.def.AeWriterVisitor#createElement(org.w3c.dom.Element, java.lang.String, java.lang.String)
    */
   protected Element createElement(Element aParentElement, String aNamespace, String aName)
   {
      if (aParentElement == null)
         return createProcessElement(aNamespace, aName);
      else
         return super.createElement(aParentElement, aNamespace, aName);
   }
      
   /**
    * Create the xml doc first, then the process element.
    * Appends the process element to the doc and returns
    * the process element.
    * @param aNamespace
    * @param aName
    * @return process element
    */
   protected Element createProcessElement(String aNamespace, String aName)
   {
      Document doc = AeXmlUtil.newDocument();
      Element element = doc.createElementNS(aNamespace, aName);
      doc.appendChild( element );
      return element;
   }
}

