// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/validation/AeDefValidationVisitor.java,v 1.55 2006/02/02 23:53:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.validation;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.wsdl.Message;
import javax.wsdl.Part;
import javax.xml.namespace.QName;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeExpressionLanguageFactory;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.AeWSDLDefHelper;
import org.activebpel.rt.bpel.IAeContextWSDLProvider;
import org.activebpel.rt.bpel.IAeExpressionLanguageFactory;
import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.AeActivityPartnerLinkBaseDef;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeCorrelationContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetDef;
import org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeEventHandlersDef;
import org.activebpel.rt.bpel.def.AeFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef;
import org.activebpel.rt.bpel.def.AeImportDef;
import org.activebpel.rt.bpel.def.AeImportsContainerDef;
import org.activebpel.rt.bpel.def.AeMessageExchangesDef;
import org.activebpel.rt.bpel.def.AeNamedDef;
import org.activebpel.rt.bpel.def.AePartnerDef;
import org.activebpel.rt.bpel.def.AePartnerLinkDef;
import org.activebpel.rt.bpel.def.AePartnerLinksContainerDef;
import org.activebpel.rt.bpel.def.AePartnersContainerDef;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.def.AeVariablesContainerDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.activity.AeActivityAssignDef;
import org.activebpel.rt.bpel.def.activity.AeActivityBreakDef;
import org.activebpel.rt.bpel.def.activity.AeActivityCompensateDef;
import org.activebpel.rt.bpel.def.activity.AeActivityContinueDef;
import org.activebpel.rt.bpel.def.activity.AeActivityEmptyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityFlowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityForEachDef;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityPickDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReplyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySequenceDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySuspendDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef;
import org.activebpel.rt.bpel.def.activity.AeActivityTerminateDef;
import org.activebpel.rt.bpel.def.activity.AeActivityThrowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWaitDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWhileDef;
import org.activebpel.rt.bpel.def.activity.IAeLoopActivityDef;
import org.activebpel.rt.bpel.def.activity.support.AeAbstractExpressionDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignToDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignVarDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachBranchesDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef;
import org.activebpel.rt.bpel.def.activity.support.AeSourceDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef;
import org.activebpel.rt.bpel.def.activity.support.AeTargetDef;
import org.activebpel.rt.bpel.def.util.AeDefUtil;
import org.activebpel.rt.bpel.def.util.AeVariableData;
import org.activebpel.rt.bpel.def.util.AeVariableProperty;
import org.activebpel.rt.bpel.def.validation.expr.AeExpressionValidationContext;
import org.activebpel.rt.bpel.def.validation.expr.IAeExpressionValidationContext;
import org.activebpel.rt.bpel.def.validation.expr.IAeExpressionValidationResult;
import org.activebpel.rt.bpel.def.validation.expr.IAeExpressionValidator;
import org.activebpel.rt.bpel.def.validation.query.AeXPathQueryValidator;
import org.activebpel.rt.bpel.def.visitors.AeCheckStartActivityVisitor;
import org.activebpel.rt.bpel.def.visitors.AeDefAssignParentVisitor;
import org.activebpel.rt.bpel.def.visitors.AeDefOneWayVisitor;
import org.activebpel.rt.bpel.def.visitors.AeDefPathVisitor;
import org.activebpel.rt.bpel.def.visitors.AeDefTraverser;
import org.activebpel.rt.bpel.def.visitors.AeMessageExchangeVisitor;
import org.activebpel.rt.bpel.def.visitors.AeTraversalVisitor;
import org.activebpel.rt.bpel.def.visitors.IAeDefVisitor;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.util.AeXmlUtil;
import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;
import org.activebpel.rt.wsdl.def.IAePartnerLinkType;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;
import org.activebpel.rt.wsdl.def.IAeRole;
import org.exolab.castor.xml.schema.ElementDecl;
import org.exolab.castor.xml.schema.XMLType;

/**
 * Validation visitor for the definition layer.<p>
 *
 * Notes:<p>
 * Types validated in visual model (other than basic validation) but whose def counterpart
 * (if any) is not visited in logical model:<br>
 * <ul>
 * <li>AevNamespaceModel
 * <ul>
 */
public class AeDefValidationVisitor implements IAeDefVisitor, IAeValidationDefs
{
   // property alias error message: part not found on message
   protected static final String NO_PART_FOUND_ON_MESSAGE = AeMessages.getString("AeDefValidationVisitor.31"); //$NON-NLS-1$

   /** URI for XPath 1.0 */
   public final static String XPATH_URI = "http://www.w3.org/TR/1999/REC-xpath-19991116" ; //$NON-NLS-1$

   /** Used in conjunction with the traversal object to traverse the object model */
   protected IAeDefVisitor mTraversalVisitor;

   /** Error reporter used during validation process. */
   private IAeBaseErrorReporter mReporter ;

   /** provides access to all context wsdl */
   private IAeContextWSDLProvider mContextProvider;

   /** The expression language factory. */
   private IAeExpressionLanguageFactory mExpressionLanguageFactory;

   /**
    * Goes to true when a Pick or Receive activity is found with createInstance
    *  flag set to true.
    */
   private boolean mCreateInstanceFound = false ;

   /** List of partner links that are actually used, by name. */
   private HashMap mUsedPartnerLinks ;

   /** List of variables that are actually used, by name. */
   private ArrayList mUsedVariables ;

   /** List of variables known to each scope - for usage test. */
   private ArrayList mKnownVariables ;

   /** List of correlation sets that are actually used, by name. */
   private ArrayList mUsedCorrSets ;

   /** List of correlation sets known to each scope - for usage test. */
   private ArrayList mKnownCorrSets ;

   /** XPath query validator. */
   private AeXPathQueryValidator mXPathQueryValidator = new AeXPathQueryValidator() ;

   /** Link validation helper for this instance. */
   private AeLinkValidator mLinkValidator ;

   /** Process def being validated */
   private AeProcessDef mProcessDef;

   /** List of Scopes defined in the process. */
   private ArrayList mKnownScopes ;

   /** List of Compensate activities encountered during validation. */
   private ArrayList mKnownCompensateActivities ;

   /** List of activities where Create Instance is set to true.  These must
    *  all use the same correlation set.
    */
   private ArrayList mKnownInstanceCreators ;

   /**
    * Returns the current error reporting object.
    *
    * @return IAeErrorReporter
    */
   private IAeBaseErrorReporter getReporter()
   {
      return mReporter;
   }

   /**
    * Set the new error reporting object.
    *
    * @param aReporter The error reporting object to set.
    */
   private void setReporter( IAeBaseErrorReporter aReporter )
   {
      mReporter = aReporter;
   }

   /**
    * Returns the current link validation object.
    *
    * @return AeLinkValidator
    */
   private AeLinkValidator getLinkValidator()
   {
      return mLinkValidator;
   }

   /**
    * Set the new link validation object.
    *
    * @param aValidator The link validation object to set.
    */
   private void setLinkValidator( AeLinkValidator aValidator )
   {
      mLinkValidator = aValidator ;
   }

   /**
    * Operational ctor.  Sets error reporting object.
    *
    * @param aReporter The error reporting object to use during validation.
    */
   public AeDefValidationVisitor( IAeValidationContext aValidationContext )
   {
      setReporter( aValidationContext.getErrorReporter() );
      mTraversalVisitor = new AeTraversalVisitor(new AeDefTraverser(), this);
      mContextProvider = aValidationContext.getContextWSDLProvider();
      setExpressionLanguageFactory(aValidationContext.getExpressionLanguageFactory());
   }

   /**
    * Uses the stack to find the element's enclosing scope, or null if there is no enclosing scope.
    *
    * @return AeScopeDef
    */
   private AeScopeDef getParentScope( AeBaseDef aChild )
   {
      return AeDefUtil.getParentScope(aChild);
   }

   /**
    * @return IAeContextWSDLProvider
    */
   public IAeContextWSDLProvider getWsdlProvider()
   {
      return mContextProvider;
   }

   /**
    * Report an exception among the validation errors.
    *
    * @param aDef The node involved.
    * @param aCause The cause of the exception, i.e., what was being validated.
    * @param aThrowable The exception thrown.
    */
   protected void reportException( AeBaseDef aDef, String aCause, Throwable aThrowable )
   {
      String[] args = new String[] { aCause, aThrowable.getLocalizedMessage()};
      getReporter().addError( EXCEPTION_DURING_VALIDATION, args, aDef );
      MessageFormat form = new MessageFormat( EXCEPTION_DURING_VALIDATION );
      String msg = form.format( args );
      AeException.logError( aThrowable, msg );
   }

   /**
    * Get the list of used correlation sets.
    *
    * @return ArrayList
    */
   public ArrayList getUsedCorrSets()
   {
      if ( mUsedCorrSets == null )
         mUsedCorrSets = new ArrayList();

      return mUsedCorrSets;
   }

   /**
    * Get the list of used partner links.
    *
    * @return ArrayList
    */
   public HashMap getUsedPartnerLinks()
   {
      if ( mUsedPartnerLinks == null )
         mUsedPartnerLinks = new HashMap();

      return mUsedPartnerLinks;
   }

   /**
    * Get the list of used variables.
    *
    * @return ArrayList
    */
   public ArrayList getUsedVariables()
   {
      if ( mUsedVariables == null )
         mUsedVariables = new ArrayList();

      return mUsedVariables;
   }

   /**
    * Get the list of known correlation sets.
    *
    * @return ArrayList
    */
   public ArrayList getKnownCorrSets()
   {
      if ( mKnownCorrSets == null )
         mKnownCorrSets = new ArrayList();

      return mKnownCorrSets;
   }

   /**
    * Get the list of known variables.
    *
    * @return ArrayList
    */
   public ArrayList getKnownVariables()
   {
      if ( mKnownVariables == null )
         mKnownVariables = new ArrayList();

      return mKnownVariables;
   }

   /**
    * Given an iterator of variable defs, this method adds only the explicit var defs to the
    * list of known variables used by the usage test.
    * @param varIterator
    */
   private void addExplicitVariables(Iterator varIterator) 
   {
      for ( Iterator iter = varIterator; iter.hasNext() ; )
      {
         AeVariableDef var = (AeVariableDef)iter.next();
         if ( ! var.isImplicit() )
            getKnownVariables().add( var);
      }
   }

   /**
    * Look for partner links, variables and correlation sets that are
    * not referenced by any of the activities in the model.  This test will only
    * be valid if it's run AFTER the entire model has been traversed.
    */
   private void checkUnusedReferences( )
   {
      checkForUnusedReferences(WARNING_PARTNER_LINK_NOT_USED, getProcessDefNode().getPartnerLinks(), getUsedPartnerLinks().values());
      checkForUnusedReferences(WARNING_VARIABLE_NOT_USED, getKnownVariables(), getUsedVariables());
      checkForUnusedReferences(WARNING_CORR_SET_NOT_USED, getKnownCorrSets(), getUsedCorrSets());
   }

   /**
    * Reports warnings for any unused references (variables, partner links, correlation sets)
    * @param aWarningMessageToken token for warning message
    * @param aAllReferences iter of objects
    * @param aUsedReferences coll of objects
    */
   private void checkForUnusedReferences(String aWarningMessageToken, Iterator aAllReferences, Collection aUsedReferences)
   {
      Set set = new HashSet();
      while(aAllReferences.hasNext())
      {
         set.add(aAllReferences.next());
      }
      checkForUnusedReferences(aWarningMessageToken, set, aUsedReferences);
   }

   /**
    * Reports warnings for any unused references (variables, partner links, correlation sets)
    * @param aWarningMessageToken token for warning message
    * @param aAllReferences coll of objects
    * @param aUsedReferences coll of objects
    */
   private void checkForUnusedReferences(String aWarningMessageToken, Collection aAllReferences, Collection aUsedReferences)
   {
      Set unusedReferences = new HashSet(aAllReferences);
      unusedReferences.removeAll(aUsedReferences);
      for (Iterator iterator = unusedReferences.iterator(); iterator.hasNext();)
      {
         AeNamedDef namedDef = (AeNamedDef) iterator.next();
         getReporter().addWarning( aWarningMessageToken,  new String[] { namedDef.getName() }, namedDef );
         
      }
   }

   /**
    * Get the prefix for the namespace URI specified.
    *
    * @param aURI The namespace URI to search.
    *
    * @return String
    */
   private String getNSPrefix( String aURI )
   {
      if ( AeUtil.isNullOrEmpty( aURI ))
         return "{no namespace}"; //$NON-NLS-1$

      for ( Iterator iter = getProcessDefNode().getNamespaceList() ; iter.hasNext() ; )
      {
         String nsKey = (String)iter.next();
         if ( aURI.equals( getProcessDefNode().findNamespace( nsKey )))
            return nsKey.substring( nsKey.indexOf(':') + 1 );
      }

      return "{" + aURI + "}" ; //$NON-NLS-1$ //$NON-NLS-2$
   }

   /**
    * Validate a supermodel.
    *
    * @param aDef The root object to validate.
    */
   public void validate( AeProcessDef aDef )
   {
      if ( getWsdlProvider() == null )
      {
         // Can't do anything w/o WSDL Provider
         //
         String cause = AeMessages.getString("AeDefValidationVisitor.ERROR_4"); //$NON-NLS-1$
         AeException ae = new AeException(AeMessages.getString("AeDefValidationVisitor.5")); //$NON-NLS-1$
         reportException( aDef, cause, ae );
         AeException.logError( ae, cause );
         return ;
      }

      setProcessDefNode( aDef );
      setLinkValidator( new AeLinkValidator( getProcessDefNode(), getReporter() ) );

      try
      {
         preprocessForValidation(aDef);
         // Validate the entire model.
         //
         if ( aDef != null )
            aDef.accept( this );
         else
            return ;

         // Check for unresolved compensator scope references.
         //
         checkCompensateActivities();

         // Check create instance correlation sets.
         //
         checkCreateInstanceCorrelations();

         // Look for unused references.
         //
         checkUnusedReferences();

         // Look for cycles and illegal boundary crosses in the links.
         //
         getLinkValidator().checkLinks();
      }
      catch ( Throwable ex )
      {
         reportException( aDef, aDef.getName(), ex );
      }
   }

   /**
    * Ensure that compensation activities are correctly placed, and that all referenced scopes exist.
    */
   private void checkCompensateActivities()
   {
      for ( Iterator iter = getKnownCompensateActivities().iterator() ; iter.hasNext() ; )
      {
         AeActivityCompensateDef comp = (AeActivityCompensateDef)iter.next();
         boolean foundCatchOrCompensateHandler = false;
         AeBaseDef parent = comp.getParent();

         while (!foundCatchOrCompensateHandler && parent != null )
         {
            if (parent instanceof AeScopeCompensatorContainerDef)
            {
               foundCatchOrCompensateHandler = true;
               break ;
            }
            if (parent instanceof AeDefaultFaultHandlerDef || parent instanceof AeFaultHandlerDef)
            {
               foundCatchOrCompensateHandler = true;
               break ;
            }

            parent = parent.getParent();
         }

         if (!foundCatchOrCompensateHandler)
         {
            getReporter().addError( ERROR_MISPLACED_COMPENSATE, null, comp );
         }
         else
         {
            // The parent found must contain a first-level Scope by the name reference in the comp,
            //  if any.
            //
            if ( !AeUtil.isNullOrEmpty( comp.getScope()) && !comp.getScope().equals( FIELD_DEFAULT ))
            {
               AeScopeDef rootScope = comp.findRootScopeForCompensation();
               if ( rootScope == null )
               {
                  getReporter().addError( ERROR_MISPLACED_COMPENSATE, null, comp );
                  continue ;
               }

               boolean found = false ;

               // Find an immediate child scope with the Comp Activity's referenced name, if any.
               //
               for ( Iterator iter2 = getKnownScopes().iterator() ; iter2.hasNext() && !found ; )
               {
                  AeActivityScopeDef scope = (AeActivityScopeDef)iter2.next();
                  AeScopeDef parentScope = getInnermostEnclosingScope( scope );
                  if ( parentScope != null &&
                        parentScope.equals( rootScope ) &&
                        scope.getName().equals( comp.getScope()))
                     found = true ;
               }

               // Failed to find scope or comp'n activity wasn't in the right place.
               //
               if ( !found )
               {
                  getReporter().addError( ERROR_SCOPE_NOT_FOUND,
                                          new String[] { comp.getScope() },
                                          comp );
               }
            }
         }
      }
   }

   /**
    * Get the def's innermost enclosing scope, if any.
    *
    * @param aDef The def to check.
    *
    * @return AeScopeDef containing the def, or null if none found.
    */
   private AeScopeDef getInnermostEnclosingScope( AeBaseDef aDef )
   {
      AeBaseDef parent = aDef.getParent();
      while ( parent != null && !( parent instanceof AeScopeDef ) )
         parent = parent.getParent();

      return (AeScopeDef)parent ;
   }

   /**
    * Ensure that all create instance activities use the same correlation set(s).
    */
   private void checkCreateInstanceCorrelations()
   {
      ArrayList corrSetNames = new ArrayList();
      boolean csListsDontMatch = false ;
      boolean refListInitialized = false ;
      if ( getKnownInstanceCreators().size() <= 1 )
         return ;

      for ( Iterator iter = getKnownInstanceCreators().iterator() ; iter.hasNext() ; )
      {
         Object ob = iter.next();
         if ( ob instanceof AeActivityPickDef )
         {
            Iterator iter2 = ((AeActivityPickDef)ob).getMessageList();
            while ( iter2.hasNext() && !csListsDontMatch )
            {
               AeOnMessageDef cnode = (AeOnMessageDef)iter2.next();
               if ( ((AeOnMessageDef)cnode).hasCorrelationList())
               {
                  Iterator csList = ((AeOnMessageDef)cnode).getCorrelationList();
                  if ( csList == null )
                  {
                     csListsDontMatch = true ;
                     break ;
                  }
                  else
                  {
                     // Check list contents to see if it matches the reference.
                     //
                     while ( csList.hasNext() )
                     {
                        if ( !refListInitialized )
                        {
                           // set the reference
                           //
                           corrSetNames.add(((AeCorrelationDef)csList.next()).getCorrelationSetName());
                        }
                        else if ( !corrSetNames.contains( ((AeCorrelationDef)csList.next()).getCorrelationSetName()))
                        {
                           csListsDontMatch = true ;
                           break ;
                        }
                     }
                  }
               }
               else
               {
                  csListsDontMatch = true ;
                  break ;
               }

               // Got through a create instance activity - we have a reference list.
               //
               refListInitialized = true ;
            }
         }
         else
         {
            if ( ((AeActivityReceiveDef)ob).hasCorrelationList())
            {
               Iterator csList = ((AeActivityReceiveDef)ob).getCorrelationList();
               if ( csList == null )
               {
                  csListsDontMatch = true ;
                  break ;
               }
               else
               {
                  // Check list contents to see if it matches the reference.
                  //
                  while ( csList.hasNext() )
                  {
                     if ( !refListInitialized )
                     {
                        // set the reference
                        //
                        corrSetNames.add(((AeCorrelationDef)csList.next()).getCorrelationSetName());
                     }
                     else if ( !corrSetNames.contains( ((AeCorrelationDef)csList.next()).getCorrelationSetName()))
                     {
                        csListsDontMatch = true ;
                        break ;
                     }
                  }
               }
            }
            else
            {
               csListsDontMatch = true ;
               break ;
            }

            // Got through a create instance activity - we have a reference list.
            //
            refListInitialized = true ;
         }
      }

      if ( csListsDontMatch )
      {
         for ( Iterator iter = getKnownInstanceCreators().iterator() ; iter.hasNext() ; )
            getReporter().addError( ERROR_CS_MISMATCH, null, (AeBaseDef)iter.next() );
      }
   }

   /**
    * Ensure that parentage and location maps are current.
    *
    * @param aDef The Process definition.
    */
   private void preprocessForValidation(AeProcessDef aDef)
   {
      // First ensure that the parent assignments and location paths have been updated.
      //
      AeDefAssignParentVisitor parentVisitor = new AeDefAssignParentVisitor();
      parentVisitor.visit( aDef );
      AeDefPathVisitor pathVisitor = new AeDefPathVisitor();
      pathVisitor.visit( aDef );
      new AeMessageExchangeVisitor().visit(aDef); 
      new AeDefOneWayVisitor(getWsdlProvider()).visit(aDef);
   }

   /**
    * Perform lowest-level validation at the node level.
    *
    * @param aDef AeBaseDef object to be validated.
    */
   private void basicValidation( AeBaseDef aDef )
   {
      // Recurse into and validate any child nodes (depth-first traversal).
      //
      aDef.accept( mTraversalVisitor );

      if ( aDef instanceof AeActivityDef )
      {
         // Check name for validity.
         if ( ! isEmptyOrValidName((AeActivityDef)aDef) )
            getReporter().addError( ERROR_INVALID_NAME, new Object[] {((AeActivityDef)aDef).getName()}, aDef );


         for ( Iterator iter = ((AeActivityDef)aDef).getSourceList() ; iter.hasNext() ; )
         {
            getLinkValidator().addSource( (AeSourceDef)iter.next(), (AeActivityDef)aDef );
         }

         for ( Iterator iter = ((AeActivityDef)aDef).getTargetList() ; iter.hasNext() ; )
         {
            getLinkValidator().addTarget( (AeTargetDef)iter.next(), (AeActivityDef)aDef );
         }

         if (AeUtil.notNullOrEmpty(((AeActivityDef) aDef).getJoinCondition()))
         {
            validateJoinConditionExpression(((AeActivityDef) aDef).getJoinCondition(), aDef);
         }
      }
      else if ( aDef instanceof AeNamedDef && !(aDef instanceof AeScopeDef) )
      {
         // Check name for validity.
         if ( ! isValidName((AeNamedDef)aDef) )
            getReporter().addError( ERROR_INVALID_NAME, new Object[] {((AeNamedDef)aDef).getName()}, aDef );

      }
   }

   /**
    * Locate a partner link definition in the process, by name.
    *
    * @param aPartnerLink The partner link name to search.
    * @param aDef The owner of the item being checked.
    *
    * @return AePartnerLinkDef if found, or null.
    */
   private AePartnerLinkDef getPartnerLinkByName( String aPartnerLink, AeBaseDef aDef )
   {
      if ( getUsedPartnerLinks().keySet().contains( aPartnerLink ))
      {
         return (AePartnerLinkDef)getUsedPartnerLinks().get( aPartnerLink );
      }
      else
      {
         for ( Iterator iter = getProcessDefNode().getPartnerLinks() ; iter.hasNext() ; )
         {
            AePartnerLinkDef pld = (AePartnerLinkDef)iter.next();
            if ( pld.getName().equals( aPartnerLink ))
            {
               // Record this PLD as used (don't record as such if just partner checking).
               //
               if ( !(aDef instanceof AePartnerDef ))
                  getUsedPartnerLinks().put( aPartnerLink, pld );

               return pld ;
            }
         }
      }

      return null ;
   }

   /**
    * Resolve a partner link name from the list known to the process.
    *
    * @param aPartnerLink The name to check.
    * @param aDef The def object that owns the partner link.
    *
    * @return AePartnerLinkDef if found, null otherwise
    */
   private AePartnerLinkDef partnerLinkExistsInProcess( String aPartnerLink, AeBaseDef aDef )
   {
      AePartnerLinkDef pld = getPartnerLinkByName( aPartnerLink, aDef );
      if ( pld != null )
      {
         return pld ;
      }
      else
      {
         // error - partner link name not resolved.
         //
         String plName = "" ; //$NON-NLS-1$
         if ( aDef instanceof AeActivityPartnerLinkBaseDef )
            plName = ((AeActivityPartnerLinkBaseDef)aDef).getPartnerLink();
         else if ( aDef instanceof AeOnMessageDef )
            plName = ((AeOnMessageDef)aDef).getPartnerLink();

         getReporter().addError( ERROR_PARTNER_LINK_NOT_FOUND,
                                 new String[] { plName },
                                 aDef );

         return null ;
      }
   }

   /**
    * Retrieve a variable from the process scope heirarchy, by name.
    *
    * @param aVariable The name of the variable to search.
    * @param aDef The def object from which to search (up).
    *
    * @return AeVariableDef or null if not found.
    */
   private AeVariableDef getVariableByName( String aVariable, AeBaseDef aDef )
   {
      // Search all enclosing scopes.
      //
      AeVariableDef def = AeDefUtil.getVariableByName(aVariable, aDef);
      if (def != null)
         getUsedVariables().add(def);
      return def;
   }

   /**
    * Resolve a variable name from the list known to the hierarchy of scopes.
    *
    * @param aVariable The name to check.
    * @param aDef The Def object that owns the variable.
    *
    * @return AeVariableDef if found, null otherwise
    */
   private AeVariableDef variableExistsInProcess( String aVariable, AeBaseDef aDef )
   {
      AeVariableDef var = getVariableByName( aVariable, aDef );
      if ( var != null )
      {
         // The variable *name* was resolved.
         //
         return var ;
      }

      // Never found the variable.
      //
      getReporter().addError( ERROR_VAR_NOT_FOUND,
                              new String[] { aVariable },
                              resolveLinkDef ( aDef ));
      return null ;
   }

   /**
    * Locate a correlation set among the process' definitions, by name.
    *
    * @param aCorrSet The name to search.
    * @param aDef The def object from which to search (up).
    *
    * @return AeCorrelationSetDef if found, otherwise null.
    */
   private AeCorrelationSetDef getCorrSetByName( String aCorrSet, AeBaseDef aDef )
   {
      AeCorrelationSetDef set = AeDefUtil.getCorrSetByName(aCorrSet, aDef);
      if (set != null)
         getUsedCorrSets().add(set);
      return set;
   }

   /**
    * Locate a correlation set among the process' definitions.
    *
    * @param aCorrSet The name to search.
    * @param aDef The owner of the corr set.
    *
    * @return AeCorrelationSetDef or null if not found.
    */
   private AeCorrelationSetDef corrSetExistsInProcess( String aCorrSet, AeBaseDef aDef )
   {
      AeCorrelationSetDef corrSet = getCorrSetByName( aCorrSet, aDef );
      if ( corrSet != null )
      {
         return corrSet ;
      }
      else
      {
         // Never found the correlation set.
         //
         getReporter().addError( ERROR_CORR_SET_NOT_FOUND,
                                 new String[] { aCorrSet },
                                 aDef );
         return null ;
      }
   }

   /**
    * Ensure that variable has property aliases defined for all correlated properties.
    *
    * @param aCorrSet The correlation set def to check.
    * @param aVariable The variable to be checked.
    * @param aDef The activity that owns the variable/corr set combo.
    */
   private void validateCorrSet( AeCorrelationSetDef aCorrSet, AeVariableDef aVariable, AeBaseDef aDef )
   {
      if (  aVariable != null &&
            aVariable.getMessageType() != null &&
            !aVariable.getMessageType().equals( EMPTY_QNAME ) &&
            aCorrSet != null )
      {
         StringBuffer buf = new StringBuffer();
         for ( Iterator iterCS = aCorrSet.getPropertiesList() ; iterCS.hasNext() ; )
         {
            QName propName = (QName)iterCS.next();
            IAePropertyAlias alias =
               AeWSDLDefHelper.getWSDLMessagePropertyAlias( getWsdlProvider(),
                                                            aVariable.getMessageType(),
                                                            propName );
            if ( alias == null )
            {
               if ( buf.length() > 0 )
                  buf.append( ", " ); //$NON-NLS-1$

               buf.append( "'"+propName.getLocalPart()+"'" ); //$NON-NLS-1$ //$NON-NLS-2$
            }
            else
            {
               // make sure a message exists for the property alias part
               if( alias.getPart() != null )
               {
                  AeBPELExtendedWSDLDef def = AeWSDLDefHelper.getWSDLDefinitionForMsg( getWsdlProvider(), alias.getMessageType() );
                  Message message = def.getMessage( alias.getMessageType() );
                  Part matchingPart = message.getPart( alias.getPart() );
                  if( matchingPart == null  )
                  {
                     Object[] args = { alias.getPart(), message.getQName(), alias.getPropertyName() };
                     getReporter().addError( NO_PART_FOUND_ON_MESSAGE, args, aDef );//$NON-NLS-1$
                  }
               }
               // check alias' part and any query against the message type.
               if ( !AeUtil.isNullOrEmpty( alias.getQuery()) && !AeUtil.isNullOrEmpty( alias.getPart()))
                  validatePartQuery( aDef, aVariable.getName(), alias.getPart(), alias.getQuery());
            }
         }

         if ( buf.length() > 0 )
         {
            getReporter().addError( ERROR_CORRELATION_PROP_ALIASES_NOT_FOUND,
                                    new String[] { aVariable.getName(),
                                                   aVariable.getMessageType().getLocalPart(),
                                                   buf.toString() },
                                    aDef );
         }
      }
   }

   /**
    * Validate that a property alias exists for the given variable type and property name
    * @param aName
    * @param aString
    * @param aDef
    */
   private void validatePropertyAlias(AeVariableDef aVarDef, QName aPropertyName, AeBaseDef aDef)
   {
      IAePropertyAlias alias = null;

      if( aVarDef.getMessageType() != null)
         alias = AeWSDLDefHelper.getWSDLMessagePropertyAlias( getWsdlProvider(),
                                                      aVarDef.getMessageType(),
                                                      aPropertyName);
      if (alias == null)
      {
         getReporter().addError( ERROR_PROP_ALIASES_NOT_FOUND,
                                 new String[] { aVarDef.getName(),
                                                getLocalPart(aVarDef.getMessageType())},
                                 resolveLinkDef( aDef ));
      }
   }

   /**
    * Util method for getting the local part from the QName or the string "null"
    * if it's null.
    * @param aName
    */
   protected String getLocalPart(QName aName)
   {
      if (aName != null)
         return aName.getLocalPart();
      return "null"; //$NON-NLS-1$
   }

   /**
    * Validate a Variable/Part combination.
    *
    * @param aVariableType The QName of the variable type to validate.
    * @param aPart The name of the variable's part to validate (which may be null).
    * @param aDef The def object that owns the variable in question.
    */
   private void validateVariableAndPart( QName aVariableType, String aPart, AeBaseDef aDef )
   {
      if ( aPart != null )
      {
         // Check for part in WSDL definition.
         //
         AeBPELExtendedWSDLDef def = AeWSDLDefHelper.getWSDLDefinitionForMsg( getWsdlProvider(), aVariableType );
         if ( def != null )
         {
            boolean found = false ;
            for ( Iterator iterParts = def.getMessageParts( aVariableType ) ; iterParts.hasNext() ; )
            {
               Part part = (Part)iterParts.next();
               if ( part.getName().equals( aPart ))
               {
                  found = true ;
                  break ;
               }
            }

            if ( !found )
            {
               getReporter().addError( ERROR_VAR_PART_NOT_FOUND,
                                       new String[] { aPart,
                                                      getNSPrefix( aVariableType.getNamespaceURI()),
                                                      aVariableType.getLocalPart()
                                                       },
                                       resolveLinkDef( aDef ))  ;
            }
         }
      }
   }

   /**
    * Process the expression validation result.  This amounts to extracting any errors or warnings from
    * the result object, as well as asking it for lists of AeVariableData and AeVariableProperty objects
    * in order to do various additional validation tasks.
    *
    * @param aResult
    * @param aDef
    */
   private void processExpressionValidationResult(IAeExpressionValidationResult aResult, AeBaseDef aDef)
   {
      // Process any info messages found by the validator.
      for ( Iterator iter = aResult.getInfoList().iterator() ; iter.hasNext() ; )
      {
         String info = (String) iter.next();
         getReporter().addInfo(info, null, resolveLinkDef(aDef));
      }
 
      // Process any errors found by the validator.
      for ( Iterator iter = aResult.getErrors().iterator() ; iter.hasNext() ; )
      {
         String error = (String) iter.next();
         getReporter().addError(error, null, resolveLinkDef(aDef));
      }

      // Process any warnings found by the validator.
      for ( Iterator iter = aResult.getWarnings().iterator() ; iter.hasNext() ; )
      {
         String warning = (String) iter.next();
         getReporter().addWarning(warning, null, resolveLinkDef(aDef));
      }

      // Check variable data list.
      for (Iterator iter = aResult.getVarDataList().iterator(); iter.hasNext();)
      {
         AeVariableData varData = (AeVariableData) iter.next();
         AeVariableDef varDef = variableExistsInProcess(varData.getVarName(), aDef);
         if (varDef != null && varData.getPart() != null)
            validateVariableAndPart(varDef.getMessageType(), varData.getPart(), aDef);
      }

      // Check variable property combos.
      for (Iterator iter = aResult.getVarPropertyList().iterator(); iter.hasNext();)
      {
         AeVariableProperty varProp = (AeVariableProperty) iter.next();
         AeVariableDef varDef = variableExistsInProcess(varProp.getVarName(), aDef);
         if (varDef != null && varProp.getProperty() != null)
            validatePropertyAlias(varDef, AeDefUtil.parseQName(aDef, varProp.getProperty()), aDef);
      }
   }

   /**
    * Creates the validation context to use during expression validation.
    *
    * @param aDef
    */
   protected IAeExpressionValidationContext createExpressionValidationContext(AeBaseDef aDef)
   {
      return new AeExpressionValidationContext(aDef);
   }

   /**
    * Validates the given query.
    *
    * @param aQuery the query to validate
    * @param aDef the def element holding the query
    */
   private void validateQuery( String aQuery, AeBaseDef aDef )
   {
      try
      {
         IAeExpressionValidator validator = getExpressionLanguageFactory().createExpressionValidator(
               AeExpressionLanguageFactory.XPATH_EXPR_LANGUAGE_URI);
         IAeExpressionValidationContext context = createExpressionValidationContext(aDef);
         IAeExpressionValidationResult vResult = validator.validateExpression(context, aQuery);
         processExpressionValidationResult(vResult, aDef);
      }
      catch (Throwable t)
      {
         getReporter().addError(AeMessages.getString("AeDefValidationVisitor.INVALID_QUERY_ERROR"), //$NON-NLS-1$
               new String[] { aQuery , t.getLocalizedMessage() }, resolveLinkDef(aDef));
      }
   }

   /**
    * Validate the given expression.
    *
    * @param aExpression The expression to validate.
    * @param aDef The def element holding the expression.
    */
   private void validateExpression( String aExpression, AeBaseDef aDef )
   {
      validateExpression(null, aExpression, aDef, aDef);
   }

   /**
    * Gets the expression language from the process def
    */
   private String getProcessDefaultExpressionLanguage()
   {
      String expressionLanguage = getProcessDefNode().getExpressionLanguage();
      return expressionLanguage;
   }

   /**
    * Validates the expression given the language, expression, def, and def to use to report an error
    * @param aExpressionLanguage optional language, defaults to the process's expression language
    * @param aExpression can be null or empty
    * @param aDef def that contains the expression
    * @param aErrorReference def to used to report the error (i.e. the parent activity or the same value as aDef)
    */
   private void validateExpression(String aExpressionLanguage, String aExpression, AeBaseDef aDef, AeBaseDef aErrorReference)
   {
      String language = AeUtil.notNullOrEmpty(aExpressionLanguage)? aExpressionLanguage : getProcessDefaultExpressionLanguage();
         
      try
      {
         IAeExpressionValidator validator = getExpressionLanguageFactory().createExpressionValidator(language);
         IAeExpressionValidationContext context = createExpressionValidationContext(aDef);
         IAeExpressionValidationResult vResult = validator.validateExpression(context, aExpression);
         processExpressionValidationResult(vResult, aDef);
      }
      catch (Throwable t)
      {
         getReporter().addError(ERROR_INVALID_EXPRESSION,
               new String[] { aExpression, t.getLocalizedMessage() }, resolveLinkDef(aDef));
      }
   }

   /**
    * Validate the given boolean expression.
    *
    * @param aExpression The expression to validate.
    * @param aDef The def element holding the expression.
    */
   private void validateBooleanExpression(String aExpression, AeBaseDef aDef)
   {
      String expressionLanguage = getProcessDefNode().getExpressionLanguage();
      try
      {
         IAeExpressionValidator validator = getExpressionLanguageFactory().createExpressionValidator(expressionLanguage);
         IAeExpressionValidationContext context = createExpressionValidationContext(aDef);
         IAeExpressionValidationResult vResult = validator.validateBooleanExpression(context, aExpression);
         processExpressionValidationResult(vResult, aDef);
      }
      catch (Throwable t)
      {
         getReporter().addError(ERROR_INVALID_EXPRESSION,
               new String[] { aExpression, t.getLocalizedMessage() }, resolveLinkDef(aDef));
      }
   }

   /**
    * Validate the given duration expression.
    *
    * @param aExpression The expression to validate.
    * @param aDef The def element holding the expression.
    */
   private void validateDurationExpression(String aExpression, AeBaseDef aDef)
   {
      String expressionLanguage = getProcessDefNode().getExpressionLanguage();
      try
      {
         IAeExpressionValidator validator = getExpressionLanguageFactory().createExpressionValidator(expressionLanguage);
         IAeExpressionValidationContext context = createExpressionValidationContext(aDef);
         IAeExpressionValidationResult vResult = validator.validateDurationExpression(context, aExpression);
         processExpressionValidationResult(vResult, aDef);
      }
      catch (Throwable t)
      {
         getReporter().addError(ERROR_INVALID_EXPRESSION,
               new String[] { aExpression, t.getLocalizedMessage() }, resolveLinkDef(aDef));
      }
   }

   /**
    * Validate the given deadline expression.
    *
    * @param aExpression The expression to validate.
    * @param aDef The def element holding the expression.
    */
   private void validateDeadlineExpression(String aExpression, AeBaseDef aDef)
   {
      String expressionLanguage = getProcessDefNode().getExpressionLanguage();
      try
      {
         IAeExpressionValidator validator = getExpressionLanguageFactory().createExpressionValidator(expressionLanguage);
         IAeExpressionValidationContext context = createExpressionValidationContext(aDef);
         IAeExpressionValidationResult vResult = validator.validateDurationExpression(context, aExpression);
         processExpressionValidationResult(vResult, aDef);
      }
      catch (Throwable t)
      {
         getReporter().addError(ERROR_INVALID_EXPRESSION,
               new String[] { aExpression, t.getLocalizedMessage() }, resolveLinkDef(aDef));
      }
   }

   /**
    * Validate the given join condition expression.
    *
    * @param aExpression The expression to validate.
    * @param aDef The def element holding the expression.
    */
   private void validateJoinConditionExpression(String aExpression, AeBaseDef aDef)
   {
      String expressionLanguage = getProcessDefNode().getExpressionLanguage();
      try
      {
         IAeExpressionValidator validator = getExpressionLanguageFactory().createExpressionValidator(expressionLanguage);
         IAeExpressionValidationContext context = createExpressionValidationContext(aDef);
         IAeExpressionValidationResult vResult = validator.validateJoinConditionExpression(context, aExpression);
         processExpressionValidationResult(vResult, aDef);
      }
      catch (Throwable t)
      {
         getReporter().addError(ERROR_INVALID_EXPRESSION,
               new String[] { aExpression, t.getLocalizedMessage() }, resolveLinkDef(aDef));
      }
   }

   /**
    * Resolve a port type from the WSDL known to the process.
    *
    * @param aPortType The portType to check.
    *
    * @return boolean <code>true</code> if the portType is resolved.
    */
   private boolean portTypeExistsInWSDL( QName aPortType )
   {
      // Ensure that the type can be resolved.
      //
      if ( null == AeWSDLDefHelper.getWSDLDefinitionForPortType( getWsdlProvider(), aPortType ))
         return false ;
      else
         return true ;
   }

   /**
    * Verify that the specified fault name is handled by some fault handler in the process.
    *
    * @param aFaultName The fault name to check.
    * @param aMessageType The message type to use as a secondary check.
    * @param aDef The def object that owns the fault being checked.
    *
    * @return boolean <code>true</code> if the fault is handled explicitly.
    *
    * TODO: AeActivityScopeImpl has code that does this - look at creating interfaces so we
    * can combine the logic someplace (good newguy project).
    */
   private boolean faultIsHandledInProcess( QName aFaultName, QName aMessageType, AeActivityThrowDef aDef )
   {
      // Verify that fault QName is known.
      //
      AeScopeDef parent = getParentScope( aDef );
      while (  parent != null )
      {
         if ( parent.getDefaultFaultHandler() != null )
            return true;

         for ( Iterator iter = parent.getFaultHandlerList() ; iter.hasNext() ; )
         {
            AeFaultHandlerDef fh = (AeFaultHandlerDef)iter.next();
            if ( fh.getFaultName() != null && fh.getFaultName().equals( aFaultName ))
               return true ;
         }

         parent = getParentScope( parent );
      }

      // Second check - look for fault variable type match.
      //
      parent = getParentScope( aDef );
      while (  parent != null )
      {
         for ( Iterator iter = parent.getFaultHandlerList() ; iter.hasNext() ; )
         {
            AeFaultHandlerDef fh = (AeFaultHandlerDef)iter.next();
            AeVariableDef varDef = getVariableByName( fh.getFaultVariable(), aDef );
            if ( varDef != null && varDef.getMessageType().equals( aMessageType ))
               return true ;
         }

         parent = getParentScope( parent );
      }

      return false ;
   }

   /**
   * Check for existence of the specified operation, including variable type checking.
   *
   * @param aPortType Port type the operation belongs to.
   * @param aOperation Name of the operation.
   * @param aInput QName for the input variable message type.
   * @param aOutput QName for the output variable message type.
   *
   * @return boolean <code>true</code> if the operation is resolved.
   */
   protected boolean operationExistsInWSDL( QName aPortType, String aOperation, QName aInput, QName aOutput )
   {
      if ( null == AeWSDLDefHelper.getWSDLDefinition( getWsdlProvider(), aPortType, aOperation, aInput, aOutput ))
         return false ;
      else
         return true ;
   }

   /**
    * Check partner, port type, operation for empty field, validate non-empty ones.
    *
    * @param aDef The def object whose fields we're validating.
    */
   private void checkPPO( AeActivityPartnerLinkBaseDef aDef )
   {
      checkPPO(aDef, aDef.getPartnerLink(), aDef.getPortType(), aDef.getOperation());
   }

   /**
    * Check partner, port type, operation for empty field, and validate non-empty ones.
    *
    * @param aDef The def object whose fields we're validating.
    * @param aPartnerLink the partner link used by this definition.
    * @param aPortType the portType defined for use in this definition.
    * @param aOperation the operation defined for use in this definition.
    */
   private void checkPPO(AeBaseDef aDef, String aPartnerLink, QName aPortType, String aOperation )
   {
      // Check required fields.
      //
      if ( isUndefined(aPartnerLink) )
      {
         getReporter().addError( ERROR_FIELD_MISSING,
                                 new String[] { AeActivityPartnerLinkBaseDef.TAG_PARTNER_LINK },
                                 aDef );
      }
      else
      {
         // Find the partner link name specified.
         //
         AePartnerLinkDef pld = partnerLinkExistsInProcess( aPartnerLink, aDef );

         if(pld != null)
         {
            // check that the partner link type is correct for the passed partner link
            //
            AeBPELExtendedWSDLDef wsdlDef = AeWSDLDefHelper.getWSDLDefinitionForPLT( getWsdlProvider(), pld.getPartnerLinkTypeName() );
            // if not null check further otherwise null partner link errors are reported elsewhere so no need here
            if ( wsdlDef != null)
            {
               boolean receive = ! (aDef instanceof AeActivityInvokeDef);
               String roleName = null;

               // if this is a receive/reply or onMessage not an invoke
               // get the myRole else get the partnerRole for invokes
               if(receive)
               {
                  roleName = pld.getMyRole();
                  if(AeUtil.isNullOrEmpty(roleName))
                  {
                     getReporter().addError( ERROR_PARTNER_LINK_MISSING_MYROLE,
                                             new String[] { aPartnerLink },
                                             aDef );
                  }
               }
               else
               {
                  roleName = pld.getPartnerRole();
                  if(AeUtil.isNullOrEmpty(roleName))
                  {
                     getReporter().addError( ERROR_PARTNER_LINK_MISSING_PARTNERROLE,
                                             new String[] { aPartnerLink },
                                             aDef );
                  }
               }

               // if we found the appropriate role
               if(! AeUtil.isNullOrEmpty(roleName))
               {
                  // Get the portType for the associated role, note plt won't be null or get wsdl check above would fail
                  IAePartnerLinkType plt = wsdlDef.getPartnerLinkType(pld.getPartnerLinkTypeName().getLocalPart());
                  QName rolePortType = null;
                  for(Iterator iter=plt.getRoleList(); iter.hasNext();)
                  {
                     IAeRole role = (IAeRole)iter.next();
                     if(roleName.equals(role.getName()))
                     {
                        // check the wsdl has a porttype for the role
                        if(role.getPortType() != null)
                        {
                           rolePortType = role.getPortType().getQName();
                        }
                        else
                        {
                           getReporter().addError( ERROR_ROLE_HAS_NO_PORTTYPE,
                                                   new String[] { roleName, plt.getName() },
                                                   pld );
                        }
                        break;
                     }
                  }

                  // if we don't have a role portType that error is reported either above or in partnerLink
                  if(rolePortType != null)
                  {
                     // check that portType passed is the same as defined in the role from plt
                     if(! AeUtil.compareObjects(aPortType, rolePortType))
                     {
                        String strRolePortType = rolePortType == null ? "": rolePortType.toString();                      //$NON-NLS-1$
                        String strDefPortType = aPortType == null ? "": aPortType.toString();                      //$NON-NLS-1$
                        getReporter().addError( ERROR_PORTTYPE_MISMATCH,
                                                new String[] { aPartnerLink, roleName, strRolePortType, strDefPortType},
                                                aDef );

                     }
                     else
                     {
                        // Validate the portType name specified can be found.
                        if ( !portTypeExistsInWSDL( aPortType ))
                        {
                           // error - portType not resolved.
                           getReporter().addError( ERROR_PORT_TYPE_NOT_FOUND,
                                                   new String[] { aPortType.getLocalPart(), aPortType.getNamespaceURI() },
                                                   aDef );
                        }

                        // validate operation for portType
                        if (  isUndefined(aOperation))
                        {
                           getReporter().addError( ERROR_FIELD_MISSING,
                                                   new String[] { AeActivityPartnerLinkBaseDef.TAG_OPERATION },
                                                   aDef );
                        }
                     }
                  }
               }
            }
         }
      }
   }

   // Visit methods.

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeProcessDef)
    */
   public void visit(AeProcessDef aDef)
   {
      // Recursively validate all nodes in the process.
      //
      basicValidation( aDef  );

      if ( aDef.getCompensatorContainer().getActivity() != null &&
            !aDef.getEnableInstanceCompensation())
      {
         getReporter().addWarning( WARN_COMPENSATION_HANDLER_NOT_ENABLED, null, aDef );
      }

      // Check for create instance found during traversal.
      //
      if ( !aDef.isAbstractProcess() && !mCreateInstanceFound )
         getReporter().addError( ERROR_NO_CREATE, null, aDef );

      // Validate that the process definition conforms to BPEL spec when CreateInstance specified
      if (mCreateInstanceFound)
      {
         AeCheckStartActivityVisitor viz = new AeCheckStartActivityVisitor(getReporter());
         viz.doValidation(getKnownInstanceCreators());
      }

      // Check for invalid expression language override.
      if (!getExpressionLanguageFactory().supportsLanguage(aDef.getExpressionLanguage()))
      {
         getReporter().addError(
               AeMessages.getString("AeDefValidationVisitor.EXPR_LANGUAGE_NOT_SUPPORTED_ERROR"), //$NON-NLS-1$
               new String[] { aDef.getExpressionLanguage() }, aDef);
      }
      else if (AeUtil.notNullOrEmpty(aDef.getExpressionLanguage()) && !AeExpressionLanguageFactory.XPATH_EXPR_LANGUAGE_URI.equals(aDef.getExpressionLanguage()))
      {
         getReporter().addInfo(
               AeMessages.getString("AeDefValidationVisitor.EXPR_LANGUAGE_NONSTANDARD_WARNING"), //$NON-NLS-1$
               new String[] { aDef.getExpressionLanguage() }, aDef);
      }

      // Finally, since the Process is also a Scope, record its known explicit variables, etc.,
      //  to check for usage later.
      addExplicitVariables( ((AeScopeDef)aDef).getVariables() );

      for ( Iterator iter = ((AeScopeDef)aDef).getCorrelationSets() ; iter.hasNext() ; )
         getKnownCorrSets().add( iter.next());
      
      new AeMessageExchangeValidationVisitor(getReporter()).visit(aDef);
      new AeExtensionElementVisitor(getReporter()).visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityAssignDef)
    */
   public void visit(AeActivityAssignDef aDef)
   {
      basicValidation( aDef );

      // Check for required Copy child.
      //
      boolean copyFound = false ;
      Iterator iter = aDef.getCopyList();
      while ( iter.hasNext())
      {
         if ( iter.next() instanceof AeAssignCopyDef )
         {
            copyFound = true ;
            break ;
         }
      }
      if ( !copyFound )
         getReporter().addError( ERROR_NO_COPY, null, aDef );

      // Check for Copy children with no "To" or no "From" (or both)
      iter = aDef.getCopyList();
      while (iter.hasNext())
      {
         Object obj = iter.next();
         if (obj instanceof AeAssignCopyDef)
         {
            AeAssignCopyDef cdef = (AeAssignCopyDef) obj;
            if (cdef.getTo() == null)
               getReporter().addError(AeMessages.getString("AeDefValidationVisitor.ASSIGNCOPY_WITH_MISSING_TO"), null, aDef); //$NON-NLS-1$
            else if (cdef.getTo().isEmpty())
               getReporter().addWarning( AeMessages.getString("AeDefValidationVisitor.ASSIGNCOPY_WITH_EMPTY_TO_ERROR"), null, aDef ); //$NON-NLS-1$

            if (cdef.getFrom() == null)
               getReporter().addError(AeMessages.getString("AeDefValidationVisitor.ASSIGNCOPY_WITH_MISSING_FROM"), null, aDef); //$NON-NLS-1$
            else if (cdef.getFrom().isEmpty())
               getReporter().addError(AeMessages.getString("AeDefValidationVisitor.ASSIGNACT_WITH_EMPTY_FROM"), null, aDef); //$NON-NLS-1$
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityCompensateDef)
    */
   public void visit(AeActivityCompensateDef aDef)
   {
      basicValidation( aDef  );

      getKnownCompensateActivities().add( aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityEmptyDef)
    */
   public void visit(AeActivityEmptyDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityFlowDef)
    */
   public void visit(AeActivityFlowDef aDef)
   {
      basicValidation( aDef  );

      if ( !aDef.getActivityList().hasNext() )
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                              new String[] { aDef.getLocationPath() },
                              aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef)
    */
   public void visit(AeActivityInvokeDef aDef)
   {
      basicValidation( aDef  );

      // Check required partner operation fields.
      checkPPO( aDef );

      String input = null,
             output = null ;
      QName inputType = EMPTY_QNAME,
            outputType = EMPTY_QNAME ;
      AeVariableDef inVarDef = null,
                     outVarDef = null ;

      // check that the input variable name is present
      if ( isUndefined(aDef.getInputVariable()))
      {
         // if this is not an abstract process then it is an error
         if ( !getProcessDefNode().isAbstractProcess() )
         {
            // Missing input variable.
            getReporter().addError( ERROR_FIELD_MISSING,
                                    new String[] { AeActivityInvokeDef.TAG_INPUT_VARIABLE },
                                    aDef );
         }
      }
      else
      {
         input = aDef.getInputVariable();
         inVarDef = variableExistsInProcess( input, aDef );
         if ( inVarDef != null && !AeUtil.isNullOrEmpty( aDef.getOperation()))
         {
            inputType = inVarDef.getMessageType();
            if ( !isUndefined(aDef.getOutputVariable()))
            {
               output = aDef.getOutputVariable();
               outVarDef = variableExistsInProcess( output, aDef );
               if ( outVarDef != null )
                  outputType = outVarDef.getMessageType();
            }

            // Validate the operation and variable type combination.
            if ( !operationExistsInWSDL( aDef.getPortType(), aDef.getOperation(), inputType, outputType ))
            {
               if ( outputType == null )
               {
                  // Input-only error
                  //
                  getReporter().addError( ERROR_OPERATION_IN_NOT_FOUND,
                                          new String[] { aDef.getOperation(),
                                                         getNSPrefix( inputType.getNamespaceURI()),
                                                         inputType.getLocalPart() },
                                          aDef );
               }
               else
               {
                  getReporter().addError( ERROR_OPERATION_INOUT_NOT_FOUND,
                                          new String[] { aDef.getOperation(),
                                                         getNSPrefix( inputType.getNamespaceURI()),
                                                         inputType.getLocalPart(),
                                                         getNSPrefix( outputType.getNamespaceURI()),
                                                         outputType.getLocalPart()
                                                         },
                                          aDef );
               }
            }
         }

         for ( Iterator iter = aDef.getCorrelationList() ; iter.hasNext() ; )
         {
            AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
            AeCorrelationSetDef corrSetDef = corrSetExistsInProcess( corrDef.getCorrelationSetName(), aDef );
            if ( corrDef.usesOutboundDataForCorrelation())
            {
               validateCorrSet( corrSetDef, inVarDef, aDef );
               if ( inVarDef == null )
                  getReporter().addError( ERROR_CORRELATION_IN_PATTERN_MISMATCH,
                        new String [] { corrDef.getPattern() }, aDef );
            }

            if ( corrDef.usesInboundDataForCorrelation())
            {
               validateCorrSet( corrSetDef, outVarDef, aDef );
               if ( outVarDef == null )
                  getReporter().addError( ERROR_CORRELATION_OUT_PATTERN_MISMATCH,
                        new String [] { corrDef.getPattern() }, aDef );
            }
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityPickDef)
    */
   public void visit(AeActivityPickDef aDef)
   {
      basicValidation( aDef );

      // Check for required OnMessage child, etc.
      //
      boolean onAlarmFound = false ;
      if ( aDef.getAlarmList().hasNext())
         onAlarmFound = true ;

      if ( aDef.getMessageList().hasNext() )
      {
         if ( aDef.isCreateInstance() && onAlarmFound )
         {
            // No alarm allowed on createInstance = yes.
            //
            getReporter().addError( ERROR_ALARM_ON_CREATEINSTANCE, null, aDef );
         }
      }
      else
      {
         // Need at least one onMessage.
         //
         getReporter().addError( ERROR_NO_ONMESSAGE, null, aDef );
      }

      if ( aDef.isCreateInstance() )
      {
         getKnownInstanceCreators().add( aDef );
         mCreateInstanceFound = true ;
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef)
    */
   public void visit(AeActivityReceiveDef aDef)
   {
      basicValidation( aDef  );

      if ( aDef.isCreateInstance() )
      {
         mCreateInstanceFound = true ;
         getKnownInstanceCreators().add(aDef);
      }

      // Check req'd partner operation fields.
      checkPPO( aDef );

      // check if the variable name is filled in and valid
      AeVariableDef varDef = null ;
      QName inputType = EMPTY_QNAME ;
      if (isUndefined(aDef.getVariable()))
      {
         // if this is not an abstract process then it is an error
         if ( !getProcessDefNode().isAbstractProcess())
         {
            getReporter().addError( ERROR_FIELD_MISSING,
                                    new String[] { AeActivityReceiveDef.TAG_VARIABLE },
                                    aDef );
         }
      }
      else
      {
         // get the variable type, if available.
         varDef = variableExistsInProcess( aDef.getVariable(), aDef );
         if ( varDef != null && !AeUtil.isNullOrEmpty( aDef.getOperation()))
         {
            inputType = varDef.getMessageType();

            // Validate the operation and variable type combination.
            //
            if ( !operationExistsInWSDL( aDef.getPortType(), aDef.getOperation(), inputType, null ))
            {
               getReporter().addError( ERROR_OPERATION_IN_NOT_FOUND,
                                       new String[] { aDef.getOperation(),
                                                      getNSPrefix( inputType.getNamespaceURI()),
                                                      inputType.getLocalPart()},
                                       aDef );
            }
         }
      }

      // warn if no corr set and not a create instance
      if ( !aDef.isCreateInstance() && !aDef.getCorrelationList().hasNext())
      {
         getReporter().addWarning( WARNING_NO_CORR_SET_NO_CREATE,
                                    new String[] { AeActivityReceiveDef.TAG_RECEIVE },
                                    aDef );
      }

      // validate the correlation sets are correct for the receives if variable was found
      if(varDef != null)
      {
         for ( Iterator iter = aDef.getCorrelationList() ; iter.hasNext() ; )
         {
            AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
            AeCorrelationSetDef corrSetDef = corrSetExistsInProcess( corrDef.getCorrelationSetName(), aDef );
            validateCorrSet( corrSetDef, varDef, aDef );
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReplyDef)
    */
   public void visit(AeActivityReplyDef aDef)
   {
      basicValidation( aDef  );

      // Check req'd partner operation fields.
      checkPPO( aDef );

      // check if the variable name is filled in
      AeVariableDef varDef = null ;
      QName outputType = EMPTY_QNAME ;
      if (isUndefined(aDef.getVariable()))
      {

         // variable name must be filled in if not an abstract process
         if (  !getProcessDefNode().isAbstractProcess())
         {
            getReporter().addError( ERROR_FIELD_MISSING,
                                    new String[] { AeActivityReplyDef.TAG_VARIABLE },
                                    aDef );
         }
      }
      else
      {
         // get the variable type, if available.
         //
         varDef = variableExistsInProcess( aDef.getVariable(), aDef );
      }

      // if the variable was found check related info
      if(varDef != null)
      {
         // check that the correlation sets are correct for variable
         for ( Iterator iter = aDef.getCorrelationList() ; iter.hasNext() ; )
         {
            AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
            AeCorrelationSetDef corrSetDef = corrSetExistsInProcess( corrDef.getCorrelationSetName(), aDef );
            validateCorrSet( corrSetDef, varDef, aDef );
         }

         // Check fault name (and associated variable).
         if ( aDef.getFaultName() != null &&
               !aDef.getFaultName().equals( EMPTY_QNAME ) &&
               !AeUtil.isNullOrEmpty( aDef.getOperation()) &&
               !AeUtil.isNullOrEmpty( aDef.getFaultName().getLocalPart()))
         {
            QName faultVarType = AeWSDLDefHelper.getFaultVarType( getWsdlProvider(),
                                                      aDef.getOperation(), aDef.getPortType(),
                                                      aDef.getFaultName());
            if ( faultVarType == null )
            {
               // Fault not found.
               getReporter().addError( ERROR_FAULT_NAME_NOT_FOUND,
                                       new String[] { getNSPrefix( aDef.getFaultName().getNamespaceURI()), aDef.getFaultName().getLocalPart() },
                                       aDef );
            }
            else
            {
               // check for msg type match
               if ( !faultVarType.equals( varDef.getMessageType()))
               {
                  // type mismatch.
                  if ( varDef.getMessageType() != null )
                  {
                     getReporter().addError( ERROR_VAR_TYPE_MISMATCH,
                                             new String[] { varDef.getName(), getNSPrefix( varDef.getMessageType().getNamespaceURI()),
                                                            varDef.getMessageType().getLocalPart(),
                                                            getNSPrefix( faultVarType.getNamespaceURI()),
                                                            faultVarType.getLocalPart()
                                                           },
                                             aDef );
                  }
                  else if ( varDef.getType() != null )
                  {
                     getReporter().addError( AeMessages.getString("AeDefValidationVisitor.23"), //$NON-NLS-1$
                                             new String[] { varDef.getName(),
                                                            getNSPrefix( varDef.getType().getNamespaceURI()),
                                                            varDef.getType().getLocalPart(),
                                                            getNSPrefix( faultVarType.getNamespaceURI()),
                                                            faultVarType.getLocalPart()
                                                           },
                                             aDef );
                  }
                  else if ( varDef.getElement() != null )
                  {
                     getReporter().addError( AeMessages.getString("AeDefValidationVisitor.24"), //$NON-NLS-1$
                                             new String[] { varDef.getName(),
                                                            varDef.getElement().getNamespaceURI(),
                                                            varDef.getElement().getLocalPart(),
                                                            getNSPrefix( faultVarType.getNamespaceURI()),
                                                            faultVarType.getLocalPart()
                                                           },
                                             aDef );
                  }
                  else
                  {
                     getReporter().addError( AeMessages.getString("AeDefValidationVisitor.25"), //$NON-NLS-1$
                                             new String[] { varDef.getName(),
                                                            getNSPrefix( faultVarType.getNamespaceURI()),
                                                            faultVarType.getLocalPart()
                                                           },
                                             aDef );
                  }
               }
            }
         }
         else
         {
            if (! AeUtil.isNullOrEmpty( aDef.getOperation()))
            {
               outputType = varDef.getMessageType();

               // Validate the operation and variable type combination.
               //
               if ( !operationExistsInWSDL( aDef.getPortType(), aDef.getOperation(), null, outputType))
               {
                  getReporter().addError( ERROR_OPERATION_OUT_NOT_FOUND,
                                          new String[] { aDef.getOperation(),
                                                         getNSPrefix( outputType.getNamespaceURI()),
                                                         outputType.getLocalPart() },
                                          aDef );
               }
            }
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivitySuspendDef)
    */
   public void visit(AeActivitySuspendDef aDef)
   {
      basicValidation(aDef);
      
      // Validate optional variable.
      if ( AeUtil.notNullOrEmpty( aDef.getVariable()) )
         variableExistsInProcess( aDef.getVariable(), aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityScopeDef)
    */
   public void visit(AeActivityScopeDef aDef)
   {
      basicValidation( aDef  );

      getKnownScopes().add( aDef );

      // No empty scopes allowed.
      //
      if ( aDef.getActivity() == null )
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                              new String[] { aDef.getLocationPath() },
                              aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationSetDef)
    */
   public void visit(AeCorrelationSetDef aDef)
   {
      basicValidation( aDef  );

      // Ensure that the Correlation Set has an assigned properties
      //  list and the properties' QNames can be resolved in WSDL.
      //
      Collection props = aDef.getProperties();
      if ( props != null && !props.isEmpty() )
      {
         for ( Iterator iterProps = props.iterator() ; iterProps.hasNext() ; )
         {
            QName propName = (QName)iterProps.next();
            AeBPELExtendedWSDLDef wsdl = AeWSDLDefHelper.getWSDLDefinitionForProp( getWsdlProvider(), propName );
            if ( wsdl == null )
            {
               getReporter().addError( ERROR_PROP_NOT_FOUND,
                                       new String[] { getNSPrefix( propName.getNamespaceURI()),
                                                      propName.getLocalPart() },
                                       aDef );
            }
         }
      }
      else
      {
         // No properties defined - error.  Added as fix for Defect #341.
         //
         getReporter().addError( ERROR_CORR_SET_PROPS_NOT_FOUND,
                                 new String[] { aDef.getName() },
                                 aDef );
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeFaultHandlerDef)
    */
   public void visit(AeFaultHandlerDef aDef)
   {
      basicValidation( aDef  );

      // ensure that catch block has either a variable or a fault name defined
      if( aDef.getFaultName() == null && AeUtil.isNullOrEmpty(aDef.getFaultVariable()) )
      {
         getReporter().addError( ERROR_NO_NAME_OR_VARIABLE_ON_FAULT_HANDLER, null, aDef );
      }

      // if the catch block has a variable name, add the variable to the used
      // variable collection so that we don't issue variable not used warnings
      // this will change in v.2.0
      if( AeUtil.notNullOrEmpty( aDef.getFaultVariable() ) )
      {
         AeVariableDef varDef = AeDefUtil.getVariableByName(aDef.getFaultVariable(), aDef);

         if( !getUsedVariables().contains(varDef) )
         {
            getUsedVariables().add( varDef );
         }
      }

      if ( aDef.getActivity() == null )
         getReporter().addError( ERROR_EMPTY_FAULT_HANDLER, null, aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef)
    */
   public void visit(AeDefaultFaultHandlerDef aDef)
   {
      basicValidation( aDef  );
      if ( aDef.getActivity() == null )
         getReporter().addError( ERROR_EMPTY_FAULT_HANDLER, null, aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeVariableDef)
    */
   public void visit(AeVariableDef aDef)
   {
      basicValidation( aDef  );

      AeBPELExtendedWSDLDef wsdl = null ;

      // verify the element exists
      if(aDef.getElement() != null)
      {
         wsdl = AeWSDLDefHelper.getWSDLDefinitionForElement( getWsdlProvider(), aDef.getElement() );
         if ( wsdl == null )
         {
            getReporter().addError( ERROR_ELEMENT_SPEC_NOT_FOUND,
                                    new String[] { getNSPrefix(aDef.getElement().getNamespaceURI()),
                                                   aDef.getElement().getLocalPart() },
                                    aDef );
         }
      }
      // else verify the type exists
      else if(aDef.getType() != null)
      {
         // don't need to locate def for simple types
         // and potentially other well known types
         QName schemaQName = aDef.getType();
         if( !IAeBPELConstants.DEFAULT_SCHEMA_NS.equals( schemaQName.getNamespaceURI() ) )
         {
            wsdl = AeWSDLDefHelper.getWSDLDefinitionForType( getWsdlProvider(), aDef.getType() );
            if ( wsdl == null )
            {
               getReporter().addError( ERROR_TYPE_SPEC_NOT_FOUND,
                                       new String[] { getNSPrefix(aDef.getType().getNamespaceURI()),
                                                      aDef.getType().getLocalPart() },
                                       aDef );
            }
         }
      }
      // else must be message so verify the message exists
      else
      {
         // Ensure that the Variable has an assigned MessageType
         //  and that it can be resolved.
         //
         QName msgType = aDef.getMessageType();
         if (  msgType == null || EMPTY_QNAME.equals( msgType ))
         {
            getReporter().addError( ERROR_VAR_HAS_NO_TYPE,
                                    new String[] { aDef.getName() },
                                    aDef );
         }
         else
         {
            wsdl = AeWSDLDefHelper.getWSDLDefinitionForMsg( getWsdlProvider(), msgType );
            if ( wsdl == null )
            {
               getReporter().addError( ERROR_MSG_SPEC_NOT_FOUND,
                                       new String[] { getNSPrefix( aDef.getMessageType().getNamespaceURI()),
                                                      aDef.getMessageType().getLocalPart() },
                                       aDef );
            }
         }

         // verify and add part info
         if ( wsdl != null )
         {
            // @todo this may be redundant to processing in preProcessStep, but wanted error funneled through here
            // save the part info to the variable def
            for (Iterator iter=wsdl.getMessageParts(aDef.getMessageType()); iter.hasNext();)
            {
               Part part = (Part)iter.next();
               try
               {
                  aDef.addPartTypeInfo(part, wsdl);
               }
               catch (AeException e)
               {
                  getReporter().addError( ERROR_DISCOVERING_PART_TYPE_SPECS,
                                          new String[] { part.getName(),
                                                         getNSPrefix( aDef.getMessageType().getNamespaceURI()),
                                                         aDef.getMessageType().getLocalPart(),
                                                         e.getMessage()},
                                          aDef );
               }
            }
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeVariablesContainerDef)
    */
   public void visit(AeVariablesContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeEventHandlersDef)
    */
   public void visit(AeEventHandlersDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef)
    */
   public void visit(AeScopeCompensatorContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef)
    */
   public void visit(AeCorrelationSetContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef)
    */
   public void visit(AeFaultHandlersContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef)
    */
   public void visit(AeOnMessageDef aDef)
   {
      basicValidation( aDef  );

      // Check for required child activity.
      //
      if ( aDef.getActivity() == null )
      {
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                                 new String[] { aDef.getLocationPath() },
                                 aDef );
      }

      // check the parternerLink/portType/Operation combinations
      checkPPO(aDef, aDef.getPartnerLink(), aDef.getPortType(), aDef.getOperation());

      // check that the variable definitions for the operation
      AeVariableDef varDef = null ;
      if ( !getProcessDefNode().isAbstractProcess())
      {
         if (isUndefined(aDef.getVariable()))
         {
            getReporter().addError( ERROR_FIELD_MISSING,
                                    new String[] { AeOnMessageDef.TAG_VARIABLE },
                                    aDef );
         }
         else
         {
            varDef = variableExistsInProcess( aDef.getVariable(), aDef );
            if ( varDef != null && !AeUtil.isNullOrEmpty( aDef.getOperation()))
            {
               QName inputType = varDef.getMessageType();

               // Validate the operation and variable type combination.
               //
               if ( !operationExistsInWSDL( aDef.getPortType(), aDef.getOperation(), inputType, null ))
               {
                  getReporter().addError( ERROR_OPERATION_IN_NOT_FOUND,
                                          new String[] { aDef.getOperation(),
                                                         getNSPrefix( inputType.getNamespaceURI()),
                                                         inputType.getLocalPart()},
                                          aDef );
               }
            }
         }
      }

      // check for corr set if parent is a Pick and not a create instance
      //
      boolean isCreateInstance = false ;
      AeBaseDef parent = aDef.getParent();
      if ( parent instanceof AeActivityPickDef )
      {
         isCreateInstance = ((AeActivityPickDef)aDef.getParent()).isCreateInstance();
      }
      
      if ( !isCreateInstance && !aDef.getCorrelationList().hasNext())
      {
         getReporter().addWarning( WARNING_NO_CORR_SET_NO_CREATE, 
                                    new String[] { AeOnMessageDef.TAG_ON_MESSAGE },
                                    aDef );
      }

      for ( Iterator iter = aDef.getCorrelationList() ; iter.hasNext() ; )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         AeCorrelationSetDef corrSetDef = corrSetExistsInProcess( corrDef.getCorrelationSetName(), aDef );
         validateCorrSet( corrSetDef, varDef, aDef );
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeMessageExchangesDef)
    */
   public void visit(AeMessageExchangesDef aDef)
   {
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef)
    */
   public void visit(AeOnAlarmDef aDef)
   {
      basicValidation( aDef  );
      String exp = aDef.getFor();
      boolean isDuration =  true;
      String errorTag = AeMessages.getString("AeDefValidationVisitor.ALARM_EXPRESSION"); //$NON-NLS-1$
      if ( AeUtil.isNullOrEmpty( exp ))
      {
         exp = aDef.getUntil();
         isDuration = false;
      }

      if (isUndefined(exp))
      {
         getReporter().addError( ERROR_FIELD_MISSING,
                                 new String[] { errorTag },
                                 aDef );
      }
      else if ( !AeUtil.isNullOrEmpty( exp ) )
      {
         if (isDuration)
         {
            validateDurationExpression( exp, aDef );
         }
         else
         {
            validateDeadlineExpression( exp, aDef );
         }
      }

      // Check for required child activity.
      //
      if ( aDef.getActivity() == null )
      {
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                                 new String[] { aDef.getLocationPath() },
                                 aDef );
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivitySequenceDef)
    */
   public void visit(AeActivitySequenceDef aDef)
   {
      basicValidation( aDef  );

      // No empty sequences allowed.
      //
      if ( !aDef.getActivityList().hasNext())
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                              new String[] { aDef.getLocationPath() },
                              aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef)
    */
   public void visit(AeActivitySwitchDef aDef)
   {
      basicValidation( aDef  );

      // Check for required Case child.
      //
      boolean caseFound = false ;
      Iterator iter = aDef.getCaseList();
      while ( iter.hasNext())
      {
         if ( iter.next() instanceof AeSwitchCaseDef )
         {
            caseFound = true ;
            break ;
         }
      }

      if ( !caseFound )
         getReporter().addError( ERROR_NO_CASE, null, aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityTerminateDef)
    */
   public void visit(AeActivityTerminateDef aDef)
   {
      basicValidation( aDef  );

      if ( getProcessDefNode().isAbstractProcess() )
      {
         getReporter().addError( ERROR_TERM_NOT_ALLOWED, null, aDef );
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityThrowDef)
    */
   public void visit(AeActivityThrowDef aDef)
   {
      basicValidation( aDef  );

      if (  aDef.getFaultName() == null ||
            aDef.getFaultName().equals(EMPTY_QNAME))
      {
         getReporter().addError( ERROR_FIELD_MISSING,
                                 new String[] { AeActivityThrowDef.TAG_FAULT_NAME },
                                 aDef );
      }
      else
      {
         AeVariableDef varDef = getVariableByName( aDef.getFaultVariable(), aDef );
         if ( varDef != null && !faultIsHandledInProcess( aDef.getFaultName(), varDef.getMessageType(), aDef ))
         {
            // Fault QName not found.
            //
            getReporter().addWarning( WARN_FAULT_NAME_NOT_CAUGHT,
                                      new String[] { aDef.getFaultName().getLocalPart(), aDef.getFaultName().getNamespaceURI() },
                                      aDef );
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityWaitDef)
    */
   public void visit(AeActivityWaitDef aDef)
   {
      basicValidation( aDef  );
      boolean isDuration = true;

      String exp = aDef.getFor();
      String errorTag = "Condition" ; //$NON-NLS-1$
      if ( AeUtil.isNullOrEmpty( exp ))
      {
         exp = aDef.getUntil();
         isDuration = false;
      }

      if (isUndefined(exp))
      {
         getReporter().addError( ERROR_FIELD_MISSING,
                                 new String[] { errorTag },
                                 aDef );
      }
      else if ( !AeUtil.isNullOrEmpty( aDef.getUntil()))
      {
         if (isDuration)
         {
            validateDurationExpression( exp, aDef );
         }
         else
         {
            validateDeadlineExpression( exp, aDef );
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityWhileDef)
    */
   public void visit(AeActivityWhileDef aDef)
   {
      basicValidation( aDef );

      // No empty whiles allowed.
      //
      if ( aDef.getActivity() == null )
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                              new String[] { aDef.getLocationPath() },
                              aDef );

      if (isUndefined(aDef.getCondition()))
      {
         getReporter().addError( ERROR_FIELD_MISSING, new String[] { "Condition" }, aDef ); //$NON-NLS-1$
      }
      else
      {
         validateBooleanExpression( aDef.getCondition(), aDef );
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityForEachDef)
    */
   public void visit(AeActivityForEachDef aDef)
   {
      basicValidation(aDef);

      //    - non-null counterName
      if ( isUndefined(aDef.getCounterName()))
      {
         getReporter().addError( ERROR_FIELD_MISSING, new String[] { AeActivityForEachDef.TAG_FOREACH_COUNTERNAME }, aDef ); //$NON-NLS-1$
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef)
    */
   public void visit(AeForEachFinalDef aDef)
   {
      validateExpressionExists(aDef, AeForEachFinalDef.TAG_FOREACH_FINALCOUNTER);
   }

   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef)
    */
   public void visit(AeForEachStartDef aDef)
   {
      validateExpressionExists(aDef, AeForEachStartDef.TAG_FOREACH_STARTCOUNTER);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachBranchesDef)
    */
   public void visit(AeForEachBranchesDef aDef)
   {
      // using the completion condition tag here since it is more descriptive than "branches" 
      validateExpressionExists(aDef, AeForEachBranchesDef.TAG_FOREACH_COMPLETION_CONDITION);

      // TODO (MF) add static validation to catch invalidBranchCondition where start / final / completionCondition are static
   }
   
   /**
    * Validates that the expression def passed in contains a valid expression
    * @param aDef
    * @param aFieldName
    */
   private void validateExpressionExists(AeAbstractExpressionDef aDef, String aFieldName)
   {
      AeActivityDef parentActivity = getActivityDef(aDef);

      if ( isUndefined(aDef.getExpression()))
      {
         getReporter().addError( ERROR_FIELD_MISSING, new String[] { aFieldName }, parentActivity ); //$NON-NLS-1$
      }
      else
      {
         validateExpression(aDef.getExpressionLanguage(), aDef.getExpression(), aDef, parentActivity);
      }
   }

   /**
    * Gets the parent activity from the expression def. This activity is used to report errors
    * found during validation since it's easier to map the errors to an activity. 
    * @param aDef
    */
   private AeActivityDef getActivityDef(AeAbstractExpressionDef aDef)
   {
      AeActivityDef parentActivity = null;
      AeBaseDef parent = aDef;
      while (parentActivity == null)
      {
         if (parent instanceof AeActivityDef)
         {
            parentActivity = (AeActivityDef) parent;
         }
         else
         {
            parent = parent.getParent();
         }
      }
      return parentActivity;
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef)
    */
   public void visit(AeForEachCompletionConditionDef aDef)
   {
      basicValidation(aDef);
   }
   
   /**
    * Convenience method that returns true if the value is null, empty, or "undefined".
    * Undefined is a value from our designer tool that lists the value as "(none)"
    *
    * @param aValue
    */
   protected boolean isUndefined(String aValue)
   {
      // TODO (MF) the designer shouldn't write bpel with "(none)". It should use
      // nulls in the object model to indicate when values haven't been set and simply
      // write nothing for the attributes.
      return AeUtil.isNullOrEmpty(aValue) || FIELD_UNDEFINED.equals(aValue);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityContinueDef)
    */
   public void visit(AeActivityContinueDef aDef)
   {
      basicValidation(aDef);
      validateLoopControl(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityBreakDef)
    */
   public void visit(AeActivityBreakDef aDef)
   {
      basicValidation(aDef);
      validateLoopControl(aDef);
   }

   /**
    * Ensures that the loop control exists within a loop container.
    * @param aDef
    */
   protected void validateLoopControl(AeBaseDef aDef)
   {
      // walk the parent axis to ensure that we find a loop container
      AeBaseDef parent = aDef.getParent();
      while(parent != null && !(parent instanceof IAeLoopActivityDef))
         parent = parent.getParent();

      if (parent == null)
      {
         getReporter().addError(AeMessages.getString("IAeValidationDefs.InvalidLocationForLoopControl"), new String[] {aDef.getLocationPath()}, aDef); //$NON-NLS-1$
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef)
    */
   public void visit(AeSwitchCaseDef aDef)
   {
      basicValidation( aDef  );

      // No empty cases allowed.
      //
      if ( aDef.getActivity() == null )
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                              new String[] { aDef.getLocationPath() },
                              aDef );

      // Needs a condition expression.
      //
      if ( AeUtil.isNullOrEmpty( aDef.getCondition()))
      {
         getReporter().addError( ERROR_FIELD_MISSING, new String[] { "Condition" }, aDef ); //$NON-NLS-1$
      }
      else
      {
         // validate the expression syntax.
         //
         validateBooleanExpression( aDef.getCondition(), aDef );
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef)
    */
   public void visit(AeOtherwiseDef aDef)
   {
      basicValidation( aDef  );

      // No empty otherwises allowed.
      //
      if ( aDef.getActivity() == null )
         getReporter().addError( ERROR_ACTIVITY_MISSING,
                              new String[] { aDef.getLocationPath() },
                              aDef );

   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerDef)
    */
   public void visit(AePartnerDef aDef)
   {
      basicValidation( aDef  );

      // check the partner links listed by this partner.
      //
      for ( Iterator iter = aDef.getPartnerLinks() ; iter.hasNext() ; )
      {
         String name = (String)iter.next();
         if ( partnerLinkExistsInProcess( name, aDef ) == null )
         {
            getReporter().addError( ERROR_PARTNER_LINK_NOT_FOUND,
                                    new String[] { name },
                                    aDef );
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerLinkDef)
    */
   public void visit(AePartnerLinkDef aDef)
   {
      basicValidation( aDef  );

      // Ensure that the type can be resolved.
      //
      QName pltName = aDef.getPartnerLinkTypeName();
      if ( pltName == null || EMPTY_QNAME.equals( pltName ))
      {
         getReporter().addError( ERROR_PARTNER_LINK_HAS_NO_TYPE,
                                 new String[] { aDef.getName()},
                                 aDef );
      }
      else
      {
         AeBPELExtendedWSDLDef wsdl = AeWSDLDefHelper.getWSDLDefinitionForPLT( getWsdlProvider(), pltName );
         if ( wsdl == null )
         {
            getReporter().addError( ERROR_PARTNER_LINK_TYPE_NOT_FOUND,
                                    new String[] { getNSPrefix( pltName.getNamespaceURI()),
                                                   pltName.getLocalPart(), },
                                    aDef );
         }
         else
         {
            // validate the roles, if defined.
            //
            String role = aDef.getMyRole();
            String plt = aDef.getPartnerLinkTypeName().getLocalPart();
            if ( !AeUtil.isNullOrEmpty( role ) && !AeUtil.isNullOrEmpty( plt ))
            {
               if ( wsdl.getPartnerLinkType( plt ).findRole( role ) == null )
               {
                  // role not found
                  getReporter().addError( ERROR_ROLE_NOT_FOUND,
                                          new String[] { role, plt },
                                          aDef );
               }
            }
            role = aDef.getPartnerRole();
            if ( !AeUtil.isNullOrEmpty( role ) && !AeUtil.isNullOrEmpty( plt ))
            {
               if ( wsdl.getPartnerLinkType( plt ).findRole( role ) == null )
               {
                  // role not found
                  getReporter().addError( ERROR_ROLE_NOT_FOUND,
                                          new String[] { role, plt },
                                          aDef );
               }
            }
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeScopeDef)
    */
   public void visit(AeScopeDef aDef)
   {
      basicValidation( aDef  );

      // Record this scope's known explicit variables, etc., to check for usage later.
      addExplicitVariables( ((AeScopeDef)aDef).getVariables() );

      for ( Iterator iter = ((AeScopeDef)aDef).getCorrelationSets() ; iter.hasNext() ; )
         getKnownCorrSets().add( iter.next());

      // Check all outer scope levels to ensure the type declaration is consistent with parent
      // TODO (RN) This should be a configurable option as this is allowed in BPEL 2.0
      validateVariableTypeOverload(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef)
    */
   public void visit(AeAssignCopyDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef)
    */
   public void visit(AeCorrelationDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeLinkDef)
    */
   public void visit(AeLinkDef aDef)
   {
      basicValidation( aDef  );
      getLinkValidator().addLink( aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSourceDef)
    */
   public void visit(AeSourceDef aDef)
   {
      basicValidation( aDef  );
      if ( !AeUtil.isNullOrEmpty( aDef.getTransitionCondition()))
         validateBooleanExpression( aDef.getTransitionCondition(), aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeTargetDef)
    */
   public void visit(AeTargetDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerLinksContainerDef)
    */
   public void visit(AePartnerLinksContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnersContainerDef)
    */
   public void visit(AePartnersContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef)
    */
   public void visit(AeLinkContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationContainerDef)
    */
   public void visit(AeCorrelationContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef)
    */
   public void visit(AeSwitchCaseContainerDef aDef)
   {
      basicValidation( aDef  );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef)
    */
   public void visit(AeAssignFromDef aDef)
   {
      basicValidation( aDef  );

      AeVariableDef varDef = null;
      if ( !AeUtil.isNullOrEmpty( aDef.getVariable()))
      {
         varDef = variableExistsInProcess( aDef.getVariable(), aDef );
         if ( varDef != null )
         {
            validateVariableAndPart(aDef, varDef);
         }
      }

      if ( !AeUtil.isNullOrEmpty( aDef.getQuery() ))
      {
         validateQuery( aDef.getQuery(), aDef );
         if(! AeUtil.isNullOrEmpty( aDef.getPart() ))
            validatePartQuery(aDef.getParent(), aDef.getVariable(), aDef.getPart(), aDef.getQuery());
         else if (varDef != null)
         {
            if (varDef.getMessageType() != null)
               getReporter().addWarning(WARNING_VARIABLE_PART_REQUIRED, new String[] {varDef.getName()}, aDef);
            else if (varDef.getType() != null)
            {
               // TODO (RN) This will be allowed for Complex Type variables in Version 2.0
               getReporter().addWarning(WARNING_VARIABLE_QUERY_NOT_SUPPORTED, new String[] {varDef.getName()}, aDef);
            }
            else if(varDef.getElement() != null)
            {
               validateElementQuery(aDef.getParent(), aDef.getVariable(), aDef.getQuery());
            }
         }
      }

      if ( !AeUtil.isNullOrEmpty( aDef.getExpression() ))
         validateExpression( aDef.getExpression(), aDef );

      // Fix for Defect #357 - add check for opaque in non-abstract process.
      //
      if ( aDef.getOpaque() && !getProcessDefNode().isAbstractProcess())
         getReporter().addError( ERROR_OPAQUE_NOT_ALLOWED, new String[0], aDef );
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignToDef)
    */
   public void visit(AeAssignToDef aDef)
   {
      basicValidation( aDef  );
      if ( !AeUtil.isNullOrEmpty( aDef.getVariable()))
      {
         AeVariableDef varDef = variableExistsInProcess( aDef.getVariable(), aDef );
         if ( varDef != null )
         {
            validateVariableAndPart(aDef, varDef);

            if ( !AeUtil.isNullOrEmpty( aDef.getQuery() ))
            {
               validateQuery( aDef.getQuery(), aDef );
               if(! AeUtil.isNullOrEmpty( aDef.getPart() ))
               {
                  validatePartQuery(aDef.getParent(), aDef.getVariable(), aDef.getPart(), aDef.getQuery());
               }
               else if (varDef.getMessageType() != null)
               {
                  getReporter().addWarning(WARNING_VARIABLE_PART_REQUIRED, new String[] {varDef.getName()}, aDef);
               }
               else if (varDef.getType() != null)
               {
                  // TODO (RN) This will be allowed for Complex Type variables in Version 2.0
                  getReporter().addWarning(WARNING_VARIABLE_QUERY_NOT_SUPPORTED, new String[] {varDef.getName()}, aDef);
               }
               else if(varDef.getElement() != null)
               {
                  validateElementQuery(aDef.getParent(), aDef.getVariable(), aDef.getQuery());
               }
            }
         }
      }
   }

   /**
    * Validateds that the variable assignment def (either a from or a to) points
    * to a valid message part. 
    * @param aDef
    * @param varDef
    */
   protected void validateVariableAndPart(AeAssignVarDef aDef, AeVariableDef varDef)
   {
      if (AeUtil.notNullOrEmpty(aDef.getPart()))
      {
         if (varDef.getMessageType() != null)
         {
            validateVariableAndPart( varDef.getMessageType(), aDef.getPart(), aDef.getParent() );
         }
         else
         {
            getReporter().addError(ERROR_PART_USAGE,
                  new String[] { aDef.getPart() },
                  aDef);
         }
      }
   }

   /**
    * Validates a part query and if in error reports it against the passed def object.
    * @param aDef The definition to report the error on.
    * @param aVar The variable used in query.
    * @param aPart The part of the message/variable used in query.
    * @param aQuery The query to validate.
    */
   private void validatePartQuery(AeBaseDef aDef, String aVar, String aPart, String aQuery)
   {
      try
      {
         AeVariableDef var = getVariableByName( aVar, aDef );
         Part part = var.getPart(aPart);
         XMLType type = var.getPartType(part.getName());
         if(type != null)
         {
            QName root = part.getElementName();
            if(root == null)
               root = new QName(null, part.getName());
            getXPathQueryValidator().validate(aQuery, type, root);
         }
      }
      catch (Exception ex)
      {
         getReporter().addError( ERROR_INVALID_XPATH,
                                 new String[] { aQuery, ex.getLocalizedMessage() },
                                 aDef );
      }
   }

   /**
    * Validates a query against an element and if in error reports it against the passed def object.
    * @param aDef The definition to report the error on.
    * @param aVar The variable used in query.
    * @param aQuery The query to validate.
    */
   private void validateElementQuery(AeBaseDef aDef, String aVar, String aQuery)
   {
      try
      {
         AeVariableDef var = getVariableByName( aVar, aDef );
         if(var.getElement() != null)
         {
            // verify the element query
            AeBPELExtendedWSDLDef wsdl = AeWSDLDefHelper.getWSDLDefinitionForElement( getWsdlProvider(), var.getElement() );
            // if wsdl is null it would have already been reported as a problem
            if ( wsdl != null )
            {
               ElementDecl elem = wsdl.findElement(var.getElement());
               if(elem != null)
               {
                  QName root = var.getElement();
                  getXPathQueryValidator().validate(aQuery, elem.getType(), root);
               }
            }
         }
      }
      catch (Exception ex)
      {
         getReporter().addError( ERROR_INVALID_XPATH,
                                 new String[] { aQuery, ex.getLocalizedMessage() },
                                 aDef );
      }
   }

   /**
    * Returns the xpath validator to use to validate xpath queries.
    */
   public AeXPathQueryValidator getXPathQueryValidator()
   {
      return mXPathQueryValidator;
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeImportsContainerDef)
    */
   public void visit(AeImportsContainerDef aDef)
   {
      // Do not visit. BPEL v1.1 does not support imports
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeImportDef)
    */
   public void visit(AeImportDef aDef)
   {
      // Do not visit. BPEL v1.1 does not support imports
   }

   /**
    * Set the process def.
    * @param aDef
    */
   private void setProcessDefNode( AeProcessDef aDef )
   {
      mProcessDef = aDef;
   }

   /**
    * Accessor for the process def.
    */
   private AeProcessDef getProcessDefNode()
   {
      return mProcessDef;
   }

   /**
    * Returns the list of known scopes.
    *
    * @return ArrayList
    */
   public ArrayList getKnownScopes()
   {
      if ( mKnownScopes == null )
         mKnownScopes = new ArrayList();

      return mKnownScopes;
   }

   /**
    * Returns the list of compensate activities encountered so far.
    *
    * @return ArrayList
    */
   public ArrayList getKnownCompensateActivities()
   {
      if ( mKnownCompensateActivities == null )
         mKnownCompensateActivities = new ArrayList();

      return mKnownCompensateActivities;
   }

   /**
    * Returns the list of know create instance activities, constructing if necessary.
    *
    * @return ArrayList
    */
   public ArrayList getKnownInstanceCreators()
   {
      if ( mKnownInstanceCreators == null )
         mKnownInstanceCreators = new ArrayList();

      return mKnownInstanceCreators;
   }

   /**
    * Convert to the source's link def if appropriate.
    *
    * @param aDef The possible source def to check.
    *
    * @return AeBaseDef
    */
   private AeBaseDef resolveLinkDef( AeBaseDef aDef )
   {
      AeBaseDef errDef = aDef ;
      if ( aDef instanceof AeSourceDef )
         errDef = getLinkValidator().findLink( ((AeSourceDef)aDef).getLinkName(), aDef );

      return errDef ;
   }

   /**
    * @return Returns the expressionLanguageFactory.
    */
   protected IAeExpressionLanguageFactory getExpressionLanguageFactory()
   {
      return mExpressionLanguageFactory;
   }
   /**
    * @param aExpressionLanguageFactory The expressionLanguageFactory to set.
    */
   protected void setExpressionLanguageFactory(IAeExpressionLanguageFactory aExpressionLanguageFactory)
   {
      mExpressionLanguageFactory = aExpressionLanguageFactory;
   }

   /**
    * Validate the type declarations for the given scope to ensure the variable
    * declarations are consistent with any declaration they may overload.
    * @param aScopeDef The scope whose variables we are currently checking
    */
   private void validateVariableTypeOverload(AeScopeDef aScopeDef)
   {
      AeBaseDef parent = aScopeDef.getParent();
      while (parent != null)
      {
         if (parent instanceof AeScopeDef)
         {
            for (Iterator iter=getKnownVariables().iterator(); iter.hasNext();)
            {
               AeVariableDef var = (AeVariableDef)iter.next();
               for (Iterator subIter=((AeScopeDef)parent).getVariables(); subIter.hasNext();)
               {
                  // If variable names are identical, ensure type match
                  AeVariableDef parentVar = (AeVariableDef)subIter.next();
                  if (var.getName().equals(parentVar.getName()))
                  {
                     if (!(AeUtil.compareObjects(var.getType(), parentVar.getType()) && 
                           AeUtil.compareObjects(var.getElement(), parentVar.getElement()) && 
                           AeUtil.compareObjects(var.getMessageType(), parentVar.getMessageType())))
                        getReporter().addWarning(WARNING_VARIABLE_TYPE_OVERLOADED, new String[] {var.getName()}, var);
                  }
               }
            }
         }

         // If we have just processed the process level scope, we are done else process next parent level
         if (parent == getProcessDefNode())
            parent = null;
         else
            parent = parent.getParent();
      }
   }

   /**
    * Helper method to determine if the name associated with the given
    * named def object is a valid XML NCName.
    * @param aDef the def object whose name is to be checked.
    * @return returns true if the name is a valid NCName, otherwise returns false.
    */
   private boolean isValidName(AeNamedDef aDef)
   {
      return AeXmlUtil.isValidNCName(aDef.getName());
   }

   /**
    * Helper method to determine if the name associated with the given
    * named def object is null/empty or is a valid XML NCName.
    * @param aDef the def object whose name is to be checked.
    * @return returns true if the name is null or empty or is valid NCName,
    * otherwise returns false.
    */
   private boolean isEmptyOrValidName(AeNamedDef aDef)
   {
      if ( AeUtil.isNullOrEmpty(aDef.getName()) || isValidName(aDef) )
         return true;

      return false;
   }
}
