//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/validation/AeExtensionElementVisitor.java,v 1.1 2006/02/01 22:44:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.validation; 

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.io.ext.IAeExtensionElement;
import org.activebpel.rt.bpel.def.visitors.AeAbstractDefVisitor;
import org.activebpel.rt.bpel.def.visitors.AeDefTraverser;
import org.activebpel.rt.bpel.def.visitors.AeTraversalVisitor;

/**
 * Visits the def model to produce warnings for any extensibility elements encountered
 * during the parse of the bpel.
 * 
 * The engine currently ignores these extensibility elements so we'll produce a warning
 * for each one encounted. If there are any within the BPEL namespace, then it's likely
 * an error in the process definition (or perhaps something we don't handle) so we'll
 * produce a different warning message for these.
 * 
 */
public class AeExtensionElementVisitor extends AeAbstractDefVisitor
{
   /** used to report the warning messages for extensibility elements */
   private IAeBaseErrorReporter mErrorReporter;
   
   /**
    * ctor
    * @param aErrorReporter
    */
   public AeExtensionElementVisitor(IAeBaseErrorReporter aErrorReporter)
   {
      setTraversalVisitor( new AeTraversalVisitor( new AeDefTraverser(), this ) );
      setErrorReporter(aErrorReporter);
   }

   /**
    * @return Returns the errorReporter.
    */
   public IAeBaseErrorReporter getErrorReporter()
   {
      return mErrorReporter;
   }

   /**
    * @param aErrorReporter The errorReporter to set.
    */
   public void setErrorReporter(IAeBaseErrorReporter aErrorReporter)
   {
      mErrorReporter = aErrorReporter;
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.AeAbstractDefVisitor#traverse(org.activebpel.rt.bpel.def.AeBaseDef)
    */
   protected void traverse(AeBaseDef aDef)
   {
      for(Iterator it=aDef.getExtensions(); it.hasNext(); )
      {
         IAeExtensionElement extElement = (IAeExtensionElement) it.next();
         addWarning(extElement.getElementQName(), aDef);
      }
      super.traverse(aDef);
   }
   
   /**
    * Adds a warning message if the def contains extensibility elements.
    * @param aQName
    * @param aDef
    */
   protected void addWarning(QName aQName, AeBaseDef aDef)
   {
      Object[] args = {aQName.getNamespaceURI(), aQName.getLocalPart(), aDef.getLocationPath()};
      String warningMessage = null;
      if (IAeBPELConstants.BPWS_NAMESPACE_URI.equals(aQName.getNamespaceURI()))
      {
         warningMessage = AeMessages.getString("AeExtensionElementVisitor.UnexpectedBpelActivity"); //$NON-NLS-1$
      }
      else
      {
         warningMessage = AeMessages.getString("AeExtensionElementVisitor.ExtensionElementFound"); //$NON-NLS-1$
      }
      getErrorReporter().addWarning(warningMessage, args, aDef);
   }
}
 
