// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/validation/IAeValidationDefs.java,v 1.29 2005/12/08 17:19:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.validation;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeMessages;


/**
 * Interface definition for error reporting, primarily for validation errors.
 */
public interface IAeValidationDefs
{
   /** Empty QName object with an empty selection localPart */
   public static final QName EMPTY_QNAME = new QName("", "(none)");  //$NON-NLS-1$ //$NON-NLS-2$

   /////////////////////////////////////////////////////////////////////////
   // Error, Warning and Information codes.  These are defined here for use
   //  by validation logic.  The AEV package can cross-reference these codes
   //  to localizable strings.
   //
   // {0}, {1}, etc., used for substitution placeholder where validate()
   //  method will insert specifics.
   /////////////////////////////////////////////////////////////////////////

   ///////////////////////////////
   // Error codes - add as needed.
   ///////////////////////////////

   public final static String ERROR_TEST = "ErrorTest" ; //$NON-NLS-1$
   public final static String FIELD_UNDEFINED = "(none)" ; //$NON-NLS-1$
   public final static String FIELD_DEFAULT = "(default)" ; //$NON-NLS-1$
   public final static String EXCEPTION_DURING_VALIDATION = AeMessages.getString("IAeValidationDefs.5") ; //$NON-NLS-1$

   /**
    * Non-abstract processes need at least one activity (either a Pick or a Receive) with the 
    * createInstance flag set to true.
    */
   public final static String ERROR_NO_CREATE = AeMessages.getString("IAeValidationDefs.6") ; //$NON-NLS-1$
   
   /**
    * A required field is missing a value.
    * 
    * Arg 0 is the field name.
    */
   public final static String ERROR_FIELD_MISSING = AeMessages.getString("IAeValidationDefs.7") ; //$NON-NLS-1$
   
   /**
    * A required activity is missing from a container.
    * 
    * Arg 0 is the container name.
    */
   public final static String ERROR_ACTIVITY_MISSING = AeMessages.getString("IAeValidationDefs.8") ; //$NON-NLS-1$
   
   /**
    * Assign: must have at least one Copy.
    */
   public final static String ERROR_NO_COPY = AeMessages.getString("IAeValidationDefs.9") ; //$NON-NLS-1$

   /**
    * Assign: from can only be opaque in abstract process.
    */   
   public final static String ERROR_OPAQUE_NOT_ALLOWED = AeMessages.getString("IAeValidationDefs.10") ; //$NON-NLS-1$

   /**
    * Switch: must have at least one Case.
    */
   public final static String ERROR_NO_CASE = AeMessages.getString("IAeValidationDefs.11") ; //$NON-NLS-1$
   
   /**
    * Pick: must have at least one onMessage.
    */
   public final static String ERROR_NO_ONMESSAGE = AeMessages.getString("IAeValidationDefs.12") ; //$NON-NLS-1$
   
   /**
    * Pick: if more than one onMessage, then there should be no onAlarm.
    */
   public final static String ERROR_ALARM_ON_CREATEINSTANCE = AeMessages.getString("IAeValidationDefs.13") ; //$NON-NLS-1$
   
   /**
    * If more than one Create Instance, their correlation sets need to match.
    */
   public final static String ERROR_CS_MISMATCH = AeMessages.getString("IAeValidationDefs.14") ; //$NON-NLS-1$
   
   /**
    * Pick: if more than one onMessage, then createInstance should be true.
    */
   public final static String ERROR_MULT_ONMSG_CREATE = AeMessages.getString("IAeValidationDefs.15") ; //$NON-NLS-1$
   
   /**
    * Link definition is invalid.
    * 
    * Arg 0 is the link name. 
    */
   public final static String ERROR_BAD_LINK = AeMessages.getString("IAeValidationDefs.16") ; //$NON-NLS-1$
   
   /**
    * Link has more than one source activity.
    * 
    * Arg 0 is the link name. 
    */
   public final static String ERROR_MULTI_SRC_LINK = AeMessages.getString("IAeValidationDefs.70") ; //$NON-NLS-1$
   
   /**
    * Link has more than one target activity.
    * 
    * Arg 0 is the link name. 
    */
   public final static String ERROR_MULTI_TARGET_LINK = AeMessages.getString("IAeValidationDefs.71") ; //$NON-NLS-1$
   
   /**
    * Link boundary crossing error.
    * 
    * Arg 0 is the link name.
    * Arg 1 is the boundary type.
    */
   public final static String ERROR_LINK_CROSSING = AeMessages.getString("IAeValidationDefs.17") ; //$NON-NLS-1$
   
   /**
    * Bad Link from Scope to a child target activity.
    * 
    * Arg 0 is the link name.
    */
   public final static String ERROR_SCOPE_LINK = AeMessages.getString("IAeValidationDefs.BAD_LINK_ERROR"); //$NON-NLS-1$
   
   /**
    * Link is part of an invalid graph cycle.
    * 
    * Arg 0 is the link name.
    */
   public final static String ERROR_LINK_CYCLE = AeMessages.getString("IAeValidationDefs.18") ; //$NON-NLS-1$

   /**
    * Terminate is not valid in an abstract process.
    */
   public final static String ERROR_TERM_NOT_ALLOWED = AeMessages.getString("IAeValidationDefs.19") ; //$NON-NLS-1$
   
   /**
    * Type specification for variable is missing.
    * 
    * Arg 0 is the variable name.
    */
   public final static String ERROR_VAR_HAS_NO_TYPE = AeMessages.getString("IAeValidationDefs.20") ; //$NON-NLS-1$
   
   /**
    * Partner link is unresolved and can't be found
    * 
    * Arg 0 is the partner link name.
    */
   public final static String ERROR_PARTNER_LINK_NOT_FOUND = AeMessages.getString("IAeValidationDefs.21") ; //$NON-NLS-1$
   
   /**
    * Partner link has no my role
    * 
    * Arg 0 is the partner link name.
    */
   public final static String ERROR_PARTNER_LINK_MISSING_MYROLE = AeMessages.getString("IAeValidationDefs.22") ; //$NON-NLS-1$
   
   /**
    * Partner link has no partner role
    * 
    * Arg 0 is the partner link name.
    */
   public final static String ERROR_PARTNER_LINK_MISSING_PARTNERROLE = AeMessages.getString("IAeValidationDefs.23") ; //$NON-NLS-1$
   
   /**
    * Partner link has no myrole
    * 
    * Arg 0 is the partner link name.
    * Arg 1 is the partner link type role.
    * Arg 2 is the portType for role.
    * Arg 3 is the portType for operation.
    */
   public final static String ERROR_PORTTYPE_MISMATCH = AeMessages.getString("IAeValidationDefs.24") ; //$NON-NLS-1$
   
   /**
    * Scope is unresolved.
    * 
    * Arg 0 is the compensate activity's scope.
    */
   public final static String ERROR_SCOPE_NOT_FOUND = AeMessages.getString("IAeValidationDefs.25"); //$NON-NLS-1$
   
   /**
    * Variable is unresolved and can't be found.
    * 
    * Arg 0 is the variable name.
    */
   public final static String ERROR_VAR_NOT_FOUND = AeMessages.getString("IAeValidationDefs.26") ; //$NON-NLS-1$
   
   /**
    * A message part is not declared and can't be found.
    * 
    * Arg 0 is the part name.
    * Arg 1:2 is the message type's QName nsURI:localPart.
    */
   public final static String ERROR_VAR_PART_NOT_FOUND = AeMessages.getString("IAeValidationDefs.27") ; //$NON-NLS-1$
   
   /**
    * A specified correlation set is unresolved and can't be found.
    * 
    * Arg 0 is the name of the correlation set.
    */
   public final static String ERROR_CORR_SET_NOT_FOUND = AeMessages.getString("IAeValidationDefs.28") ; //$NON-NLS-1$
   
   /**
    * A partner link is missing partner link type.
    * 
    * Arg 0 is the partner link's name.
    */
   public final static String ERROR_PARTNER_LINK_HAS_NO_TYPE = AeMessages.getString("IAeValidationDefs.29") ; //$NON-NLS-1$
   
   /**
    * A partner link type is unresolved and can't be found.
    * 
    * Arg 0:1 is the partner link type's QName nsURI:localPart.
    */
   public final static String ERROR_PARTNER_LINK_TYPE_NOT_FOUND = AeMessages.getString("IAeValidationDefs.30") ; //$NON-NLS-1$
   
   /**
    * A specified role is unresolved and can't be found.
    * 
    * Arg 0 is the role's name.
    * Arg 1 is the partner link type local name.
    */
   public final static String ERROR_ROLE_NOT_FOUND = AeMessages.getString("IAeValidationDefs.31") ; //$NON-NLS-1$
   
   /**
    * A specified role has no port type assignment.
    * 
    * Arg 0 is the role's name.
    * Arg 1 is the partner link type local name.
    */
   public final static String ERROR_ROLE_HAS_NO_PORTTYPE = AeMessages.getString("IAeValidationDefs.32") ; //$NON-NLS-1$
   
   /**
    * A specified port type is unresolved and can't be found.
    * 
    * Arg 0:1 is the port type's QName nsURI:localPart.
    */
   public final static String ERROR_PORT_TYPE_NOT_FOUND = AeMessages.getString("IAeValidationDefs.33") ; //$NON-NLS-1$
   
   /**
    * A specified operation with given input and is unresolved and can't be found.
    * 
    * Arg 0 is the operation's name.
    * Arg 1:2 is the QName of the invalid input message type.
    * Arg 3:4 is the QName of the invalid output message type
    */
   public final static String ERROR_OPERATION_INOUT_NOT_FOUND = AeMessages.getString("IAeValidationDefs.34") ; //$NON-NLS-1$
   
   /**
    * A specified operation with given input is unresolved and can't be found.
    * 
    * Arg 0 is the operation's name.
    * Arg 1:2 is the QName of the invalid input message type.
    */
   public final static String ERROR_OPERATION_IN_NOT_FOUND = AeMessages.getString("IAeValidationDefs.35") ; //$NON-NLS-1$

   /**
    * A specified operation with given output is unresolved and can't be found.
    * 
    * Arg 0 is the operation's name.
    * Arg 1:2 is the QName of the invalid output message type
    */
   public final static String ERROR_OPERATION_OUT_NOT_FOUND = AeMessages.getString("IAeValidationDefs.36") ; //$NON-NLS-1$
   
   /**
    * A correlation pattern is set to IN, OUT-IN, but the input variable definition doesn't exist.
    * 
    */
   public final static String ERROR_CORRELATION_IN_PATTERN_MISMATCH = AeMessages.getString("IAeValidationDefs.37") ; //$NON-NLS-1$

   /**
    * A correlation pattern is set to OUT or OUT-IN, but the output variable definitions doesn't exist.
    * 
    */
   public final static String ERROR_CORRELATION_OUT_PATTERN_MISMATCH = AeMessages.getString("IAeValidationDefs.38") ; //$NON-NLS-1$

   /**
    * A specified property is unresolved and can't be found.
    * 
    * Arg 0:1 is the property's QName nsURI:localPart.
    */
   public final static String ERROR_PROP_NOT_FOUND = AeMessages.getString("IAeValidationDefs.39") ; //$NON-NLS-1$
   
   /**
    * A specified message type is unresolved and can't be found.
    * 
    * Arg 0:1 is the message type's QName nsURI:localPart.
    */
   public final static String ERROR_MSG_SPEC_NOT_FOUND = AeMessages.getString("IAeValidationDefs.40") ; //$NON-NLS-1$

   /**
    * A specified element is unresolved and can't be found.
    * 
    * Arg 0:1 is the element's QName nsURI:localPart.
    */
   public final static String ERROR_ELEMENT_SPEC_NOT_FOUND = AeMessages.getString("IAeValidationDefs.41") ; //$NON-NLS-1$
   
   /**
    * A specified type specification is unresolved and can't be found.
    * 
    * Arg 0:1 is the type specification's QName nsURI:localPart.
    */
   public final static String ERROR_TYPE_SPEC_NOT_FOUND = AeMessages.getString("IAeValidationDefs.42") ; //$NON-NLS-1$
   
   /**
    * One or more property aliases for a variable are undefined.
    * 
    * Arg 0 is the variable name.
    * Arg 1 is the variable's type name.
    * Arg 2 is a comma-delimited list of the names of properties that are missing aliases.
    */
   public final static String ERROR_CORRELATION_PROP_ALIASES_NOT_FOUND = 
      AeMessages.getString("IAeValidationDefs.0") ; //$NON-NLS-1$
      
   /**
    * Correlation set has no properties
    * 
    * Arg 0 is the correlation set name.
    */
   public final static String ERROR_CORR_SET_PROPS_NOT_FOUND =
      AeMessages.getString("IAeValidationDefs.45") ; //$NON-NLS-1$
      
   /**
    * A PropertyAlias wasn't found for the given variable and property type.
    * 
    * Arg 0 is the variable name.
    * Arg 1 is the variable's type name.
    * Arg 2 is a comma-delimited list of the names of properties that are missing aliases.
    */
   public final static String ERROR_PROP_ALIASES_NOT_FOUND = 
      AeMessages.getString("IAeValidationDefs.46") ; //$NON-NLS-1$

   /**
    * An invalid expression (unexpected error) was encountered. Arg 0 is the invalid expression. Arg 1 is the reason.
    */
   public final static String ERROR_INVALID_EXPRESSION = AeMessages.getString("IAeValidationDefs.INVALID_EXPRESSION"); //$NON-NLS-1$

   /**
    * An invalid XPath expression was encountered. Arg 0 is the invalid expression. Arg 1 is the reason.
    */
   public final static String ERROR_INVALID_XPATH = AeMessages.getString("AeXPathExpressionValidator.INVALID_XPATH"); //$NON-NLS-1$

   /** signals an illegal part attribute for a copy operation's from/to */
   public final static String ERROR_PART_USAGE = AeMessages.getString("IAeValidationDefs.ERROR_PART_USAGE"); //$NON-NLS-1$
   
   /**
    * Variable Type Mismatch for an Activity (i.e., the variable used doesn't match the type 
    * needed by the activity.
    * 
    * Arg 0 is the variable name.
    * Arg 1:2 is the QName of the variable's type.
    * Arg 3:4 is the QName of the activity's required type.
    */
   public final static String ERROR_VAR_TYPE_MISMATCH = AeMessages.getString("IAeValidationDefs.49") ; //$NON-NLS-1$

   /**
    * A fault name is unresolved and can't be found.
    * 
    * Arg 0 is the fault name.
    */
   public final static String ERROR_FAULT_NAME_NOT_FOUND = AeMessages.getString("IAeValidationDefs.50") ; //$NON-NLS-1$
   
   /**
    * A fault handler has not activity.
    */
   public final static String ERROR_EMPTY_FAULT_HANDLER = AeMessages.getString("IAeValidationDefs.51") ; //$NON-NLS-1$

   /**
    * A fault handler has no fault name or fault variable.
    */
   public final static String ERROR_NO_NAME_OR_VARIABLE_ON_FAULT_HANDLER = AeMessages.getString("IAeValidationDefs.79" ); //$NON-NLS-1$
   
   /**
    * A compensate activity is in the wrong place.
    */
   public final static String ERROR_MISPLACED_COMPENSATE = AeMessages.getString("IAeValidationDefs.52") ; //$NON-NLS-1$

   /**
    * A part type specification is unresolved or can't be found.
    * 
    * Arg 0 is partname
    * Arg 1:2 is the message type specification's QName nsURI:localPart.
    * Arg 3 Exception message
    */
   public final static String ERROR_DISCOVERING_PART_TYPE_SPECS = AeMessages.getString("IAeValidationDefs.53") ; //$NON-NLS-1$

   /**
    * The object name is invalid, ie: not a valid NCName.
    * 
    * Arg 0 The invalid name
    */
   public final static String ERROR_INVALID_NAME = AeMessages.getString("IAeValidationDefs.69") ; //$NON-NLS-1$
   
   /**
    * No validator could be found for the expression language.
    * 
    * Arg 0 - the expression language.
    */
   public final static String ERROR_NO_VALIDATOR_FOR_LANGUAGE = AeMessages.getString("IAeValidationDefs.NO_VALIDATOR_FOR_EXPRLANG_ERROR"); //$NON-NLS-1$
   
   /////////////////////////////////
   // Warning codes - add as needed.
   /////////////////////////////////

   public final static String WARNING_TEST = "WarningTest" ; //$NON-NLS-1$

   /**
    * A defined partner link is never used.
    * 
    * Arg 0 is the partner link name.
    */
   public final static String WARNING_PARTNER_LINK_NOT_USED = AeMessages.getString("IAeValidationDefs.56") ; //$NON-NLS-1$

   /**
    * A defined variable is never used.
    * 
    * Arg 0 is the variable name.
    */
   public final static String WARNING_VARIABLE_NOT_USED = AeMessages.getString("IAeValidationDefs.57") ; //$NON-NLS-1$
   
   /**
    * A defined correlation set is never used.
    * 
    * Arg 0 is the correlation set name.
    */
   public final static String WARNING_CORR_SET_NOT_USED = AeMessages.getString("IAeValidationDefs.58") ; //$NON-NLS-1$

   /**
    * Import is missing it's location specifier.
    */
   public final static String WARNING_MISSING_IMPORT_LOCATION = AeMessages.getString("IAeValidationDefs.59") ; //$NON-NLS-1$
   
   /**
    * Namespace has an invalid location specifier.
    * 
    * Arg 0 is the namespace prefix.
    */
   public final static String WARNING_INVALID_IMPORT_LOCATION = AeMessages.getString("IAeValidationDefs.60") ; //$NON-NLS-1$
   
   /**
    * Receive: no correlation set assigned and the Receive is not a create instance.
    */
   public final static String WARNING_NO_CORR_SET_NO_CREATE = AeMessages.getString("IAeValidationDefs.61") ; //$NON-NLS-1$

   /**
    * A fault name is unresolved and can't be found.
    * 
    * Arg 0 is the fault name.
    */
   public final static String WARN_FAULT_NAME_NOT_CAUGHT = AeMessages.getString("IAeValidationDefs.62") ; //$NON-NLS-1$
   
   /**
    * Enabld instance compensation set to No - compensation handler invalid.
    */
   public final static String WARN_COMPENSATION_HANDLER_NOT_ENABLED = AeMessages.getString("IAeValidationDefs.63") ; //$NON-NLS-1$
   
   /**
    * Missing single quotes to specify a constant variable name.
    * 
    * Arg 1 is the variable name.
    */
   public final static String WARN_NON_CONST_VARNAME = AeMessages.getString("IAeValidationDefs.64") ; //$NON-NLS-1$

   /**
    * A variable type has been overloaded within a scope definition
    * 
    * Arg 0 is the variable name.
    */
   public final static String WARNING_VARIABLE_TYPE_OVERLOADED = AeMessages.getString("IAeValidationDefs.65") ; //$NON-NLS-1$

   /**
    * A variable has supplied a query expression without a required part name
    * 
    * Arg 0 is the variable name.
    */
   public final static String WARNING_VARIABLE_PART_REQUIRED = AeMessages.getString("IAeValidationDefs.66") ; //$NON-NLS-1$

   /**
    * A variable has supplied a query expression for a simple type variable
    * 
    * Arg 0 is the variable name.
    */
   public final static String WARNING_VARIABLE_QUERY_NOT_SUPPORTED = AeMessages.getString("IAeValidationDefs.67") ; //$NON-NLS-1$
   
   /**
    * Invalid literal found in join conditio expression.
    * 
    * Arg 0 is the invalid literal.
    */
   public static final String INVALID_LITERAL_IN_JOIN_CONDITION = AeMessages.getString("IAeValidationDefs.75"); //$NON-NLS-1$

   /**
    * Link in joinCondition could not be found.
    * 
    * Arg 0 is the link name.
    * Arg 1 is the joinCondition expression 
    */
   public static final String NO_LINK_FOUND_FOR_JOIN_CONDITION = AeMessages.getString("IAeValidationDefs.76"); //$NON-NLS-1$
   
   /**
    * Invalid match between number of link names and number of getLinkStatus function calls.
    * 
    * Arg 0 is the joinCondition expression 
    */
   public static final String INVALID_JOIN_CONDITION = AeMessages.getString("IAeValidationDefs.77"); //$NON-NLS-1$
   
   /**
    * Found a getLinkStatus with no args.
    * 
    * Arg 0 is the entire joinCondition expression.
    */
   public static final String EMPTY_GET_LINK_STATUS_FUNCTION = AeMessages.getString("IAeValidationDefs.72"); //$NON-NLS-1$

   /**
    * Invalid nesting of functions inside joinCondition expression.
    * 
    * Arg 0 is the entire joinCondition expression.
    */
   public static final String INVALID_NESTING_IN_LINK_STATUS_FUNCTION = AeMessages.getString("IAeValidationDefs.73"); //$NON-NLS-1$
   
   /**
    * Invalid arg present in getLinkStatus function of joinCondition.
    * 
    * Arg 0 is the joinCondition expression.
    */
   public static final String INVALID_ARG_IN_LINK_STATUS_FUNCTION = AeMessages.getString("IAeValidationDefs.74"); //$NON-NLS-1$
   
   /////////////////////////////////////
   // Information codes - add as needed.
   /////////////////////////////////////

   public final static String INFO_TEST = "InfoTest" ; //$NON-NLS-1$

}
