//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/validation/expr/xpath/AeXPathExpressionValidator.java,v 1.1 2005/06/08 12:50:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.validation.expr.xpath;

import java.util.Iterator;

import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.def.expr.IAeExpressionParseResult;
import org.activebpel.rt.bpel.def.expr.IAeExpressionParser;
import org.activebpel.rt.bpel.def.expr.IAeExpressionParserContext;
import org.activebpel.rt.bpel.def.expr.xpath.AeXPathExpressionParser;
import org.activebpel.rt.bpel.def.expr.xpath.AeXPathParseResult;
import org.activebpel.rt.bpel.def.expr.xpath.AeXPathParseHandler.AeXPathFunctionNode;
import org.activebpel.rt.bpel.def.expr.xpath.AeXPathParseHandler.AeXPathLiteralNode;
import org.activebpel.rt.bpel.def.expr.xpath.AeXPathParseHandler.AeXPathNode;
import org.activebpel.rt.bpel.def.validation.expr.AeAbstractExpressionValidator;
import org.activebpel.rt.bpel.def.validation.expr.AeExpressionValidationResult;
import org.activebpel.rt.bpel.def.validation.expr.IAeExpressionValidationContext;

/**
 * Implements an expression validator for the XPath 1.0 expression language. This is the default languge used
 * for BPEL 1.0 (when no expression language is specified).
 */
public class AeXPathExpressionValidator extends AeAbstractExpressionValidator
{
   /**
    * Constructs an xpath expression validator.
    */
   public AeXPathExpressionValidator()
   {
   }

   /**
    * Overrides method to do additional validation on the xpath parse tree for join conditions.  This
    * method delegates to the superclass to do most of the validation, but then does a little bit extra.
    * 
    * @see org.activebpel.rt.bpel.def.validation.expr.AeAbstractExpressionValidator#doJoinConditionValidation(org.activebpel.rt.bpel.def.expr.IAeExpressionParseResult, org.activebpel.rt.bpel.def.validation.expr.AeExpressionValidationResult, org.activebpel.rt.bpel.def.validation.expr.IAeExpressionValidationContext)
    */
   protected void doJoinConditionValidation(IAeExpressionParseResult aParseResult,
         AeExpressionValidationResult aValidationResult, IAeExpressionValidationContext aContext)
   {
      super.doJoinConditionValidation(aParseResult, aValidationResult, aContext);

      AeXPathNode node = ((AeXPathParseResult) aParseResult).getHandler().getTree();
      validateXPathNodeForJoinCondition(node, aParseResult, aValidationResult);
   }

   /**
    * Validates that the given node is ok for a join condition.  This specifically looks for 
    * literal nodes that are not contained within function calls.
    * 
    * @param aNode
    * @param aParseResult
    * @param aValidationResult
    */
   protected void validateXPathNodeForJoinCondition(AeXPathNode aNode, IAeExpressionParseResult aParseResult,
         AeExpressionValidationResult aValidationResult)
   {
      if (aNode instanceof AeXPathFunctionNode)
      {
         return;
      }
      else if (aNode instanceof AeXPathLiteralNode)
      {
         addError(aValidationResult,
               AeMessages.getString("AeXPathExpressionValidator.INVALID_LITERAL_IN_JOINCONDITION_ERROR"),  //$NON-NLS-1$
               new Object [] { aNode.getValue(), aParseResult.getExpression() });
         return;
      }

      for (Iterator iter = aNode.getChildren().iterator(); iter.hasNext(); )
      {
         AeXPathNode child = (AeXPathNode) iter.next();
         validateXPathNodeForJoinCondition(child, aParseResult, aValidationResult);
      }
   }

   /**
    * Overrides method to supply the xpath expression parser.
    * 
    * @see org.activebpel.rt.bpel.def.validation.expr.AeAbstractExpressionValidator#createExpressionParser(org.activebpel.rt.bpel.def.expr.IAeExpressionParserContext)
    */
   protected IAeExpressionParser createExpressionParser(IAeExpressionParserContext aContext)
   {
      return new AeXPathExpressionParser(aContext);
   }
}
