// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/visitors/AeDefPathVisitor.java,v 1.13 2006/01/11 21:01:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.visitors;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeCorrelationContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetDef;
import org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeEventHandlersDef;
import org.activebpel.rt.bpel.def.AeFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef;
import org.activebpel.rt.bpel.def.AeImportDef;
import org.activebpel.rt.bpel.def.AeImportsContainerDef;
import org.activebpel.rt.bpel.def.AeMessageExchangesDef;
import org.activebpel.rt.bpel.def.AeNamedDef;
import org.activebpel.rt.bpel.def.AePartnerDef;
import org.activebpel.rt.bpel.def.AePartnerLinkDef;
import org.activebpel.rt.bpel.def.AePartnerLinksContainerDef;
import org.activebpel.rt.bpel.def.AePartnersContainerDef;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.def.AeVariablesContainerDef;
import org.activebpel.rt.bpel.def.activity.AeActivityAssignDef;
import org.activebpel.rt.bpel.def.activity.AeActivityBreakDef;
import org.activebpel.rt.bpel.def.activity.AeActivityCompensateDef;
import org.activebpel.rt.bpel.def.activity.AeActivityContinueDef;
import org.activebpel.rt.bpel.def.activity.AeActivityEmptyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityFlowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityForEachDef;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityPickDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReplyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySequenceDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySuspendDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef;
import org.activebpel.rt.bpel.def.activity.AeActivityTerminateDef;
import org.activebpel.rt.bpel.def.activity.AeActivityThrowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWaitDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWhileDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignToDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachBranchesDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef;
import org.activebpel.rt.bpel.def.activity.support.AeSourceDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef;
import org.activebpel.rt.bpel.def.activity.support.AeTargetDef;
import org.activebpel.rt.util.AeUtil;

/**
 * Visitor responsible for creating location paths for process definition objects.
 * User must call the createPaths method to execute location path creation.
 * @todo may want a static convienence method for ctor and createPaths call.
 */
public class AeDefPathVisitor implements IAeDefVisitor
{
   /** Used in conjunction with the traversal object to traverse the object model */
   protected IAeDefVisitor mTraversalVisitor;
   /** Current path */
   protected String mPath=""; //$NON-NLS-1$
   /** Path Maps */
   protected HashSet mPaths = new HashSet();
   /** Next available location id */
   private int mNextLocationId = 1;
   /** Maps location paths to location ids */
   private final Map mLocationPathMap = new HashMap();
   
   /**
    * Ctor for the path visitor allocates traverser. User must call
    * the <CODE>createPaths</CODE> method to execute location path creation.
    */
   public AeDefPathVisitor()
   {
      mTraversalVisitor = new AeTraversalVisitor(new AeDefTraverser(), this);
   }
      
   /**
    * @return Returns the path.
    */
   protected String getPath()
   {
      return mPath;
   }
   
   /**
    * @param aPath The path to set.
    */
   protected void setPath(String aPath)
   {
      mPath = aPath;
   }
   /**
    * Creates a unique location path for the definition and traverses its children.
    * The unique path is based on the current path of parent and the passed 
    * piece to append.  After creating and setting the unique path for the passed
    * definition it then traverses the definitions children after making its path 
    * current. It then restores the parent's path before returning.
    * @param aDef The definition to associate with passed path.
    * @param aAppendPath The path piece to append to the current path to uniquely 
    *                     represent def, if null only traverse is performed.
    */
   protected void createPathAndTraverse(AeBaseDef aDef, String aAppendPath)
   {
      createPathAndTraverse(aDef, aAppendPath, null);
   }

   /**
    * Accepts the extra arg of a name path which will be non null for AeNamedDef 
    * instances that had a non-null value for their name. This allows for more
    * user friendly xpath names while not running into problem of generating
    * inaccurate xpath.
    * @param aDef - the definition object we're visiting
    * @param aAppendPath - the path to the def object
    * @param aNamePath - the path with the name attribute in it
    */
   protected void createPathAndTraverse(AeBaseDef aDef, String aAppendPath, String aNamePath)
   {
      if(aAppendPath != null)
      {
         String savePath = mPath;
         if (AeUtil.isNullOrEmpty(aNamePath))
         {
            mPath = createUniquePath(aDef, aAppendPath);
         }
         else
         {
            // calling createUniquePath TWICE here so we'll have a record of the
            // path in both its named and unnamed form. This addresses the issue
            // of having multiple named objects as child elements and only having
            // SOME of them contain names. 
            // i.e.
            // <scope name="one"/>
            // <scope/>
            // <scope name="three"/>
            // without this extra call, the path for the second scope will be
            // "/scope" since there was no other scope recorded with that path. 
            // It SHOULD be /scope[2].
            createUniquePath(aDef, aAppendPath);
            mPath = createUniquePath(aDef, aNamePath);
         }
         int locationId = getNextLocationId();
         mLocationPathMap.put(mPath, new Integer(locationId));
         recordLocationPathAndId(aDef, mPath, locationId);
         aDef.accept(mTraversalVisitor);
         mPath = savePath;
      }
      else
      {
         aDef.accept(mTraversalVisitor);
      }
   }
   
   /**
    * Sets the location path and id on the def object. 
    * @param aDef
    * @param aLocationPath
    * @param aLocationId
    */
   protected void recordLocationPathAndId(AeBaseDef aDef, String aLocationPath, int aLocationId)
   {
      aDef.setLocationPath(aLocationPath);
      aDef.setLocationId(aLocationId);
   }

   /**
    * Appends the def's name to the xpath to make a more human readable xpath.
    * @param aDef
    * @param aAppendPath
    */
   protected void createPathAndTraverse(AeNamedDef aDef, String aAppendPath)
   {
      String name = aDef.getName();
      if (!AeUtil.isNullOrEmpty(name))
      {
         StringBuffer buffer = new StringBuffer(aAppendPath);
         buffer.append("[@name='"); //$NON-NLS-1$
         buffer.append(name);
         buffer.append("']");  //$NON-NLS-1$
         createPathAndTraverse((AeBaseDef)aDef, aAppendPath, buffer.toString());
      }
      else
      {
         createPathAndTraverse((AeBaseDef)aDef, aAppendPath);
      }
   }
   
   
   /**
    * Checks map of paths created by vistors and appends indexes to make unique
    * @param aDef The def object we're creating the path for
    * @param aAppendPath path to append to main path
    * @return the unique path
    */
   protected String createUniquePath(AeBaseDef aDef, String aAppendPath)
   {
      StringBuffer testPath = new StringBuffer(mPath);
      testPath.append("/"); //$NON-NLS-1$
      testPath.append(aAppendPath);
      int initLen = testPath.length();
      
      // test path for uniqueness in set and loop until unique
      for(int i = 2; mPaths.contains(testPath.toString()); ++i)
      {
         testPath.setLength(initLen);
         testPath.append("[").append(i).append("]"); //$NON-NLS-1$ //$NON-NLS-2$
      }
      
      // good path found so save the path in set and return the good path
      String goodPath = testPath.toString();
      mPaths.add(goodPath);
      return goodPath;
   }

   /**
    * Returns next available location id.
    */
   protected int getNextLocationId()
   {
      int nextId = mNextLocationId;
      setNextLocationId(nextId + 1);
      return nextId;
   }
   
   /**
    * Setter for the location id
    * 
    * @param aId
    */
   protected void setNextLocationId(int aId)
   {
      mNextLocationId = aId; 
   }

   /**
    * Returns set of generated location paths.
    */
   public Set getLocationPaths()
   {
      return Collections.unmodifiableSet(mLocationPathMap.keySet());
   }
   
   /**
    * Returns location id corresponding to a location path.
    */
   public int getLocationId(String aLocationPath)
   {
      Integer id = (Integer) mLocationPathMap.get(aLocationPath);
      return id.intValue();
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeProcessDef)
    */
   public void visit(AeProcessDef aDef)
   {
      // Do the cast here to AeBaseDef so we won't have the name predicate included
      // in the path. There is no need to have the name in place for the process
      // element since there is only one per document.
      createPathAndTraverse((AeBaseDef)aDef, AeProcessDef.TAG_PROCESS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityScopeDef)
    */
   public void visit(AeActivityScopeDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityScopeDef.TAG_SCOPE);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeEventHandlersDef)
    */
   public void visit(AeEventHandlersDef aDef)
   {
      createPathAndTraverse(aDef, AeEventHandlersDef.TAG_EVENT_HANDLERS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef)
    */
   public void visit(AeFaultHandlersContainerDef aDef)
   {
      createPathAndTraverse(aDef, AeFaultHandlersContainerDef.TAG_FAULT_HANDLERS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef)
    */
   public void visit(AeScopeCompensatorContainerDef aDef)
   {
      createPathAndTraverse(aDef, AeScopeCompensatorContainerDef.TAG_COMPENSATION_HANDLER);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeVariablesContainerDef)
    */
   public void visit(AeVariablesContainerDef aDef)
   {
      createPathAndTraverse(aDef, AeVariablesContainerDef.TAG_VARIABLES);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityAssignDef)
    */
   public void visit(AeActivityAssignDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityAssignDef.TAG_ASSIGN);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityCompensateDef)
    */
   public void visit(AeActivityCompensateDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityCompensateDef.TAG_COMPENSATE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityEmptyDef)
    */
   public void visit(AeActivityEmptyDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityEmptyDef.TAG_EMPTY);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityFlowDef)
    */
   public void visit(AeActivityFlowDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityFlowDef.TAG_FLOW);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef)
    */
   public void visit(AeActivityInvokeDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityInvokeDef.TAG_INVOKE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityPickDef)
    */
   public void visit(AeActivityPickDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityPickDef.TAG_PICK);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef)
    */
   public void visit(AeActivityReceiveDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityReceiveDef.TAG_RECEIVE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReplyDef)
    */
   public void visit(AeActivityReplyDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityReplyDef.TAG_REPLY);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivitySuspendDef)
    */
   public void visit(AeActivitySuspendDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityReplyDef.TAG_SUSPEND);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivitySequenceDef)
    */
   public void visit(AeActivitySequenceDef aDef)
   {
      createPathAndTraverse(aDef, AeActivitySequenceDef.TAG_SEQUENCE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef)
    */
   public void visit(AeActivitySwitchDef aDef)
   {
      createPathAndTraverse(aDef, AeActivitySwitchDef.TAG_SWITCH);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityTerminateDef)
    */
   public void visit(AeActivityTerminateDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityTerminateDef.TAG_TERMINATE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityThrowDef)
    */
   public void visit(AeActivityThrowDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityThrowDef.TAG_THROW);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityWaitDef)
    */
   public void visit(AeActivityWaitDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityWaitDef.TAG_WAIT);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityForEachDef)
    */
   public void visit(AeActivityForEachDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityForEachDef.TAG_FOREACH);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef)
    */
   public void visit(AeForEachFinalDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityForEachDef.TAG_FOREACH_FINALCOUNTER);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef)
    */
   public void visit(AeForEachStartDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityForEachDef.TAG_FOREACH_STARTCOUNTER);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachBranchesDef)
    */
   public void visit(AeForEachBranchesDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityForEachDef.TAG_FOREACH_BRANCHES);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef)
    */
   public void visit(AeForEachCompletionConditionDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityForEachDef.TAG_FOREACH_COMPLETION_CONDITION);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityWhileDef)
    */
   public void visit(AeActivityWhileDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityWhileDef.TAG_WHILE);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityContinueDef)
    */
   public void visit(AeActivityContinueDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityContinueDef.TAG_CONTINUE);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityBreakDef)
    */
   public void visit(AeActivityBreakDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityBreakDef.TAG_BREAK);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationSetDef)
    */
   public void visit(AeCorrelationSetDef aDef)
   {
      createPathAndTraverse(aDef, AeCorrelationSetDef.TAG_CORRELATION_SET);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeFaultHandlerDef)
    */
   public void visit(AeFaultHandlerDef aDef)
   {
      createPathAndTraverse(aDef, AeFaultHandlerDef.TAG_CATCH);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeMessageExchangesDef)
    */
   public void visit(AeMessageExchangesDef aDef)
   {
      // don't need paths for message exchanges
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef)
    */
   public void visit(AeOnAlarmDef aDef)
   {
      createPathAndTraverse(aDef, AeOnAlarmDef.TAG_ON_ALARM);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef)
    */
   public void visit(AeOnMessageDef aDef)
   {
      createPathAndTraverse(aDef, AeOnMessageDef.TAG_ON_MESSAGE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeVariableDef)
    */
   public void visit(AeVariableDef aDef)
   {
      createPathAndTraverse(aDef, AeVariableDef.TAG_VARIABLE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef)
    */
   public void visit(AeDefaultFaultHandlerDef aDef)
   {
      createPathAndTraverse(aDef, AeDefaultFaultHandlerDef.TAG_CATCH_ALL);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef)
    */
   public void visit(AeSwitchCaseDef aDef)
   {
      createPathAndTraverse(aDef, AeActivitySwitchDef.TAG_CASE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef)
    */
   public void visit(AeOtherwiseDef aDef)
   {
      createPathAndTraverse(aDef, AeActivitySwitchDef.TAG_OTHERWISE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef)
    */
   public void visit(AeSwitchCaseContainerDef aDef)
   {
      createPathAndTraverse(aDef, null);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef)
    */
   public void visit(AeAssignCopyDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityAssignDef.TAG_COPY);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef)
    */
   public void visit(AeCorrelationDef aDef)
   {
      createPathAndTraverse(aDef, AeCorrelationDef.TAG_CORRELATION);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeLinkDef)
    */
   public void visit(AeLinkDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityFlowDef.TAG_LINK);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerDef)
    */
   public void visit(AePartnerDef aDef)
   {
      createPathAndTraverse(aDef, AePartnerDef.TAG_PARTNER);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerLinkDef)
    */
   public void visit(AePartnerLinkDef aDef)
   {
      createPathAndTraverse(aDef, AePartnerLinkDef.TAG_PARTNER_LINK);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeScopeDef)
    */
   public void visit(AeScopeDef aDef)
   {
      createPathAndTraverse(aDef, null);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnerLinksContainerDef)
    */
   public void visit(AePartnerLinksContainerDef aDef)
   {
      createPathAndTraverse(aDef, AePartnerLinkDef.TAG_PARTNER_LINKS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AePartnersContainerDef)
    */
   public void visit(AePartnersContainerDef aDef)
   {
      createPathAndTraverse(aDef, AePartnerDef.TAG_PARTNERS);
   }

   public void visit(AeLinkContainerDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityFlowDef.TAG_LINKS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationContainerDef)
    */
   public void visit(AeCorrelationContainerDef aDef)
   {
      createPathAndTraverse(aDef, AeCorrelationDef.TAG_CORRELATIONS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef)
    */
   public void visit(AeAssignFromDef aDef)
   {
      createPathAndTraverse(aDef, AeAssignCopyDef.TAG_FROM);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignToDef)
    */
   public void visit(AeAssignToDef aDef)
   {
      createPathAndTraverse(aDef, AeAssignCopyDef.TAG_TO);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef)
    */
   public void visit(AeCorrelationSetContainerDef aDef)
   {
      createPathAndTraverse(aDef, AeCorrelationSetDef.TAG_CORRELATION_SETS);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSourceDef)
    */
   public void visit(AeSourceDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityDef.TAG_SOURCE);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeTargetDef)
    */
   public void visit(AeTargetDef aDef)
   {
      createPathAndTraverse(aDef, AeActivityDef.TAG_TARGET);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeImportsContainerDef)
    */
   public void visit(AeImportsContainerDef aDef)
   {
      // Do not visit. BPEL v1.1 does not support imports
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeImportDef)
    */
   public void visit(AeImportDef aDef)
   {
      // Do not visit. BPEL v1.1 does not support imports
   }
}
