// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/visitors/AeDefTraverser.java,v 1.11 2006/01/11 21:01:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.visitors;

import java.util.Iterator;

import org.activebpel.rt.bpel.def.AeActivityDef;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeCorrelationContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef;
import org.activebpel.rt.bpel.def.AeCorrelationSetDef;
import org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeEventHandlersDef;
import org.activebpel.rt.bpel.def.AeFaultHandlerDef;
import org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef;
import org.activebpel.rt.bpel.def.AeMessageExchangesDef;
import org.activebpel.rt.bpel.def.AePartnerDef;
import org.activebpel.rt.bpel.def.AePartnerLinkDef;
import org.activebpel.rt.bpel.def.AePartnerLinksContainerDef;
import org.activebpel.rt.bpel.def.AePartnersContainerDef;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.def.AeVariablesContainerDef;
import org.activebpel.rt.bpel.def.activity.AeActivityAssignDef;
import org.activebpel.rt.bpel.def.activity.AeActivityBreakDef;
import org.activebpel.rt.bpel.def.activity.AeActivityCompensateDef;
import org.activebpel.rt.bpel.def.activity.AeActivityContinueDef;
import org.activebpel.rt.bpel.def.activity.AeActivityEmptyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityFlowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityForEachDef;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityPickDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReplyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySequenceDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySuspendDef;
import org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef;
import org.activebpel.rt.bpel.def.activity.AeActivityTerminateDef;
import org.activebpel.rt.bpel.def.activity.AeActivityThrowDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWaitDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWhileDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignToDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachBranchesDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef;
import org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeLinkDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef;
import org.activebpel.rt.bpel.def.activity.support.AeSourceDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef;
import org.activebpel.rt.bpel.def.activity.support.AeTargetDef;

/**
 * The external iterator for the IAeDefVisitor interface. Provides the logic for
 * traversing the BPEL definition objects. 
 */
public class AeDefTraverser implements IAeDefTraverser
{
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeProcessDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeProcessDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getPartnerLinksContainer(), aVisitor);
      callAccept(aDef.getPartnersContainer(), aVisitor);
      
      traverse((AeScopeDef)aDef, aVisitor);
   }

   /**
    * Nothing to traverse on the <code>AeActivityScopeDef</code> proper since the real work
    * is done in the <code>AeScopeDef</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityScopeDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityScopeDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      AeScopeDef scope = aDef.getScope();
      scope.accept(aVisitor);
   }

   /**
    * Walk each of the <code>AeCorrelationSetDef</code> and call accept.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeCorrelationSetContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(
      AeCorrelationSetContainerDef aDef,
      IAeDefVisitor aVisitor)
   {
      // @todo these classes should impl common interface as per chris's
      //       original idea. 
      callAccept(aDef.getValues(), aVisitor);
   }
   
   /**
    * Walk each of the <code>AeOnMessageDef</code> and <code>AeOnAlarmDef</code>
    * and call accept.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeEventHandlersDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeEventHandlersDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getMessageList(), aVisitor);
      callAccept(aDef.getAlarmList(), aVisitor);
   }

   /**
    * Walk all of the <code>AeFaultHandlerDef</code> as well as the
    * <code>AeDefaultFaultHandlerDef</code> and call accept
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeFaultHandlersContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(
      AeFaultHandlersContainerDef aDef,
      IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getFaultHandlers(), aVisitor);
      callAccept(aDef.getDefaultFaultHandler(), aVisitor);
   }

   /**
    * Calls accept on the single child of the compensation handler
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeScopeCompensatorContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(
      AeScopeCompensatorContainerDef aDef,
      IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getActivity(), aVisitor);
   }

   /**
    * Walk all of the <code>AeVariableDef</code> and call accept.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeVariablesContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeVariablesContainerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getValues(), aVisitor);
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeCorrelationSetDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeCorrelationSetDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * Calls <code>accept</code> on the child activity for the fault
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeFaultHandlerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeFaultHandlerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getActivity(), aVisitor);
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeVariableDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeVariableDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * Calls <code>accept</code> on the child activity for the alarm.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeOnAlarmDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getActivity(), aVisitor);
   }

   /**
    * Calls <code>accept</code> on the correlations and the child activity
    * for the message.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeOnMessageDef aDef, IAeDefVisitor aVisitor)
   {
      if (aDef.hasCorrelationList())
      {
         callAccept(aDef.getCorrelationContainer(), aVisitor);
      }
      callAccept(aDef.getActivity(), aVisitor);
   }
   
   /**
    * Calls <code>accept</code> on the <code>AeAssignCopyDef</code> children.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityAssignDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityAssignDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getCopyList(), aVisitor);
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityCompensateDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityCompensateDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityEmptyDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityEmptyDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivitySuspendDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivitySuspendDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityContinueDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityContinueDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityBreakDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityBreakDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * Walk all of the <code>AeLinkDef</code> and <code>AeActivityDef</code> 
    * and call <code>accept</code>
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityFlowDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityFlowDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getLinkContainer(), aVisitor);
      callAccept(aDef.getActivityList(), aVisitor);
   }

   /**
    * Walks all of the <code>AeInvokeDef</code> and calls <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityInvokeDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      if (aDef.hasCorrelationList())
      {
         callAccept(aDef.getCorrelationContainer(), aVisitor);
      }
   }

   /**
    * Walk the <code>AeOnMessageDef</code> and <code>AeOnAlarmDef</code> and
    * call <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityPickDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityPickDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getMessageList(), aVisitor);
      callAccept(aDef.getAlarmList(), aVisitor);
   }

   /**
    * Walks all of the <code>AeCorrelationDef</code> and calls <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityReceiveDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      if (aDef.hasCorrelationList())
      {
         callAccept(aDef.getCorrelationContainer(), aVisitor);
      }
   }

   /**
    * Walks all of the <code>AeCorrelationDef</code> and calls <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityReplyDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityReplyDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      if (aDef.hasCorrelationList())
      {
         callAccept(aDef.getCorrelationContainer(), aVisitor);
      }
   }

   /**
    * Walk all of the child activities and call <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivitySequenceDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivitySequenceDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getActivityList(), aVisitor);
   }

   /**
    * Walk each of the <code>AeSwitchCaseDef</code> and call <code>accept</code>
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivitySwitchDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getSwitchCaseContainer(), aVisitor);
      if (aDef.getOtherwise() != null)
      {
         callAccept(aDef.getOtherwise(), aVisitor);
      }
   }

   /**
    * Traverse terminate for inbound links
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityTerminateDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityTerminateDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * Nothing to traverse here.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityThrowDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityThrowDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * Nothing to traverse here.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityWaitDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityWaitDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
   }

   /**
    * Call <code>accept</code> on the child activity.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityWhileDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityWhileDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getActivity(), aVisitor);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.AeActivityForEachDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeActivityForEachDef aDef, IAeDefVisitor aVisitor)
   {
      traverseSourceAndTargetLinks(aDef, aVisitor);
      callAccept(aDef.getStartDef(), aVisitor);
      callAccept(aDef.getFinalDef(), aVisitor);
      callAccept(aDef.getCompletionCondition(), aVisitor);
      callAccept(aDef.getActivity(), aVisitor);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeForEachFinalDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeForEachFinalDef aDef, IAeDefVisitor aVisitor)
   {
      // dead end
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeForEachStartDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeForEachStartDef aDef, IAeDefVisitor aVisitor)
   {
      // dead end
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeForEachBranchesDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeForEachBranchesDef aDef, IAeDefVisitor aVisitor)
   {
      // dead end
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeForEachCompletionConditionDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeForEachCompletionConditionDef aDef,
         IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getBranches(), aVisitor);
   }

   /**
    * Call <code>accept</code> on the child activity.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeDefaultFaultHandlerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getActivity(), aVisitor);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeSwitchCaseContainerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getValues(), aVisitor);
   }

   /**
    * Call <code>accept</code> on the child activity.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeSwitchCaseDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getActivity(), aVisitor);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeOtherwiseDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeOtherwiseDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getActivity(), aVisitor);
   }
   

   /**
    * Calls <code>accept</code> on the From and To
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeAssignCopyDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getFrom(), aVisitor);
      callAccept(aDef.getTo(), aVisitor);
   }
   
   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeAssignFromDef aDef, IAeDefVisitor aVisitor)
   {
   }
   
   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeAssignToDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeAssignToDef aDef, IAeDefVisitor aVisitor)
   {
   }

   

   /**
    * Walks all of the <code>correlation</code> objects and calls accept.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeCorrelationContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeCorrelationContainerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getValues(), aVisitor);
   }

   

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeCorrelationDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeLinkDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeLinkDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AePartnerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AePartnerDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AePartnerLinkDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AePartnerLinkDef aDef, IAeDefVisitor aVisitor)
   {
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeMessageExchangesDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeMessageExchangesDef aDef, IAeDefVisitor aVisitor)
   {
      // there are no defs for the individual messageExchange values
   }

   /**
    * Calls <code>accept</code> on the fault handler, compensation handler,
    * variable container, correlationset container, event handler, and the
    * child activity.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AeScopeDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeScopeDef aDef, IAeDefVisitor aVisitor)
   {
      if (aDef.hasVariables())
      {
         callAccept(aDef.getVariablesContainer(), aVisitor);
      }
      
      if (aDef.hasCorrelationSets())
      {
         callAccept(aDef.getCorrelationSetContainer(), aVisitor);
      }
      
      if (aDef.hasFaultHandlers())
      {
         callAccept(aDef.getFaultHandlerContainer(), aVisitor);
      }

      if (aDef.hasCompensator())
      {
         callAccept(aDef.getCompensatorContainer(), aVisitor);
      }
      
      if (aDef.hasEventHandlers())
      {
         callAccept(aDef.getEventHandlers(), aVisitor);
      }

      if (aDef.hasMessageExchanges())
      {
         callAccept(aDef.getMessageExchangesDef(), aVisitor);
      }

      callAccept(aDef.getActivity(), aVisitor);
   }

   /**
    * Walk all of the child partnerLinks and call <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AePartnerLinksContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(
      AePartnerLinksContainerDef aDef,
      IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getValues(), aVisitor);
   }

   /**
    * Walk all of the child partners and call <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.AePartnersContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AePartnersContainerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getValues(), aVisitor);
   }

   /**
    * Walk all of the child links and call <code>accept</code>.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeLinkContainerDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeLinkContainerDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getLinks(), aVisitor);
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeSourceDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeSourceDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * Nothing to traverse.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefTraverser#traverse(org.activebpel.rt.bpel.def.activity.support.AeTargetDef, org.activebpel.rt.bpel.def.visitors.IAeDefVisitor)
    */
   public void traverse(AeTargetDef aDef, IAeDefVisitor aVisitor)
   {
   }

   /**
    * @param aDef
    * @param aVisitor
    */
   protected void traverseSourceAndTargetLinks(AeActivityDef aDef, IAeDefVisitor aVisitor)
   {
      callAccept(aDef.getTargetList(), aVisitor);
      callAccept(aDef.getSourceList(), aVisitor);
   }

   /**
    * Calls <code>accept</code> on each of the definition objects in the Iterator
    * @param aIterator
    * @param aVisitor
    */
   protected void callAccept(Iterator aIterator, IAeDefVisitor aVisitor)
   {
      while (aIterator.hasNext())
      {
         AeBaseDef def = (AeBaseDef) aIterator.next();
         callAccept(def, aVisitor);
      }
   }

   /**
    * Calls <code>accept</code> on the def. 
    * @param aDef can be null.
    * @param aVisitor
    */
   protected void callAccept(AeBaseDef aDef, IAeDefVisitor aVisitor)
   {
      if (aDef != null)
      {
         aDef.accept(aVisitor);
      }
   }

}
