// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/visitors/AeDefVariableTypeVisitor.java,v 1.10 2005/06/22 16:43:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.visitors;

import java.text.MessageFormat;
import java.util.Iterator;

import javax.wsdl.Part;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeWSDLDefHelper;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeWSDLProvider;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;

/**
 * Visitor that sets the Part and Type data for each variable.
 */
public class AeDefVariableTypeVisitor extends AeAbstractDefVisitor
{
   /** The WSDL provider set during visitor creation. */
   private IAeWSDLProvider mWSDLProvider;
   /** Flag used to track if errors have occurred. */
   private boolean mHasErrors;  

   /**
    * Constructor for visitor used to set type info for variables.
    * @param aProvider the WSDL provider to facilitate in finding type info.
    */
   public AeDefVariableTypeVisitor(IAeWSDLProvider aProvider)
   {
      mWSDLProvider = aProvider;
      setTraversalVisitor(new AeTraversalVisitor(new AeDefTraverser(), this));
   }

   /**
    * Initiates the visitation process.
    * @param aProcess the process to be visited.
    */
   public void findTypeInfo(AeProcessDef aProcess) throws AeBusinessProcessException
   {
      visit(aProcess);
      
      if (mHasErrors)
         throw new AeBusinessProcessException(AeMessages.getString("AeDefVariableTypeVisitor.ERROR_0")); //$NON-NLS-1$
   }

   /**
    * Handle visitation to the variable. 
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeVariableDef)
    */
   public void visit(AeVariableDef aVariable)
   {
      try
      {
         // only assign types for messages (element and type vars are not validated)
         if(aVariable.getMessageType() != null)
         {
            AeBPELExtendedWSDLDef def = AeWSDLDefHelper.getWSDLDefinitionForMsg(mWSDLProvider, aVariable.getMessageType());
            if (def != null)
            {
               for (Iterator iter=def.getMessageParts(aVariable.getMessageType()); iter.hasNext();)
                  aVariable.addPartTypeInfo((Part)iter.next(), def);
            }
            else
            { 
               throw new AeException(MessageFormat.format(AeMessages.getString("AeDefVariableTypeVisitor.ERROR_1"), //$NON-NLS-1$
                                                          new Object[] {aVariable.getName(), aVariable.getMessageType()}));
            }
         }
      }
      catch (AeException e)
      {
         mHasErrors = true;
         AeException.logError(e, AeMessages.getString("AeDefVariableTypeVisitor.ERROR_3") + aVariable.getName()); //$NON-NLS-1$
      }
      
      traverse(aVariable);
   }
}
