// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/def/visitors/AeInlinePropertyAliasVisitor.java,v 1.7 2005/10/12 19:26:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.def.visitors;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeWSDLDefHelper;
import org.activebpel.rt.bpel.IAeContextWSDLProvider;
import org.activebpel.rt.bpel.IAeExpressionLanguageFactory;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeBaseDefNamespaceContext;
import org.activebpel.rt.bpel.def.AeCorrelationSetDef;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef;
import org.activebpel.rt.bpel.def.activity.AeActivityReplyDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWaitDef;
import org.activebpel.rt.bpel.def.activity.AeActivityWhileDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignToDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignVarDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.def.activity.support.AeSourceDef;
import org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef;
import org.activebpel.rt.bpel.def.expr.AeExpressionAnalyzerContext;
import org.activebpel.rt.bpel.def.expr.IAeExpressionAnalyzer;
import org.activebpel.rt.bpel.def.expr.IAeExpressionAnalyzerContext;
import org.activebpel.rt.bpel.def.util.AeDefUtil;
import org.activebpel.rt.bpel.def.util.AeVariableProperty;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.wsdl.def.IAeProperty;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;

/**
 * Visits the def to inline usage of property alias's where possible. This will
 * reduce the overhead of having to search for these prop aliases at runtime. 
 */
public class AeInlinePropertyAliasVisitor extends AeAbstractDefVisitor
{
   /** The WSDL provider set during visitor creation. */
   private IAeContextWSDLProvider mWSDLProvider;
   /** The process def that we're visiting */   
   private AeProcessDef mProcessDef;
   /** The expression language factory. */
   private IAeExpressionLanguageFactory mExpressionLanguageFactory;
   
   /**
    * Constructor for the visitor
    * @param aProvider
    * @param aExpressionLanguageFactory
    */
   public AeInlinePropertyAliasVisitor(IAeContextWSDLProvider aProvider, IAeExpressionLanguageFactory aExpressionLanguageFactory)
   {
      mWSDLProvider = aProvider;
      setExpressionLanguageFactory(aExpressionLanguageFactory);
      setTraversalVisitor(new AeTraversalVisitor(new AeDefTraverser(), this));
   }
   
   /**
    * Finds any propertyAliases needed to resolve the correlationSets used in either
    * the input and output variables of an invoke.
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef)
    */
   public void visit(AeActivityInvokeDef aDef)
   {
      AeVariableDef inputVariableDef = AeDefUtil.getVariableByName(aDef.getInputVariable(), aDef);
      AeVariableDef outputVariableDef = null;
      if (!AeUtil.isNullOrEmpty(aDef.getOutputVariable()))
         outputVariableDef = AeDefUtil.getVariableByName(aDef.getOutputVariable(), aDef);
         
      for ( Iterator iter = aDef.getCorrelationList() ; iter.hasNext() ; )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         AeCorrelationSetDef corrSetDef = AeDefUtil.getCorrSetByName( corrDef.getCorrelationSetName(), aDef );
         if ( corrDef.usesOutboundDataForCorrelation())
            walkCorrelationSet( corrSetDef, inputVariableDef);
         if ( corrDef.usesInboundDataForCorrelation())
            walkCorrelationSet( corrSetDef, outputVariableDef);
      }
      
      super.visit(aDef);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReceiveDef)
    */
   public void visit(AeActivityReceiveDef aDef)
   {
      walkCorrelationIterator(aDef, AeDefUtil.getVariableByName(aDef.getVariable(), aDef), aDef.getCorrelationList());
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityReplyDef)
    */
   public void visit(AeActivityReplyDef aDef)
   {
      walkCorrelationIterator(aDef, AeDefUtil.getVariableByName(aDef.getVariable(), aDef), aDef.getCorrelationList());
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef)
    */
   public void visit(AeOnMessageDef aDef)
   {
      walkCorrelationIterator(aDef, AeDefUtil.getVariableByName(aDef.getVariable(), aDef), aDef.getCorrelationList());
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.AeProcessDef)
    */
   public void visit(AeProcessDef aDef)
   {
      mProcessDef = aDef;
      super.visit(aDef);
   }
   
   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef)
    */
   public void visit(AeAssignFromDef aDef)
   {
      visitFromDef(aDef);
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeAssignToDef)
    */
   public void visit(AeAssignToDef aDef)
   {
      visitToDef(aDef);
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityWaitDef)
    */
   public void visit(AeActivityWaitDef aDef)
   {
      extractPropAliasFromExpression(aDef.getFor(), aDef);
      extractPropAliasFromExpression(aDef.getUntil(), aDef);
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.AeActivityWhileDef)
    */
   public void visit(AeActivityWhileDef aDef)
   {
      extractPropAliasFromExpression(aDef.getCondition(), aDef);
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef)
    */
   public void visit(AeOnAlarmDef aDef)
   {
      extractPropAliasFromExpression(aDef.getFor(), aDef);
      extractPropAliasFromExpression(aDef.getUntil(), aDef);
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSourceDef)
    */
   public void visit(AeSourceDef aDef)
   {
      extractPropAliasFromExpression(aDef.getTransitionCondition(), aDef);
      super.visit(aDef);
   }

   /**
    * @see org.activebpel.rt.bpel.def.visitors.IAeDefVisitor#visit(org.activebpel.rt.bpel.def.activity.support.AeSwitchCaseDef)
    */
   public void visit(AeSwitchCaseDef aDef)
   {
      extractPropAliasFromExpression(aDef.getCondition(), aDef);
      super.visit(aDef);
   }

   /**
    * Walks the iter of correlation usages looking for property aliases.
    * @param aDef The activity that's providing the context for this search
    * @param variableDef The variable used in the activity
    * @param correlationIter Iter of correlation defs
    */
   protected void walkCorrelationIterator(AeBaseDef aDef, AeVariableDef aVariableDef, Iterator aCorrelationIter)
   {
      while ( aCorrelationIter.hasNext() )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)aCorrelationIter.next();
         AeCorrelationSetDef corrSetDef = AeDefUtil.getCorrSetByName( corrDef.getCorrelationSetName(), aDef );
         walkCorrelationSet( corrSetDef, aVariableDef);
      }
   }

   /**
    * Finds all of the property aliases for the correlation set's properties and
    * the given variable type.
    * @param aCorrSetDef
    * @param aVariableDef
    */
   protected void walkCorrelationSet(AeCorrelationSetDef aCorrSetDef, AeVariableDef aVariableDef)
   {
      QName messageType = aVariableDef.getMessageType();
      for ( Iterator iter = aCorrSetDef.getPropertiesList() ; iter.hasNext() ; )
      {
         QName propName = (QName)iter.next();
         cachePropertyAlias(messageType, propName);
      }
   }

   /**
    * Caches a property alias on the process def if it's found.
    * @param aMessageType
    * @param aPropName
    */
   protected void cachePropertyAlias(QName aMessageType, QName aPropName)
   {
      if (mProcessDef.getPropertyAlias(aMessageType, aPropName) == null)
      {
         IAePropertyAlias alias = 
            AeWSDLDefHelper.getWSDLMessagePropertyAlias( getWSDLProvider(), 
                                                         aMessageType, 
                                                         aPropName );
         // @todo should have been caught already by static analysis
         if (alias != null) 
         {
            mProcessDef.cachePropertyAlias(alias);
            IAeProperty prop = AeWSDLDefHelper.getProperty(getWSDLProvider(), alias.getPropertyName());
            // Check if property is null, i.e. this property alias is referencing a non-existent property in the WSDL.
            // @todo we should be checking for this in the def validation visitor during static analysis.
            if ( prop != null )
               mProcessDef.cachePropertyType(alias.getPropertyName(), prop.getTypeName());
         }
      }
   }
   
   /**
    * Getter for the wsdl provider.
    */
   protected IAeContextWSDLProvider getWSDLProvider()
   {
      return mWSDLProvider;
   }

   /**
    * Visits the assign var def in order to cache property aliases.
    * 
    * @param aDef
    */
   protected void visitVarDef(AeAssignVarDef aDef)
   {
      if (!AeUtil.isNullOrEmpty(aDef.getVariable()) && !AeUtil.isNullOrEmpty(aDef.getProperty()))
      {
         AeVariableDef varDef = AeDefUtil.getVariableByName(aDef.getVariable(), aDef);
         QName propName = AeDefUtil.parseQName(aDef, aDef.getProperty());
         cachePropertyAlias(varDef.getMessageType(), propName); 
      }
   }

   /**
    * Attempts to cache a property alias if it's used in the variable assignment
    * @param aDef
    */
   protected void visitToDef(AeAssignToDef aDef)
   {
      visitVarDef(aDef);
   }

   /**
    * Attempts to cache a property alias if it's used in the variable assignment
    * @param aDef
    */
   protected void visitFromDef(AeAssignFromDef aDef)
   {
      visitVarDef(aDef);

      if (!AeUtil.isNullOrEmpty(aDef.getExpression()))
      {
         extractPropAliasFromExpression(aDef.getExpression(), aDef);
      }
   }
   
   
   /**
    * Searches for bpws:getVariableProperty usage within the expression and attempts
    * to inline the property alias.
    * @param aExpression
    * @param aDef
    */
   protected void extractPropAliasFromExpression(String aExpression, AeBaseDef aDef)
   {
      if (AeUtil.isNullOrEmpty( aExpression ))
         return;

      try
      {
         String language = AeDefUtil.getProcessDef(aDef).getExpressionLanguage();
         IAeExpressionAnalyzer analyzer = getExpressionLanguageFactory().createExpressionAnalyzer(language);
         IAeExpressionAnalyzerContext ctx = new AeExpressionAnalyzerContext(new AeBaseDefNamespaceContext(aDef));

         for (Iterator iter = analyzer.getVarPropertyList(ctx, aExpression).iterator(); iter.hasNext();)
         {
            AeVariableProperty varProp = (AeVariableProperty) iter.next();
            AeVariableDef varDef = AeDefUtil.getVariableByName(varProp.getVarName(), aDef);
            QName propName = AeDefUtil.parseQName(aDef, varProp.getProperty());
            cachePropertyAlias(varDef.getMessageType(), propName);
         }
      }
      catch (AeException e)
      {
         AeException.logError(e, e.getLocalizedMessage());
      }
   }

   /**
    * @return Returns the expressionLanguageFactory.
    */
   protected IAeExpressionLanguageFactory getExpressionLanguageFactory()
   {
      return mExpressionLanguageFactory;
   }

   /**
    * @param aExpressionLanguageFactory The expressionLanguageFactory to set.
    */
   protected void setExpressionLanguageFactory(IAeExpressionLanguageFactory aExpressionLanguageFactory)
   {
      mExpressionLanguageFactory = aExpressionLanguageFactory;
   }
}
