// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/AeBusinessProcess.java,v 1.126 2006/02/17 01:05:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.AeWSDLDefHelper;
import org.activebpel.rt.bpel.IAeActivity;
import org.activebpel.rt.bpel.IAeBusinessProcess;
import org.activebpel.rt.bpel.IAeContextWSDLProvider;
import org.activebpel.rt.bpel.IAeEndpointReference;
import org.activebpel.rt.bpel.IAeEngineAlert;
import org.activebpel.rt.bpel.IAeEngineEvent;
import org.activebpel.rt.bpel.IAeExpressionLanguageFactory;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.IAeInvokeResponseReceiver;
import org.activebpel.rt.bpel.IAeLocatableObject;
import org.activebpel.rt.bpel.IAePartnerLink;
import org.activebpel.rt.bpel.IAeProcessEvent;
import org.activebpel.rt.bpel.IAeProcessInfoEvent;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.config.IAeEngineConfiguration;
import org.activebpel.rt.bpel.coord.IAeCoordinating;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;
import org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity;
import org.activebpel.rt.bpel.impl.activity.support.AeCompInfo;
import org.activebpel.rt.bpel.impl.activity.support.AeCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.activity.support.AeProcessCompensationCallbackWrapper;
import org.activebpel.rt.bpel.impl.activity.support.AeScopeSnapshot;
import org.activebpel.rt.bpel.impl.activity.support.IAeCompensationCallback;
import org.activebpel.rt.bpel.impl.fastdom.AeDocumentBuilder;
import org.activebpel.rt.bpel.impl.fastdom.AeFastDocument;
import org.activebpel.rt.bpel.impl.fastdom.AeFastElement;
import org.activebpel.rt.bpel.impl.lock.AeNoopVariableLocker;
import org.activebpel.rt.bpel.impl.lock.AeVariableLocker;
import org.activebpel.rt.bpel.impl.lock.IAeVariableLockCallback;
import org.activebpel.rt.bpel.impl.lock.IAeVariableLocker;
import org.activebpel.rt.bpel.impl.queue.AeInboundReceive;
import org.activebpel.rt.bpel.impl.queue.AeInvoke;
import org.activebpel.rt.bpel.impl.queue.AeMessageReceiver;
import org.activebpel.rt.bpel.impl.queue.AeReply;
import org.activebpel.rt.bpel.impl.storage.AeFaultSerializer;
import org.activebpel.rt.bpel.impl.storage.AeProcessSnapshot;
import org.activebpel.rt.bpel.impl.storage.AeRestoreImplState;
import org.activebpel.rt.bpel.impl.storage.IAeProcessSnapshot;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.bpel.impl.visitors.IAeVisitable;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeIntMap;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * Describes the interface used for interacting with business processes
 */
public class AeBusinessProcess extends AeActivityScopeImpl implements IAeBusinessProcess, IAeVariableLockCallback, IAeVisitable
{
   /** The message that started the process. Gets set during startup and then consumed by the start activity. */
   private AeInboundReceive mStartMessage;
   /** The engine we are running within. */
   private IAeBusinessProcessEngineInternal mEngine;
   /** Our instances process id within the engine. */
   private long mProcessId;
   /** Our process state (running, suspended, etc.) */
   private int mProcessState;
   /** Map of source location for the Bpel implementation objects. */
   private Map mBpelObjects = new HashMap();
   /** Map of source locations to process variables. */
   private Map mProcessVariables = new HashMap();
   /** Our map of associated partner links for this process. */
   private Map mPartnerLinks = new HashMap();
   /** Queue used to execute objects. Prevents multiple executes from happening concurrently */
   private final AeExecutionQueue mExecutionQueue = new AeExecutionQueue(this);
   /** The context wsdl provider */
   protected IAeContextWSDLProvider mContextWsdlProvider;
   /** Used to control concurrent access to variables when there are serializable scopes present */
   private IAeVariableLocker mVariableLocker;
   /** The date/time that the process was started */
   private Date mDateStarted = new Date();
   /** The date/time that the process was ended */
   private Date mDateEnded;
   /** process plan */
   protected IAeProcessPlan mProcessPlan;
   /** business process instance properties */
   private HashMap mBusinessProcessProperties;
   /** utility class for handling resume/suspend logic */
   private AeProcessSuspendResumeHandler mProcessAdministrator;
   /** location paths for any activities that are in the faulting state */
   private List mFaultingActivityLocationPaths;
   /** maps location id to location path */
   private AeIntMap mLocationIdToPath = new AeIntMap();
   /** maps location path to location id */
   private Map mLocationPathToId = new HashMap();
   /** max location id used for creating new bpel objects dynamically as in parallel forEach */
   private int mMaxLocationId = -1;
   /** tracks the queued receives locally to check for bpws:conflictingReceive */
   private Set mQueuedReceives = new HashSet();
   /** counter used to store the next version number to use for a variable */
   private int mNextVariableVersionNumber = 1;
   /** Reason code for process state, default to none */
   private int mProcessStateReason = IAeBusinessProcess.PROCESS_REASON_NONE;

   /** maps state change values to their engine event values.  */
   private static final int[] STATE_TO_EVENT_MAPPING =
   {
      // TODO ! we should have these values matched up so there is no xlation necessary
      /*  Inactive  */        IAeProcessEvent.INACTIVE,
      /*  Ready     */        IAeProcessEvent.READY_TO_EXECUTE,
      /*  Executing */        IAeProcessEvent.EXECUTING,
      /*  Finished  */        IAeProcessEvent.EXECUTE_COMPLETE,
      /*  Faulted   */        IAeProcessEvent.EXECUTE_FAULT,
      /*  Dead Path */        IAeProcessEvent.DEAD_PATH_STATUS,
      /*  Queued    */        IAeProcessEvent.INACTIVE, // we're skipping over this
      /*  Terminated */       IAeProcessEvent.TERMINATED,
      /*  Faulting   */       IAeProcessEvent.FAULTING
   };

   /**
    * Construct a new business process from a definition object and associated process id.
    * @param aPid The process id to associate with this process
    * @param aEngine The engine this process will be owned by
    * @param aPlan The process plan.
    */
   protected AeBusinessProcess(long aPid, IAeBusinessProcessEngineInternal aEngine, IAeProcessPlan aPlan)
   {
      super(aPlan.getProcessDef());
      if(aPlan.getProcessDef().containsSerializableScopes())
         mVariableLocker = new AeVariableLocker();
      else
         mVariableLocker = new AeNoopVariableLocker();
      setProcessId(aPid);
      setEngine(aEngine);
      setProcessState(IAeBusinessProcess.PROCESS_LOADED);
      mProcessPlan = aPlan;
      mFaultingActivityLocationPaths = new ArrayList();
      mProcessAdministrator = new AeProcessSuspendResumeHandler( this );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getNextVariableVersionNumber()
    */
   public int getNextVariableVersionNumber()
   {
      return mNextVariableVersionNumber;
   }
   
   /**
    * Setter for the next variable id. This is public for the process restore code.
    * 
    * @param aId
    */
   public void setNextVariableVersionNumber(int aId)
   {
      mNextVariableVersionNumber = aId;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }

   /** get the state of the process */
   public int getProcessState()
   {
      return mProcessState;
   }

   /**
    * The processStateReason to set.
    */
   public void setProcessStateReason(int aProcessStateReason)
   {
      mProcessStateReason = aProcessStateReason;
   }
   
   /**
    * Get the reason code of process state
    */
   public int getProcessStateReason()
   {
      return mProcessStateReason;
   }
   
   /** set the state of the process */
   public void setProcessState(int aState)
   {
      mProcessState = aState;
   }

   /** Helper method to check if process is suspended. */
   public boolean isSuspended()
   {
      return getProcessState() == PROCESS_SUSPENDED;
   }
   
   /** Helper method to check if process is running. */
   public boolean isRunning()
   {
      return getProcessState() == PROCESS_RUNNING;
   }

   /** Adds a partner link to the process. */
   public void addPartnerLink(AePartnerLink aLink)
   {
      mPartnerLinks.put(aLink.getName(), aLink);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getProcessDefinition()
    */
   public AeProcessDef getProcessDefinition()
   {
      return (AeProcessDef) getDefinition();
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getProcessId()
    */
   public long getProcessId()
   {
      return mProcessId;
   }

   /**
    * Setter for the process id
    * @param aProcessId
    */
   protected void setProcessId(long aProcessId)
   {
      mProcessId = aProcessId;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      setProcessState(PROCESS_RUNNING);
      getEngine().fireEngineEvent(new AeEngineEvent(getProcessId(), IAeEngineEvent.PROCESS_STARTED, getName()));

      // events will get started after the start message has been consumed.
      getProcess().queueObjectToExecute(getActivity());
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getEngine()
    */
   public IAeBusinessProcessEngineInternal getEngine()
   {
      return mEngine;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setEngine(org.activebpel.rt.bpel.impl.IAeBusinessProcessEngineInternal)
    */
   public void setEngine(IAeBusinessProcessEngineInternal aEngine)
   {
      mEngine = aEngine;
   }

   /**
    * Getter for the owning process, return this.
    */
   public IAeBusinessProcess getProcess()
   {
      return this;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getPartnerLink(java.lang.String)
    */
   public AePartnerLink getPartnerLink(String aName)
   {
      return (AePartnerLink)mPartnerLinks.get(aName);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#queueObjectToExecute(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void queueObjectToExecute(IAeBpelObject aObject) throws AeBusinessProcessException
   {
      aObject.setState(AeBpelState.QUEUED_BY_PARENT);
      if(aObject.isReadyToExecute())
      {
         readyToExecuteObject(aObject);
      }
   }
   
   /**
    * This method allows a runnable object to be queued for execution.
    * @param aObject 
    */
   public void queueObjectToExecute(Runnable aRunnable) throws AeBusinessProcessException
   {
      if (aRunnable != null)
      {
         getExecutionQueue().addRunnable(aRunnable);
      }
   }
      
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#linkStatusChanged(org.activebpel.rt.bpel.impl.IAeLink)
    */
   public void linkStatusChanged(IAeLink aLink)
   {
      // fire link status change event
      String linkPath = aLink.getLocationPath();
      fireEvent(linkPath, IAeProcessEvent.LINK_STATUS, Boolean.toString(aLink.getStatus()));

      // process target activity
      IAeActivity activity = aLink.getTargetActivity();
      if( activity.isReadyToExecute())
      {
         try
         {
            readyToExecuteObject(activity);
         }
         catch(AeBpelException e)
         {
            propagateFault(activity, e.getFault());
         }
         catch(AeBusinessProcessException bpe)
         {
            AeException.logError(bpe, bpe.getLocalizedMessage());
            String msg = MessageFormat.format(AeMessages.getString("AeBusinessProcess.ERROR_EXECUTING_ACTIVITY_ERROR"),  //$NON-NLS-1$
                  new Object [] { activity.getLocationPath(), bpe.getLocalizedMessage() });
            IAeFault fault = AeFaultFactory.getSystemErrorFault(bpe, msg);
            propagateFault(activity, fault);
         }
         catch(Throwable ex)
         {
            // TODO should we not catch everything, for example maybe throwable,
            // i.e. non-planned bpel exceptions  should suspend the process
            // rather than fault
         }
      }
   }

   /**
    * Propagates the fault to the object passed in. Broke this out from
    * linkStatusChanged to avoid having the ugliness of a nested try/catch block.
    * An object that says it's ready to execute but throws a fault
    * @param aBpelObject
    * @param aFault
    */
   private void propagateFault(IAeBpelObject aBpelObject, IAeFault aFault)
   {
      try
      {
         objectCompletedWithFault(aBpelObject, aFault);
      }
      catch (AeBusinessProcessException e)
      {
         e.logError();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#isSuppressJoinConditionFailure()
    */
   public boolean isSuppressJoinConditionFailure()
   {
      AeProcessDef def = (AeProcessDef) getDefinition();
      return def.getSuppressJoinFailure();
   }

   /**
    * If the passed object is not associated with a dead path it will execute
    * the bpel object passed after firing the ready to execute event.
    * If after the event the process is suspened the activity is queued otherwise
    * it is passed to executObject method.
    * @throws AeBusinessProcessException - either a fatal error or an unsuppressed join failure
    */
   protected void readyToExecuteObject(IAeBpelObject aObject) throws AeBusinessProcessException
   {
      if(aObject.isNotDeadPath())
      {
            aObject.setState(AeBpelState.READY_TO_EXECUTE);
            executeObject(aObject);
      }
      else
      {
         // we may be in a terminated state, don't want to move from dead path to terminated
         aObject.setState(AeBpelState.DEAD_PATH);
      }
   }

   /**
    * Getter for the execution queue.
    */
   protected AeExecutionQueue getExecutionQueue()
   {
      return mExecutionQueue;
   }

   /**
    * We're only going to execute one object at a time. This will simplify our
    * execution by avoiding recursive calls to execute and hopefully avoid
    * any subtle bugs with the ordering of event firing and state changes.
    */
   protected void executeObject(IAeBpelObject aObject) throws AeBusinessProcessException
   {
      // add the object to our execution queue. If the queue is being executed, the
      // our addition will be queued, otherwise it'll execute immediately
      getExecutionQueue().add((IAeExecutableBpelObject) aObject);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#suspend()
    */
   public void suspend(AeSuspendReason aReason) throws AeBusinessProcessException
   {
      if (aReason == null || aReason.getReasonCode() == AeSuspendReason.SUSPEND_CODE_MANUAL)
      {
         setProcessStateReason(AeSuspendReason.SUSPEND_CODE_MANUAL);
         getProcessSuspendResumeHandler().suspend();
      }
      else
      {
         if (aReason.getReasonCode() == AeSuspendReason.SUSPEND_CODE_AUTOMATIC)
         {
            setProcessStateReason(AeSuspendReason.SUSPEND_CODE_AUTOMATIC);
            IAeFault uncaughtFault = ((AeAbstractBpelObject)findBpelObjectOrThrow(aReason.getLocationPath())).getFault(); 
            getProcessSuspendResumeHandler().suspendBecauseOfUncaughtFault(aReason.getLocationPath(), uncaughtFault );
            alertFaulting(aReason.getLocationPath(), uncaughtFault);
         }
         else if (aReason.getReasonCode() == AeSuspendReason.SUSPEND_CODE_LOGICAL)
         {
            setProcessStateReason(AeSuspendReason.SUSPEND_CODE_LOGICAL);
            getProcessSuspendResumeHandler().suspend();
            AeEngineAlert alert = createAlert(IAeEngineAlert.PROCESS_ALERT_SUSPENDED, aReason.getLocationPath(), aReason.getVariable());
            getEngine().fireEngineAlert(alert);
            
         }
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#resume(boolean)
    */
   public void resume(boolean aExecute) throws AeBusinessProcessException
   {
      getProcessSuspendResumeHandler().resume( aExecute );
   }
   
   /**
    * Check the suspended execution queue and resume the object represented
    * by the passed path.
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#resume(java.lang.String)
    */
   public void resume(String aLocation) throws AeBusinessProcessException
   {
      getProcessSuspendResumeHandler().resume( aLocation );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#retryActivity(java.lang.String, boolean)
    */
   public void retryActivity( String aLocation, boolean aAtScope ) throws AeBusinessProcessException
   {
      getProcessSuspendResumeHandler().retryActivity( aLocation, aAtScope );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#completeActivity(java.lang.String)
    */
   public void completeActivity(String aLocation) throws AeBusinessProcessException
   {
      getProcessSuspendResumeHandler().completeActivity( aLocation );
   }

   /**
    * Note: Bpel activities shouldn't call this method directly. Doing so will
    * bypass the behavior in the base class that handles the link processing.
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#objectCompleted(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void objectCompleted(IAeBpelObject aObject) throws AeBusinessProcessException
   {
      if (aObject == this)
      {
         processEnded(null);
      }
      else
      {
         // if dead path we don't want to declare as finished in a normal way
         if (! aObject.getState().isFinal())
         {
            aObject.setState(AeBpelState.FINISHED);
         }
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#objectCompletedWithFault(org.activebpel.rt.bpel.impl.IAeBpelObject, org.activebpel.rt.bpel.IAeFault)
    */
   public void objectCompletedWithFault(IAeBpelObject aObject, IAeFault aFaultObject) throws AeBusinessProcessException
   {  
      if(aObject != this && !aObject.getParent().getState().isFinal())
      {
         ((IAeExecutableBpelObject)aObject.getParent()).childIsFaulting(aObject, aFaultObject);
      }
      
      aObject.setFaultedState(aFaultObject);

      if ( aFaultObject != null && aFaultObject.getSource() == null )
      {
         aFaultObject.setSource( aObject );
      }

      if(aObject != this && !aObject.getParent().getState().isFinal())
      {
         ((IAeExecutableBpelObject)aObject.getParent()).childCompleteWithFault(aObject, aFaultObject);
      }

      if (aObject == this)
      {
         processEnded(aFaultObject);
      }
   }

   /** 
    * Overrides method to so that the compInfo is not cleared at the process level. 
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl#clearCompInfo()
    */
   protected void clearCompInfo()
   {
     // The compInfo for compensation at the process (marked for enableInstanceCompensation) level 
     // should not be cleared.      
   }
   
   /** 
    * Overrides method to return <code>AeCompInfo</code> and creates the variable/correlationSet
    * snapshot for the data at the process level (needed during process instance compensation).
    * @see org.activebpel.rt.bpel.impl.IAeCompensatableActivity#getCompInfo()
    */
   public AeCompInfo getCompInfo()
   {
      AeCompInfo compInfo = super.getCompInfo();
      // check if a snapshot is created -  if not create it.
      if (compInfo.getSnapshot() == null)
      {
         // Note: calling AeCompInfo::recordSnapshot() creates a new snapshot.
         compInfo.recordSnapshot();
      }
      return compInfo;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#compensate(org.activebpel.rt.bpel.impl.activity.support.IAeCompensationCallback)
    */
   public void compensate(IAeCompensationCallback aCallback) throws AeBusinessProcessException
   {
      if (getProcessState() != IAeBusinessProcess.PROCESS_COMPENSATABLE)
      {
         throw new AeBusinessProcessException(AeMessages.format("AeBusinessProcess.NotCompensatable", String.valueOf(getProcessId()))); //$NON-NLS-1$
      }
      
      setProcessState(IAeBusinessProcess.PROCESS_RUNNING);
      
      getExecutionQueue().resume(true);

      AeCompensationHandler compHandler = getCompensationHandler();
      // use a wrapper for the callback to ensure that our state changes to complete after compensation is done.
      compHandler.setCallback(new AeProcessCompensationCallbackWrapper(aCallback));
      compHandler.setCompInfo(getCompInfo());
      
      AeProcessInfoEvent event = new AeProcessInfoEvent(getProcessId(), getLocationPath(), IAeProcessInfoEvent.INFO_PROCESS_COMPENSATION_STARTED);
      getEngine().fireInfoEvent(event);
      queueObjectToExecute(compHandler);
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#releaseCompensationResources()
    */
   public void releaseCompensationResources()
   {
      if (getProcessState() == IAeBusinessProcess.PROCESS_COMPENSATABLE)
      {
         // TODO (MF) optimization here to trim any remaining comp info objects from state.
         setProcessState(IAeBusinessProcess.PROCESS_COMPLETE);
      }
   }

   /** 
    * Overrides method to add Process level variable and correlationSet data by reference.
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl#recordScopeSnapshot(org.activebpel.rt.bpel.impl.activity.support.AeScopeSnapshot)
    */
   public void recordScopeSnapshot(AeScopeSnapshot aSnapshot)
   {
      // add the process's variable and correlation data by reference. 
      // (the scope's impl. adds the data as a copy/clone is it needs to maintain a real snapshot).
      aSnapshot.addVariables( getVariablesMap() );
      aSnapshot.addCorrelationSet( getCorrelationSetMap() );
   }
   
   /**
    * Overload method for engine events with no fault to report.
    */
   protected void fireEvent(String aPath, int aEventId, String aOtherInfo)
   {
      AeProcessEvent event = new AeProcessEvent(getProcessId(), aPath, aEventId, "", aOtherInfo, getName()); //$NON-NLS-1$
      getEngine().fireEvent(event);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#isReceiveDataQueued(java.lang.String, javax.xml.namespace.QName, java.lang.String)
    */
   public boolean isReceiveDataQueued(String aPartnerLinkName, QName aPortType, String aOperation)
   {
      if (mStartMessage != null)
      {
         return AeUtil.compareObjects(aPartnerLinkName, mStartMessage.getPartnerLinkName()) &&
         AeUtil.compareObjects(aPortType, mStartMessage.getPortType()) &&
         AeUtil.compareObjects(aOperation, mStartMessage.getOperation());
      }
      return false;
   }

   /**
    * Report engine events - includes fault parameter.
    */
   protected void fireEvent(String aPath, int aEventId, String aFault, String aOtherInfo)
   {
      AeProcessEvent event = new AeProcessEvent(getProcessId(), aPath, aEventId, aFault, aOtherInfo, getName());
      getEngine().fireEvent(event);
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#queueMessageReceiver(org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity, int, java.lang.String)
    */
   public void queueMessageReceiver(IAeMessageReceiverActivity aMessageReceiver,
         int aGroupId, String aMessageExchange)
         throws AeBusinessProcessException
   {
      Map correlation = new HashMap();
      Set correlationSetPaths = new HashSet();
      getCorrelationData(aMessageReceiver, correlationSetPaths, correlation);
      
      int receiverPathId = getLocationId(aMessageReceiver.getLocationPath());
      AeMessageReceiver messageQueueObject = new AeMessageReceiver(getProcessId(), getName(),
            aMessageReceiver.getPartnerLink().getName(), aMessageReceiver.getPortType(), 
            aMessageReceiver.getOperation(), correlation, receiverPathId, aGroupId, aMessageExchange);

       // If the start message is still present, then the executing activity MUST be
       // a create instance activity (although it doesn't necessarily have to match against
       // the start message).
      if (mStartMessage != null)
      {
         // If it's not a create instance, then throw an exception because we're dealing with bad bpel
         if (!aMessageReceiver.canCreateInstance())
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeBusinessProcess.ERROR_1")); //$NON-NLS-1$
         }
         // check to see if it matches the start message.
         // it should match because we don't allow other receives in a multi-start
         // bpel to queue unless they're the start activity
         else if (messageQueueObject.matches(mStartMessage))
         {
            // it's safe to start the events running now since they won't execute
            // until after the receive is done executing.
            executeEventHandler();

            // mStartMessage will get set to null during the callback in checkIfCreateInstance()
            IAeMessageData messageData = mStartMessage.getMessageData();
            mStartMessage = null;
            aMessageReceiver.onMessage(messageData);
         }
         // We should never get here since the multi-start code avoids executing the activity
         // unless it's the one that created the process. To reach here indicates that the
         // multi-start code failed somehow. This is bad because the activity would enter the
         // queue without its correlation data set and would likely never get matched.
         else
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeBusinessProcess.ERROR_2")); //$NON-NLS-1$
         }
      }
      else
      {
         addReceiverKeyForConflictingReceives(aMessageReceiver, correlationSetPaths);
         getEngine().addMessageReceiver(messageQueueObject);
      }
   }

   /**
    * Walks the message receiver populating the set with correlation paths that are
    * referenced by the message receiver as well as populating the Map with the
    * correlation values. 
    * 
    * @param aMessageReceiver The receive or onMessage activity
    * @param aSet - a non-null set that will be populated with correlation set paths
    * @param aCorrelation - optional map to store the correlation values
    * @throws AeBusinessProcessException
    */
   protected void getCorrelationData(IAeMessageReceiverActivity aMessageReceiver, Set aCorrelationSetPaths, Map aCorrelation) throws AeBusinessProcessException
   {
      for(Iterator iter = aMessageReceiver.getCorrelationList(); iter.hasNext(); )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         AeCorrelationSet corrSet = aMessageReceiver.getCorrelationSet(corrDef.getCorrelationSetName());
         aCorrelationSetPaths.add(corrSet.getLocationPath());
         if (aCorrelation != null)
         {
            if(corrSet.isInitialized() || corrDef.shouldBeInitialized() )
            {
               aCorrelation.putAll(corrSet.getPropertyValues());            
            }
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dequeueMessageReceiver(org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity)
    */
   public void dequeueMessageReceiver(IAeMessageReceiverActivity aMessageReceiver) throws AeBusinessProcessException
   {
      removeReceiverKeyForConflictingReceives(aMessageReceiver);
      
      int locationPathId = getLocationId(aMessageReceiver.getLocationPath());
      getEngine().removeMessageReceiver(getProcessId(), locationPathId);
   }

   /**
    * Queues an alarm through the engine.
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#queueAlarm(org.activebpel.rt.bpel.impl.IAeAlarmReceiver, java.util.Date)
    */
   public void queueAlarm(IAeAlarmReceiver aAlarm, Date aDeadline) throws AeBusinessProcessException
   {
      getEngine().scheduleAlarm(getProcessId(), aAlarm.getLocationId(), aAlarm.getGroupId(), aDeadline);
   }

   /**
    * Dequeues an alarm set in the engine.
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dequeueAlarm(org.activebpel.rt.bpel.impl.IAeAlarmReceiver)
    */
   public void dequeueAlarm(IAeAlarmReceiver aAlarm) throws AeBusinessProcessException
   {
      getEngine().removeAlarm(getProcessId(), aAlarm.getLocationId());
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#queueReply(org.activebpel.rt.message.IAeMessageData, javax.xml.namespace.QName, java.lang.String, javax.xml.namespace.QName, java.lang.String, java.lang.String, java.util.Map)
    */
   // TODO (MF) should remove port type from this method and the underlying AeReply
   public void queueReply(IAeMessageData aInputMessage, QName aFaultName, String aPartnerLinkName,
                           QName aPortType, String aOperation, String aMessageExchange, Map aCorrelationMap)
                           throws AeBusinessProcessException
   {
      AeReply replyObject = new AeReply(getProcessId(), aPartnerLinkName, aPortType, aOperation, aMessageExchange);
      Map propertiesSnapshot = cloneBusinessProcessProperties();
      getEngine().sendReply(getProcessId(), replyObject, aInputMessage, aFaultName, propertiesSnapshot);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#queueInvoke(org.activebpel.rt.bpel.IAeInvokeResponseReceiver, org.activebpel.rt.message.IAeMessageData, java.lang.String, javax.xml.namespace.QName, java.lang.String)
    */
   public void queueInvoke(IAeInvokeResponseReceiver aInvoke, IAeMessageData aInputMessage,
         String aPartnerLinkName, QName aPortType, String aOperation) throws AeBusinessProcessException
   {
      AeInvoke messageQueueObject = createInvokeQueueObject(aInvoke, aInputMessage, aPartnerLinkName, aPortType, aOperation);
      getEngine().addInvoke(getProcessPlan(), messageQueueObject);
   }

   /**
    * Creates an invoke queue object that will be added to the queue manager.
    * 
    * @param aInvoke
    * @param aInputMessage
    * @param aPartnerLinkName
    * @param aPortType
    * @param aOperation
    * @throws AeBusinessProcessException
    */
   protected AeInvoke createInvokeQueueObject(IAeInvokeResponseReceiver aInvoke, IAeMessageData aInputMessage, String aPartnerLinkName, QName aPortType, String aOperation) throws AeBusinessProcessException
   {
      IAePartnerLink plink = getPartnerLink(aPartnerLinkName);
      Map propertiesSnapshot = cloneBusinessProcessProperties();

      AeInvoke messageQueueObject =
            new AeInvoke(
               getProcessId(),
               getName(),
               plink,
               aPortType,
               aOperation,
               aInputMessage,
               aInvoke,
               propertiesSnapshot );

      messageQueueObject.setOneWay(aInvoke.isOneWay());
      messageQueueObject.setResponseReceiver(aInvoke);
      return messageQueueObject;
   }

   /**
    * Shallow clone of the properties map.
    */
   public Map cloneBusinessProcessProperties()
   {
      // shallow copy of properties added to invoke layer
      return (HashMap)getBusinessProcessPropertiesMap().clone();
   }

   /**
    * Adds a reference for the object at the given location path
    * @param aLocationPath an XPath location string of the object
    * @param aObject the Bpel implementation object
    */
   public void addBpelObject(String aLocationPath, IAeBpelObject aObject)
   {
      if (aLocationPath != null && aObject != null)
      {
         mBpelObjects.put(aLocationPath, aObject);

         addMappings(aObject);
      }
   }

   /**
    * Records the mapping of the location path to the location id if this object
    * has a custom location path set on it. In this case, we can't rely on the
    * definition to provide the mapping.
    * @param aObject
    */
   protected void addMappings(IAeLocatableObject aObject)
   {
      if (aObject.hasCustomLocationPath())
      {
         if (aObject.getLocationId() != -1)
         {
            mLocationIdToPath.put(aObject.getLocationId(), aObject.getLocationPath());
            mLocationPathToId.put(aObject.getLocationPath(), new Integer(aObject.getLocationId()));
         }
      }
   }

   /**
    * Adds a reference for the variable at the given location path
    * @param aLocationPath an XPath location string of the object
    * @param aVar the process variable
    */
   public void addVariableMapping(String aLocationPath, IAeVariable aVar )
   {
      if (aLocationPath != null && aVar != null)
      {
         mProcessVariables.put(aLocationPath, aVar);
         addMappings(aVar);
      }
   }

   /**
    * Returns the Bpel implementation object specified by the location path string.
    * @param aLocationPath the XPath location of the Bpel object
    * @return the Bpel impl object or null if not found
    */
   public IAeBpelObject findBpelObject(String aLocationPath)
   {
      return (IAeBpelObject)mBpelObjects.get(aLocationPath);
   }
   
   /**
    * Finds the bpel object by path or throws an exception
    * @param aLocationPath
    * @throws AeBusinessProcessException
    */
   public IAeBpelObject findBpelObjectOrThrow(String aLocationPath) throws AeBusinessProcessException
   {
      IAeBpelObject obj = findBpelObject(aLocationPath);
      if (obj == null)
      {
         Object[] errMsgParams = { aLocationPath, getName().toString(), String.valueOf( getProcessId() ) }; 
         throw new AeBusinessProcessException(AeMessages.format("AeBusinessProcess.ERROR_LOCATING_BPEL_OBJECT", errMsgParams)); //$NON-NLS-1$
      }
      return obj;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#findBpelObject(int)
    */
   public IAeBpelObject findBpelObject(int aLocationId)
   {
      String locationPath = getLocationPath(aLocationId);
      return findBpelObject(locationPath);
   }

   /**
    * Returns the variable specified by the location path string.
    * @param aLocationPath the XPath location of the Bpel object
    * @return the variable impl or null if not found
    */
   public IAeVariable findProcessVariable(String aLocationPath)
   {
      return (IAeVariable)mProcessVariables.get(aLocationPath);
   }

   /**
    * Returns the data for the variable being referenced in the activity location path.
    * An optional part may be specified when dealing with a message type variable.
    * @param aActivityLoc the location path for the activity which references this variable
    * @param aVarName the name of the variable being referenced
    * @param aPart the message part we want data for or null if not message type variable
    * @return the data for the given variable which will a String for simple types,
    *          an Element for Element types and a Document for message types.
    */
   public Object getVariableData(String aActivityLoc, String aVarName, String aPart)
   {
      Object data = null;

      IAeBpelObject activity = findBpelObject(aActivityLoc);
      if (activity != null)
      {
         IAeVariable var = activity.getVariable(aVarName);
         if (var != null)
         {
            if (var.getTypeData() != null)
               data = var.getTypeData();
            else if (var.getElementData() != null)
               data = var.getElementData();
            else if (var.hasMessageData())
            {
               try
               {
                  data = var.getMessageData().getData(aPart);
               }
               catch (AeBusinessProcessException e)
               {
                  // we should NEVER get here because we're checking hasMessageData()
                  // above to ensure that the variable has been set.
                  throw new InternalError(e.getMessage());
               }
            }
         }
      }
      return data;
   }
   
   /**
    * Fires an event that the object's status is changing. If the object is a
    * dead path then we'll notify the parent. This is really only for Sequence
    * which needs to move onto the next child in the sequence if one reaches a
    * dead path.
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#objectStateChanged(org.activebpel.rt.bpel.impl.IAeBpelObject, org.activebpel.rt.bpel.impl.AeBpelState, org.activebpel.rt.bpel.impl.IAeStateChangeDetail)
    */
   public void objectStateChanged(IAeBpelObject aObject, AeBpelState aOldState, IAeStateChangeDetail aDetailsObject) throws AeBusinessProcessException
   {
      if (aObject.getState() != AeBpelState.QUEUED_BY_PARENT)
      {
         int code = STATE_TO_EVENT_MAPPING[aObject.getState().getCode()];
         fireEvent(aObject.getLocationPath(), code, aDetailsObject.getFaultName(), aDetailsObject.getAdditionalInfo());
      }

      boolean objectIsBeingRetried = isBeingRetried(aObject);
      
      // Special handling of an object that is being retried from the Exception Management API
      if (aObject.getState() == AeBpelState.TERMINATED && objectIsBeingRetried)
      {
         getProcessSuspendResumeHandler().terminationComplete((AeAbstractBpelObject) aObject);
      }
      else
      {
         // if the object is in a final state then we release its locks
         if (aObject.getState().isFinal())
         {
            releaseLocks(aObject.getLocationPath());
         }
   
         /*
          * todo Sequence and Flow both need to know about a child activity going dead
          * path. The other container activities don't care. In fact, some of the
          * other containers require a special check in order to prevent them
          * from misinterpreting this childComplete as a normal child completion
          * vs dead path. Perhaps we're back to considering adding a childCompleteWithDeadPath()
          * call ?
          */
         if (aObject.getState() == AeBpelState.DEAD_PATH && !aObject.getParent().getState().isFinal() && !aObject.getParent().isTerminating())
         {
            ((IAeExecutableBpelObject)aObject.getParent()).childComplete(aObject);
         }
   
         // if the termination is the result of a retry, we don't
         // need to notify the parent - just retry the terminated activity
         if( aObject.getState() == AeBpelState.TERMINATED )
         {
            if (aObject.getParent() != null && !aObject.getParent().getState().isFinal())
            {
               ((IAeExecutableBpelObject)aObject.getParent()).childTerminated(aObject);
            }
         }
      }
   }
   
   /**
    * Return true if we DO NOT need to notify the parent of a terminate state change
    * (because of activity retry).
    * @param aObject
    */
   protected boolean isBeingRetried( IAeBpelObject aObject )
   {
      // was checking for isSuspended but needed to allow the process to resume in order to execute compensation handlers for scopes being retried
      return getFaultingActivityLocationPaths().contains( aObject.getLocationPath() );
   }
   
   /**
    * Overriding setState to catch our transition to terminated. 
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl#setState(org.activebpel.rt.bpel.impl.AeBpelState)
    */
   public void setState(AeBpelState aNewState) throws AeBusinessProcessException
   {
      if (aNewState == AeBpelState.TERMINATED)
      {
         processTerminated();
      }
      else
      {
         super.setState(aNewState);
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setCreateMessage(org.activebpel.rt.bpel.impl.queue.AeInboundReceive)
    */
   public void setCreateMessage(AeInboundReceive aReceiveQueueObject)
   {
      mStartMessage = aReceiveQueueObject;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getWSDLDefinition(javax.xml.namespace.QName)
    */
   public AeBPELExtendedWSDLDef getWSDLDefinition(QName aName) throws AeBusinessProcessException
   {
      return AeWSDLDefHelper.getWSDLDefinitionForMsg(mContextWsdlProvider , aName);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getMessageProperty(javax.xml.namespace.QName, javax.xml.namespace.QName)
    */
   public IAePropertyAlias getMessageProperty(QName aMsgType, QName aMsgProp) throws AeBusinessProcessException
   {
      AeProcessDef def = (AeProcessDef) getDefinition();
      return def.getPropertyAlias(aMsgType, aMsgProp);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setContextWSDLProvider(org.activebpel.rt.bpel.IAeContextWSDLProvider)
    */
   public void setContextWSDLProvider( IAeContextWSDLProvider aProvider )
   {
      mContextWsdlProvider = aProvider;
   }

   /**
    * Returns the WSDL definition given the message name or null if not found.
    * @param aDef The process definition
    * @param aMsgName the message name we are interested in.
    * @throws AeBusinessProcessException
    */
   public AeBPELExtendedWSDLDef getWSDLDefinition(AeProcessDef aDef, QName aMsgName) throws AeBusinessProcessException
   {
      if (mContextWsdlProvider == null)
         return null;

      return AeWSDLDefHelper.getWSDLDefinitionForMsg(mContextWsdlProvider , aMsgName);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.lock.IAeVariableLockCallback#variableLocksAcquired(java.lang.String)
    */
   public void variableLocksAcquired(String aOwnerPath) throws AeBusinessProcessException
   {
      readyToExecuteObject(findBpelObjectOrThrow(aOwnerPath));
   }

   /**
    * Getter for the variable locker.
    */
   protected IAeVariableLocker getVariableLocker()
   {
      return mVariableLocker;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#addExclusiveLock(java.util.Set, java.lang.String)
    */
   public boolean addExclusiveLock(Set aSetOfVariablePaths, String aOwnerXPath)
   {
      return getVariableLocker().addExclusiveLock(aSetOfVariablePaths, aOwnerXPath, this);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#addSharedLock(java.util.Set, java.lang.String)
    */
   public boolean addSharedLock(Set aSetOfVariablePaths, String aOwnerXPath)
   {
      return getVariableLocker().addSharedLock(aSetOfVariablePaths, aOwnerXPath, this);
   }

   /**
    * Releases any variable locks that were acquired with this location path.
    * @param aOwnerXPath
    */
   private void releaseLocks(String aOwnerXPath) throws AeBusinessProcessException
   {
      getVariableLocker().releaseLocks(aOwnerXPath);
   }

   /**
    * Overridden as a side effect of extending scope. The process never needs to
    * lock variables, that's left to the individual activities to worry about.
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityImpl#acquireVariableLocks()
    */
   protected boolean acquireVariableLocks()
   {
      return true;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getStartDate()
    */
   public Date getStartDate()
   {
      return mDateStarted;
   }

   /**
    * Handles any processing that needs to happen as a result of the process ending.
    */
   protected void processEnded(IAeFault aFault) throws AeBusinessProcessException
   {
      // TODO (MF) This code is run once when the process completes its normal execution. Should revist this to account for the process completing from its compensation handler.
      if (aFault == null)
      {
         setState(AeBpelState.FINISHED);
      }
      else
      {
         setFault(aFault);
      }
      mDateEnded = new Date();
      if (getFault() != null)
      {
         setProcessState(IAeBusinessProcess.PROCESS_FAULTED);
      }
      else if (isNormalCompletion() && isCoordinated() && getCompensationHandler().getState() == AeBpelState.INACTIVE)
      {
         // if it's a sub process and it is compensatable, then record that as the process state. This process will
         // remain compensatable until it gets compensated or until the parent process completes normally.
         setProcessState(IAeBusinessProcess.PROCESS_COMPENSATABLE);
      }
      else
      {
         setProcessState(IAeBusinessProcess.PROCESS_COMPLETE);
      }
      getEngine().fireEngineEvent(new AeEngineEvent(getProcessId(), IAeEngineEvent.PROCESS_TERMINATED, getName()));
      getEngine().getEngineCallback().processEnded(this);
   }
   
   /**
    * Returns true if the process properties contain a coordination id
    */
   protected boolean isCoordinated()
   {
      return AeUtil.notNullOrEmpty(getBusinessProcessProperty(IAeCoordinating.WSCOORD_ID));
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getEndDate()
    */
   public Date getEndDate()
   {
      return mDateEnded;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getName()
    */
   public QName getName()
   {
      AeProcessDef def = (AeProcessDef) getDefinition();
      return def.getQName();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#terminate()
    */
   public void terminate() throws AeBusinessProcessException
   {
      // Clear any faulting locations to avoid process interpreting the termination of a faulting activity as a signal to retry the activity
      getFaultingActivityLocationPaths().clear();      
      
      // terminate process iff process's bpel state is *not* final. Otherwise, there maybe
      // race conditions where the process has completed and unwinding when the termination  is
      // applied.
      if ( getState().isTerminatable() )
      {
         setTerminating(true);
         terminateNextChild();
         // in case we were suspended when asked to terminate
         resume(true);
      }
      else if ( getState().isFinal() )
      {
         // Coordinated Subprocess's instance compensation:
         // If this activity was executed in a process level compensation handler (process instance compensation),
         // the process will not terminate since at this point the process has already completed. (i.e. process
         // is not terminatable. The thread then unwinds - but with out the AeCompensationHandler object/child
         // completing. Hence, the compHandler callback does not get notified and therefore the coordinator
         // will not get the call back that the subprocess is done with the instance compensation.
         // At this point, we should check if a process instance compensantion handler is running,
         // and if so, fault it so that that call back is invoked.
         AeCompensationHandler compHandler = getCompensationHandler();
         if (compHandler != null && compHandler.getCallback() != null && compHandler.getState().isTerminatable())
         {
			 compHandler.terminate();        
         }
      }      
   }
   
   /** 
    * Overrides method to 
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#cancelProcess()
    */
   public void cancelProcess() throws AeBusinessProcessException
   {
   	// todo: add new fault specific to cancellation e.g: AE_PROCESS_CANCELED
      handleFault(AeFaultFactory.getFault(AeFaultFactory.FORCED_TERMINATION));
   }

   /**
    * Serializes the process state to an instance of <code>AeFastDocument</code>.
    *
    * @param aForPersistence <code>true</code> to serialize for persistence.
    * @return AeFastDocument
    * @throws AeBusinessProcessException
    */
   public AeFastDocument fastSerializeState(boolean aForPersistence) throws AeBusinessProcessException
   {
      AeProcessImplState serializer = new AeProcessImplState();
      serializer.setForPersistence(aForPersistence);
      serializer.setProcess(this);
      return serializer.getProcessDocument();
   }

   /**
    * Serializes the specified variable to an instance of <code>AeFastDocument</code>.
    *
    * @return AeFastDocument
    * @throws AeBusinessProcessException
    */
   public AeFastDocument fastSerializeVariable(IAeVariable aVariable) throws AeBusinessProcessException
   {
      AeVariableSerializer serializer = new AeVariableSerializer(getEngine().getTypeMapping());
      serializer.setVariable(aVariable);
      return serializer.getVariableDocument();
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#serializeState(boolean)
    */
   public Document serializeState(boolean aForPersistence) throws AeBusinessProcessException
   {
      AeFastDocument document = fastSerializeState(aForPersistence);
      return new AeDocumentBuilder().build(document);
   }

   /**
    * Returns an XML <code>Document</code> that represents the specified variable.
    *
    * @param aVariable
    * @throws AeBusinessProcessException
    */
   public Document serializeVariable(IAeVariable aVariable) throws AeBusinessProcessException
   {
      AeFastDocument document = fastSerializeVariable(aVariable);
      return new AeDocumentBuilder().build(document);
   }
   
   /**
    * Serializes the specified fault to an instance of <code>AeFastElement</code>.
    * @param aFault
    * @throws AeBusinessProcessException
    */
   public AeFastElement fastSerializeFault(IAeFault aFault) throws AeBusinessProcessException
   {
      AeFaultSerializer serializer = new AeFaultSerializer();
      serializer.setFault(aFault);
      serializer.setTypeMapping(getEngine().getTypeMapping());

      AeFastElement faultElement = serializer.getFaultElement();
      return faultElement;
   }
   
   /**
    * Serializes the specified variable to an instance of <code>AeFastElement</code>.
    * @param aFault
    * @throws AeBusinessProcessException
    */
   protected Document serializeFault(IAeFault aFault) throws AeBusinessProcessException
   {
      AeFastElement element = fastSerializeFault(aFault);
      AeFastDocument doc = new AeFastDocument(element);
      return new AeDocumentBuilder().build(doc);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#serializeVariable(java.lang.String)
    */
   public Document serializeVariable(String aLocationPath) throws AeBusinessProcessException
   {
      IAeVariable variable = findProcessVariable(aLocationPath);

      if ( variable == null)
         throw new AeBusinessProcessException(
            MessageFormat.format(AeMessages.getString("AeBusinessProcess.ERROR_FINDING_PROCESS_VARIABLE"),  //$NON-NLS-1$
               new Object [] { aLocationPath })
         );      

      return serializeVariable(variable);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#noMoreChildrenToTerminate()
    */
   protected void noMoreChildrenToTerminate() throws AeBusinessProcessException
   {
      if (isTerminating())
      {
         processTerminated();
      }
      else
      {
         super.noMoreChildrenToTerminate();
      }
   }

   /**
    * Called when the process has completed its termination handling
    */
   protected void processTerminated() throws AeBusinessProcessException
   {
      IAeFault fault = AeFaultFactory.getFault(AeFaultFactory.FORCED_TERMINATION);
      setFaultedState(fault);
      processEnded(fault);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#setTerminating(boolean)
    */
   public void setTerminating(boolean aTerminatingFlag)
   {
      if (!isTerminating())
      {
         super.setTerminating(aTerminatingFlag);
      }
   }

   /**
    * Returns a list of the location paths of the object's in this process's execution queue.
    */
   public List getExecutionQueuePaths()
   {
      return getExecutionQueue().getLocationPaths();
   }

   /**
    * Returns serialization for this process's variable locker.
    *
    * @throws AeBusinessProcessException
    */
   public DocumentFragment getVariableLockerData() throws AeBusinessProcessException
   {
      return getVariableLocker().getLockerData(this);
   }

   /**
    * Sets the process's end date.
    */
   public void setEndDate(Date aEndDate)
   {
      mDateEnded = aEndDate;
   }

   /**
    * Sets the process's execution queue from a list of implementation object
    * location paths.
    *
    * @param aSuspended whether the execution queue is suspended
    * @param aExecutionQueuePaths the list of implementation object location paths
    * @throws AeBusinessProcessException
    */
   public void setExecutionQueue(boolean aSuspended, List aExecutionQueuePaths) throws AeBusinessProcessException
   {
      List queue = new LinkedList();

      // Convert location paths to implementation objects.
      for (Iterator i = aExecutionQueuePaths.iterator(); i.hasNext(); )
      {
         String locationPath = (String) i.next();
         IAeBpelObject impl = findBpelObjectOrThrow(locationPath);

         queue.add(impl);
      }

      mExecutionQueue.setQueueData(aSuspended, queue);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setProcessData(org.w3c.dom.Document)
    */
   public void setProcessData(Document aDocument) throws AeBusinessProcessException
   {
      AeRestoreImplState state = new AeRestoreImplState();
      state.setProcess(this);
      state.restoreState(aDocument);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setVariableData(java.lang.String, org.w3c.dom.Document, boolean)
    */
   public void setVariableData(String aLocationPath, Document aDocument, boolean aValidate) throws AeBusinessProcessException
   {
      IAeVariable variable = findProcessVariable(aLocationPath);
      
      if( variable == null )
      {
         Object [] errMsgParams = new Object[] { aLocationPath, getName().toString(), String.valueOf( getProcessId() ) };
         throw new AeBusinessProcessException(MessageFormat.format( AeMessages.getString("AeBusinessProcess.NO_VARIABLE_FOR_PATH"), errMsgParams)); //$NON-NLS-1$
      }
      
      AeVariableDeserializer variableSetter = null;

      // get the right type of variable setter based on the validate flag
      if( aValidate )
      {
         variableSetter = new AeSetVariableHandler( getEngine() );
      }
      else
      {
         variableSetter = new AeVariableDeserializer( getEngine() );
      }
      variableSetter.setVariable(variable);
      variableSetter.restoreVariable(aDocument);
   }
   
   /**
    * Sets the process's start date.
    */
   public void setStartDate(Date aStartDate)
   {
      mDateStarted = aStartDate;
   }

   /**
    * Sets this process's variable locker state from a serialized variable locker document.
    */
   public void setVariableLockerData(Node aNode) throws AeBusinessProcessException
   {
      getVariableLocker().setLockerData(aNode, this);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dispatchAlarm(int)
    */
   public void dispatchAlarm(int aLocationPathId)
   {
      dispatchAlarm(getLocationPath(aLocationPathId));
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dispatchAlarm(java.lang.String)
    */
   public void dispatchAlarm(String aLocationPath)
   {
      try
      {
         IAeAlarmReceiver receiver = (IAeAlarmReceiver) findBpelObjectOrThrow(aLocationPath);
         if (receiver.isQueued())
         {
            try
            {
               getExecutionQueue().dispatchAlarm(receiver);
            }
            catch (AeBusinessProcessException e)
            {
               objectCompletedWithFault((IAeBpelObject) receiver, AeFaultFactory.getSystemErrorFault(e));
            }
         }
      }
      catch (AeBusinessProcessException e)
      {
         e.logError();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dispatchInvokeData(java.lang.String, org.activebpel.rt.message.IAeMessageData)
    */
   public void dispatchInvokeData(String aLocationPath, IAeMessageData aData)
   {
      try
      {
         // find the receiver and if the receiver is not found then throw an exception
         IAeMessageReceiver receiver = (IAeMessageReceiver) findBpelObjectOrThrow(aLocationPath);
         
         // dispatch the invoke return if throws an exception while processing then fault the bpel object 
         try
         {
            getExecutionQueue().dispatchInvokeData(receiver, aData);
         }
         catch (AeBusinessProcessException e)
         {
            objectCompletedWithFault((IAeBpelObject) receiver, AeFaultFactory.getSystemErrorFault(e));
         }
         
      }
      catch (AeBusinessProcessException e)
      {
         AeBusinessProcessException.logError(e, AeMessages.format("AeBusinessProcess.ERROR_DISPATCHING_INVOKE", new Object[] {aLocationPath, Long.toString(getProcessId()) }) ); //$NON-NLS-1$
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dispatchInvokeFault(java.lang.String, org.activebpel.rt.bpel.IAeFault)
    */
   public void dispatchInvokeFault(String aLocationPath, IAeFault aFault)
   {
      try
      {
         IAeMessageReceiver receiver = (IAeMessageReceiver) findBpelObjectOrThrow(aLocationPath);
         try
         {
            getExecutionQueue().dispatchInvokeFault(receiver, aFault);
         }
         catch (AeBusinessProcessException e)
         {
            objectCompletedWithFault((IAeBpelObject) receiver, AeFaultFactory.getSystemErrorFault(e));
         }
      }
      catch (AeBusinessProcessException e)
      {
         e.logError();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dispatchReceiveData(java.lang.String, org.activebpel.rt.message.IAeMessageData)
    */
   public void dispatchReceiveData(String aLocationPath, IAeMessageData aData)
   {
      try
      {
         IAeMessageReceiverActivity receiver = (IAeMessageReceiverActivity) findBpelObjectOrThrow(aLocationPath);
         try
         {
            getExecutionQueue().dispatchReceiveData(receiver, aData);
         }
         catch (AeBusinessProcessException e)
         {
            objectCompletedWithFault((IAeBpelObject) receiver, AeFaultFactory.getSystemErrorFault(e));
         }
      }
      catch (AeBusinessProcessException e)
      {
         e.logError();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#dispatchReceiveData(int, org.activebpel.rt.message.IAeMessageData)
    */
   public void dispatchReceiveData(int aLocationPathId, IAeMessageData aData)
   {
      String locationPath = getLocationPath(aLocationPathId);
      dispatchReceiveData(locationPath, aData);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getCreateMessage()
    */
   public AeInboundReceive getCreateMessage()
   {
      return mStartMessage;
   }

   /**
    * The process will attempt to resolve the location path to an id using its
    * def object and then fallback to the instance mappings which contain the 
    * dynamically created objects.
    * 
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getLocationId(java.lang.String)
    */
   public int getLocationId(String aLocationPath)
   {
      AeProcessDef def = (AeProcessDef) getDefinition();
      int id = def.getLocationId(aLocationPath);
      if (id == -1)
      {
         Integer eyeD = (Integer) mLocationPathToId.get(aLocationPath);
         if (eyeD != null)
            id = eyeD.intValue();
      }
      return id;
   }

   /**
    * Checks with the definition object first to see if it has a mapping for the
    * location id and then falls back to the instance mappings which contain the
    * dynamically created objects. 
    * 
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getLocationPath(int)
    */
   public String getLocationPath(int aLocationId)
   {
      AeProcessDef def = (AeProcessDef) getDefinition();
      String path = def.getLocationPath(aLocationId);
      if (path == null)
      {
         path = (String) mLocationIdToPath.get(aLocationId);
      }
      return path;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getProcessSnapshot()
    */
   public IAeProcessSnapshot getProcessSnapshot() throws AeBusinessProcessException
   {
      return new AeProcessSnapshot(this);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#isStartMessageAvailable()
    */
   public boolean isStartMessageAvailable()
   {
      return mStartMessage != null;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getProcessPlan()
    */
   public IAeProcessPlan getProcessPlan()
   {
      return mProcessPlan;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getExpressionLanguageFactory()
    */
   public IAeExpressionLanguageFactory getExpressionLanguageFactory() throws AeException
   {
      IAeEngineConfiguration engineConfig = getEngine().getEngineConfiguration();
      return engineConfig.getExpressionLanguageFactory();
   }

   /**
    * Getter for the business process properties <code>Map</code>.
    */
   protected HashMap getBusinessProcessPropertiesMap()
   {
      if( mBusinessProcessProperties == null )
      {
         mBusinessProcessProperties = new HashMap();
      }
      return mBusinessProcessProperties;
   }

   /**
    * Add all of the properties in the map to the business process.  The arg
    * map should only contain <code>String</code> name/value pairs.
    * @param aMap
    */
   public void addBusinessProcessProperties( Map aMap )
   {
      getBusinessProcessPropertiesMap().putAll( aMap );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#removeBusinessProcessProperty(java.lang.String)
    */
   public String removeBusinessProcessProperty( String aName )
   {
      return (String)getBusinessProcessPropertiesMap().remove( aName );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setBusinessProcessProperty(java.lang.String, java.lang.String)
    */
   public void setBusinessProcessProperty( String aName, String aProperty )
   {
      getBusinessProcessPropertiesMap().put( aName, aProperty );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getBusinessProcessProperty(java.lang.String)
    */
   public String getBusinessProcessProperty( String aName )
   {
      return (String)getBusinessProcessPropertiesMap().get( aName );
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getBusinessProcessPropertyNames()
    */
   public Iterator getBusinessProcessPropertyNames()
   {
      return getBusinessProcessPropertiesMap().keySet().iterator();
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getFaultingActivityLocationPaths()
    */
   public List getFaultingActivityLocationPaths()
   {
      return mFaultingActivityLocationPaths;
   }
   
   /**
    * @return Returns the processAdministrator.
    */
   protected AeProcessSuspendResumeHandler getProcessSuspendResumeHandler()
   {
      return mProcessAdministrator;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setPartnerLinkData(boolean, java.lang.String, org.w3c.dom.Document)
    */
   public void setPartnerLinkData( boolean aIsPartnerRole, String aPartnerLinkPath, Document aPartnerEndpointRef )
   throws AeBusinessProcessException
   {
      AePartnerLink partnerLink = getPartnerLinkByLocationPath( aPartnerLinkPath, mPartnerLinks.values() );
      if( partnerLink != null )
      {
         if(aIsPartnerRole)
            partnerLink.getPartnerReference().setReferenceData( aPartnerEndpointRef.getDocumentElement() );
         else
            partnerLink.getMyReference().setReferenceData( aPartnerEndpointRef.getDocumentElement() );
      }
      else
      {
         Object[] errMsgParams = { aPartnerLinkPath, getName().toString(), String.valueOf( getProcessId() ) }; 
         throw new AeBusinessProcessException( AeMessages.format("AeBusinessProcess.NO_PARTNERLINK_FOR_PATH", errMsgParams ) ); //$NON-NLS-1$
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getPartnerRoleEndpointReference(java.lang.String)
    */
   public IAeEndpointReference getPartnerRoleEndpointReference(String aPartnerLinkPath) throws AeBusinessProcessException
   {
      AePartnerLink partnerLink = getPartnerLinkByLocationPath( aPartnerLinkPath, mPartnerLinks.values() );

      if( partnerLink != null && partnerLink.getPartnerReference() != null )
      {
         return partnerLink.getPartnerReference();
      }
      else
      {
         Object[] errMsgParams = { aPartnerLinkPath, getName().toString(), String.valueOf( getProcessId() ) }; 
         throw new AeBusinessProcessException( AeMessages.format("AeBusinessProcess.NO_PARTNERLINK_FOR_PATH", errMsgParams ) ); //$NON-NLS-1$
      }
   }

   /**
    * Find the matching <code>AePartnerLink</code> object.
    * @param aLocationPath
    * @param aPartnerLinks
    */
   protected AePartnerLink getPartnerLinkByLocationPath( String aLocationPath, Collection aPartnerLinks )
   {
      for( Iterator iter = aPartnerLinks.iterator(); iter.hasNext(); )
      {
         AePartnerLink partnerLink = (AePartnerLink)iter.next();
         String objLocationPath = partnerLink.getDefinition().getLocationPath();
         if( aLocationPath.equals( objLocationPath ) )
         {
            return partnerLink;
         }
      }
      return null;
   }

   /** 
    * Finds the correlationset given the location path.
    */
   protected AeCorrelationSet findCorrelationSet( String aLocationPath) throws AeBusinessProcessException
   {
      AeCorrelationSet correlationSet = (AeCorrelationSet)findBpelObjectOrThrow( aLocationPath );
      if( correlationSet != null )
      {
         return correlationSet;
      }
      else
      {
         Object[] errMsgParams = { aLocationPath, getName().toString(), String.valueOf( getProcessId() ) }; 
         throw new AeBusinessProcessException( AeMessages.format("AeBusinessProcess.NO_CORRELATION_SET_FOR_PATH", errMsgParams ) ); //$NON-NLS-1$
      }
   }
      
   /** 
    * Overrides method to 
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getCorrelationData(java.lang.String)
    */
   public Map getCorrelationData( String aLocationPath) throws AeBusinessProcessException
   {
      AeCorrelationSet correlationSet = findCorrelationSet( aLocationPath );
      return new HashMap(correlationSet.getPropertyValues());
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#setCorrelationData(java.lang.String, java.util.Map)
    */
   public void setCorrelationData( String aLocationPath, Map aCorrelationData )
   throws AeBusinessProcessException
   {
      AeCorrelationSet correlationSet = findCorrelationSet( aLocationPath );
      correlationSet.setPropertyValues( aCorrelationData );
   }
   
   /**
    * Getter for the max location id. If no objects have been created outside of the
    * initial visitation then this will simply return the value from the def.
    * Otherwise, we'll have a record locally of the max location id.
    */
   public int getMaxLocationId()
   {
      if (mMaxLocationId == -1)
      {
         return ((AeProcessDef)getDefinition()).getMaxLocationId();
      }
      return mMaxLocationId;
   }
   
   /**
    * Called after we've created some objects dynamically (ie through parallel 
    * forEach execution). 
    * @param aMaxValue
    */
   public void setMaxLocationId(int aMaxValue)
   {
      mMaxLocationId = aMaxValue;
   }
   
   /**
    * Gets the scope definition for this scope.  
    */
   protected AeScopeDef getScopeDef()
   {
      return (AeScopeDef) getDefinition();
   }
   
   /**
    * Informs the alert manager that the process is faulting.
    * @param aLocationPath
    * @param aFault
    */
   protected void alertFaulting(String aLocationPath, IAeFault aFault)
   {
      AeEngineAlert alert = createAlert(IAeEngineAlert.PROCESS_ALERT_FAULTING, aLocationPath, aFault);
      getEngine().fireEngineAlert(alert);
   }
   
   /**
    * Creates an AeEngineAlert with the info provided.
    * @param aReason either faulting or suspend constant
    * @param aLocationPath location of the activity causing the alert
    * @param aDetails optional details of the alert
    */
   protected AeEngineAlert createAlert(int aReason, String aLocationPath, Object aDetails)
   {
      QName faultName = null;
      Document doc = null;
      try
      {
         if (aDetails instanceof IAeVariable)
         {
            IAeVariable variable = (IAeVariable) aDetails;
            // TODO (MF) should defer the serialization of the variable until we know that there's someone listening
            if (variable != null && variable.hasData())
            {
               doc = serializeVariable(variable);
            }
         }
         else if (aDetails instanceof IAeFault)
         {
            IAeFault fault = (IAeFault) aDetails;
            faultName = fault.getFaultName();
            if (fault.getMessageData() != null)
            {
               doc = serializeFault(fault);
            }
         }
      }
      catch(AeException e)
      {
         e.logError();
      }
      return new AeEngineAlert(getProcessId(), aReason, getName(), aLocationPath, faultName, doc );
   }

   /**
    * Adds the message receiver to our collection of receivers. We track the receivers
    * within the process to prevent multiple concurrent receivers for the same
    * tupple of partner link, operation, and correlation sets.
    * @param aMessageReceiver
    * @param aCorrelationSetPaths
    * @throws AeBusinessProcessException
    */
   public void addReceiverKeyForConflictingReceives(IAeMessageReceiverActivity aMessageReceiver, Collection aCorrelationSetPaths) 
      throws AeBusinessProcessException
   {
      AeQueuedReceiveKey key = new AeQueuedReceiveKey(getLocationId(aMessageReceiver.getLocationPath()), aMessageReceiver.getPartnerLink().getName(), 
                     aMessageReceiver.getOperation(), aCorrelationSetPaths);
      boolean wasAdded = getQueuedReceives().add(key);
      if (!wasAdded)
      {
         throw new AeConflictingReceiveException(); 
      }
   }
   
   /**
    * Adds the message receiver to our collection of receivers. We track the receivers
    * within the process to prevent multiple concurrent receivers for the same
    * tupple of partner link, operation, and correlation sets.
    * @param aMessageReceiver
    * @throws AeBusinessProcessException
    */
   public void addReceiverKeyForConflictingReceives(IAeMessageReceiverActivity aMessageReceiver) 
      throws AeBusinessProcessException
   {
      Set correlationSetPaths = new HashSet();
      getCorrelationData(aMessageReceiver, correlationSetPaths, null);
      addReceiverKeyForConflictingReceives(aMessageReceiver, correlationSetPaths);
   }
   
   /**
    * Removes the message receiver from the collection of executing receivers.
    * @param aMessageReceiver
    */
   public void removeReceiverKeyForConflictingReceives(IAeMessageReceiverActivity aMessageReceiver)
   {
      // TODO (MF) use simple remove and add complexity to equals() in AeQueuedReceiveKey
      int locationId = getLocationId(aMessageReceiver.getLocationPath());
      for (Iterator iter = getQueuedReceives().iterator(); iter.hasNext();)
      {
         AeQueuedReceiveKey key = (AeQueuedReceiveKey) iter.next();
         if (key.getLocationId() == locationId)
         {
            iter.remove();
            break;
         }
      }
   }
   
   /**
    * @return Returns the queuedReceives.
    */
   protected Set getQueuedReceives()
   {
      return mQueuedReceives;
   }

   /**
    * Value used for the key in a map of currently executing receives. This map 
    * is used to prevent receives with the same partner link, 
    */
   protected static class AeQueuedReceiveKey
   {
      /** name of the partner link */
      private String mPartnerLink;
      /** name of the operation */
      private String mOperation;
      /** set comprised of location paths for the correlation sets used for the receive */
      private Set mCorrelationSets = new HashSet();
      /** cached value of the hashcode */
      private int mHashcode;
      /** the location id for the activity */
      private int mLocationId;
      
      /**
       * Constructor accepts all of the values for the key
       * 
       * @param aLocationId
       * @param aPartnerLink
       * @param aOperation
       * @param aCorrelationSetPaths
       */
      public AeQueuedReceiveKey(int aLocationId, String aPartnerLink, String aOperation, Collection aCorrelationSetPaths)
      {
         mLocationId = aLocationId;
         mPartnerLink = aPartnerLink;
         mOperation = aOperation;
         mCorrelationSets.addAll(aCorrelationSetPaths);
         mHashcode = aPartnerLink.hashCode() + aOperation.hashCode() + mCorrelationSets.hashCode();
      }
      
      /**
       * @return Returns the correlationSets.
       */
      protected Set getCorrelationSets()
      {
         return mCorrelationSets;
      }

      /**
       * @return Returns the hashcode.
       */
      protected int getHashcode()
      {
         return mHashcode;
      }
      
      /**
       * @return Returns the operation.
       */
      protected String getOperation()
      {
         return mOperation;
      }
      
      /**
       * @return Returns the partnerLink.
       */
      protected String getPartnerLink()
      {
         return mPartnerLink;
      }
      
      /**
       * @see java.lang.Object#equals(java.lang.Object)
       */
      public boolean equals(Object aObj)
      {
         if (aObj instanceof AeQueuedReceiveKey)
         {
            AeQueuedReceiveKey other = (AeQueuedReceiveKey) aObj;
            return AeUtil.compareObjects(getPartnerLink(), other.getPartnerLink()) &&
                   AeUtil.compareObjects(getOperation(), other.getOperation()) &&
                   AeUtil.compareObjects(getCorrelationSets(), other.getCorrelationSets());
         }
         return false;
      }
      
      /**
       * Getter for the location id
       */
      public int getLocationId()
      {
         return mLocationId;
      }
      
      /**
       * @see java.lang.Object#hashCode()
       */
      public int hashCode()
      {
         return mHashcode;
      }
   }
}

