//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/AeDataConverter.java,v 1.2 2005/04/06 21:34:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl; 

import java.util.Iterator;

import org.activebpel.rt.bpel.AeMessageDataFactory;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.wsio.AeWebServiceMessageData;
import org.activebpel.wsio.IAeWebServiceMessageData;

/**
 * Utility class for converting between wsio data and our standard data container.
 * The wsio packaging provides a standalone set of interfaces and support classes
 * for dispatching messages to the engine and for invoking external web services.
 * In order to simplify the packaging, the wsio project does not have ties to the
 * rest of the activebpel codebase. One side effect is that we're left with two
 * different interfaces 
 */
public class AeDataConverter
{
   /**
    * Converts a message data object to a wsio message data. 
    * Returns <code>null</code> if input is <code>null</code>.
    * 
    * @param aData
    */
   public static IAeWebServiceMessageData convert(IAeMessageData aData)
   {
      AeWebServiceMessageData wsMsg;
      
      if (aData == null)
      {
         wsMsg = null;
      }
      else
      {
         wsMsg = new AeWebServiceMessageData( aData.getMessageType() );

         for( Iterator iter = aData.getPartNames(); iter.hasNext(); )
         {
            String partName = (String)iter.next();
            wsMsg.setData( partName, aData.getData(partName) );
         }
      }
      
      return wsMsg;
   }
   
   /**
    * Converts a wsio message to a message data.
    * Returns <code>null</code> if input is <code>null</code>.
    * 
    * @param aData
    */
   public static IAeMessageData convert(IAeWebServiceMessageData aData)
   {
      return (aData == null)
         ? null
         : AeMessageDataFactory.instance().createMessageData(aData.getMessageType(), aData.getMessageData());
   }
}
 
