//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/AeProcessPlanFixture.java,v 1.4 2005/10/17 19:44:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl; 

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;

/**
 * A no-op fixture for process plans. Implement things as you need them for testing. 
 */
public class AeProcessPlanFixture implements IAeProcessPlan
{
   /** maps partner link names to their port types */
   private Map mMyRolePortTypes = new HashMap();
   
   /** process def for the plan */
   private AeProcessDef mProcessDef;
   
   /**
    * No arg ctor 
    */
   public AeProcessPlanFixture()
   {
   }
   
   /**
    * Sets the process def in place.
    * 
    * @param aDef
    */
   public AeProcessPlanFixture(AeProcessDef aDef)
   {
      setProcessDef(aDef);
   }

   /**
    * Adds the mapping of the partner link name to the port type.
    * 
    * @param aPartnerLinkName
    * @param aPortType
    */
   public void addMyRolePortType(String aPartnerLinkName, QName aPortType)
   {
      mMyRolePortTypes.put(aPartnerLinkName, aPortType);
   }
   
   /**
    * Setter for the process def.
    * 
    * @param aDef
    */
   public void setProcessDef(AeProcessDef aDef)
   {
      mProcessDef = aDef;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeProcessPlan#isCreateInstance(java.lang.String, java.lang.String)
    */
   public boolean isCreateInstance(String aPartnerLinkName, String aOperation)
   {
      return false;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeProcessPlan#getCorrelatedPropertyNames(java.lang.String, java.lang.String)
    */
   public Collection getCorrelatedPropertyNames(String aPartnerLinkName, String aOperation)
   {
      return null;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeProcessPlan#getProcessDef()
    */
   public AeProcessDef getProcessDef()
   {
      return mProcessDef;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeProcessPlan#getMyRolePortType(java.lang.String)
    */
   public QName getMyRolePortType(String aPartnerLinkName)
   {
      return (QName) mMyRolePortTypes.get(aPartnerLinkName);
   }

   /**
    * @see org.activebpel.rt.bpel.IAeContextWSDLProvider#getWSDLIterator()
    */
   public Iterator getWSDLIterator()
   {
      return null;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeWSDLProvider#getWSDLIterator(java.lang.String)
    */
   public Iterator getWSDLIterator(String aNamespaceUri)
   {
      return null;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeWSDLProvider#dereferenceIteration(java.lang.Object)
    */
   public AeBPELExtendedWSDLDef dereferenceIteration(Object aIteration)
   {
      return null;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeProcessPlan#isSuspendProcessOnUncaughtFaultEnabled()
    */
   public boolean isSuspendProcessOnUncaughtFaultEnabled()
   {
      return false;
   }
}
 
