// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/AeVariableSerializer.java,v 1.3 2005/02/08 15:33:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.impl.fastdom.AeFastDocument;
import org.activebpel.rt.bpel.impl.fastdom.AeFastElement;
import org.activebpel.rt.bpel.impl.fastdom.AeFastText;
import org.activebpel.rt.bpel.impl.fastdom.AeForeignNode;
import org.w3c.dom.Element;

/**
 * Serializes a variable to an <code>AeFastElement</code> or
 * <code>AeFastDocument</code>.
 */
public class AeVariableSerializer extends AeMessageDataSerializer implements IAeImplStateNames
{
   /** The variable to serialize. */
   private IAeVariable mVariable;

   /** The resulting serialization. */
   private AeFastElement mVariableElement;

   /**
    * Constructor.
    *
    * @param aTypeMapping The type mapping for simple types.
    */
   public AeVariableSerializer(AeTypeMapping aTypeMapping)
   {
      super(aTypeMapping);
   }

   /**
    * Serializes the specified variable to an <code>AeFastElement</code>.
    *
    * @param aVariable
    */
   protected AeFastElement createVariableElement(IAeVariable aVariable) throws AeBusinessProcessException
   {
      AeFastElement variableElement = new AeFastElement(STATE_VAR);
      boolean hasData = aVariable.hasData();

      variableElement.setAttribute(STATE_NAME   , aVariable.getName());
      variableElement.setAttribute(STATE_DATA   , "yes"); //$NON-NLS-1$
      variableElement.setAttribute(STATE_HASDATA, "" + hasData); //$NON-NLS-1$
      variableElement.setAttribute(STATE_VERSION, "" + aVariable.getVersionNumber()); //$NON-NLS-1$

      if (hasData)
      {
         if (aVariable.getType() != null)
         {
            String data = getTypeMapping().serialize(aVariable.getTypeData());
            variableElement.appendChild(new AeFastText(data));
         }
         else if (aVariable.getElement() != null)
         {
            Element data = aVariable.getElementData();
            variableElement.appendChild(new AeForeignNode(data));
         }
         else if (aVariable.getMessageData() != null)
         {
            appendMessageDataParts(variableElement, aVariable.getMessageData());
         }
         else
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeVariableSerializer.ERROR_3")); //$NON-NLS-1$
         }
      }

      return variableElement;
   }

   /**
    * Returns an <code>AeFastDocument</code> representing the variable.
    */
   public AeFastDocument getVariableDocument() throws AeBusinessProcessException
   {
      return new AeFastDocument(getVariableElement());
   }

   /**
    * Returns an <code>AeFastElement</code> representing the variable.
    */
   public AeFastElement getVariableElement() throws AeBusinessProcessException
   {
      if (mVariableElement == null)
      {
         if (mVariable == null)
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeVariableSerializer.ERROR_4")); //$NON-NLS-1$
         }

         mVariableElement = createVariableElement(mVariable);
      }

      return mVariableElement;
   }

   /**
    * Sets the variable to serialize.
    *
    * @param aVariable
    */
   public void setVariable(IAeVariable aVariable)
   {
      mVariable = aVariable;
      mVariableElement = null;
   }
}
