// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/IAeBusinessProcessEngineInternal.java,v 1.44 2006/01/10 21:04:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl;

import java.util.Date;
import java.util.Map;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeBusinessProcess;
import org.activebpel.rt.bpel.IAeBusinessProcessEngine;
import org.activebpel.rt.bpel.IAeEngineAlert;
import org.activebpel.rt.bpel.IAeEngineEvent;
import org.activebpel.rt.bpel.IAePlanManager;
import org.activebpel.rt.bpel.IAeProcessEvent;
import org.activebpel.rt.bpel.IAeProcessInfoEvent;
import org.activebpel.rt.bpel.impl.queue.AeInboundReceive;
import org.activebpel.rt.bpel.impl.queue.AeMessageReceiver;
import org.activebpel.rt.bpel.impl.queue.AeReply;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.wsio.IAeWebServiceResponse;
import org.activebpel.wsio.invoke.IAeInvoke;
import org.activebpel.wsio.receive.AeTimeoutException;
import org.activebpel.wsio.receive.IAeMessageContext;

/**
 * Internal extensions to business process engine allows classes within
 * the implemenation to use the engine while still hiding the implementor.
 */
public interface IAeBusinessProcessEngineInternal extends IAeBusinessProcessEngine, IAeBusinessProcessFactory
{
   /**
    * Gets the lock manager.
    */
   public IAeLockManager getLockManager();

   /**
    * Gets the queue manager that's used to queue activities and request data
    */
   public IAeQueueManager getQueueManager();

   /** Getter for the process manager. */
   public IAeProcessManager getProcessManager();
   
   /** 
    * @return Returns the coordination manager.
    */
   public IAeCoordinationManagerInternal getCoordinationManager();  
   
   /**
    * Returns true if message context indicates that invoke request should be coordinated.
    * @param aContext
    * @return returns true if the message context's business properties has a coordination context
    */
   public boolean isCoordinated(IAeMessageContext aContext);
   
    /**
    * Registers the given context for coordination with the coordination manager.
    * @param aContext message context
    * @param aProcess process
    * @throws AeBusinessProcessException
    */
   public void registerForCoordination(IAeMessageContext aContextt, IAeBusinessProcess aProcess) throws AeBusinessProcessException;
   
   /**
    * Returns the interface to the engine callback.
    * @return engine call back interface.
    */
   public IAeBusinessProcessEngineCallback getEngineCallback();
   
   /**
    * Returns handler for responsible for process coordination operations.
    */
   public IAeProcessCoordination getProcessCoordination();
   
   /**
    * Fire an information event to report the results of an expression evaluation.
    * @param aPID The process ID.
    * @param aExpression The expression to evaluate.
    * @param aActivity The activity instance, used as Function and Namespace context, etc.
    * @param aEventID The event ID for this evaluation.
    * @param aNodePath The node path for the object performing the evaluation.
    * @param aResult The result of the evaluation.
    */
   public void fireEvaluationEvent(long aPID, String aExpression, AeAbstractBpelObject aActivity, int aEventID, String aNodePath, String aResult);

   /**
    * Fires the information event passed to all process listeners.
    * @param aInfoEvent The information event to be fired.
    */
   public void fireInfoEvent(IAeProcessInfoEvent aInfoEvent);

   /**
    * Fires the event passed to all process listeners.
    * @param aEvent The event to be fired.
    */
   public void fireEvent(IAeProcessEvent aEvent);

   /**
    * Fires the event to all engine listeners.
    * @param aEvent The event to be fired.
    */
   public void fireEngineEvent(IAeEngineEvent aEvent);
   
   /**
    * Fires the alert to all engine listeners
    * @param aEvent
    */
   public void fireEngineAlert(IAeEngineAlert aEvent);


   /**
    * Allows an external service to instruct the engine to handle a matched receive.  This method
    * is called when an incoming receive is matched to a queued receive.
    *
    * @param aInboundReceive
    * @param aFound
    * @param aReplyWaitingFlag
    * @throws AeBusinessProcessException
    */
   public void handleMatchedReceive( AeInboundReceive aInboundReceive,
                                     AeMessageReceiver aFound,
                                     boolean aReplyWaitingFlag)
         throws AeBusinessProcessException;

   /**
    * Sets the plan manager for the engine. This is used to resolve a
    * process QName to a process plan that provides hints in the form of
    * correlated properties and create instance.
    * @param aPlanManager
    */
   public void setPlanManager(IAePlanManager aPlanManager);

   /**
    * Starts the engine.
    */
   public void start() throws AeBusinessProcessException;

   /**
    * Stops the engine.
    */
   public void stop() throws AeBusinessProcessException;

   /**
    * Shuts the engine down.
    */
   public void shutDown() throws AeBusinessProcessException;

   /**
    * Dispatches alarm to a process by the location path of the alarm receiver.
    *
    * @param aProcessId The process id of the owner process.
    * @param aLocationPathId The location path id of the alarm receiver.
    * @param aGroupId The alarm's group id.
    */
   public void dispatchAlarm(long aProcessId, int aLocationPathId, int aGroupId) throws AeBusinessProcessException;

   /**
    * Returns the strategy for managing partner links.
    */
   public IAeEnginePartnerLinkStrategy getPartnerLinkStrategy();

   /**
    * Schedules an alarm for execution.
    *
    * @param aProcessId The ID of the BPEL process this alarm is associated with.
    * @param aLocationPath The location path id of the alarm receiver.
    * @param aGroupId The alarm's group id.
    * @param aDeadline The alarm deadline.
    */
   public void scheduleAlarm(long aProcessId, int aLocationPathId, int aGroupId, Date aDeadline) throws AeBusinessProcessException;

   /**
    * Removes an alarm.  The removal of an alarm requires it to be unscheduled (cancelled)
    * in addition to whatever other bookkeeping must be done by the alarm manager
    * implementation.  Note that <code>removeAlarm</code> will be called by the
    * engine when an alarm should be cancelled (not fired) as well as when the alarm
    * was fired and must now be removed from whatever internal storage the alarm manager
    * implementation is using.
    *
    * @param aProcessId The ID of the BPEL process this alarm is associated with.
    * @param aLocationPathId The location path id of the alarm receiver.
    * @return True if the alarm was successfully removed.
    * @throws AeBusinessProcessException
    */
   public boolean removeAlarm(long aProcessId, int aLocationPathId) throws AeBusinessProcessException;

   /**
    * The invoke queue object contains all of the necessary information to perform
    * an invocation on a target endpoint. The actual invoke operation is handled
    * at a different layer asynchronously.
    * @param aPlan may contain additional information from the deployment layer like invokeHandler or addressing information
    * @param aInvokeQueueObject The entry to add.
    */
   public void addInvoke(IAeProcessPlan aPlan, IAeInvoke aInvokeQueueObject) throws AeBusinessProcessException;

   /**
    * Queues a message receiver. This message receiver (e.g. receive, onMessage)
    * will stay queued until a message arrives for it from the outside world
    * or until it is explicity dequeued by the process.
    * @param aMessageReceiver The entry to add.
    */
   public void addMessageReceiver(AeMessageReceiver aMessageReceiver) throws AeBusinessProcessException;

   /**
    * Finds a matching message queue entry for the receiver and dequeues it.
    * @param aProcessId process Id for the message receiver
    * @param aMessageReceiverPathId The path id to the receiver
    * @return boolean True if the matching element was found and dequeued.
    * @throws AeBusinessProcessException
    */
   public boolean removeMessageReceiver(long aProcessId, int aMessageReceiverPathId) throws AeBusinessProcessException;

   /**
    * Sends the data to the response receiver that's waiting for the reply.
    * @param aProcessId The process id of the process sending the reply.
    * @param aReplyObject Identifies the reply object
    * @param aData data to send
    * @param aFaultName fault name, if not null then the reply is a fault
    * @param aProcessProperties The business process properties.
    * @return True iff the reply was successfully sent.
    * @throws AeInvalidReplyException if no reply was found to be waiting
    * @throws AeBusinessProcessException if there is some communication error executing the reply
    */
   public boolean sendReply(long aProcessId, AeReply aReplyObject, IAeMessageData aData, QName aFaultName, Map aProcessProperties) throws AeBusinessProcessException;
   
   /**
    * Specialized version of queue receive data that accepts the process plan.
    * This method is intended to be used by our internal web service layer to
    * route messages to the engine. In that layer, we need to load the deployment
    * plan for the process in order to deserialize the SOAP message into our
    * message data container.
    * 
    * @param aPlan Process plan for routing the message
    * @param aContext provides contextual information for message like partner link being targeted
    * @param aData contains the data extracted from the inbound message
    * @throws AeBusinessProcessException
    * @throws AeTimeoutException
    */
   public IAeWebServiceResponse queueReceiveData(IAeProcessPlan aPlan, IAeMessageContext aContext, IAeMessageData aData) 
         throws AeBusinessProcessException, AeTimeoutException;
   
   
   /**
    * Returns the locationPath string given the locationId and the processId
    * @param aProcessId process id
    * @param aLocationId location id of the BPEL object.
    * @return
    * @throws AeBusinessProcessException
    */
   public String getLocationPathById(long aProcessId, int aLocationId) throws AeBusinessProcessException;
}
