// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/IAeProcessPlan.java,v 1.10 2005/10/17 19:44:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl;

import java.util.Collection;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.IAeContextWSDLProvider;
import org.activebpel.rt.bpel.def.AeProcessDef;

/**
 * Describes some basic characteristics of a process that we need in order to
 * for the engine to dispatch a message to a specific process (or possibly create
 * a process instance). 
 */
public interface IAeProcessPlan extends IAeContextWSDLProvider
{
   /**
    * Returns a flag indicating if this activity should create the process instance.  
    * @param aPartnerLinkName the name of the partner link 
    * @param aOperation the operation for the partner link
    * @return boolean flag indicating if the operation is capable of creating the process
    */
   public boolean isCreateInstance(String aPartnerLinkName, String aOperation);
                            
   /**
    * Returns a List of correlation properties for the given partnerLink, portType and operation.
    * If no list has been set an empty collection will be returned.  
    * @param aPartnerLinkName the name of the partner link 
    * @param aOperation the operation for the partner link
    */
   public Collection getCorrelatedPropertyNames(String aPartnerLinkName, String aOperation);
            
   /**
    * Gets the process definition.
    */
   public AeProcessDef getProcessDef();
   
   /**
    * Return the port type for the my role and partner link.
    * 
    * @param aPartnerLinkName
    */
   public QName getMyRolePortType(String aPartnerLinkName);
   
   /**
    * Return true if the process should be suspended if it encounters
    * an uncaught fault.
    */
   public boolean isSuspendProcessOnUncaughtFaultEnabled();
}
