// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/AeActivityAssignImpl.java,v 1.55 2006/02/13 23:09:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import javax.xml.namespace.QName;

import org.activebpel.rt.IAeConstants;
import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessageDataFactory;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeEndpointReference;
import org.activebpel.rt.bpel.IAeProcessInfoEvent;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.activity.AeActivityAssignDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignCopyDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignFromDef;
import org.activebpel.rt.bpel.def.activity.support.AeAssignToDef;
import org.activebpel.rt.bpel.impl.AeBadXPathException;
import org.activebpel.rt.bpel.impl.AeBpelException;
import org.activebpel.rt.bpel.impl.AeFaultFactory;
import org.activebpel.rt.bpel.impl.AeNamespaceResolver;
import org.activebpel.rt.bpel.impl.AePartnerLink;
import org.activebpel.rt.bpel.impl.AeProcessInfoEvent;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.expr.xpath.AeXPathFunctionContext;
import org.activebpel.rt.bpel.impl.expr.xpath.AeXPathNamespaceContext;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.bpel.xpath.AeXPathHelper;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.util.AeXmlUtil;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;
import org.activebpel.rt.xml.AeXMLParserBase;
import org.exolab.castor.xml.schema.XMLType;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

/**
 * Implementation of the BPEL assign activity.
 */
public class AeActivityAssignImpl extends AeActivityImpl
{
   /** Variable not found error message. */
   protected static final String VAR_NOT_FOUND = AeMessages.getString("AeActivityAssignImpl.0"); //$NON-NLS-1$
   /** Unable to locate property alias error message. */
   protected static final String NO_PROP_ALIAS = AeMessages.getString("AeActivityAssignImpl.1"); //$NON-NLS-1$
   /** Data missing for assign error message. */
   protected static final String MISSING_DATA = AeMessages.getString("AeActivityAssignImpl.2"); //$NON-NLS-1$
   /** Data mismatch error message. */
   protected static final String DATA_MISMATCH = AeMessages.getString("AeActivityAssignImpl.3"); //$NON-NLS-1$

   /**
    * Default constructor. 
    * @param aAssign the definition of the assign activity. 
    * @param aParent the parent of the assign activity.
    */
   public AeActivityAssignImpl(AeActivityAssignDef aAssign, IAeActivityParent aParent)
   {
      super(aAssign, aParent);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept(IAeImplVisitor aVisitor) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }

   /**
    * Perform execution of the assign activity.
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      // Map to hold data in case we need to restore it in case of failure.
      HashMap dataMap = new HashMap();
      
      // Maintain current copy being processed in case of error
      int currentCopy = 1;
      
      try
      {
         // Perform copy operations for the assignment
         AeActivityAssignDef assign = (AeActivityAssignDef) getDefinition();
         for (Iterator iter = assign.getCopyList(); iter.hasNext(); currentCopy++)
         {
            AeAssignCopyDef copy = (AeAssignCopyDef) iter.next();
            storeData(dataMap, copy.getTo());
            setCopyData(copy.getTo(), getCopyData(copy.getFrom()));
         }

         // Now validate that the data assignment yielded a correct DOM
         currentCopy = 1;
         for (Iterator iter = assign.getCopyList(); iter.hasNext(); currentCopy++)
         {
            AeAssignToDef copyTgt = ((AeAssignCopyDef) iter.next()).getTo();
            if (!AeUtil.isNullOrEmpty(copyTgt.getVariable()))
            {
               // Make sure we can find the variable
               IAeVariable var = getVariable(copyTgt.getVariable());
               if (var == null)
                  throw new AeBpelException(VAR_NOT_FOUND, AeFaultFactory.BAD_PROCESS);

               if (!isDataValid(var))
                  throw new AeBpelException(DATA_MISMATCH, AeFaultFactory.MISMATCHED_ASSIGNMENT_FAILURE);
            }
         }

         // Signal successful completion
         objectCompleted();
      }
      catch (AeBpelException e)
      {
         // Log info error message to give user clue as to which operation failed.
         // Note we are sending the index of the copy operation which is translated in the msg formatter
         AeProcessInfoEvent evt = new AeProcessInfoEvent(getProcess().getProcessId(), 
                                                 getLocationPath(), 
                                                 IAeProcessInfoEvent.ERROR_ASSIGN_ACTIVITY, 
                                                 "", //$NON-NLS-1$
                                                 Integer.toString(currentCopy));
         getProcess().getEngine().fireInfoEvent(evt);
         
         // Restore data to the initial state and signal a fault
         restoreData(dataMap);
         objectCompletedWithFault(e.getFault());
      }
   }

   /**
    * Returns data represented by the source copy operation or null if unable to 
    * retrieve data.
    * @param aFrom the source of the copy operation
    * @throws AeBusinessProcessException
    */
   private Object getCopyData(AeAssignFromDef aFrom) throws AeBusinessProcessException
   {
      Object data = null;

      if (!AeUtil.isNullOrEmpty(aFrom.getVariable()))
         data = getCopyVariableData(aFrom);
      else if (!AeUtil.isNullOrEmpty(aFrom.getPartnerLink()))
      {
         AePartnerLink partnerLink = getPartnerLink(aFrom.getPartnerLink());
         if (partnerLink != null)
         {
            // get the endpoint reference of the [artner link
            IAeEndpointReference ref = null;
            if (IAeBPELConstants.TAG_MY_ROLE.equals(aFrom.getEndpointReference()))
               ref = partnerLink.getMyReference();
            else if (IAeBPELConstants.TAG_PARTNER_ROLE.equals(aFrom.getEndpointReference()))
               ref = partnerLink.getPartnerReference();
            
            // if the reference is not null then get it as an element
            if(ref != null)
            {
               Document doc = ref.toDocument(); 
               if(doc != null)
                  data = doc.getDocumentElement();
            }
         }
      }
      else if (!AeUtil.isNullOrEmpty(aFrom.getExpression()))
      {
         data = executeExpression(aFrom.getExpression());
      }
      else if(aFrom.getLiteral() != null)
      {
         // Need to synchronize on the document fragment we are copying, or bad things can happen under load
         synchronized(aFrom.getLiteral())
         {
            Document doc = AeXmlUtil.newDocument();
            Node node = doc.importNode(aFrom.getLiteral(), true);
            if (node instanceof Element)
            {
               Map namespaceMap = new HashMap();
               AeXmlUtil.getDeclaredNamespaces((Element)aFrom.getLiteral(), namespaceMap);
               AeXmlUtil.declareNamespacePrefixes((Element)node, namespaceMap);
               
               doc.appendChild(node);
               data = doc.getDocumentElement();
            }
            else
            {
               data = node;
            }
         }
      }

      // Make sure we found the data, otherwise signal appropriate error unless configuration overrides
      if (data == null && (!isEmptyQuerySelectionAllowed()))
         throw new AeBpelException(MISSING_DATA, AeFaultFactory.UNINITIALIZED_VARIABLE);

      return data;
   }

   /**
    * Returns data represented by the source copy operation for variable type 
    * operations. Null will be returned if unable to retrieve data.
    * @param aFrom the source of the copy operation
    * @throws AeBusinessProcessException
    */
   private Object getCopyVariableData(AeAssignFromDef aFrom) throws AeBusinessProcessException
   {
      Object data = null;

      // Make sure we can find the variable for our variable processing.
      IAeVariable var = getVariable(aFrom.getVariable());
      if (var == null)
         throw new AeBpelException(VAR_NOT_FOUND, AeFaultFactory.BAD_PROCESS);

      // Set the data based upon the type which has been set, if any
      if (var.getElementData() != null)
         data = var.getElementData();
      else if (var.getTypeData() != null)
         data = var.getTypeData();
      else if (var.getMessageData() != null) // this throws unitialized variable if it is null 
      {
         if (!AeUtil.isNullOrEmpty(aFrom.getPart()))
         {
            // Get the data for the given message part, and process query if necessary
            data = var.getMessageData().getData(aFrom.getPart());
         }
         else if (!AeUtil.isNullOrEmpty(aFrom.getProperty()))
         {
            // Get the property alias for the property and get data for its part
            IAePropertyAlias propAlias = getPropertyAlias(var, aFrom.getProperty());
            if (propAlias == null)
               throw new AeBpelException(NO_PROP_ALIAS, AeFaultFactory.BAD_PROCESS);
            else
            {
               data = var.getMessageData().getData(propAlias.getPart());
               if (data instanceof Document && !AeUtil.isNullOrEmpty(propAlias.getQuery()))
               {
                  Element docElem = ((Document) data).getDocumentElement();
                  data = executeQuery(propAlias.getQuery(), docElem, new AeNamespaceResolver(propAlias));
                  data = AeXPathHelper.unwrapXPathValue(data, isEmptyQuerySelectionAllowed());
               }
            }
         }
         else
         {
            // If no part or property specified we're copying the entire variable contents
            data = var.getMessageData();
         }
      }
      
      // execute the query against the data if not empty
      if (!AeUtil.isNullOrEmpty(aFrom.getQuery()))
      {
         // If data is a document set the context at the root element
         if (data instanceof Document)
            data = ((Document)data).getDocumentElement();
         // Execute query against context data, this MUST return exactly one node
         data = executeQuery(aFrom.getQuery(), data);
         data = AeXPathHelper.unwrapXPathValue(data, isEmptyQuerySelectionAllowed());
      }

      return data;
   }

   /**
    * Sets the data for the target copy operation. 
    * @param aTo the target of the copy operation
    * @param aData the data to be set in the target 
    * @throws AeBusinessProcessException
    */
   private void setCopyData(AeAssignToDef aTo, Object aData) throws AeBusinessProcessException
   {
      if (!AeUtil.isNullOrEmpty(aTo.getVariable()))
      {
         // Make sure we can find the variable for our variable processing.
         IAeVariable var = getVariable(aTo.getVariable());
         if (var == null)
            throw new AeBpelException(VAR_NOT_FOUND, AeFaultFactory.BAD_PROCESS);

         var.dataUpdated();

         if (var.getMessageType() != null)
         {
            // Create target message data container if necessary
            if (!var.hasMessageData())
               var.setMessageData(AeMessageDataFactory.instance().createMessageData(var.getMessageType()));

            if (!AeUtil.isNullOrEmpty(aTo.getPart()))
               setVariablePartData(aTo, aData);
            else if (!AeUtil.isNullOrEmpty(aTo.getProperty()))
               setVariablePropertyData(aTo, aData);
            else
            {
               // When no part name is specified, we are copying the entire message
               // Also make sure message types are compatible types 
               if (aData instanceof IAeMessageData
                  && ((IAeMessageData) aData).getMessageType().equals(var.getMessageType()))
               {
                  // Move the data for each part to the target variable
                  for (Iterator iter = ((IAeMessageData) aData).getPartNames(); iter.hasNext();)
                  {
                     String partName = iter.next().toString();
                     Object data = coerceData(var, partName, ((IAeMessageData) aData).getData(partName));
                     var.getMessageData().setData(partName, data);
                  }
               }
               else
               {
                  throw new AeBpelException(DATA_MISMATCH, AeFaultFactory.MISMATCHED_ASSIGNMENT_FAILURE);
               }
            }
         }
         else if (var.getElement() != null)
         {
            boolean isValid = false;

            // create the element data if it doesn't exist
            if(var.getElementData() == null)
            {
               Document doc = AeXmlUtil.newDocument();
               String prefix = "aetgt"; //$NON-NLS-1$
               String elemName = prefix + ":" + var.getElement().getLocalPart(); //$NON-NLS-1$
               String namespaceDecl = "xmlns:" + prefix; //$NON-NLS-1$
               Element targetElem = doc.createElementNS(var.getElement().getNamespaceURI(), elemName);
               doc.appendChild(targetElem);
               targetElem.setAttributeNS(IAeConstants.W3C_XMLNS, namespaceDecl, var.getElement().getNamespaceURI());
               var.setElementData(targetElem);
            }
         
            // if the source data is a document get its root element
            if(aData instanceof Document)
               aData = ((Document)aData).getDocumentElement();
            
            // check if query on target, otherwise it must be element to element
            if (aData instanceof Element && AeUtil.isNullOrEmpty(aTo.getQuery()))
            {
               // if they're trying to set the entire element, then the src and target
               // elements MUST match, checked during static analysis.
               AeXmlUtil.copyNodeContents((Node)aData, var.getElementData());
               isValid = true;
            }
            else
            {
               setVariableData(aData, var.getElementData().getOwnerDocument(), aTo.getQuery());
               isValid = true;
            }

            if (!isValid)
               throw new AeBpelException(DATA_MISMATCH, AeFaultFactory.MISMATCHED_ASSIGNMENT_FAILURE);

         }
         else if (var.getType() != null)
         {
            Object data = extractData(aData);
            if (data instanceof Element)
               data = AeXmlUtil.getText((Element)data);
            // coerce data if neccesary
            if(data instanceof String)
               data = getProcess().getEngine().getTypeMapping().deserialize(var.getType(), (String)data);
            var.setTypeData(data);
         }
      }
      else if (!AeUtil.isNullOrEmpty(aTo.getPartnerLink()))
      {
         AePartnerLink partnerLink = getPartnerLink(aTo.getPartnerLink());
         if (partnerLink != null)
         {
            // Only possible to set the partner reference during assignment
            boolean illegalAssign = false;
            if (partnerLink.getPartnerReference() == null)
               illegalAssign = true;
            else if (aData instanceof IAeEndpointReference)
               partnerLink.getPartnerReference().setReferenceData((IAeEndpointReference) aData);
            else if (aData instanceof Element)
               partnerLink.getPartnerReference().setReferenceData((Element)aData);
            else if (aData instanceof Document)
               partnerLink.getPartnerReference().setReferenceData(((Document)aData).getDocumentElement());
            else
               illegalAssign = true;
            
            // Signal a fault if we attempted an illegal assignment
            if (illegalAssign)
               throw new AeBpelException(DATA_MISMATCH, AeFaultFactory.MISMATCHED_ASSIGNMENT_FAILURE);
         }
      }
   }

   /**
    * Sets the variable data for a copy operation with a query argument. 
    * @param aTo the target of the copy operation
    * @param aData the data to be set in the target 
    * @throws AeBusinessProcessException
    */
   private void setVariablePartData(AeAssignToDef aTo, Object aData) throws AeBusinessProcessException
   {
      // If no query component, set entire contents with data
      IAeVariable var = getVariable(aTo.getVariable());
      if (AeUtil.isNullOrEmpty(aTo.getQuery()))
      {
         Object data = coerceData(var, aTo.getPart(), extractData(aData));
         var.getMessageData().setData(aTo.getPart(), data);
      }
      else
      {
         // Dealing with query, make sure we're operating against a DOM
         Object partData = var.getMessageData().getData(aTo.getPart());

         if (partData == null)
         {
            Document doc = (new AeXMLParserBase()).createDocument();
            partData = doc;
            var.getMessageData().setData(aTo.getPart(), partData);
         }
         else if (!(partData instanceof Document))
         {
            throw new AeBadXPathException();
         }
         setVariableData(aData, (Document)partData, aTo.getQuery());
      }
   }

   /**
    * Copies the source data to the location within the target document specified
    * by the query.
    * @param aSourceData
    * @param aTargetDoc
    * @param aQuery
    * @throws AeBusinessProcessException
    * @throws AeBadXPathException
    */
   protected void setVariableData(Object aSourceData, Document aTargetDoc, String aQuery) throws AeBusinessProcessException, AeBadXPathException
   {
      boolean dataModified = false;
      
      Node tgtNode = null;

      // if we have a document element then search for the path
      if (aTargetDoc.getDocumentElement() != null)
      {
         // Locate the proper node from the query
         Object selection = executeQuery(aQuery, aTargetDoc.getDocumentElement());
         if (selection instanceof List && ((List) selection).size() == 1)
            tgtNode = (Node) ((List) selection).get(0);
      }

      // if we didn't find the node and the config says create then create it
      if (tgtNode == null && allowCreateXPath())
         tgtNode = findOrCreateXPath(aQuery, aTargetDoc);

      // if we have a node then assign it
      if (tgtNode != null)
      {
         // Remove all child nodes before replacing with our data, only remove attributes
         // if we will be replacing with a node
         Object data = extractData(aSourceData);
         AeXmlUtil.removeNodeContents(tgtNode, (data instanceof Node));

         // Create appropriate target node. Note if src was attribute we just grab its value 
         if(tgtNode.getNodeType() == Node.ATTRIBUTE_NODE)
         {
            if (data instanceof Element)
               tgtNode.setNodeValue(AeXmlUtil.getText((Element)data));
            else if(data != null)
               tgtNode.setNodeValue(data.toString());
            else
               ((Attr)tgtNode).getOwnerElement().removeAttributeNode((Attr)tgtNode);
         }
         else
         {
            if (data instanceof Document)
            {
               // change the src to be the document element or we'll have 
               // problems with importing a document into the target node.
               data = ((Document)data).getDocumentElement();
            }
            
            if (data instanceof Node)
               AeXmlUtil.copyNodeContents((Node) data, tgtNode);
            else if (data != null)
            {
               if (tgtNode instanceof Text)
               {
                  tgtNode.setNodeValue(data.toString());
               }
               else
               {
                  tgtNode.appendChild(tgtNode.getOwnerDocument().createTextNode(data.toString()));
               }
            }
            else
               tgtNode.getParentNode().removeChild(tgtNode);
         }
         dataModified = true;
      }

      // Data was not modified during operation, must be an error in selection
      if (!dataModified)
         throw new AeBadXPathException();
   }

   /**
    * Sets the variable data for a copy operation whose target is a variable property. 
    * @param aTo the target of the copy operation
    * @param aData the data to be set in the target 
    * @throws AeBusinessProcessException
    */
   private void setVariablePropertyData(AeAssignToDef aTo, Object aData) throws AeBusinessProcessException
   {
      IAeVariable var = getVariable(aTo.getVariable());
      IAePropertyAlias propAlias = getPropertyAlias(var, aTo.getProperty());
      if (propAlias == null)
         throw new AeBpelException(NO_PROP_ALIAS, AeFaultFactory.BAD_PROCESS);
      else
      {
         // If no data is set in the target variable need to check if we can just
         // set new data. If the target is a simple message part, indicated by
         // no query arg in prop alias then this is legal. If there is a query arg 
         // then this is considered a selection failure error.
         Object data = var.getMessageData().getData(propAlias.getPart());
         if (data == null)
         {
            String query = propAlias.getQuery();
            if (query != null && query.equals("/" + propAlias.getPart())) //$NON-NLS-1$
               query = null;
               
            // if no query then set the part data, otherwise we have a problem
            if (query == null)
            {
               var.getMessageData().setData(propAlias.getPart(), aData);
            }
            else
            {
               // @todo we should create xpaths here if we are configured as such
               throw new AeBadXPathException();
            }
         }
         else if (data instanceof Document && !AeUtil.isNullOrEmpty(propAlias.getQuery()))
         {
            Element docElem = ((Document) data).getDocumentElement();
            data = executeQuery(propAlias.getQuery(), docElem, new AeNamespaceResolver(propAlias));
            if (data instanceof List)
            {
               // property should only evaluate to a single, node otherwise it is an error
               if (((List) data).size() != 1)
                  throw new AeBadXPathException();
               else if (((List) data).get(0) instanceof Node)
               {
                  Node tgtNode = (Node) ((List) data).get(0);
                  // should be element or attribute
                  if(tgtNode.getNodeType() == Node.ELEMENT_NODE)
                  {
                     // Remove all child nodes before replacing with our data
                     AeXmlUtil.removeNodeContents(tgtNode, false);
                     tgtNode.appendChild(tgtNode.getOwnerDocument().createTextNode(extractData(aData).toString()));
                  }
                  else if(tgtNode.getNodeType() == Node.ATTRIBUTE_NODE)
                  {
                     ((Attr)tgtNode).setValue(extractData(aData).toString());
                  }
                  else
                  {
                     throw new AeBadXPathException();
                  }
               }
            }
         }
      }
   }

   /**
    * Method used to extract data from given data object. When data represents a 
    * Node element and only contains text nodes, a concatentation of the data will
    * be returned. Otherwise the element itself will be returned. For PartnerLink
    * references, a document fragment of the reference will be returned. 
    * @param aData The data which may need to be extracted.
    */
   private Object extractData(Object aData)
   {
      Object extractedData = aData;
      if (aData instanceof IAeEndpointReference)
      {
         extractedData = ((IAeEndpointReference) aData).toDocument();
      }
      else if (aData instanceof Node)
      {
         // For attributes get the underlying data, and doc fragments get the first element         
         Node node = (Node)aData;
         if (node.getNodeType() == Node.ATTRIBUTE_NODE ||
             node.getNodeType() == Node.TEXT_NODE ||
             node.getNodeType() == Node.CDATA_SECTION_NODE)
            extractedData = node.getNodeValue();
         else if (node.getNodeType() == Node.DOCUMENT_FRAGMENT_NODE || node.getNodeType() == Node.DOCUMENT_NODE)
            extractedData = AeXmlUtil.getFirstSubElement((Node)aData); 
      }
      
      // @todo should we check type before coercing to string for reformatting
      if(extractedData != null && !(extractedData instanceof Node))
         extractedData = getProcess().getEngine().getTypeMapping().serialize(extractedData);

      return extractedData;
   }

   /**
    * Used during the copy of DOM data between message parts. If copying
    * from an element to a type it may be necessary to insert a type node
    * at the root. This will also clone the document if this is a doc part copy.
    *  
    * @param aTargetVar the variable we are copying to 
    * @param aPart the part of the variable we are copying to
    * @param aData the data to be copied
    */
   private Object coerceData(IAeVariable aTargetVar, String aPart, Object aData) throws AeBusinessProcessException
   {
      boolean cloned = false;
      if (aData instanceof Element)
      {
         // if it is an element or a type which is complex or any then handle as xml 
         if ( aTargetVar.getDefinition().isPartElement(aPart) || 
              AeXmlUtil.isComplexOrAny(aTargetVar.getDefinition().getPartType(aPart)) )
         {
            Node srcNode = (Node) aData; 
            Document doc = (new AeXMLParserBase()).createDocument();
            Element tgtElement = doc.createElementNS(srcNode.getNamespaceURI(), srcNode.getNodeName());
            doc.appendChild(tgtElement);
            AeXmlUtil.copyNodeContents(srcNode, tgtElement);
            aData = doc;
            cloned = true;
         }
         else
         {
            aData = AeXmlUtil.getText((Element)aData);
         }
      }

      // process document data
      if (aData instanceof Document)
      {
         if(! aTargetVar.getDefinition().isPartElement(aPart))
         {
            // If type node is not present we must now create it 
            // @todo isn't this already taken care over by receive and invoke processing
            Element docElement = ((Document)aData).getDocumentElement();
            if (! aPart.equals(docElement.getLocalName()))
            {
               Document doc = (new AeXMLParserBase()).createDocument();
               Element newRoot = doc.createElementNS(null, aPart);
               doc.appendChild(newRoot);
               AeXmlUtil.copyNodeContents(docElement, newRoot);
               aData = doc;
               cloned = true;
            }
         }
         // clone the document for new part if it isn't already a copy
         if(! cloned)
            aData = ((Document)aData).cloneNode(true); 
      }
      else if(aData instanceof String)
      {
         XMLType type = aTargetVar.getDefinition().getPartType(aPart);
         aData = getProcess().getEngine().getTypeMapping().deserialize(type, (String)aData);
      }

      return aData;
   }

   /**
    * Returns a flag indicating if the data associated with the variable is valid
    * or not.
    * @param aVariable the variable we are checking
    */
   private boolean isDataValid(IAeVariable aVariable)
   {
      boolean valid = true;

      // TODO - Hook in validation code
      //      if (aVariable.getMessageData() != null)
      //      {
      //         for (Iterator iter=aVariable.getMessageData().getPartNames(); iter.hasNext();)
      //         {
      //            String partName = iter.next().toString();            
      //         }
      //      }

      return valid;
   }

   /**
    * Returns the property alias for a given variable and property. This method
    * returns null if alias cannot be found. 
    * @param aVar the variable we are referencing
    * @param aProperty the property we are looking for (QName qualified)
    */
   private IAePropertyAlias getPropertyAlias(IAeVariable aVar, String aProperty) throws AeBusinessProcessException
   {
      IAePropertyAlias propAlias = null;

      QName msgType = aVar.getDefinition().getMessageType();
      QName msgProp = parseQName(aProperty);

      if (msgType != null && msgProp != null)
         propAlias = getMessageProperty(msgType, msgProp);

      return propAlias;
   }

   /**
    * Makes a backup copy of the variable data which may be restored in case of
    * failure during any part of operation. 
    * @param aDataMap the data map to be populated
    * @param aTo the target of the assign copy operation
    */
   private void storeData(Map aDataMap, AeAssignToDef aTo) throws AeBusinessProcessException
   {
      if (!AeUtil.isNullOrEmpty(aTo.getVariable()))
      {
         // Only store variable data if it has not already been mapped 
         IAeVariable var = getVariable(aTo.getVariable());
         if (var != null && aDataMap.get(var) == null)
         {
            AeVariableDef def = var.getDefinition();
            if (def != null)
            {
               if (var.hasMessageData())
                  aDataMap.put(var, var.getMessageData());
               else if (def.getElement() != null)
                  aDataMap.put(var, var.getElementData());
               else if (def.getType() != null)
                  aDataMap.put(var, var.getTypeData());
            }
         }
      }
      else if (!AeUtil.isNullOrEmpty(aTo.getPartnerLink()))
      {
         // Only store partner link data if it has not already been mapped 
         AePartnerLink partnerLink = getPartnerLink(aTo.getPartnerLink());
         if (partnerLink != null && aDataMap.get(partnerLink) == null)
            aDataMap.put(partnerLink, partnerLink.getPartnerReference());
      }
   }

   /**
    * Restore the data for the variables contained in our data map.
    * @param aDataMap contains data mappings to be restored
    */
   private void restoreData(Map aDataMap)
   {
      for (Iterator iter = aDataMap.keySet().iterator(); iter.hasNext();)
      {
         Object key = iter.next();
         Object data = aDataMap.get(key);
         if (key instanceof AePartnerLink)
             ((AePartnerLink) key).getPartnerReference().setReferenceData((IAeEndpointReference) data);
         else if (key instanceof IAeVariable)
         {
            AeVariableDef def = ((IAeVariable)key).getDefinition();
            if (def.getMessageType() != null)
                ((IAeVariable) key).setMessageData((IAeMessageData) data);
            else if (def.getElement() != null)
                ((IAeVariable) key).setElementData((Element)data);
            else
                ((IAeVariable) key).setTypeData(data);
         }
      }
   }

   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   // TODO The following code was extracted/modified from win32/AeMessageObject and should be refactored to rt layer for resuse.
   // This parser should be moved into a standalone class.
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

   /**
    * Finds or creates the node indicated by the passed path and returns it.
    * Acceptable paths are of a form /a/b[#index]/@abc.
    * @param aPath The path to find or create.
    * @param aRoot The context root to create the path within.
    * @return The node found or created matching the path.
    * @throws AeBusinessProcessException Thrown if problem with path.
    */
   protected Node findOrCreateXPath(String aPath, Node aRoot) throws AeBusinessProcessException
   {
      StringTokenizer strtok = new StringTokenizer(aPath, "/[]@", true); //$NON-NLS-1$
      String nextToken;
      Node currentNode = aRoot;
      while (strtok.hasMoreTokens())
      {
         nextToken = strtok.nextToken();
         if (nextToken.equals("/")) //$NON-NLS-1$
         {
            ; // ignore
         }
         else if (nextToken.equals("]")) //$NON-NLS-1$
         {
            ; // ignore
         }
         else if (nextToken.equals("[")) //$NON-NLS-1$
         {
            if (strtok.hasMoreTokens())
            {
               String indexExpr = strtok.nextToken().trim();
               currentNode = createIndexedNode(indexExpr, currentNode);
            }
            else
            {
               throw new AeBusinessProcessException(AeMessages.getString("AeActivityAssignImpl.ERROR_9")); //$NON-NLS-1$
            }
         }
         else if (nextToken.equals("@")) //$NON-NLS-1$
         {
            // if we have more tokens then get the attribute name
            if (strtok.hasMoreTokens())
            {
               // grab the attribute name qualifier
               nextToken = strtok.nextToken();
               currentNode = createAttributeNode(nextToken, currentNode);
            }
         }
         else
         {
            currentNode = createElementNode(nextToken, currentNode);
         }
      }

      return currentNode;
   }
   
   /**
    * Parses the next token for creation of an element node in the 
    * passed currentNode.
    * @param aToken
    * @param aCurrentNode
    * @return Element the new element node
    * @throws AeBusinessProcessException if the node can not be created
    */
   protected Element createElementNode(String aToken, Node aCurrentNode) 
      throws AeBusinessProcessException
   {
      Element newElement = null;
      
      // get the namespace info based on path (may be empty - null)
      String localName, namespaceURI, prefix;
      int colonOffset;
      if ((colonOffset = aToken.indexOf(":")) >= 0) //$NON-NLS-1$
      {
         if (colonOffset > 0 && colonOffset < (aToken.length() - 1))
         {
            prefix = aToken.substring(0, colonOffset);
            localName = aToken.substring(colonOffset + 1);
            namespaceURI = translateNamespacePrefixToUri(prefix);
         }
         else
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeActivityAssignImpl.ERROR_12") + aToken); //$NON-NLS-1$
         }
      }
      else
      {
         localName = aToken;
         namespaceURI = null;
         prefix = null;
      }
      
      // check if we are at root of document or in an element
      if (aCurrentNode.getNodeType() == Node.DOCUMENT_NODE)
      {
         Document doc = (Document)aCurrentNode;
         if (doc.getDocumentElement() == null)
         {
            newElement = doc.createElementNS(namespaceURI, aToken);
            doc.appendChild(newElement);
            if (namespaceURI != null)
               newElement.setAttributeNS(IAeConstants.W3C_XMLNS, "xmlns:" + prefix, namespaceURI); //$NON-NLS-1$
         }
         else
         {
            // TODO make sure they didn't mess up and have a different name or namespace
            newElement = doc.getDocumentElement();
         }
      }
      else if(aCurrentNode.getNodeType() == Node.ELEMENT_NODE)
      {
         Element element = (Element) aCurrentNode;
         boolean found = false;
         for (Node child = element.getFirstChild();(!found) && (child != null); child = child.getNextSibling())
         {
            if (localName.equals(child.getLocalName()))
            {
               // if both null or matching
               if (AeUtil.compareObjects(namespaceURI, child.getNamespaceURI()))
               {
                  newElement = (Element)child;
                  found = true;
                  break;
               }
            }
         }
         // if we didn't find the node then create it now
         if (!found)
         {
            // check if namespace has a prefix and if so use it
            if(namespaceURI != null)
            {
               String newPrefix = AeXmlUtil.getPrefixForNamespace(element, namespaceURI);
               if(newPrefix == null)
               {
                  newElement = aCurrentNode.getOwnerDocument().createElementNS(namespaceURI, aToken);
                  newElement.setAttributeNS(IAeConstants.W3C_XMLNS, "xmlns:" + prefix, namespaceURI); //$NON-NLS-1$
               }
               else if(newPrefix.equals("")) //$NON-NLS-1$
               {
                  newElement = aCurrentNode.getOwnerDocument().createElementNS(namespaceURI, localName);
               }
               else
               {
                  newElement = aCurrentNode.getOwnerDocument().createElementNS(namespaceURI, newPrefix + ":" + localName); //$NON-NLS-1$
               }
            }
            else
            {
               newElement = aCurrentNode.getOwnerDocument().createElementNS(namespaceURI, localName);
            }
            element.appendChild(newElement);
         }
      }
      else
      {
         throw new AeBusinessProcessException(AeMessages.getString("AeActivityAssignImpl.ERROR_17") + aToken); //$NON-NLS-1$
      }
      return newElement;
   }
   
   /**
    * Parses the next token for creation of an attribute node in the 
    * passed currentNode.
    * @param aToken
    * @param aCurrentNode
    * @return Attr the new attribute node
    * @throws AeBusinessProcessException if the node can not be created
    */
   protected Attr createAttributeNode(String aToken, Node aCurrentNode) 
      throws AeBusinessProcessException
   {
      Attr newAttribute = null;
      
      // get the namespace info based on path (may be empty - null)
      String localName, namespaceURI, prefix;
      int colonOffset;
      if ((colonOffset = aToken.indexOf(":")) >= 0) //$NON-NLS-1$
      {
         if (colonOffset > 0 && colonOffset < (aToken.length() - 1))
         {
            prefix = aToken.substring(0, colonOffset);
            localName = aToken.substring(colonOffset + 1);
            namespaceURI = translateNamespacePrefixToUri(prefix);
         }
         else
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeActivityAssignImpl.ERROR_19") + aToken); //$NON-NLS-1$
         }
      }
      else
      {
         localName = aToken;
         namespaceURI = null;
         prefix = null;
      }
      
      // check that we are at an element, since this is an attribute
      if (aCurrentNode.getNodeType() == Node.ELEMENT_NODE)
      {
         Element element = (Element)aCurrentNode;
         // check if namespace has a prefix and if so use it
         if(namespaceURI != null)
         {
            String newPrefix = AeXmlUtil.getPrefixForNamespace(element, namespaceURI);
            if(AeUtil.isNullOrEmpty(newPrefix))
            {
               newAttribute = element.getOwnerDocument().createAttributeNS(namespaceURI, aToken);
               element.setAttributeNS(IAeConstants.W3C_XMLNS, "xmlns:" + prefix, namespaceURI); //$NON-NLS-1$
            }
            else
            {
               newAttribute = aCurrentNode.getOwnerDocument().createAttributeNS(namespaceURI, newPrefix + ":" + localName); //$NON-NLS-1$
            }
         }
         else
         {
            newAttribute = aCurrentNode.getOwnerDocument().createAttributeNS(namespaceURI, localName);
         }
         element.setAttributeNodeNS(newAttribute);
      }
      else
      {
         throw new AeBusinessProcessException(AeMessages.getString("AeActivityAssignImpl.ERROR_22") + aToken); //$NON-NLS-1$
      }
      return newAttribute;
   }
   
   /**
    * Accepts an expression representing an index number for the replication
    * or finding within the parent of the passed node.
    * @param aIndexExpr The index expression to evaluate
    * @param aCurrentNode The node which will be replicated until index is reached
    * @return the indexed node
    * @throws AeBusinessProcessException If error creating or finding the nodes
    */
   protected Node createIndexedNode(String aIndexExpr, Node aCurrentNode) throws AeBusinessProcessException
   {
      // get the index of the element we are to find or create
      int index = convertIndex(aIndexExpr, aCurrentNode);
      
      // check the children to see if we already have the indexed element
      Node parent = aCurrentNode.getParentNode();
      boolean found = false;
      int count = 0;
      for (Node child = parent.getFirstChild(); child != null; child = child.getNextSibling())
      {
         if (aCurrentNode.getLocalName().equals(child.getLocalName()))
         {
            if (AeUtil.compareObjects(aCurrentNode.getNamespaceURI(), child.getNamespaceURI()))
            {
               if (++count == index)
               {
                  aCurrentNode = child;
                  found = true;
                  break;
               }
            }
         }
      }
      
      // if indexed element not found then create all neccesary elements inbetween
      if (! found)
      {
         String newPrefix = null;
         String namespaceURI = aCurrentNode.getNamespaceURI();
         String newName = null;
         // check if namespace has a prefix and if so use it
         if(namespaceURI != null)
         {
            newPrefix = AeXmlUtil.getPrefixForNamespace((Element)parent, namespaceURI);
            if(newPrefix == null)
            {
               newPrefix = "ans"; //$NON-NLS-1$
               newName = newPrefix + ":" + aCurrentNode.getLocalName(); //$NON-NLS-1$
            }
            else if(newPrefix.equals("")) //$NON-NLS-1$
            {
               newName = aCurrentNode.getLocalName();
               newPrefix = null;
            }
            else
            {
               newName = newPrefix + ":" + aCurrentNode.getLocalName(); //$NON-NLS-1$
            }
         }
         else
         {
            newName = aCurrentNode.getLocalName();
         }
         // create new entries up to and including the index requested
         Document doc = aCurrentNode.getOwnerDocument();
         while (count < index)
         {
            Element element = doc.createElementNS(namespaceURI, newName);
            if(newPrefix != null)
               element.setAttributeNS(IAeConstants.W3C_XMLNS, "xmlns:" + newPrefix, namespaceURI); //$NON-NLS-1$
            aCurrentNode = parent.appendChild(element);
            count++;
         }
      }

      return aCurrentNode;
   }
      
   /**
    * Converts the passed expression string to the integer number it represents.
    * @param aExpr the expression to convert
    * @return int the index value
    */
   public int convertIndex(String aExpr, Node aContext) throws AeBusinessProcessException
   {
      int index = 0;
      boolean numberExpr = true;
      
      // check each character for digits if not break and return
      for (int i = 0; i < aExpr.length(); ++i)
      {
         int digit = Character.digit(aExpr.charAt(i), 10);
         if (digit >= 0)
         {
            index = (index * 10) + digit;
         }
         else
         {
            numberExpr = false;
            break;
         }
      }

      // if this was not just a numeric expression then evaluate through rhino scope
      if(! numberExpr)
      {
         try
         {
            Object data = AeXPathHelper.executeXPathExpression(aExpr, aContext, 
                  new AeXPathFunctionContext(createFunctionExecutionContext(aContext)),
                  new AeXPathNamespaceContext(this, (Node) aContext));
            data = AeXPathHelper.unwrapXPathValue(data, isEmptyQuerySelectionAllowed());
            if(data instanceof Number)
               index = ((Number)data).intValue();
            else
            	// Was index = Integer.parseInt(data.toString());
            	// except that one of the test cases processes constructs the variable offset dynamically
            	// and XPath returns a double.
               index = Double.valueOf(data.toString()).intValue();
         }
         catch(Exception ex)
         {
            throw new AeBusinessProcessException(AeMessages.getString("AeActivityAssignImpl.ERROR_28") + aExpr + " : " + ex.getMessage(), ex); //$NON-NLS-1$ //$NON-NLS-2$
         }
      }

      return index;
   }
   
   /**
    * Return true if the mutable config allows XPath creation.
    */
   protected boolean allowCreateXPath()
   {
      return getProcess().getEngine().getEngineConfiguration().getUpdatableEngineConfig().allowCreateXPath();
   }
}
