//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/AeActivityForEachParallelImpl.java,v 1.7 2006/02/16 01:17:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity; 

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeActivity;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.activity.AeActivityForEachDef;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.def.visitors.AeDefPathParallelVisitor;
import org.activebpel.rt.bpel.def.visitors.AeDefToImplVisitor;
import org.activebpel.rt.bpel.impl.AeAbstractBpelObject;
import org.activebpel.rt.bpel.impl.AeBusinessProcess;
import org.activebpel.rt.bpel.impl.AeFaultFactory;
import org.activebpel.rt.bpel.impl.AeVariable;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.visitors.AeImplTraversingVisitor;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;

/**
 * An extension of the forEach activity that executes all of its iterations
 * in parallel.
 */
public class AeActivityForEachParallelImpl extends AeActivityForEachImpl
{
   /** list of child scope instances created during execute routine */
   private List mChildren = new ArrayList();

   /** list of child scope instances that have been restored for compensation purposes */
   private List mCompensatableChildren = new ArrayList();

   /** value for the next scope instance created for this parallel forEach */
   private int mInstanceValue = 1;
   
   /**
    * @param aForEachDef
    * @param aParent
    */
   public AeActivityForEachParallelImpl(AeActivityForEachDef aForEachDef, IAeActivityParent aParent)
   {
      super(aForEachDef, aParent);
   }
   
   /**
    * @return Returns the instance value.
    */
   public int getInstanceValue()
   {
      return mInstanceValue;
   }

   /**
    * @param aInstanceValue The instance value to set.
    */
   public void setInstanceValue(int aInstanceValue)
   {
      mInstanceValue = aInstanceValue;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept(IAeImplVisitor aVisitor)
         throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeActivityParent#addActivity(org.activebpel.rt.bpel.IAeActivity)
    */
   public void addActivity(IAeActivity aActivity)
   {
      getChildren().add(aActivity);
   }
   
   /**
    * Getter for the collection of children
    */
   public List getChildren()
   {
      return mChildren;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableQueueItem#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      // Record the compensatable scopes locally within a separate collection.
      // This ensures that the process state document will contain any references
      // to compensating scopes. The use case is a process state document getting 
      // built after the forEach has executed multiple times but before any state
      // restoration. The result is a state document which won't contain any info
      // for the currently compensating scope. By adding the compensatable scopes
      // to the separate collection, they'll get persisted. 
      for (Iterator iter = getChildren().iterator(); iter.hasNext();)
      {
         AeActivityScopeImpl scope = (AeActivityScopeImpl) iter.next();
         if (scope.isNormalCompletion())
         {
            getCompensatableChildren().add(scope);
         }
      }
      getChildren().clear();
      super.execute();
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl#startLoop()
    */
   protected void startLoop() throws AeBusinessProcessException
   {
      // we only have work to do if the counter value is w/in range
      if (isCounterWithinRange())
      {
         // creates all of the children
         int startInstance = getInstanceValue();
         int finalInstance = startInstance + (getFinalValue() - getStartValue());
         createChildren(true, startInstance, finalInstance);
         // set the instance value for the next time we execute
         setInstanceValue(finalInstance + 1);
         setCounterValue(getFinalValue() + 1);

         for (Iterator it=getChildren().iterator(); it.hasNext();)
         {
            getProcess().queueObjectToExecute((IAeBpelObject) it.next());
         }
      }
      else
      {
         objectCompleted();
      }
   }
   
   /**
    * Creates the implementation objects for the children based on the child def
    * for this forEach. This method is used during the execution of the activity
    * and during the restoring of the activity from a process state document.
    * The only differences relate to notifying the process of the newly created
    * objects and incrementing the process's maxLocationId value. In the case of
    * restoring a forEach, we don't want to notify the process of the newly restored
    * objects until their full state has been restored and they have the correct
    * location ids. 
    * 
    * @param aCreateModeFlag differentiates between create mode and restore mode.
    * @param aStartInstance starting instance value to create
    * @param aFinalInstance final instance value to create
    * @throws AeBusinessProcessException
    */
   public void createChildren(boolean aCreateModeFlag, int aStartInstance, int aFinalInstance) throws AeBusinessProcessException
   {
      // our custom visitor for creating the impl's
      AeDefToImplParallelVisitor objectCreationVisitor = new AeDefToImplParallelVisitor((AeBusinessProcess) getProcess(), this);
      
      int counter = getCounterValue();
      
      // keep looping for all of the iterations 
      for( int i=aStartInstance; i<= aFinalInstance; i++)
      {
         // create the child scope impl and all of its descendents
         getDef().getChildScope().accept(objectCreationVisitor);
         
         // get the most recently added child scope
         AeActivityScopeImpl instanceScope = (AeActivityScopeImpl) getChildren().get(getChildren().size()-1);
         // visit that instance w/ an impl visitor to set the location paths and ids on all
         // of the activities and other implementation objects (like variables and correlation sets)
         AeLocationPathImplVisitor locationVisitor =
                  new AeLocationPathImplVisitor((AeBusinessProcess) getProcess(), 
                                       instanceScope, i, aCreateModeFlag);
         locationVisitor.startVisiting();
         
         IAeVariable variable = (IAeVariable) instanceScope.getVariable(getDef().getCounterName());
         variable.setTypeData(new Integer(counter++));
      }
      
      if (aCreateModeFlag)
      {
         // make the process aware of all of the variables we've created
         objectCreationVisitor.reportVariables();
         objectCreationVisitor.reportBpelObjects();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childComplete(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void childComplete(IAeBpelObject aChild)
         throws AeBusinessProcessException
   {
      handleLoopCompletion(aChild);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl#completedIteration()
    */
   protected void completedIteration() throws AeBusinessProcessException
   {
      if (childrenAreDone())
      {
         objectCompleted();
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl#getNumberOfIterationsRemaining()
    */
   protected int getNumberOfIterationsRemaining()
   {
      int stillExecuting = 0;
      for (Iterator iter = getChildren().iterator(); iter.hasNext();)
      {
         AeActivityScopeImpl scope = (AeActivityScopeImpl) iter.next();
         if (!scope.getState().isFinal())
         {
            stillExecuting++;
         }
      }
      return stillExecuting;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl#completionConditionFailure()
    */
   protected void completionConditionFailure() throws AeBusinessProcessException
   {
      setFault(AeFaultFactory.getFault(AeFaultFactory.FOREACH_COMPLETION_CONDITION_FAILURE));
      terminateNextChild();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl#completionConditionMet()
    */
   protected void completionConditionMet() throws AeBusinessProcessException
   {
      runEarlyTerminationOnChildren();
   }

   /**
    * A continue call causes the parent instance of the loop control to terminate
    * but does not affect any other instances.
    * 
    * @see org.activebpel.rt.bpel.impl.activity.support.IAeLoopActivity#onContinue(org.activebpel.rt.bpel.impl.activity.IAeLoopControl)
    */
   public void onContinue(IAeLoopControl aControl)
         throws AeBusinessProcessException
   {
      AeActivityScopeImpl impl = (AeActivityScopeImpl) findScopeInstance(aControl);
      
      impl.terminateEarly();
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.activity.support.IAeLoopActivity#onBreak(org.activebpel.rt.bpel.impl.activity.IAeLoopControl)
    */
   public void onBreak(IAeLoopControl aControl)
         throws AeBusinessProcessException
   {
      runEarlyTerminationOnChildren();
   }

   /**
    * Walks all of the children and calls <code>terminateEarly</code> on all that
    * are eligible. This is called from the break and when the optional completionCondition
    * evaluates to true.
    */
   protected void runEarlyTerminationOnChildren() throws AeBusinessProcessException
   {
      boolean terminatedChild = false;
      for (Iterator iter = getChildren().iterator(); iter.hasNext();)
      {
         AeActivityScopeImpl impl = (AeActivityScopeImpl) iter.next();
         if (!impl.getState().isFinal())
         {
            impl.terminateEarly();
            terminatedChild = true;
         }
      }
      
      // if we didn't actually terminate any children, then we should just
      // signal that we're done.
      if (!terminatedChild)
      {
         objectCompleted();
      }
   }
   
   /**
    * Finds the enclosing scope for the object that is also our child instance.
    * 
    * @param aObject
    */
   protected IAeBpelObject findScopeInstance(IAeBpelObject aObject)
   {
      if (aObject.getParent() == this)
      {
         return aObject;
      }
      else
      {
         return findScopeInstance(aObject.getParent());
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getChildrenForStateChange()
    */
   public Iterator getChildrenForStateChange()
   {
      return getChildren().iterator();
   }
   
   /**
    * Getter for the list of children that are compensatable
    */
   public List getCompensatableChildren()
   {
      return mCompensatableChildren;
   }

   /**
    * Modified version of the def to impl visitor that defers notifying the process
    * of the instances created until we've had a chance to set their proper location
    * paths and location ids.  
    */
   protected static class AeDefToImplParallelVisitor extends AeDefToImplVisitor
   {
      /** collection of variables we've created */
      private Collection mVariables = null;
      /** collection of bpel objects we've created */
      private Collection mBpelObjects = new ArrayList();
      
      /**
       * Primes the pump with the forEach as the parent
       * 
       * @param aProcess
       * @param aParent
       */
      public AeDefToImplParallelVisitor(AeBusinessProcess aProcess, AeActivityForEachParallelImpl aParent)
      {
         setProcess(aProcess);
         push(aParent);
         init();
      }
      
      /**
       * We keep track of the variables that get created but defer notifying the process
       * until we've had a chance to update their location paths.
       * 
       * @see org.activebpel.rt.bpel.def.visitors.AeDefToImplVisitor#recordVariableWithProcess(org.activebpel.rt.bpel.impl.AeVariable)
       */
      protected void recordVariableWithProcess(AeVariable aVar)
      {
         if (mVariables == null)
         {
            mVariables = new ArrayList();
         }
         mVariables.add(aVar);
      }
      
      /**
       * @see org.activebpel.rt.bpel.def.visitors.AeDefToImplVisitor#recordBpelObjectWithProcess(org.activebpel.rt.bpel.def.AeBaseDef, java.lang.Object)
       */
      protected void recordBpelObjectWithProcess(AeBaseDef aDef, Object aImpl)
      {
         mBpelObjects.add(aImpl);
      }

      /**
       * Calls into the process to notify it of the newly created variables 
       */
      public void reportVariables()
      {
         if (mVariables != null)
         {
            for (Iterator iter = mVariables.iterator(); iter.hasNext();)
            {
               AeVariable variable = (AeVariable) iter.next();
               getProcess().addVariableMapping(variable.getLocationPath(), variable);
            }
         }
      }
      
      /**
       * Calls into the process to notify it of the newly created bpel objects 
       */
      public void reportBpelObjects()
      {
         for (Iterator iter = mBpelObjects.iterator(); iter.hasNext();)
         {
            IAeBpelObject obj = (IAeBpelObject) iter.next();
            getProcess().addBpelObject(obj.getLocationPath(), obj);
         }
      }
   }
   
   /**
    * Extension of the location path visitor that creates a custom path for the
    * root scope of the forEach. All of the other nodes visited will get their
    * standard path which will include the scope's special instance predicate which
    * serves to make the paths unique. 
    */
   protected static class AeImplPathParallelVisitor extends AeDefPathParallelVisitor
   {
      /** map of def objects to their path information */
      private Map mDefToPathMap = new HashMap();
      /** reference to the process in case we need to increment the maxLocationId on the process  */
      private AeBusinessProcess mProcess;
      /** flag that tells us if we're in create mode or not */
      private boolean mCreateMode;
      
      /**
       * Ctor
       * 
       * @param aProcess provides the max location id and allows us to call back into it to increment that id as necessary
       * @param aBasePath the base path that provides the context of our instance
       * @param aDef the root def for our instance path
       * @param aCreateFlag if true, we're creating new instances as opposed to restoring existing ones
       */
      public AeImplPathParallelVisitor(AeBusinessProcess aProcess, String aBasePath, AeBaseDef aDef, boolean aCreateFlag)
      {
         super(aBasePath, aDef);
         mProcess = aProcess;
         setNextLocationId(mProcess.getMaxLocationId() + 1);
         mCreateMode = aCreateFlag;
      }
            
      /**
       * @see org.activebpel.rt.bpel.def.visitors.AeDefPathVisitor#recordLocationPathAndId(org.activebpel.rt.bpel.def.AeBaseDef, java.lang.String, int)
       */
      protected void recordLocationPathAndId(AeBaseDef aDef,
            String aLocationPath, int aLocationId)
      {
         // the mapping of location path to id will be done already
         // we just need to record the mapping of the def to its location path
         mDefToPathMap.put(aDef, aLocationPath);
      }
      
      /**
       * Getter for the location path
       * 
       * @param aDef
       */
      protected String getLocationPath(AeBaseDef aDef)
      {
         return (String) mDefToPathMap.get(aDef);
      }
      
      /**
       * If we're being called as part of the forEach's execution, then we create
       * the objects and set their location ids in place. We callback onto the
       * process to record each new location id since the process will need to track
       * the ids to ensure that they're unique.
       * 
       * If we're being called from the restoreChildren() method then we create
       * the child instances with a location id of -1 since the real id will be
       * set by the restore visitor from the state document. 
       * 
       * @see org.activebpel.rt.bpel.def.visitors.AeDefPathVisitor#getNextLocationId()
       */
      protected int getNextLocationId()
      {
         if (mCreateMode)
         {
            int id = super.getNextLocationId();
            mProcess.setMaxLocationId(id);
            return id;
         }
         else
         {
            return -1;
         }
      }
   }
   
   /**
    * Visits each of the implementation objects and sets a custom location path
    * and id on each object. 
    */
   protected static class AeLocationPathImplVisitor extends AeImplTraversingVisitor
   {
      /** our modified path visitor that creates a custom path with the [instance()=n] predicate */
      protected AeImplPathParallelVisitor mPathVisitor;
      /** reference to the process since we might be asked to report all of the mappings */
      protected AeBusinessProcess mProcess;
      /** reference to the scope instance from the forEach */
      protected AeActivityScopeImpl mScope;
      
      /**
       * Creates the visitor and starts the process
       * 
       * @param aProcess
       * @param aImpl
       * @param aInstanceValue
       * @param aCreateMode
       * @throws AeBusinessProcessException
       */
      public AeLocationPathImplVisitor(AeBusinessProcess aProcess, AeActivityScopeImpl aImpl, int aInstanceValue, boolean aCreateMode) throws AeBusinessProcessException
      {
         mProcess = aProcess;
         mPathVisitor = new AeImplPathParallelVisitor(aProcess, aImpl.getParent().getLocationPath(), aImpl.getDefinition(), aCreateMode);
         mPathVisitor.setInstanceValue(aInstanceValue);
         mScope = aImpl;
      }
      
      /**
       * Visits the scope's def with the path generator and then the impl visitor
       * sets the newly generated paths.
       * 
       * @throws AeBusinessProcessException
       */
      public void startVisiting() throws AeBusinessProcessException
      {
         mScope.getDefinition().accept(mPathVisitor);
         mScope.accept(this);
      }

      /**
       * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl)
       */
      public void visit(AeActivityScopeImpl aImpl)
            throws AeBusinessProcessException
      {
         super.visit(aImpl);
         
         AeActivityScopeDef def = (AeActivityScopeDef) aImpl.getDefinition();
         if (def.getScope().hasVariables())
         {
            for (Iterator iter = aImpl.getVariablesMap().values().iterator(); iter.hasNext();)
            {
               AeVariable v = (AeVariable) iter.next();
               setVariablePath(v);
            }
         }
         
         if (def.getScope().hasCorrelationSets())
         {
            for (Iterator iter = aImpl.getCorrelationSetMap().values().iterator(); iter.hasNext();)
            {
               AeCorrelationSet corrSet = (AeCorrelationSet) iter.next();
               traverse(corrSet);
            }
         }
      }
      
      /**
       * Sets the variable's location path and id
       * @param aVariable
       */
      protected void setVariablePath(AeVariable aVariable)
      {
         String path = mPathVisitor.getLocationPath(aVariable.getDefinition());
         aVariable.setLocationPath(path);
      }
      
      /**
       * Sets the bpel object's location path and id and then traverses.
       * 
       * @see org.activebpel.rt.bpel.impl.visitors.AeImplTraversingVisitor#traverse(org.activebpel.rt.bpel.impl.AeAbstractBpelObject)
       */
      protected void traverse(AeAbstractBpelObject aImpl)
            throws AeBusinessProcessException
      {
         String path = mPathVisitor.getLocationPath(aImpl.getDefinition());
         aImpl.setLocationPath(path);
         int id = mPathVisitor.getLocationId(path);
         aImpl.setLocationId(id);

         super.traverse(aImpl);
      }
   }
}
 
