// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/AeActivityInvokeImpl.java,v 1.32 2006/01/03 19:55:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity;

import java.util.Date;
import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.AeException;
import org.activebpel.rt.IAeConstants;
import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeEndpointReference;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.IAeInvokeResponseReceiver;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.activity.AeActivityInvokeDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.impl.AeBpelState;
import org.activebpel.rt.bpel.impl.AeFaultFactory;
import org.activebpel.rt.bpel.impl.AePartnerLink;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.IAeAlarmReceiver;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.util.AeXmlUtil;
import org.activebpel.rt.xml.AeXMLParserBase;
import org.activebpel.wsio.IAeWebServiceResponse;
import org.activebpel.wsio.receive.AeMessageContext;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Implementation of the bpel invoke activity.
 */
public class AeActivityInvokeImpl extends AeActivityImpl implements IAeInvokeResponseReceiver, IAeAlarmReceiver
{
   /** ActiveBPEL policy namespace. */
   public static final String ABPEL_POLICY_NS = "http://www.activebpel.org/policies"; //$NON-NLS-1$
   /** Retry policy tag name. */
   public static final String RETRY_POLICY_TAG = "retry"; //$NON-NLS-1$
   /** Retry attempts attr name. */
   public static final String RETRY_ATTEMPTS_ATTR = "attempts"; //$NON-NLS-1$
   /** Retry interval attr name. */
   public static final String RETRY_INTERVAL_ATTR = "interval"; //$NON-NLS-1$
   /** Retry service name attr name. */
   public static final String PROCESS_SERVICE_NAME_ATTR = "service"; //$NON-NLS-1$
   /** Retry yes or no element returned from service. */
   public static final String RETRY_TAG = "retry"; //$NON-NLS-1$
   /** Retry interval element returned from service. */
   public static final String INTERVAL_TAG = "interval"; //$NON-NLS-1$
   /** Retry fault list attr name. */
   public static final String FAULT_LIST_ATTR = "faultList"; //$NON-NLS-1$
   /** Retry fault exclusion list attr name. */
   public static final String FAULT_EXCLUSION_LIST_ATTR = "faultExclusionList"; //$NON-NLS-1$
   
   /** Wild card for qname tests. */
   public static final String QNAME_WILDCARD = "*"; //$NON-NLS-1$
   
   /** ActiveBPEL retry check document namespace. */
   public static final String ABPEL_RETRY_CHECK_NS = "http://www.activebpel.org/services/retry"; //$NON-NLS-1$
   /** Retry check input element tag name. */
   public static final String RETRY_CHECK_INPUT_TAG = "retryCheckInput"; //$NON-NLS-1$
   /** Retry check input processId element tag name. */
   public static final String RETRY_CHECK_PROCESS_ID_TAG = "retryCheckInput"; //$NON-NLS-1$
   /** Retry check output part name. */
   public static final String RETRY_CHECK_OUTPUT_PART = "output"; //$NON-NLS-1$
   

   /** Boolean indicating that the invoke has been queued to be tried at a later time. */
   private boolean mQueued;
   
   /** Count of times that alarms have fired to retry this activity. */
   private int mRetries;
   
   /** default constructor for activity */
   public AeActivityInvokeImpl(AeActivityInvokeDef aActivityDef, IAeActivityParent aParent)
   {
      super(aActivityDef, aParent);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      // if we are executing in a loop make sure to clear the state of retries
      setRetries(0);
      setQueued(false);
      
      // get the variable we're working with (if there is one)
      IAeMessageData inputMessage = getInputMessageData();

      // initiates any correlation sets that are defined to initiate with OUTBOUND data      
      initiateCorrelationSet(getDef().getOutboundCorrelationDefs(), getInputVariable());

      // Queue our invocation of the partner operation we will be called back when it is done
      // note that we may be called back via onMessage or onFault before this returns
      getProcess().queueInvoke(this,
                               inputMessage, 
                               getDef().getPartnerLink(), 
                               getDef().getPortType(), 
                               getDef().getOperation()); 
   }
   
   /**
    * If the correlation set definition says that the correlation set must be
    * initialized with this part of the activity (either sending the invoke
    * or receiving the reply) then we assert that the correlation set is either
    * ALREADY initialized or that we can initialize it with the variable provided.
    * @param aIterator Iter of correlation definitions that indicates whether we need to initiate.
    * @param aVariable Either the input or output variable depending on where this is called from.
    * @throws AeBusinessProcessException
    */
   private void initiateCorrelationSet(
      Iterator aIterator,
      IAeVariable aVariable)
      throws AeBusinessProcessException
   {
      while(aIterator.hasNext())
      {
         AeCorrelationDef corrDef = (AeCorrelationDef) aIterator.next();
         AeCorrelationSet corrSet = getCorrelationSet(corrDef.getCorrelationSetName());
         corrSet.initiateOrValidate(aVariable, corrDef.isInitiate());
      }
   }

   /**
    * Gets the data for the invoke. If there is a variable defined but not initialized
    * then you'll get an exception.
    */
   private IAeMessageData getInputMessageData() throws AeBusinessProcessException
   {
      IAeMessageData inputMessage = null;
      if(! AeUtil.isNullOrEmpty(getDef().getInputVariable()))
      {
         IAeVariable var = getVariable(getDef().getInputVariable());
         inputMessage = var.getMessageData();
            
         if( isValidationOfServiceMessagesEnabled() )
            var.validate();            
      }
      return inputMessage;
   }
   
   /**
    * Convenience method to avoid having to cast the definition object in order
    * to access methods provided by invoke def.
    */
   private AeActivityInvokeDef getDef()
   {
      AeActivityInvokeDef def = (AeActivityInvokeDef)getDefinition();
      return def;
   }

   /**
    * Handle the receipt of our output message.
    * @see org.activebpel.rt.bpel.impl.IAeMessageReceiver#onMessage(org.activebpel.rt.message.IAeMessageData)
    */
   public void onMessage(IAeMessageData aMessage) throws AeBusinessProcessException
   {
      if (getState().isFinal())
         return;
      
      if (aMessage != null)
      {
         if(! AeUtil.isNullOrEmpty(getDef().getOutputVariable()))
         {
            IAeVariable variable = getVariable(getDef().getOutputVariable()); 
            variable.setMessageData(aMessage);
            if( isValidationOfServiceMessagesEnabled() )
            {
               try
               {
                  variable.validate();
               }
               catch(AeBusinessProcessException ex)
               {
                  objectCompletedWithFault(AeFaultFactory.getFault(AeFaultFactory.VALIDATION_ERROR));
                  return;
               }
            }
            else
            {
               variable.declarePartTypes();
            }
         }
   
         // initiates any correlation sets that are defined to initiate with INBOUND data      
         initiateCorrelationSet(getDef().getInboundCorrelationDefs(), getOutputVariable());
      }
      
      // we are done
      objectCompleted();
   }

   /**
    * Convenience method for getting the input variable. If none is defined, then
    * we return null
    */
   private IAeVariable getInputVariable()
   {
      if (getDef().getInputVariable() != null)
         return getVariable(getDef().getInputVariable());
      else
         return null;
   }

   /**
    * Convenience method for getting the output variable. If none is defined, then
    * we return null
    */
   public IAeVariable getOutputVariable()
   {
      if (getDef().getOutputVariable() != null)
         return getVariable(getDef().getOutputVariable());
      else
         return null;
   }
   
   /**
    * Handle the receipt of a fault from our invoke.
    * @see org.activebpel.rt.bpel.impl.IAeMessageReceiver#onFault(org.activebpel.rt.bpel.IAeFault)
    */
   public void onFault(IAeFault aFault) throws AeBusinessProcessException
   {
      if (!getState().isFinal())
      {
         if(! reschedule(aFault))
         {
            objectCompletedWithFault(aFault);
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.IAeInvokeResponseReceiver#isOneWay()
    */
   public boolean isOneWay()
   {
      return getDef().isOneWay();
   }

   /**
    * Tests the policies associated with this endpoint to see if the invoke 
    * should be rescheduled for a later attempt. Ideally this could be handled 
    * by an invoke handler itself, but for internal policy handling it is 
    * simpler to do here since all the neccesary info is at hand.
    * @param aFault The fault which occured diuring the invoke.
    * @return true if the invoke has been rescheduled and the fault should be ignored.
    */
   protected boolean reschedule(IAeFault aFault) throws AeBusinessProcessException
   {
      AePartnerLink partnerLink = getPartnerLink(getDef().getPartnerLink());
      IAeEndpointReference epr = partnerLink.getPartnerReference();
      
      // test the endpoint reference downtime policies if any
      if(epr != null && epr.getPolicies() != null && epr.getPolicies().size() > 0)
      {
         for(Iterator iter=epr.getPolicies().iterator(); iter.hasNext(); )
         {
            Element policyEl = (Element)iter.next();
            
            // test retry assertions
            NodeList children = policyEl.getElementsByTagNameNS(ABPEL_POLICY_NS, RETRY_POLICY_TAG);
            for(int i = 0; children != null && i < children.getLength(); ++i)
            {
               Element assertionEl = (Element)children.item(i);
               if(rescheduleFromRetryAssertion(aFault, assertionEl))
                  return true;
            }
         }
      }
    
      return false;
   }
   
   /**
    * Returns true if based on the passed retry assertion and fault the current invoke
    * has rescheduled itself.
    * @param aFault the fault returned from invoke.
    * @param aRetryAssertionElem The assertion element to test
    * @return true if the invoke has been rescheduled.
    * @throws AeBusinessProcessException
    */
   protected boolean rescheduleFromRetryAssertion(IAeFault aFault, Element aRetryAssertionElem) throws AeBusinessProcessException
   {
      // check if this fault is eligible for a retry
      if(isFaultEligibleForRetry(aFault, aRetryAssertionElem))
      {
         boolean retry = false;
         int intervalValue = 0;
         // check if service name is present
         String service = aRetryAssertionElem.getAttribute(PROCESS_SERVICE_NAME_ATTR);
         if(AeUtil.notNullOrEmpty(service))
         {
            intervalValue = callRetryCheckService(service, aFault);
            if(intervalValue < 0)
               retry = false;
            else
               retry = true;
         }
         else
         {
            // no service so user must provide attempts and interval in policy
            
            // check if there are any attempts left
            String attempts = aRetryAssertionElem.getAttribute(RETRY_ATTEMPTS_ATTR);
            if(AeUtil.notNullOrEmpty(attempts) && (getRetries() + 1) < Integer.parseInt(attempts))
            {
               retry = true;
               String interval = aRetryAssertionElem.getAttribute(RETRY_INTERVAL_ATTR);
               if(AeUtil.notNullOrEmpty(interval))
                  intervalValue = Integer.parseInt(interval);
            }
         }
         
         // if we are retrying then schedule it
         if(retry)
         {
            // TODO (ck) add an info event to log that have scheduled a retry
            Date deadline = new Date(System.currentTimeMillis() + (intervalValue * 1000));
            setQueued(true);
            getProcess().queueAlarm(this, deadline);
            return true;
         }
      }
      return false;
   }

   /**
    * Checks the current fault against the list of faults for inclusion or exclusion
    * to determine if the current fault is elegible for retry.
    * @param aFault
    * @param aAssertionEl
    */
   protected boolean isFaultEligibleForRetry(IAeFault aFault, Element aAssertionEl)
   {
      // if fault list is given it must match something in the fault list
      String faultList = aAssertionEl.getAttribute(FAULT_LIST_ATTR);
      if(AeUtil.notNullOrEmpty(faultList))
      {
         // split list on whitespace and test the indvidual entries
         String[] qnameList = faultList.split("\\s"); //$NON-NLS-1$
         for(int i=0; i < qnameList.length; ++i)
         {
            if(isFaultNameMatch(aFault.getFaultName(), qnameList[i]))
               return true;
         }
         return false;
      }
      
      // if fault exlusion list is given if the passed construct matches then we will return false
      String faultExclusionList = aAssertionEl.getAttribute(FAULT_EXCLUSION_LIST_ATTR);
      if(AeUtil.notNullOrEmpty(faultExclusionList))
      {
         // split list on whitespace and test the indvidual entries
         String[] qnameList = faultExclusionList.split("\\s"); //$NON-NLS-1$
         for(int i=0; i < qnameList.length; ++i)
         {
            if(isFaultNameMatch(aFault.getFaultName(), qnameList[i]))
               return false;
         }
         return true;
      }
      else
      {
         // there is an automatic exclusion of portType namespace faults
         StringBuffer buf = (new StringBuffer("{")).append(getDef().getPortType().getNamespaceURI()).append("}").append(QNAME_WILDCARD); //$NON-NLS-1$ //$NON-NLS-2$
         if(isFaultNameMatch(aFault.getFaultName(), buf.toString()))
            return false;
      }
      
      return true;
   }

   /**
    * Tests if the passed qname and string qname match.  The string qname can contain wildcards
    * in the form of an asterisk.  The string qname is in the format <CODE>{namespace}localpart</CODE>.
    * @param aFaultName the fault name to test.
    * @param aString qname string in form that can be passed to valurOf in QName. The format is <CODE>{namespace}localpart</CODE>.
    * @return boolean true if the fault names match. 
    */
   private boolean isFaultNameMatch(QName aFaultName, String aString)
   {
      try
      {
         QName testName = QName.valueOf(aString);
         if(QNAME_WILDCARD.equals(testName.getNamespaceURI()) ||
            AeUtil.compareObjects(aFaultName.getNamespaceURI(), testName.getNamespaceURI()) )
         {
            if(QNAME_WILDCARD.equals(testName.getLocalPart()) ||
               AeUtil.compareObjects(aFaultName.getLocalPart(), testName.getLocalPart()) )
               return true;
         }
      }
      catch (Exception ex)
      {
         // ignore exception on parse or test of fault names
         AeException.logError(ex, ex.getLocalizedMessage());
      }
      return false;
   }

   /**
    * Calls the passed service to decide whether and when the invoke should be retried.
    * @param aService the service to invoke.
    * @param aFault the fault which is being checked for retry.
    * @return int which will be the number of seconds before retrying, -1 means no retry,
    */
   protected int callRetryCheckService(String aService, IAeFault aFault)
   {
      int interval = -1;
      try
      {
         // build message context to call the process service and call it
         AeMessageContext context = new AeMessageContext();
         context.setServiceName(aService);
         Document[] array = new Document[1];
         array[0] = createRetryCheckInput(aFault);
         IAeWebServiceResponse resp = getProcess().getEngine().queueReceiveData(context, array);

         // if the service returned a good response then process it
         if(! resp.isFaultResponse())
         {
            Element elem = ((Document)resp.getMessageData().getMessageData().get(RETRY_CHECK_OUTPUT_PART)).getDocumentElement();
            String retry = AeXmlUtil.getText(AeXmlUtil.findSubElement(elem, RETRY_TAG));
            if("yes".equals(retry)) //$NON-NLS-1$
            {
               String intervalStr = AeXmlUtil.getText(AeXmlUtil.findSubElement(elem, INTERVAL_TAG));
               if(AeUtil.notNullOrEmpty(intervalStr))
                  interval = Integer.parseInt(intervalStr);
            }
         }
         else
         {
            AeException.logWarning(AeMessages.format("AeActivityInvokeImpl.ERROR_RETRY_SERVICE_FAILURE", aService)); //$NON-NLS-1$
         }
      }
      catch (Exception e)
      {
         // should never happen since the alert invoke is a one-way
         AeException.logError(e, AeMessages.format("AeActivityInvokeImpl.ERROR_RETRY_SERVICE_FAILURE", aService)); //$NON-NLS-1$
      }
      return interval;
   }

   /**
    * @return the document to be passed to the retry check service operation.
    */
   protected Document createRetryCheckInput(IAeFault aFault) throws AeException
   {
      AeXMLParserBase parser = new AeXMLParserBase();
      Document doc = parser.createDocument();
      Element root = doc.createElementNS(ABPEL_RETRY_CHECK_NS, RETRY_CHECK_INPUT_TAG);
      doc.appendChild(root);
      root.setAttributeNS(IAeConstants.W3C_XMLNS, "xmlns", ABPEL_RETRY_CHECK_NS); //$NON-NLS-1$
      AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "faultName", aFault.getFaultName().toString()); //$NON-NLS-1$
      AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "processId", "" + getProcess().getProcessId()); //$NON-NLS-1$ //$NON-NLS-2$
      AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "processName", getProcess().getName().toString()); //$NON-NLS-1$
      AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "invokePath", getLocationPath()); //$NON-NLS-1$
      AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "attempts", "" + (getRetries() + 1)); //$NON-NLS-1$ //$NON-NLS-2$
      AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "partnerLinkName", getDef().getPartnerLink()); //$NON-NLS-1$
      Element pl = AeXmlUtil.createChildElementWithText(root, ABPEL_RETRY_CHECK_NS, "partnerLink", null); //$NON-NLS-1$
      Document epr = getPartnerLink(getDef().getPartnerLink()).getPartnerReference().toDocument();
      pl.appendChild(doc.importNode(epr.getDocumentElement(), true));
      return doc;
   }

   /**
    * Implements method by dispatching an invoke retry after marking ourselves as dequeued
    * and incrementing the count of times we have attempted retries.
    * @see org.activebpel.rt.bpel.impl.IAeAlarmReceiver#onAlarm()
    */
   public void onAlarm() throws AeBusinessProcessException
   {
      // if for some reason we are not queued then this alarm is in error
      if(isQueued())
      {
         // TODO (ck) add an info event to log that we are retrying
         setQueued(false);
         setRetries(getRetries() + 1);
   
         // retry the invoke...
         
         // Queue our invocation of the partner operation we will be called back when it is done
         // note that we may be called back via onMessage or onFault before this returns
         getProcess().queueInvoke(this,
                                  getInputMessageData(), 
                                  getDef().getPartnerLink(), 
                                  getDef().getPortType(), 
                                  getDef().getOperation());
      }
   }

   /**
    * Overrides method to extend base in order to dequeue any queued alarm. So if we are 
    * terminating or completing with queued alarms they should be dequeued. 
    * @see org.activebpel.rt.bpel.impl.IAeExecutableQueueItem#setState(org.activebpel.rt.bpel.impl.AeBpelState)
    */
   public void setState(AeBpelState aNewState) throws AeBusinessProcessException
   {
      dequeue();
      super.setState(aNewState);
   }
   
   /**
    * Dequeues the alarm from the process.
    */
   protected void dequeue() throws AeBusinessProcessException
   {
      if (isQueued())
      {
         getProcess().dequeueAlarm(this);
         setQueued(false);
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeAlarmReceiver#getGroupId()
    */
   public int getGroupId()
   {
      return getLocationId();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeAlarmReceiver#isQueued()
    */
   public boolean isQueued()
   {
      return mQueued;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeAlarmReceiver#isQueued()
    */
   public void setQueued(boolean aQueued)
   {
      mQueued = aQueued;
   }
   
   /**
    * @return Returns the retries.
    */
   public int getRetries()
   {
      return mRetries;
   }
   
   /**
    * @param aRetries The retries to set.
    */
   public void setRetries(int aRetries)
   {
      mRetries = aRetries;
   }
}
