// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/AeActivityReplyImpl.java,v 1.18 2006/01/11 21:01:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity;

import java.util.HashMap;
import java.util.Iterator;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.activity.AeActivityReplyDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeUtil;

/**
 * Implementation of the bpel reply activity.
 */
public class AeActivityReplyImpl extends AeActivityImpl
{
   /** default constructor for activity */
   public AeActivityReplyImpl(AeActivityReplyDef aActivityDef, IAeActivityParent aParent)
   {
      super(aActivityDef, aParent);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }
   
   /**
    * Convenience method to avoid casting of definition.
    */
   private AeActivityReplyDef getDef()
   {
      return (AeActivityReplyDef) getDefinition();
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      AeActivityReplyDef def = getDef();
      
      // get the correlation sets we are creating via input and initiate
      HashMap correlation = new HashMap(); 
      for(Iterator iter = def.getCorrelationList(); iter.hasNext(); )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         AeCorrelationSet corrSet = getCorrelationSet(corrDef.getCorrelationSetName());
         corrSet.initiateOrValidate(getVariable(def.getVariable()), corrDef.isInitiate());
         correlation.putAll(corrSet.getPropertyValues());            
      }

      IAeMessageData inputMessage = null;
      if(! AeUtil.isNullOrEmpty(def.getVariable()))
      {
         IAeVariable var = getVariable(def.getVariable());
         inputMessage = var.getMessageData();

         if( isValidationOfServiceMessagesEnabled() )
         {
            var.validate();
         }
         else
         {
            var.declarePartTypes();
         }
      }
      
      String messageExchange = findEnclosingScope().getMessageExchangePath(getDef().getMessageExchange());
      

      // Queue our invocation of the partner operation we will be called back when it is done
      // note that we may be called back via onMessage or onFault before this returns
      getProcess().queueReply( inputMessage,
                               def.getFaultName(), 
                               def.getPartnerLink(), 
                               def.getPortType(), 
                               def.getOperation(),
                               messageExchange,                               
                               correlation); 

      // Once the reply has finished, decrement the scope's outstanding reply count.
      findEnclosingScope().decrementOutstandingReplyCount(getDef().getMessageExchange());

      // @todo ? might want to introduce a callback so we can be sure that the reply
      //         was successful.                   
      objectCompleted();
   }

}
