// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/AeActivityScopeImpl.java,v 1.48 2006/01/24 23:46:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeActivity;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeScopeDef;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.def.activity.AeActivityScopeDef;
import org.activebpel.rt.bpel.impl.AeBpelState;
import org.activebpel.rt.bpel.impl.AeBusinessProcess;
import org.activebpel.rt.bpel.impl.AeFaultFactory;
import org.activebpel.rt.bpel.impl.AeVariable;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.bpel.impl.IAeCompensatableActivity;
import org.activebpel.rt.bpel.impl.activity.support.AeCompInfo;
import org.activebpel.rt.bpel.impl.activity.support.AeCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCoordinationContainer;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.activity.support.AeDefaultFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeEventHandlers;
import org.activebpel.rt.bpel.impl.activity.support.AeEventHandlersContainer;
import org.activebpel.rt.bpel.impl.activity.support.AeFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeImplicitCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeImplicitFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeOnAlarm;
import org.activebpel.rt.bpel.impl.activity.support.AeOnMessage;
import org.activebpel.rt.bpel.impl.activity.support.AeScopeSnapshot;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeSequenceIterator;
import org.activebpel.rt.util.AeUtil;

/**
 * Implementation of the bpel scope activity.
 */
public class AeActivityScopeImpl extends AeActivityImpl implements IAeActivityParent, IAeCompensatableActivity
{
   /** flag that indicates the scope has completed normally. Scopes that catch
    * faults w/o rethrowing are not eligible for compensation */
   private boolean mNormalCompletion = true;

   /** maps variable names to variable objects */
   private Map mVariables;

   /** collection of fault implementation object */
   private Collection mFaultHandlers;

   /** catchAll fault handler */
   private AeDefaultFaultHandler mCatchAll;

   /** maps correlation set name to implementation object */
   private Map mCorrelationSets;

   /** container for the compensation handler */
   private AeCompensationHandler mCompensationHandler;
   
   /** container for the events */
   private AeEventHandlersContainer mEvents;

   /** child activity */
   private IAeActivity mChild;

   /** contains all of the compensation info for this scope. */
   private AeCompInfo mCompInfo;

   /** currently executing fault handler */
   private IAeBpelObject mFaultHandler;
   
   /**  Support container for coordinations. */
   private AeCoordinationContainer mCoordinationContainer;
   
   /** True if the scope has recorded its variables from a previous execution */
   private boolean mSnapshotRecorded = false;

   /** Number of outstanding replies for message exchanges declared within this scope. */
   private int mOutstandingReplies = 0;
   
   /** default constructor for activity */
   public AeActivityScopeImpl(AeActivityScopeDef aActivityDef, IAeActivityParent aParent)
   {
      super(aActivityDef, aParent);
   }

   /**
    * This constructor is here to allow Process to extend Scope.
    * @param aDef
    */
   protected AeActivityScopeImpl(AeBaseDef aDef)
   {
      super(aDef);
   }
   
   /**
    * Returns true if a variable snapshot has been recorded. If this is true, then
    * a subsequent execution of the scope should clone the local variables to ensure
    * that they are different objects than the ones in the snapshot. 
    */
   public boolean isSnapshotRecorded()
   {
      // the previous impl used to clone the variable at the time of the snapshot
      // this flag defers the cloning until the scope executes again (if at all) 
      return mSnapshotRecorded;
   }
   
   /**
    * Setter for the variable snapshot flag
    * @param aBool
    */
   public void setSnapshotRecorded(boolean aBool)
   {
      mSnapshotRecorded = aBool;
   }
   
   /**
    * Callback from the events container to signal the completion of an alarm.
    * If the child activity for the scope is in a final state then the scope
    * must have been waiting for the completion of the alarm (or some other event)
    * so we'll kick off the disable call again and try to complete the scope.
    * @param aAlarm
    * @throws AeBusinessProcessException
    */
   public void alarmCompleted(AeOnAlarm aAlarm) throws AeBusinessProcessException
   {
      if (isScopeOkToComplete())
      {
         completeEvents();
      }
   }
   
   /**
    * Callback from the events container to signal the completion of an event.
    * If the child activity for the scope is in a final state and there are no
    * active coordinations, then the scope must have been waiting for the 
    * completion of the event (or an alarm) so we'll kick off the disable call 
    * again and try to complete the scope. Otherwise, we'll requeue the event.
    * @param aAlarm
    * @throws AeBusinessProcessException
    */
   public void eventCompleted(AeOnMessage aMessage) throws AeBusinessProcessException
   {
      if (isScopeOkToComplete())
      {
         completeEvents();
      }
      else
      {
         // if the scope's activity is still executing OR if we're waiting for a 
         // coordination event to complete then we should requeue the event
         
         // TODO (MF) We won't need to do this for BPEL 2.0 since the events will remain queued.
         aMessage.setState(AeBpelState.INACTIVE);
         getProcess().queueObjectToExecute(aMessage);
      }
   }
   
   /**
    * Returns true if the scope's child activity is in a final state and there are
    * no active coordinations
    */
   protected boolean isScopeOkToComplete()
   {
      // scope is ok to complete if the following conditions are met:
      //
      // it's not already in a final state
      //       AND
      // it's not currently executing a fault handler
      //       AND
      // its child IS in a final state
      //       AND
      // there are no active coordinations
      return !isExecutingFaultHandler() && !getState().isFinal() && getActivity().getState().isFinal() && !hasActiveCoordinations();
   }

   /**
    * A messageExchange path is the value of the declaring scope's location path
    * plus /aMessageExchangeValue. This method will check this scope and then
    * continuing walking up the enclosing scopes (including the process) until it
    * finds the scope that declares the messageExchange. This method will never
    * return null since a null indicates a problem with the BPEL process that
    * should have been caught during static analysis.
    * @param aMessageExchangeValue
    */
   public String getMessageExchangePath(String aMessageExchangeValue) throws AeBusinessProcessException
   {
      AeScopeDef def = getScopeDef();
      
      if (def.declaresMessageExchange(aMessageExchangeValue))
      {
         return getLocationPath() + "/" + aMessageExchangeValue; //$NON-NLS-1$
      }
      else
      {
         AeActivityScopeImpl enclosingScope = findEnclosingScope();
         if (enclosingScope == null)
         {
            staticAnalysisFailure(AeMessages.format("AeActivityScopeImpl.NO_DECL_FOR_MESSAGE_EXCHANGE", aMessageExchangeValue)); //$NON-NLS-1$
         }
         return enclosingScope.getMessageExchangePath(aMessageExchangeValue);
      }
   }
   
   /**
    * Gets the scope definition for this scope.  
    */
   protected AeScopeDef getScopeDef()
   {
      return ((AeActivityScopeDef) getDefinition()).getScope();
   }

   /**
    * Getter for the compensation handler. If there was none defined then we create
    * an instance of the implicit compensation handler.
    */
   public AeCompensationHandler getCompensationHandler()
   {
      if (mCompensationHandler == null)
      {
         mCompensationHandler = new AeImplicitCompensationHandler(this);
      }
      return mCompensationHandler;
   }

   /**
    * Returns <code>true</code> if and only if this scope has an implicit
    * compensation handler but does not create the compensation handler (unlike
    * {@link #getCompensationHandler()}).
    */
   public boolean hasImplicitCompensationHandler()
   {
      return mCompensationHandler instanceof AeImplicitCompensationHandler;
   }

   /**
    * Returns <code>true</code> if and only if this scope has an explicit
    * compensation handler.
    */
   public boolean hasExplicitCompensationHandler()
   {
      return hasCompensationHandler() && !hasImplicitCompensationHandler();
   }

   /**
    * Returns true if a compensation handler has been created for this activity.
    * @return true if compensationHandler is not null.
    */
   public boolean hasCompensationHandler()   
   {
      return mCompensationHandler != null;
   }
   
   /**
    * Returns true if the scope is currently in the middle of compensating.
    */
   public boolean isCompensating()
   {
      boolean compensating = false;

      if (hasCompensationHandler())
      {
         // comp handler is not in a final state and its not inactive, ergo
         // it must be ready to execute, executing, or faulting
         if (!getCompensationHandler().getState().isFinal() && getCompensationHandler().getState() != AeBpelState.INACTIVE)
         {
            compensating = true;
         }
      }
      
      return compensating;
   }

   /** 
    * Overrides method to terminate the current (if installed) compensation handler if it
    * is in a terminatable state. 
    * @see org.activebpel.rt.bpel.impl.IAeCompensatableActivity#terminateCompensationHandler()
    */
   public void terminateCompensationHandler() throws AeBusinessProcessException
   {
      if (mCompensationHandler != null && mCompensationHandler.isTerminatable())
      {
         mCompensationHandler.terminate();
      }
   }
   
   /**
    * Returns the coordination support container. Creates a new instance if it does not have one.
    * @return Returns the coordinationContainer. 
    */
   public AeCoordinationContainer getCoordinationContainer()
   {
      if (mCoordinationContainer == null)
      {
         mCoordinationContainer = new AeCoordinationContainer(this);
      }
      return mCoordinationContainer;
   }

   /**
    * Sets the coordination container.
    * @param aContainer
    */
   public void setCoordinationContainer(AeCoordinationContainer aContainer)
   {
      mCoordinationContainer = aContainer;
   }
   
   /**
    * Returns true if the scope has any activities under 'active' coordination. An activity
    * that has 'completed' is not considered active.
    * @return Returns true if the scope has any activities under 'active' coordination.
    */
   public boolean hasActiveCoordinations()
   {
      return ( mCoordinationContainer != null && mCoordinationContainer.hasActiveCoordinations() );
   }

   /**
    * Returns true if the scope has any activities under coordination, 
    * including completed, and faulted activities.
    * 
    * @return Returns true if the scope has any activities under coordination.
    */
   public boolean hasCoordinations()
   {
      return ( mCoordinationContainer != null);
   }   
   
   /**
    * A callback method that signals that all of the scope's active coordinations have
    * completed executing. The scope can now complete.
    */
   public void coordinationCompleted() throws AeBusinessProcessException
   {
      // the check for isExecutingFaultHandler is to avoid illegal state 
      // transition of fault handler from executing to dead path
      
      if (isScopeOkToComplete())
      {
         completeScope();
      }
   }
   
   /**
    * Completing a scope sets its fault handlers to dead path (since they're no
    * longer reachable and shouldn't have been executed) and then completes
    * the events. If there are events currently executing, then the scope will
    * wait for those events to complete.
    */
   protected void completeScope() throws AeBusinessProcessException
   {
      completeEvents();
   }

   /**
    * If the scope has events then we'll signal the events container that they
    * should be disabled. Otherwise, we'll call the eventsCompleted method which
    * signals the completion of the scope.
    * @throws AeBusinessProcessException
    */
   protected void completeEvents() throws AeBusinessProcessException
   {
      if (hasEvents())
      {
         getEventHandlersContainer().completeEvents();
      }
      else
      {
         eventsCompleted();
      }
   }
   
   /**
    * Invoked by the coordination framework once the participant under coordination
    * has completed successfully and is compensatable.
    * @param aCoordinationId
    * @param aCompInfo compensation information
    */
   public void coordinatedActivityCompleted(String aCoordinationId, AeCompInfo aCompInfo)
   {
      if (hasCoordinations())
      {
         synchronized( getCoordinationContainer() )
         {
            if (getCoordinationContainer().hasCoordinationId(aCoordinationId))
            {
               // install compensation handler
               enclosedScopeCompleted(aCompInfo);
               // move the coordination 'state' from active to completed.
               getCoordinationContainer().coordinationCompleted(aCoordinationId);                                 
            }
         }// synch
      }// if
   }

   /**
    * Invoked by the coordination framework once the participant under coordination
    * has faulted and is not available for compensatable.
    * @param aCoordinationId
    * @param aFault fault.
    */   
   public void coordinatedActivityCompletedWithFault(String aCoordinationId, IAeFault aFault)
   {
      if (hasCoordinations())
      {
         synchronized( getCoordinationContainer() )
         {      
            if (getCoordinationContainer().hasCoordinationId(aCoordinationId))
            {
               // what happens when fault occurs during compensation?
               getCoordinationContainer().coordinationCompletedWithFault(aCoordinationId);
            }
         }// synch
      }// if
   }

   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }

   /**
    * Setter for the compensation handler.
    * @param aHandler
    */
   public void setCompensationHandler(AeCompensationHandler aHandler)
   {
      mCompensationHandler = aHandler;
   }

   /**
    * Getter for the variables map. Lazily creates the map.
    */
   public Map getVariablesMap()
   {
      if (mVariables == null)
      {
         setVariablesMap(new HashMap());
      }
      return mVariables;
   }

   /**
    * Setter for the variables map.
    * @param aMap
    */
   protected void setVariablesMap(Map aMap)
   {
      mVariables = aMap;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getCorrelationSet(java.lang.String)
    */
   public AeCorrelationSet getCorrelationSet(String aName) throws AeBusinessProcessException
   {
      AeCorrelationSet set = (AeCorrelationSet) getCorrelationSetMap().get(aName);
      if (set == null && getParent() != null)
      {
         set = super.getCorrelationSet(aName);
      }
      if (set == null)
      {
         staticAnalysisFailure(AeMessages.getString("AeActivityScopeImpl.0")); //$NON-NLS-1$
      }
      return set;
   }

   /**
    * Adds a correlation set implementation to the scope.
    * @param aCorrelationSet
    */
   public void addCorrelationSet(AeCorrelationSet aCorrelationSet)
   {
      getCorrelationSetMap().put(aCorrelationSet.getName(), aCorrelationSet);
   }

   /**
    * Getter for the correlation set map. Lazily creates the map.
    */
   protected Map getCorrelationSetMap()
   {
      if (mCorrelationSets == null)
      {
         mCorrelationSets = new HashMap();
      }
      return mCorrelationSets;
   }

   /**
    * Gets the variable by name. Delegates to the parent if it's not handled
    * by this scope.
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getVariable(java.lang.String)
    */
   public IAeVariable getVariable(String aName)
   {
      IAeVariable var = (IAeVariable) getVariablesMap().get(aName);
      // added null check here because the parent could be null if this scope
      // is actually a process. could override the method in process but this
      // works fine.
      if (var == null && getParent() != null)
      {
         var = super.getVariable(aName);
      }
      return var;
   }

   /**
    * Getter for the fault handlers map. Lazily creates the map.
    */
   protected Collection getFaultHandlersCollection()
   {
      if (mFaultHandlers == null)
      {
         mFaultHandlers = new ArrayList();
      }
      return mFaultHandlers;
   }

   /**
    * Helper method to get the iterator of fault handlers or the empty iterator
    * if there are no fault handlers.
    */
   private Iterator getFaultHandlersIterator()
   {
      if (mFaultHandlers == null)
      {
         return Collections.EMPTY_SET.iterator();
      }
      return getFaultHandlersCollection().iterator();
   }

   /**
    * Adds a fault handler to the scope.
    * @param aFaultHandler
    */
   public void addFaultHandler(AeFaultHandler aFaultHandler)
   {
      getFaultHandlersCollection().add(aFaultHandler);
   }

   /**
    * Adds a variable to the scope.
    * @param aVariable
    */
   public void addVariable(AeVariable aVariable)
   {
      getVariablesMap().put(aVariable.getName(), aVariable);
   }

   /**
    * Sets the <code>catchAll</code> fault handler on this scope.
    */
   public void setDefaultFaultHandler(AeDefaultFaultHandler aCatchAll)
   {
      mCatchAll = aCatchAll;
   }

   /**
    * Getter for the default fault handler (aka catchAll).
    */
   public AeDefaultFaultHandler getDefaultFaultHandler()
   {
      return mCatchAll;
   }

   /**
    * Setter for the events handler container.
    */
   public void setEventHandlersContainer(AeEventHandlersContainer aEvents)
   {
      mEvents = aEvents;
   }

   /**
    * Getter for the events handler container.
    * @return AeEventHandlersContainer or null
    */
   public AeEventHandlersContainer getEventHandlersContainer()
   {
      return mEvents;
   }

   /**
    * Getter for the event handlers.
    * @return AeEventHandlers or null if there are no events associated with
    *          this scope.
    */
   protected AeEventHandlers getEventHandlers()
   {
      if ( ! hasEvents() )
      {
         return null;
      }

      return mEvents.getEventHandlers();
   }

   /**
    * Convenience routine that returns true if the scope has events.
    * @return true if there is a event container present.
    */
   protected boolean hasEvents()
   {
      return mEvents != null;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeActivityParent#addActivity(org.activebpel.rt.bpel.IAeActivity)
    */
   public void addActivity(IAeActivity aActivity)
   {
      mChild = aActivity;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.AeActivityImpl#acquireVariableLocks()
    */
   protected boolean acquireVariableLocks()
   {
      AeActivityScopeDef def = (AeActivityScopeDef) getDefinition();
      if (def.getVariableAccessSerializable())
         return getProcess().addExclusiveLock(getConvertedVariableLockPaths(), getLocationPath());
      else
         return getProcess().addSharedLock(getConvertedVariableLockPaths(), getLocationPath());
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      // reset the completion flag in case we're being executed multiple times from
      // a loop
      setNormalCompletion(true);
      setOutstandingReplyCount(0);
      executeEventHandler(); 
      getProcess().queueObjectToExecute(getActivity());
   }

   /**
    * Executes the events and alarms for the scope's event handler if present.
    * @throws AeBusinessProcessException
    */
   protected void executeEventHandler() throws AeBusinessProcessException
   {
      if (hasEvents())
      {
         // we still queue the container to execute so it'll transition through
         // its states cleanly.
         getProcess().queueObjectToExecute(getEventHandlersContainer());

         // The container's execute() is a no-op. We're installing the alarms and
         // messages here to ensure that they're present on the execution queue
         // BEFORE any other activities that may get queued as a result of
         // executing the scope's child activity.
         getEventHandlersContainer().installAlarms();
         getEventHandlersContainer().installMessages();

      }
   }

   /**
    * Variables get re-initialized each time the scope executes.
    */
   private void clearVariableState(boolean aCloneFlag)
   {
      for (Iterator iter = getVariablesMap().entrySet().iterator(); iter.hasNext();)
      {
         Map.Entry entry = (Entry) iter.next();
         AeVariable var = (AeVariable) entry.getValue();
         if (aCloneFlag)
         {
            var = (AeVariable) var.clone();
         }
         if (!var.getDefinition().isImplicit())
         {
            var.clear();
         }
         entry.setValue(var);
      }
   }

   /**
    * Correlation Sets get re-initialized each time the scope executes.
    */
   private void clearCorrelationSetState(boolean aCloneFlag)
   {
      for (Iterator iter = getCorrelationSetMap().entrySet().iterator(); iter.hasNext();)
      {
         Map.Entry entry = (Map.Entry) iter.next();
         AeCorrelationSet corrSet = (AeCorrelationSet) entry.getValue();
         if (aCloneFlag)
         {
            corrSet = (AeCorrelationSet) corrSet.clone(); 
         }
         corrSet.clear();
         entry.setValue(corrSet);
      }
   }

   /**
    * Scopes only have a single child, ergo this should mark us as completed.
    * Need to uninstall any events and alarms and then wait for any active
    * events.
    * @param aChild
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childComplete(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void childComplete(IAeBpelObject aChild) throws AeBusinessProcessException
   {
      // checking to see if the child is our activity. This is to avoid triggering the
      // complete code for fault handlers that go to dead path
      if (aChild == getActivity() && aChild.getState() != AeBpelState.DEAD_PATH)
      {
         if (isScopeOkToComplete())
         {
            completeScope();
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childCompleteWithFault(org.activebpel.rt.bpel.impl.IAeBpelObject, org.activebpel.rt.bpel.IAeFault)
    */
   public void childCompleteWithFault(IAeBpelObject aChild, IAeFault aFault)
   throws AeBusinessProcessException
   {
      handleFault(aFault);
   }

   /**
    * Returns true if the scope is currently executing a fault handler. This is
    * to signal that the scope is ineligible for executing any other fault handlers.
    */
   public boolean isExecutingFaultHandler()
   {
      return getFaultHandler() != null;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#terminateForRetry()
    */
   public void terminateForRetry() throws AeBusinessProcessException
   {
      IAeFault retryFault = AeFaultFactory.getFault(AeFaultFactory.RETRY_FAULT);
      retryFault.setRethrowable(false);
      handleFault(retryFault);
   }

   /**
    * Handles the fault passed in. This involves finding or creating the fault
    * handler, terminating the child, and then executing the fault handler.
    * @param aFault
    * @throws AeBusinessProcessException
    */
   protected void handleFault(IAeFault aFault) throws AeBusinessProcessException
   {
      if ( ! isExecutingFaultHandler() )
      {
         setNormalCompletion(false);

         setFault(aFault);

         terminateNextChild();
      }
   }

   /**
    * Callback method from the fault handler. Indicates that the fault handler completed
    * execution w/o rethrowing a fault. The Scope's state remains at faulted, but
    * its links are still evaluated and regular execution continues.
    * @throws AeBusinessProcessException
    */
   public void faultHandlerComplete() throws AeBusinessProcessException
   {
      setFaultHandler(null);
      
      if (isFaultingOrRetrying() || isTerminating() || getProcess().isTerminating())
      {
         setState(AeBpelState.TERMINATED);
      }
      else if (hasOutstandingReplies())
      {
         faultHandlerCompleteWithFault(AeFaultFactory.getFault(AeFaultFactory.MISSING_REPLY));
      }
      else
      {
         objectCompleted();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#isFaultingOrRetrying()
    */
   public boolean isFaultingOrRetrying()
   {
      return super.isFaultingOrRetrying() || getProcess().getFaultingActivityLocationPaths().contains(getLocationPath());
   }

   /**
    * Callback method from the fault handler that signals a rethrowing of a fault.
    * This causes the activity to complete with a fault and propagate upwards.
    * @param aFault
    * @throws AeBusinessProcessException
    */
   public void faultHandlerCompleteWithFault(IAeFault aFault) throws AeBusinessProcessException
   {
      setFaultHandler(null);
      /*
       * At this point, we've already gone through our fault handling code - which
       * is probably obvious from the method name. If we call the base class's impl
       * of objectCompletedWithFault, then we'll take another cycle through the
       * fault handling and termination code which isn't something we want to do.
       * This is why we're calling objectCompletedWithFault on the process and
       * we're the only other class outside of the business process and base class
       * to do such.
       */
      getProcess().objectCompletedWithFault(this, aFault);
   }
   
   /**
    * Return true if this scope handles the fault.
    * @param aFault
    */
   public boolean isFaultHandledByScope( IAeFault aFault )
   {
      boolean retry = getProcess().getFaultingActivityLocationPaths().contains(getLocationPath()); 

      // added check for retry in order to consume any faults raised during compensating a scope if its enclosing scope is retrying
      if( getDefaultFaultHandler() != null || retry)
      {
         return true;
      }
      else
      {
         return findMatchingCatch( aFault ) != null;
      }
   }

   /**
    * Find the matching fault handler by scanning the <code>catch</code> blocks
    * in our definition. If there is no match, then we fall back to the
    * <code>catchAll</code>. If there is no <code>catchAll</code>, then we
    * create the implicit fault handler and return that.
    * @param aFault
    * @return IAeBpelObject - the fault handler (will never be null)
    * @throws AeBusinessProcessException
    */
   private IAeBpelObject findOrCreateMatchingFaultHandler(IAeFault aFault)
      throws AeBusinessProcessException
   {
      IAeBpelObject matched = null;

      // special check for our internal retry fault. This fault should trigger
      // compensation for the eligible scopes but cannot be caught by an fault
      // handler - even a catch all.
      if (aFault.getFaultName().getNamespaceURI().equals(IAeBPELConstants.AE_NAMESPACE_URI) && aFault.getFaultName().getLocalPart().equals(AeFaultFactory.RETRY_FAULT))
      {
         matched = createImplicitFaultHandler(aFault);
      }
      else
      {
         AeFaultHandler matchedFaultHandler = findMatchingCatch(aFault);
         matched = matchedFaultHandler;
         if (matched == null )
         {
            if (getDefaultFaultHandler() != null)
            {
               matched = getDefaultFaultHandler();
            }
            else
            {
               matched = createImplicitFaultHandler(aFault);
            }
         }
         else
         {
            // check to see if there is variable data to copy over
            IAeVariable faultVar = matchedFaultHandler.getFaultVariable();
            if (faultVar != null)
               faultVar.setMessageData(aFault.getMessageData());
         }
      }

      setAllOtherFaultHandlersToDeadPath(matched);

      return matched;
   }

   /**
    * Sets all of the other fault handlers to dead path except for the one that
    * we matched against and are about to execute.
    * @param aMatched
    */
   private void setAllOtherFaultHandlersToDeadPath(IAeBpelObject aMatched) throws AeBusinessProcessException
   {
      Iterator it = AeUtil.join(getFaultHandlersIterator(), getDefaultFaultHandler());
      setAllOtherToDeadPath(aMatched, it);
   }

   /**
    * Broken out to allow Process to override since the process terminates on
    * any uncaught fault.
    * @param aFault
    */
   public IAeBpelObject createImplicitFaultHandler(IAeFault aFault)
   {
      return new AeImplicitFaultHandler(this, aFault);
   }

   /**
    * Finds the matching fault handler to execute or returns null if there is
    * no match.
    * @param aFault
    */
   protected AeFaultHandler findMatchingCatch(IAeFault aFault)
   {
      // describes the fault handler we're looking for
      AeFaultHandlerSpec spec =
         new AeFaultHandlerSpec(aFault.getFaultName(), aFault.getMessageData());

      return spec.findMatchingCatch(getFaultHandlersCollection());
   }
   
   /**
    * A callback method that signals that all of the scope's active events have
    * completed executing. The scope can now complete.
    */
   public void eventsCompleted() throws AeBusinessProcessException
   {
      if (hasOutstandingReplies())
      {
         handleFault(AeFaultFactory.getFault(AeFaultFactory.MISSING_REPLY));
      }
      else
      {
         setAllOtherFaultHandlersToDeadPath(null);
         objectCompleted();
      }
   }

   /**
    * Returns the associated main scope activity.
    */
   public IAeActivity getActivity()
   {
      return mChild;
   }

   /**
    * Scopes only have a single child activity but this method includes the fault
    * handlers, default fault handler, and compensation handler because these
    * bpel objects need to have the state change propagated to them.
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getChildrenForStateChange()
    */
   public Iterator getChildrenForStateChange()
   {
      AeSequenceIterator seqIterator = new AeSequenceIterator();
      seqIterator.add(getActivity());
      seqIterator.add(mCompensationHandler);
      seqIterator.add(getFaultHandlersIterator());
      seqIterator.add(getDefaultFaultHandler());
      seqIterator.add(getEventHandlersContainer());

      // If the current fault handler is the implicit one, then it hasn't been
      // added to the iterator yet.
      if (getFaultHandler() instanceof AeImplicitFaultHandler)
      {
         seqIterator.add(getFaultHandler());
      }
      // If coordinated, then add the coordination container.
      if (hasCoordinations())
      {
         seqIterator.add(getCoordinationContainer());
      }
      return seqIterator;
   }

   /**
    * If the scope completes normally then it will notify the enclosing scope
    * of the successful completion and of any available compensation information.
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#objectCompleted()
    */
   protected void objectCompleted() throws AeBusinessProcessException
   {
      if ( isNormalCompletion() )
      {
         // find the enclosing scope and add notify it that we've completed.
         AeActivityScopeImpl enclosingScope = findEnclosingScope();
         if (enclosingScope != null)
         {
            enclosingScope.enclosedScopeCompleted(getCompInfo());
            setSnapshotRecorded(true);
         }
      }
      clearCompInfo();     
      super.objectCompleted();

   }

   /**
    * Sets the comp info to null. We clear the reference after we've successfully
    * completed and notified the enclosing scope of our completion. The parent
    * scope is responsible for maintaining our compensation information. This
    * reference is only non-null during the execution of the scope.
    */
   protected void clearCompInfo()
   {
      mCompInfo = null;
   }

   /**
    * Getter for the AeCompInfo object.
    */
   public AeCompInfo getCompInfo()
   {
      if (mCompInfo == null)
      {
         mCompInfo = new AeCompInfo(this);
      }
      return mCompInfo;
   }

   /**
    * Returns <code>true</code>if and only if this scope has compensation info.
    */
   public boolean hasCompInfo()
   {
      return mCompInfo != null;
   }

   /**
    * Records the scope's variable data and corr sets on the snapshot object
    * @param aSnapshot
    */
   public void recordScopeSnapshot(AeScopeSnapshot aSnapshot)
   {
      if (mVariables != null)
         aSnapshot.addVariables(mVariables.values().iterator());
      if (mCorrelationSets != null)
         aSnapshot.addCorrelationSets(mCorrelationSets.values().iterator());
   }

   /**
    * Returns true if the scope completed normally. An abnormal completion will
    * prevent the compensation handler from being installed for this execution.
    */
   public boolean isNormalCompletion()
   {
      return mNormalCompletion;
   }

   /**
    * Setter for the normal termination flag. A false value will prevent the
    * compensation handler from being installed for this execution of the scope.
    * @param aB
    */
   public void setNormalCompletion(boolean aB)
   {
      mNormalCompletion = aB;
   }

   /**
    * Adds the compensation info object to the scope's existing compensation
    * info. Once this scope completes, it'll notify its enclosing scope through
    * this same method and further propagate the compensation info.
    * @param aCompInfo
    */
   public void enclosedScopeCompleted(AeCompInfo aCompInfo)
   {
      getCompInfo().add(aCompInfo);
   }

   /**
    * Set by the compensate activity during compensation.
    */
   public void setCompInfo(AeCompInfo aCompInfo)
   {
      mCompInfo = aCompInfo;
   }

   /**
    * Scopes handle terminate calls by generating a <code>bpws:forcedTermination</code>
    * exception and then handling that exception through the regular fault handling
    * process. This will terminate the active child activity as well as any
    * events that have been queued. After the termination completes, the scope
    * will then execute the matched fault handler for the  <code>bpws:forcedTermination</code>
    * fault.
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#terminate()
    */
   public void terminate() throws AeBusinessProcessException
   {
      if (!isTerminatable())
      {
         return;
      }

      if (isExecutingFaultHandler() || isProcessTerminating())
      {
         // we're currently in the middle of catching a fault so we can't issue
         // a forced termination, therefore we'll just terminate ourselves directly.
         super.terminate();
      }
      else
      {
         setTerminating(true);
         IAeFault fault = AeFaultFactory.getFault(AeFaultFactory.FORCED_TERMINATION);
         fault.setRethrowable(false);
         handleFault(fault);
      }      
   }
   
   /**
    * Returns the next child available for termination. While Scopes only have a
    * single primary activity, a scope's events (if any) must be terminated when
    * the scope terminated. As such, this method will first return the primary
    * child if it's available for termination and then the event container.
    * @return IAeBpelObject or null if there are no active bpel objects that require termination.
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#getChildForTermination()
    */
   protected IAeBpelObject getChildForTermination() throws AeBusinessProcessException
   {
      IAeBpelObject childToTerminate = null;
      if (getActivity().getState().canTransitionToDeadPath())
      {
         getActivity().setState(AeBpelState.DEAD_PATH);
      }
      
      if (getActivity().getState().isTerminatable())
      {
         childToTerminate = getActivity();
      }
      else if (hasEvents() && getEventHandlersContainer().getState().isTerminatable())
      {
         childToTerminate = getEventHandlersContainer();
      }
      else if (isProcessTerminating() || isExecutingFaultHandler())
      {
         childToTerminate = getFaultHandler();
         setFaultHandler(null);
      }
      return childToTerminate;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#noMoreChildrenToTerminate()
    */
   protected void noMoreChildrenToTerminate() throws AeBusinessProcessException
   {
      if (isProcessTerminating() || getFault() == null)
      {
         setState(AeBpelState.TERMINATED);
      }
      else
      {
         IAeBpelObject matched = findOrCreateMatchingFaultHandler(getFault());
         setFaultHandler(matched);
         setFault(null);
         getProcess().queueObjectToExecute(matched);
      }
   }

   /**
    * Returns true if the process is terminating
    */
   private boolean isProcessTerminating()
   {
      return ((AeBusinessProcess)getProcess()).isTerminating();
   }

   /**
    * Overriding this in order to clear the variable state for the scope when it
    * is ready to execute. This was previously accomplished by overriding isReadyToExecute()
    * but that method now has to handle serializable scopes and as such will not
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#setState(org.activebpel.rt.bpel.impl.AeBpelState)
    */
   public void setState(AeBpelState aNewState)
      throws AeBusinessProcessException
   {
      super.setState(aNewState);
      if (aNewState == AeBpelState.READY_TO_EXECUTE)
      {
         boolean makeClones = isSnapshotRecorded();
         clearVariableState(makeClones);
         clearCorrelationSetState(makeClones);
         setSnapshotRecorded(false);
      }
   }

   /**
    * Returns currently executing fault handler.
    */
   public IAeBpelObject getFaultHandler()
   {
      return mFaultHandler;
   }

   /**
    * Sets currently executing fault handler.
    *
    * @param aFaultHandler the fault handler to set.
    */
   public void setFaultHandler(IAeBpelObject aFaultHandler)
   {
      mFaultHandler = aFaultHandler;
   }

   /**
    * Returns true if there are outstanding replies
    */
   public boolean hasOutstandingReplies()
   {
      return getOutstandingReplyCount() > 0;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeBusinessProcess#getOutstandingReplyCount()
    */
   public int getOutstandingReplyCount()
   {
      return mOutstandingReplies;
   }

   /**
    * Sets the process's outstanding reply count.
    */
   public void setOutstandingReplyCount(int aCount)
   {
      mOutstandingReplies = aCount;
   }

   /**
    * If this scope declares the messageExchange value then it increments its outstanding
    * reply count to track that there is an open message exchange that needs a reply.
    * Otherwise, the scope delegates to its enclosing scope.
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#incrementOutstandingReplyCount(java.lang.String)
    */
   public void incrementOutstandingReplyCount(String aMessageExchange)
   {
      AeScopeDef def = getScopeDef();
      if (def.declaresMessageExchange(aMessageExchange))
      {
         mOutstandingReplies++;
      }
      else
      {
         findEnclosingScope().incrementOutstandingReplyCount(aMessageExchange);
      }
   }

   /**
    * If this scope declares the messageExchange value then it decrements its
    * outstanding reply count. Otherwise the scope delegates to its enclosing scope.
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#decrementOutstandingReplyCount(java.lang.String)
    */
   public void decrementOutstandingReplyCount(String aMessageExchange)
   {
      AeScopeDef def = getScopeDef();
      if (def.declaresMessageExchange(aMessageExchange))
      {
         mOutstandingReplies--;
      }
      else
      {
         findEnclosingScope().decrementOutstandingReplyCount(aMessageExchange);
      }
   }  
}

/**
 * Describes the fault handler that we're looking for based on the fault
 * that's been thrown.
 */
class AeFaultHandlerSpec
{
   /** the name of the fault we're looking for */
   private QName mFaultName;
   /** message type we're looking for */
   private QName mMessageType;

   /**
    * Ctor takes name and variable.
    * @param aFaultName if null then variable cannot be null
    * @param aMessageData if null then fault name cannot be null
    */
   AeFaultHandlerSpec(QName aFaultName, IAeMessageData aMessageData)
   {
      if (aFaultName == null && aMessageData == null)
      {
         throw new IllegalArgumentException(AeMessages.getString("AeActivityScopeImpl.ERROR_1")); //$NON-NLS-1$
      }

      setFaultName(aFaultName);
      if (aMessageData != null)
      {
         setMessageType(aMessageData.getMessageType());
      }
   }

   /**
    * Walks the collection looking for a match based on the match rules in BPEL
    * spec. The matching process may take multiple passes through the collection
    * looking for an exact match first and then falling back on a less exact match
    * if not found.
    * @param aColl
    * @return AeFaultHandler or null if none matched
    */
   protected AeFaultHandler findMatchingCatch(Collection aColl)
   {
      /*
         Rules for matching the fault, from BPEL 1.1 spec...

         1. If the fault has no associated fault data, a catch activity that
            specifies a matching faultName value will be selected if present.
            Otherwise, the default catchAll handler is selected if present.
         2. If the fault has associated fault data, a catch activity specifying
            a matching faultName value and a faultVariable whose type
            (WSDL message type) matches the type of the fault's data will be
            selected if present. Otherwise, a catch activity with no specified
            faultName and with a faultVariable whose type matches the type of
            the fault data will be selected if present. Otherwise, the default
            catchAll handler is selected if present.

         Whenever a fault handler (for any fault) .... is missing for any given
         scope, they are implicitly created with the following behavior:

             - Run all available compensation handlers for immediately enclosed
               scopes in the reverse order of completion of the corresponding
               scopes.
             - Rethrow the fault to the next enclosing scope.
      */

      AeFaultHandler matched = null;

      if (getMessageType() != null)
      {
         // try to find exact match on name AND variable
         matched = pickFault(aColl.iterator());
         // if not found, look for variable only
         if (matched == null)
         {
            setFaultName(null);
            matched = pickFault(aColl.iterator());
         }
         // if still not found, there is no match
      }
      else
      {
         // since there is no variable data, we can only go by the name
         matched = pickFault(aColl.iterator());
      }

      return matched;
   }

   /**
    * Walks the iterator looking for a matching catch block. Returns match or
    * null if not found.
    * @param aIter
    */
   private AeFaultHandler pickFault(Iterator aIter)
   {
      while(aIter.hasNext())
      {
         AeFaultHandler fh = (AeFaultHandler) aIter.next();
         if (matches(fh))
         {
            return fh;
         }
      }
      return null;
   }

   /**
    * Returns true if the handler matches the spec.
    * @param aFaultHandler
    */
   protected boolean matches(AeFaultHandler aFaultHandler)
   {
      return faultNameMatches(aFaultHandler) && faultVariableMatches(aFaultHandler);
   }

   /**
    * Returns true if the variable message type matches the handler
    * @param aFaultHandler
    * @return true if both types are null or if they're not null and equal
    */
   private boolean faultVariableMatches(AeFaultHandler aFaultHandler)
   {
      return AeUtil.compareObjects(getMessageType(), aFaultHandler.getFaultVariableMessageType());
   }

   /**
    * Returns true if the fault name matches the handler
    * @param aFaultHandler
    * @return true if both names are null or if they're not null and equal
    */
   private boolean faultNameMatches(AeFaultHandler aFaultHandler)
   {
      return AeUtil.compareObjects(getFaultName(), aFaultHandler.getFaultName());
   }

   /**
    * Setter for the fault name
    * @param faultName
    */
   protected void setFaultName(QName faultName)
   {
      mFaultName = faultName;
   }

   /**
    * Getter for the fault name
    */
   protected QName getFaultName()
   {
      return mFaultName;
   }

   /**
    * Setter for the message type
    * @param messageType
    */
   protected void setMessageType(QName messageType)
   {
      mMessageType = messageType;
   }

   /**
    * Getter for the message type
    */
   protected QName getMessageType()
   {
      return mMessageType;
   }
   
      
}
