// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/AeActivitySwitchImpl.java,v 1.13 2004/08/10 19:09:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.activity.AeActivitySwitchDef;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.bpel.impl.activity.support.AeSwitchCase;
import org.activebpel.rt.bpel.impl.activity.support.AeSwitchOtherwise;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.util.AeUtil;

/**
 * Implementation of the bpel switch activity.
 */
public class AeActivitySwitchImpl extends AeActivityImpl
{
   /** list of cases for the switch */
   private List mCases = new ArrayList();
   
   /** otherwise case if none of the other cases matched */
   private AeSwitchOtherwise mOtherwise;
   
   /** default constructor for activity */
   public AeActivitySwitchImpl(AeActivitySwitchDef aActivityDef, IAeActivityParent aParent)
   {
      super(aActivityDef, aParent);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }
   
   /**
    * Adds the case to the switch.
    * @param aCase
    */
   public void addCase(AeSwitchCase aCase)
   {
      getCaseList().add(aCase);
   }

   /**
    * Sets the otherwise case on the switch.
    * @param aOtherwise
    */
   public void setOtherwise(AeSwitchOtherwise aOtherwise)
   {
      mOtherwise = aOtherwise;
   }
   
   /**
    * Getter for the otherwise case.
    */
   public AeSwitchOtherwise getOtherwise()
   {
      return mOtherwise;
   }
   
   /**
    * Getter for the case list.
    */
   protected List getCaseList()
   {
      return mCases;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getChildrenForStateChange()
    */
   public Iterator getChildrenForStateChange()
   {
      return AeUtil.join(mCases.iterator(), mOtherwise);
   }

   /**
    * Switch activity checks all cases and executes the applicable case container
    * or default if none are true.
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      IAeBpelObject child = findTrueCase();
      
      // note if child is null then calling this will complete execution
      // as all cases will be dead paths 
      setAllOtherChildrenToDeadPath(child);

      if (child != null)
      {
         getProcess().queueObjectToExecute(child);
      }
      
   }
   
   /**
    * Sets all of the other child objects to dead path except for the one passed in
    * which will be executed.
    * @param aChild can be null
    */
   private void setAllOtherChildrenToDeadPath(IAeBpelObject aChild) throws AeBusinessProcessException
   {
      setAllOtherToDeadPath(aChild, getChildrenForStateChange());
   }

   /**
    * Walks the case list looking for a case that evaluates to true. If a case
    * evaluates to false then it becomes a dead path. If it evaluates to true,
    * we'll return it. If none of the cases eval to true, then we'll fall back
    * on the otherwise. If there is no otherwise, then we'll return null.
    * @return IAeBpelObject a case that eval'd to true, the otherwise, or null if
    *         none eval'd to true.
    * @throws AeBusinessProcessException
    */
   private IAeBpelObject findTrueCase() throws AeBusinessProcessException
   {
      IAeBpelObject foundCase = null;
      for(Iterator iter = getCaseList().iterator(); iter.hasNext();)
      {
         AeSwitchCase caseObj = (AeSwitchCase)iter.next();
         if(caseObj.isEvalTrue())
         {
            foundCase = caseObj;
            break;
         }
      }

      if ( foundCase == null )
      {
         foundCase = getOtherwise();
      }
      
      return foundCase;
   }
   
   /**
    * Handles a child completion by completing switch activity.
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childComplete(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void childComplete(IAeBpelObject aChild) throws AeBusinessProcessException
   {
      if (childrenAreDone())
      {
         objectCompleted();
      }
   }
}
