// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/IAeMessageReceiverActivity.java,v 1.2 2005/12/06 22:27:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeBusinessProcess;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.IAePartnerLink;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.activity.support.IAeCorrelationListener;
import org.activebpel.rt.message.IAeMessageData;

/**
 * Provides the necessary methods in order to share some common code between
 * <code>receive</code> and <code>onMessage</code> since both have special
 * multi-start behavior. 
 */
public interface IAeMessageReceiverActivity extends IAeCorrelationListener
{
   /**
    * Notifies the activity that its data has arrived.
    * 
    * @param aMessage
    * @throws AeBusinessProcessException
    */
   public void onMessage(IAeMessageData aMessage) throws AeBusinessProcessException;
   
   /**
    * Used to report a fault back to the message receive activity. At this point,
    * the only fault that will get reported will be a bpws:conflictingRequest 
    * 
    * @param aFault
    * @throws AeBusinessProcessException
    */
   public void onFault(IAeFault aFault) throws AeBusinessProcessException;
   
   /**
    * Returns the xpath location of the activity
    */
   public String getLocationPath();
   
   /**
    * Gets the correlation def objects for the activity
    */
   public Iterator getCorrelationList();
   
   /**
    * Getter for the process (pass through to AeAbstractBpelObject's impl)
    */
   public IAeBusinessProcess getProcess();
   
   /**
    * Getter for the partner link (pass through to AeAbstractBpelObject's impl)
    */
   public IAePartnerLink getPartnerLink() throws AeBusinessProcessException;
   
   /**
    * Gets the implementation of the correlation set (pass through to AeAbstractBpelObject's impl)
    * @param aString
    */
   public AeCorrelationSet getCorrelationSet(String aString) throws AeBusinessProcessException;

   /**
    * Gets the name of the operation from the underlying def object
    */
   public String getOperation();

   /**
    * Gets the port type from the underlying def object
    */
   public QName getPortType();
   
   /**
    * Returns true if the def's createInstance attribute is 'yes'
    */
   public boolean canCreateInstance();
   
}
