// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeBaseEvent.java,v 1.4 2004/08/05 16:48:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.impl.AeBpelState;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.bpel.impl.activity.IAeEventParent;

/**
 * Base class for message and alarm events. 
 */
abstract public class AeBaseEvent extends AeActivityParent
{
   /** flag for the active state of the event */
   private boolean mActive = false;

   /** True if we have queued our receiver. */
   private boolean mQueued = false;
   
   /**
    * @param aDef
    * @param aParent
    */
   public AeBaseEvent(AeBaseDef aDef, IAeBpelObject aParent)
   {
      super(aDef, aParent);
   }

   /**
    * Overrides the base class by allowing the execute state to transition
    * to dead path and during that state transition we will dequeue our
    * receiver if queued.
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#validateStateChange(org.activebpel.rt.bpel.impl.AeBpelState)
    */
   protected void validateStateChange(AeBpelState aNewState) throws AeBusinessProcessException
   {
      // always dequeue queued entries on a state change (terminate, deadpath, etc).
      if(isQueued())
      {
         dequeue();
         setQueued(false);
      }
         
      // Allow change from executing to dead path otherwise verify via base class
      if(! (AeBpelState.EXECUTING.equals(getState()) &&
            AeBpelState.DEAD_PATH.equals(aNewState)))
      {
         super.validateStateChange(aNewState);
      }
   }
      
   /**
    * This is an abstract method to dequeue our receiver if it has been queued 
    * derived classes should implement the actual dequeue code.
    * @throws AeBusinessProcessException
    */
   abstract protected void dequeue() throws AeBusinessProcessException;
   
   /**
    * Sets the event to active, notifies parent then queues the child activity
    * to execute. 
    * @throws AeBusinessProcessException
    */
   protected void executeChild() throws AeBusinessProcessException
   {
      // make sure that no race condition causes a timer or message to be
      // dispatched to us after we have been canceled
      if(! getState().isFinal())
      {
         setActive(true);
         ((IAeEventParent)getParent()).childActive(this);
         getProcess().queueObjectToExecute(mChild);
      }
   }
   
   /**
    * Sets the event to not active and calls that we are complete.
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childComplete(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void childComplete(IAeBpelObject aChild)
      throws AeBusinessProcessException
   {
      // set that we are no longer actively executing
      setActive(false);
      
      // when our child is complete we are in turn completed
      objectCompleted();
   }

   /**
    * Getter for the enabled. Enabled events are capable of being activated.
    */
   public boolean isActive()
   {
      return mActive;
   }

   /**
    * Setter for the active flag. Enabled events are capable of being activated.
    * @param aB
    */
   public void setActive(boolean aB)
   {
      mActive = aB;
   }
   
   /**
    * @return boolean True if the receiver has been queued.
    */
   public boolean isQueued()
   {
      return mQueued;
   }

   /**
    * @param aQueued Sets that we have a receiver queued if true.
    */
   public void setQueued(boolean aQueued)
   {
      mQueued = aQueued;
   }

}
