//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeBaseHandler.java,v 1.1 2005/10/28 21:07:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.bpel.impl.activity.AeActivityCompensateImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;

/**
 * Base class for Compensation or Fault handlers.
 */
public abstract class AeBaseHandler extends AeActivityParent
{
   /**
    * Implicit compensator activity for compensating coordinated activities.
    */
   private AeActivityCompensateImpl mCoordinatedCompensator;
      
   /**
    * Child activity fault object.
    */
   private IAeFault mChildFaultObject;
   
   /**
    * Default construct.
    * @param aDef
    * @param aParent
    */
   public AeBaseHandler(AeBaseDef aDef, IAeBpelObject aParent)
   {
      super(aDef, aParent);   
   }
      
   /**
    * @return Returns the childFaultObject.
    */
   protected IAeFault getChildFaultObject()
   {
      return mChildFaultObject;
   }
      
   /** 
    * @return true if there is an implicit compensator created for coordinated activities.
    */
   public boolean hasCoordinatedCompensator()
   {
      return (mCoordinatedCompensator != null);
   }
   
   /** 
    * Returns a compensator with coordinated CompInfo objects. This method creates one
    * if the current member variable is null.
    * @return Returns a compensator with coordinated CompInfo objects.
    */
   public AeActivityCompensateImpl getCoordinatedActivityCompensator()
   {
      if (mCoordinatedCompensator == null)
      {
         mCoordinatedCompensator = AeActivityCompensateImpl.createImplicitCompensation(this, true);
      }
      return mCoordinatedCompensator;
   }
   
   /**
    * @return Returns true if the coordinated activities (invokes) 
    * should also be compensated after executing the normal list of CompInfo objects.
    */
   protected abstract boolean isEnableCoordinatedActivityCompensation();
 
   /** 
    * Overrides method to 
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#getChildrenForStateChange()
    */
   public Iterator getChildrenForStateChange()
   {
      List list = new ArrayList();
      list.add(getActivity());
      if (hasCoordinatedCompensator())
      {
         list.add(getCoordinatedActivityCompensator());
      }
      return list.iterator();
   }   
   
   /**
    * Queues an implicit compensate activity to compensate coordinated CompInfo objects.
    * @param aChild
    * @param aChildFaultObject
    * @return true if an implicit compensate activity was queued.
    * @throws AeBusinessProcessException
    */
   protected boolean queueCoordinatedCompensator(IAeBpelObject aChild, IAeFault aChildFaultObject) throws AeBusinessProcessException
   {
      boolean rVal = false;
      
      // "install" compensator for coordinated activities if the aChild is
      // the child of this handler; and the compensation for coordinated activites is enabled.
      
      if ( getActivity() == aChild )
      {  
         // save the child fault
         setFault(aChildFaultObject);
         
         // get the enclosing scope
         AeActivityScopeImpl scope = findEnclosingScope();
         // queue implicit compensator iff scope has coordinated activities
         if (scope != null && scope.hasCoordinations())
         {
            // comp or cancel active coordinations
            scope.getCoordinationContainer().cancelActiveCoordinations();
            // queue implicit compensate activity if needed for the activities which are eligible for compensation.
            // Note: we queue up an implicit compensate activity only if the coordinated comp info objects
            // have not been run before to avoid repeated-comp-fault [scope.getCompInfo().hasEnabledCoordinatedChildren()]            
            if ( isEnableCoordinatedActivityCompensation() && !hasCoordinatedCompensator() 
                  && scope.getCompInfo().hasEnabledCoordinatedChildren())
            {
               getProcess().queueObjectToExecute(getCoordinatedActivityCompensator());
               rVal = true;
            }
         }
         else if (scope == null)
         {
            staticAnalysisFailure(AeMessages.getString("AeBaseHandler.ENCLOSING_SCOPE_NOT_FOUND")); //$NON-NLS-1$  
         }                    
      }
      return rVal;
   }   
}
