// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeDefaultFaultHandler.java,v 1.14 2006/01/24 23:46:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.def.AeBaseDef;
import org.activebpel.rt.bpel.def.AeDefaultFaultHandlerDef;
import org.activebpel.rt.bpel.impl.AeBpelState;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;

/**
 * Implementation of CatchAll fault handler. 
 */
public class AeDefaultFaultHandler extends AeBaseHandler
{
   /**
    * Accepts the base def and scope as parent.
    * @param aDef
    * @param aParent
    */
   public AeDefaultFaultHandler(AeDefaultFaultHandlerDef aDef, AeActivityScopeImpl aParent)
   {
      super(aDef, aParent);      
   }

   /**
    * Protected ctor for use by sub class. This allows the other fault handlers.
    * @param aDef
    * @param aParent
    */
   protected AeDefaultFaultHandler(AeBaseDef aDef, IAeBpelObject aParent)
   {
      super(aDef, aParent);
   }
    
   /**
    * Returns true since the coordinated activities (invokes) should also be compensated 
    * after executing the normal list of CompInfo objects. 
    * @return this method returns true
    */
   protected boolean isEnableCoordinatedActivityCompensation()
   {
      return true;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      getProcess().queueObjectToExecute(getActivity());
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childComplete(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void childComplete(IAeBpelObject aChild)
      throws AeBusinessProcessException
   {
      if (queueCoordinatedCompensator(aChild, null))
      {
         // compensator for coordinated activities (invokes) have been queued up, so activity is not yet complete. 
         return;
      }
      // since fault handlers can only have one child, getActivity() is the equivalent
      // to aChild.
      if ( getActivity().getState() != AeBpelState.DEAD_PATH )
      {
         setState(AeBpelState.FINISHED);
         AeActivityScopeImpl scope = (AeActivityScopeImpl) getParent();
         scope.faultHandlerComplete();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#childCompleteWithFault(org.activebpel.rt.bpel.impl.IAeBpelObject, org.activebpel.rt.bpel.IAeFault)
    */
   public void childCompleteWithFault(
      IAeBpelObject aChild,
      IAeFault aFaultObject)
      throws AeBusinessProcessException
   {
      if (queueCoordinatedCompensator(aChild, aFaultObject))
      {
         return;
      }      
      if ( isAllowedToRethrow() )
      {
         setFaultedState( getFault() );
         AeActivityScopeImpl scope = (AeActivityScopeImpl) getParent();
         scope.faultHandlerCompleteWithFault(getFault() );
      }
      else
      {
         childComplete( getActivity() );
      }
   }
   
   /**
    * Getter for the rethrow flag. 
    */
   public boolean isAllowedToRethrow()
   {
      // the fault should not be null, null check just in case
      return getFault() == null || getFault().isRethrowable();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#noMoreChildrenToTerminate()
    */
   protected void noMoreChildrenToTerminate() throws AeBusinessProcessException
   {
      super.noMoreChildrenToTerminate();
      if (getState() == AeBpelState.DEAD_PATH)
      {
         ((AeActivityScopeImpl)getParent()).faultHandlerComplete();
      }
   }

   /**
    * Overrides to do nothing. Fault handlers are allowed to complete once they
    * start executing. If their parent scope is terminating, then the termination
    * of the scope will complete once the fault handler is done executing.
    *  
    * @see org.activebpel.rt.bpel.impl.AeAbstractBpelObject#terminate()
    */
   public void terminate() throws AeBusinessProcessException
   {
      if (getProcess().isTerminating())
      {
         super.terminate();
      }
   }
}
