// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeFault.java,v 1.13 2006/01/24 23:46:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.impl.IAeBpelObject;
import org.activebpel.rt.message.IAeMessageData;

/**
 * Models BPEL fault objects. These faults can be generated through the
 * execution of an <code>invoke</code>, <code>throw</code>, or through a BPEL
 * error like a <code>bpws:correlationViolation</code> (among others).
 */
public class AeFault implements IAeFault
{
   /** fault name */
   private QName mName;
   /** fault data */
   private IAeMessageData mMessageData;
   /** fault source */
   private IAeBpelObject mSource ;
   /** fault info, from exception, if any */
   private String mInfo = ""; //$NON-NLS-1$
   /** stacktrace from exception, if any */
   private String mDetailedInfo;
   /** return true if this fault has not caused a process to suspend */
   private boolean mSuspendable = true;
   /** return true if this fault is rethrowable */
   private boolean mRethrowable = true;

   /**
    * Ctor takes name and variable.
    * @param aName
    * @param aMessageData can be null if name is supplied
    */
   public AeFault(QName aName, IAeMessageData aMessageData)
   {
      mName = aName;
      mMessageData = aMessageData;
   }
   
   /**
    * Getter for the fault name
    * @return QName
    */
   public QName getFaultName()
   {
      return mName;
   }
   
   /**
    * Getter for the message
    * @return IAeMessageData or null
    */
   public IAeMessageData getMessageData()
   {
      return mMessageData;
   }
   
   /**
    * Returns true if the fault contains message data.
    */
   public boolean hasMessageData()
   {
      return getMessageData() != null;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeFault#isRethrowable()
    */
   public boolean isRethrowable()
   {
      return mRethrowable;
   }

   /**
    * @param aRethrowable The rethrowable to set.
    */
   public void setRethrowable(boolean aRethrowable)
   {
      mRethrowable = aRethrowable;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeFault#getSource()
    */
   public IAeBpelObject getSource()
   {
      return mSource ;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeFault#setSource(org.activebpel.rt.bpel.impl.IAeBpelObject)
    */
   public void setSource(IAeBpelObject aSource)
   {
      mSource = aSource ;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeFault#getInfo()
    */
   public String getInfo()
   {
      return mInfo ;
   }

   /**
    * @see org.activebpel.rt.bpel.IAeFault#setInfo(java.lang.String)
    */
   public void setInfo(String aInfo)
   {
      mInfo = aInfo ;
   }
   
   
   /**
    * @see org.activebpel.rt.bpel.IAeFault#getDetailedInfo()
    */
   public String getDetailedInfo()
   {
      return mDetailedInfo;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeFault#setDetailedInfo(java.lang.String)
    */
   public void setDetailedInfo(String aDetailedInfo)
   {
      mDetailedInfo = aDetailedInfo;
   }
   
   
   /**
    * @see org.activebpel.rt.bpel.IAeFault#isSuspendable()
    */
   public boolean isSuspendable()
   {
      return mSuspendable;
   }
   
   /**
    * @see org.activebpel.rt.bpel.IAeFault#setSuspendable(boolean)
    */
   public void setSuspendable(boolean aValue)
   {
      mSuspendable = aValue;
   }
}
