// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeFaultHandler.java,v 1.7 2005/12/03 01:01:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.AeFaultHandlerDef;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.util.AeUtil;

/**
 * Models the <code>catch</code> and <code>catchAll</code> providing a place to
 * store the child activity for the fault.
 */
public class AeFaultHandler extends AeDefaultFaultHandler
{
   /**
    * Ctor for creating <code>catch</code> implementation.
    * @param aDef
    * @param aParent
    */
   public AeFaultHandler(AeFaultHandlerDef aDef, AeActivityScopeImpl aParent)
   {
      super(aDef, aParent);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept(IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }   
   
   /**
    * Returns true since the coordinated activities (invokes) should also be compensated 
    * after executing the normal list of CompInfo objects. 
    * @return true for explicit compensation handler.
    */
   protected boolean isEnableCoordinatedActivityCompensation()
   {
      // return true since the coordinated invokes should be compensated after running
      // the fault handler
      return true;
   }   
   
   /**
    * Convenience getter for the fault name, delegates to the def object.
    */
   public QName getFaultName()
   {
      AeFaultHandlerDef def = (AeFaultHandlerDef) getDefinition();
      return def.getFaultName(); 
   }
   
   /**
    * Gets the message type for the fault variable or null if there is no
    * fault variable defined for this handler.
    */
   public QName getFaultVariableMessageType()
   {
      IAeVariable variable = getFaultVariable();
      if (variable != null)
      {
         return variable.getMessageType(); 
      }
      
      return null;
   }

   /**
    * Gets the variable for this catch block or null if there is not fault variable
    * defined here.
    */
   public IAeVariable getFaultVariable()
   {
      IAeVariable variable = null;
      AeFaultHandlerDef def = (AeFaultHandlerDef) getDefinition();
      String variableName = def.getFaultVariable();
      if (!AeUtil.isNullOrEmpty(variableName))
      {
         variable = getVariable(variableName);
      }
      return variable;
   }
}
