// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeOnAlarm.java,v 1.14 2006/02/01 22:44:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.def.activity.support.AeOnAlarmDef;
import org.activebpel.rt.bpel.impl.AeBpelState;
import org.activebpel.rt.bpel.impl.IAeAlarmReceiver;
import org.activebpel.rt.bpel.impl.activity.AeActivityPickImpl;
import org.activebpel.rt.bpel.impl.activity.IAeEventParent;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;

import java.util.Date;

/**
 * Models the <code>onAlarm</code> in a <code>pick</code> or in the <code>eventHandlers</code>.
 */
public class AeOnAlarm extends AeBaseEvent implements IAeAlarmReceiver
{
   /**
    * Requires the alarm def and parent to create.
    * @param aDef
    * @param aParent
    */
   public AeOnAlarm(AeOnAlarmDef aDef, IAeEventParent aParent)
   {
      super(aDef, aParent);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.IAeAlarmReceiver#getGroupId()
    */
   public int getGroupId()
   {
      if (getParent() instanceof AeActivityPickImpl)
      {
         return getParent().getLocationId();
      }
      else
      {
         return getLocationId();
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }

   /**
    * Fire an info event when we go to Ready To Execute.
    *
    * @see org.activebpel.rt.bpel.impl.IAeBpelObject#setState(org.activebpel.rt.bpel.impl.AeBpelState)
    */
   public void setState( AeBpelState aState ) throws AeBusinessProcessException
   {
      super.setState(aState);
   }

   /**
    * Implements the alarm execution logic.  The alarm queues a deadline
    * alarm with the engine.  The engine will then call back the alarm
    * when the deadline is reached.
    *
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      AeOnAlarmDef def = (AeOnAlarmDef)getDefinition();
      Date deadline = extractDeadline(def);

      setQueued(true);
      getProcess().queueAlarm(this, deadline);
   }

   /**
    * Handle the recipt of our alarm going off.
    */
   public void onAlarm() throws AeBusinessProcessException
   {
      if (isQueued())
      {
         // set that we are no longer queued since we have been called
         setQueued(false);
   
         // queue our child activity
         executeChild();
      }
   }

   /**
    * Dequeue our alarm.
    * @see org.activebpel.rt.bpel.impl.activity.support.AeBaseEvent#dequeue()
    */
   protected void dequeue() throws AeBusinessProcessException
   {
      if (isQueued())
      {
         getProcess().dequeueAlarm(this);
         setQueued(false);
      }
   }

}
