// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/activity/support/AeOnMessage.java,v 1.27 2006/01/11 21:01:1
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.activity.support;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.IAePartnerLink;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.activity.AeActivityPickDef;
import org.activebpel.rt.bpel.def.activity.support.AeCorrelationDef;
import org.activebpel.rt.bpel.def.activity.support.AeOnMessageDef;
import org.activebpel.rt.bpel.impl.IAeActivityParent;
import org.activebpel.rt.bpel.impl.activity.AeActivityPickImpl;
import org.activebpel.rt.bpel.impl.activity.AeMultiStartHelper;
import org.activebpel.rt.bpel.impl.activity.IAeEventParent;
import org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity;
import org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor;
import org.activebpel.rt.message.IAeMessageData;

/**
 * Models the <code>onMessage</code> that is part of a <code>pick</code>
 * or the <code>eventHandlers</code>.
 */
public class AeOnMessage extends AeBaseEvent 
      implements IAeActivityParent, IAeMessageReceiverActivity
{
   /**
    * Requires the def object and parent.
    * @param aDef
    * @param aParent
    */
   public AeOnMessage(AeOnMessageDef aDef, IAeEventParent aParent)
   {
      super(aDef, aParent);
   }

   /**
    * Convenience method to avoid casting
    */
   private AeOnMessageDef getDef()
   {
      return (AeOnMessageDef) getDefinition();
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeVisitable#accept(org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor)
    */
   public void accept( IAeImplVisitor aVisitor ) throws AeBusinessProcessException
   {
      aVisitor.visit(this);
   }
   
   /**
    * Implements on message execution logic.  The on message queues a waiting for 
    * message with the engine.  The engine will then call back the on message
    * when its message is ready.
    * @see org.activebpel.rt.bpel.impl.IAeExecutableBpelObject#execute()
    */
   public void execute() throws AeBusinessProcessException
   {
      boolean okToQueue = AeMultiStartHelper.checkForMultiStartBehavior(this);
      
      if (okToQueue)
      {
         queueReceive();
      }
   }

   /**
    * Queues the message receiver.
    */
   private void queueReceive() throws AeBusinessProcessException
   {
      setQueued(true);
      
      String messageExchange = findEnclosingScope().getMessageExchangePath(getDef().getMessageExchange());
      
      getProcess().queueMessageReceiver(this, getGroupId(), messageExchange);
   }

   /**
    * Gets the OnMessage's group id.
    */
   protected int getGroupId()
   {
      if (getParent() instanceof AeActivityPickImpl)
      {
         return getParent().getLocationId();
      }
      else
      {
         return getLocationId();
      }
   }

   /**
    * Handle the receipt of our correlated message.
    * @see org.activebpel.rt.bpel.impl.IAeMessageReceiver#onMessage(org.activebpel.rt.message.IAeMessageData)
    */
   public void onMessage(IAeMessageData aMessage) throws AeBusinessProcessException
   {
      // TODO (MF) BPEL 2.0 don't remove or dequeue the onMessage if it's an event 
      getProcess().removeReceiverKeyForConflictingReceives(this);
      
      AeOnMessageDef def = (AeOnMessageDef)getDefinition();
      
      // set that we are no longer queued since we have been called
      setQueued(false);

      // Increment the "outstanding receives" counter on the process.
      if (!getDef().isOneWay())
      {
         findEnclosingScope().incrementOutstandingReplyCount(getDef().getMessageExchange());
      }

      // put the data into our associated variable
      IAeVariable variable = getVariable(def.getVariable());
      variable.setMessageData(aMessage);
      if (getProcess().getEngine().getEngineConfiguration().getUpdatableEngineConfig().validateServiceMessages())
      {
         variable.validate();
      }
      
      for(Iterator iter = def.getCorrelationList(); iter.hasNext(); )
      {
         AeCorrelationDef corrDef = (AeCorrelationDef)iter.next();
         if(corrDef.isInitiate())
         {
            AeCorrelationSet corrSet = getCorrelationSet(corrDef.getCorrelationSetName());
            corrSet.initiateOrValidate(variable, corrDef.isInitiate());            
         }
      }
      
      // queue our child to execute
      executeChild();
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity#onFault(org.activebpel.rt.bpel.IAeFault)
    */
   public void onFault(IAeFault aFault) throws AeBusinessProcessException
   {
      objectCompletedWithFault(aFault);
   }
   
   /**
    * Dequeue our message receiver.
    * @see org.activebpel.rt.bpel.impl.activity.support.AeBaseEvent#dequeue()
    */
   protected void dequeue() throws AeBusinessProcessException
   {
      if (isQueued())
      {
         getProcess().dequeueMessageReceiver(this);
         setQueued(false);
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity#canCreateInstance()
    */
   public boolean canCreateInstance()
   {
      boolean canCreate = false;
      if (getParent() instanceof AeActivityPickImpl)
      {
         AeActivityPickImpl pick = (AeActivityPickImpl) getParent();
         AeActivityPickDef pickDef = (AeActivityPickDef) pick.getDefinition();
         canCreate = pickDef.isCreateInstance();
      }
      return canCreate;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity#getCorrelationList()
    */
   public Iterator getCorrelationList()
   {
      return getDef().getCorrelationList();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity#getOperation()
    */
   public String getOperation()
   {
      return getDef().getOperation();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity#getPartnerLink()
    */
   public IAePartnerLink getPartnerLink() throws AeBusinessProcessException
   {
      return getPartnerLink(getDef().getPartnerLink());
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.IAeMessageReceiverActivity#getPortType()
    */
   public QName getPortType()
   {
      return getDef().getPortType();
   }

   /**
    * @see org.activebpel.rt.bpel.impl.activity.support.IAeCorrelationListener#correlationSetInitialized(org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet)
    */
   public void correlationSetInitialized(AeCorrelationSet aSet)
      throws AeBusinessProcessException
   {
      aSet.removeCorrelationListener(this);
      
      if (AeMultiStartHelper.isCorrelatedDataAvailable(this))
      {
         queueReceive();
      }
   }
}
