// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/endpoints/AeWSAddressingEndpointDeserializer.java,v 1.11 2006/02/06 21:39:4
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.endpoints;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeEndpointReference;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.bpel.impl.AeEndpointReference;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.util.AeXmlUtil;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Creates an IAeEndpointReference instance based on the xml format defined in
 * the WS-Addressing spec. 
 */
public class AeWSAddressingEndpointDeserializer implements IAeEndpointDeserializer
{
   /** singleton instance */
   private static final AeWSAddressingEndpointDeserializer sSingleton = new AeWSAddressingEndpointDeserializer();
   
   /**
    * Private ctor for singleton pattern 
    */
   private AeWSAddressingEndpointDeserializer()
   {
   }
   
   /**
    * Getter for the singleton instance
    */
   public static AeWSAddressingEndpointDeserializer getInstance()
   {
      return sSingleton;
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.endpoints.IAeEndpointDeserializer#deserializeEndpoint(org.w3c.dom.Element)
    */
   public IAeEndpointReference deserializeEndpoint(Element aData)
      throws AeBusinessProcessException
   {
      return deserializeEndpoint(aData, null);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.endpoints.IAeEndpointDeserializer#deserializeEndpoint(org.w3c.dom.Element, org.activebpel.rt.bpel.IAeEndpointReference)
    */
   public IAeEndpointReference deserializeEndpoint(Element aData, IAeEndpointReference aRef)
      throws AeBusinessProcessException
   {
      IAeEndpointReference ref = aRef == null ? new AeEndpointReference() : aRef;
      
      ref.setSourceNamespace(IAeBPELConstants.WSA_NAMESPACE_URI);
      
      NodeList children = aData.getChildNodes();
      for (int i=0, len=children.getLength(); i < len; i++)
      {
         Node child = children.item(i);
         if (child.getNodeType() != Node.ELEMENT_NODE)
            continue;
      
         if (IAeBPELConstants.WSA_NAMESPACE_URI.equals(child.getNamespaceURI()))
         {
            if ("Address".equals(child.getLocalName())) //$NON-NLS-1$
               ref.setAddress(AeXmlUtil.getText((Element) child));
            else if ("PortType".equals(child.getLocalName())) //$NON-NLS-1$
               ref.setPortType(extractQNameData((Element)child));
            else if ("ReferenceProperties".equals(child.getLocalName())) //$NON-NLS-1$
            {
               NodeList refProps = child.getChildNodes();
               for (int j=0, length=refProps.getLength(); j < length; j++)
               {

                  Node prop = refProps.item(j);                  
                  
                  if (prop.getNodeType() != Node.ELEMENT_NODE)
                        continue;
           
                  // add name/value pair 
                  // this does not preserve attributes of reference property
                  QName key = new QName(prop.getNamespaceURI(), prop.getLocalName());
                     ref.addProperty(key, AeXmlUtil.getText((Element) prop));

                  // add element to reference properties for header propagation
                  ref.addReferenceProperty((Element) prop);
                     
               }
               
            }
            else if ("ServiceName".equals(child.getLocalName())) //$NON-NLS-1$
            {
               String portName = ((Element)child).getAttribute("PortName"); //$NON-NLS-1$
               if (! AeUtil.isNullOrEmpty(portName))
                  ref.setServicePort(portName);
               
               ref.setServiceName(extractQNameData((Element)child));
            }
         }
         else if (IAeBPELConstants.WSP_NAMESPACE_URI.equals(child.getNamespaceURI()))
         {
            if ("Policy".equals(child.getLocalName())) //$NON-NLS-1$
               ref.addPolicyElement((Element)child);
         }
         else
         {
            // Extensibility element
            ref.addExtensibilityElement((Element)child);
         }
      }
      return ref;
   }

   /**
    * Extracts the text value from this element which is expected to be a QName. 
    * @param aElement
    * @throws AeBusinessProcessException
    */
   protected QName extractQNameData(Element aElement) throws AeBusinessProcessException
   {
      String qname  = AeXmlUtil.getText(aElement);
      if (AeUtil.isNullOrEmpty(qname) && aElement instanceof SOAPElement)
         qname = ((SOAPElement)aElement).getValue();
      String prefix = AeXmlUtil.extractPrefix(qname);
      String nsURI  = null;
      if (prefix != null)
      {
         if (aElement instanceof SOAPElement)
         {
            SOAPElement soapEl = (SOAPElement) aElement;
            nsURI = soapEl.getNamespaceURI(prefix);
         }
         else
         {
            nsURI = AeXmlUtil.getNamespaceForPrefix(aElement, prefix);
         }
      }
      
      if (nsURI == null)
         throw new AeBusinessProcessException(AeMessages.getString("AeWSAddressingEndpointDeserializer.ERROR_6") + qname); //$NON-NLS-1$
      
      return new QName(nsURI, AeXmlUtil.extractLocalPart(qname));
   }
}
