// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/endpoints/AeWSAddressingEndpointSerializer.java,v 1.15 2006/01/26 18:11:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.endpoints;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.namespace.QName;

import org.activebpel.rt.IAeConstants;
import org.activebpel.rt.bpel.def.IAeBPELConstants;
import org.activebpel.rt.util.AeXmlUtil;
import org.activebpel.wsio.IAeWebServiceEndpointReference;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Endpoint serializer for WS-Addressing endpoints. 
 */
public class AeWSAddressingEndpointSerializer implements IAeEndpointSerializer
{
   /** singleton instance */
   private static final AeWSAddressingEndpointSerializer sSingleton = new AeWSAddressingEndpointSerializer();
   
   /**
    * Private ctor for singleton pattern 
    */
   private AeWSAddressingEndpointSerializer()
   {
   }
   
   /**
    * Getter for the singleton
    */
   public static AeWSAddressingEndpointSerializer getInstance()
   {
      return sSingleton;
   }

   /**
    * @see org.activebpel.rt.bpel.impl.endpoints.IAeEndpointSerializer#serializeEndpoint(org.activebpel.wsio.IAeWebServiceEndpointReference)
    */
   public Document serializeEndpoint(IAeWebServiceEndpointReference aRef)
   {
      Document doc = AeXmlUtil.newDocument();
      
      Map qnameToPrefixMap = new HashMap();
         
      Element er = AeXmlUtil.addElementNS(doc, IAeBPELConstants.WSA_NAMESPACE_URI, "wsa:EndpointReference", null); //$NON-NLS-1$
      
      // prime the pump w/ the wsa ns prefix mapping. it'll get added to the doc below
      qnameToPrefixMap.put(IAeBPELConstants.WSA_NAMESPACE_URI, "wsa"); //$NON-NLS-1$

      // Add the address, which is mandatory
      AeXmlUtil.addElementNS(er, IAeBPELConstants.WSA_NAMESPACE_URI, "wsa:Address", aRef.getAddress()); //$NON-NLS-1$

      
      // Add the port type if one was specified
      if (aRef.getPortType() != null)
      {
         String portType = getQNameStr(aRef.getPortType(), qnameToPrefixMap);
         AeXmlUtil.addElementNS(er, IAeBPELConstants.WSA_NAMESPACE_URI, "wsa:PortType", portType); //$NON-NLS-1$
      }
      
      // Add the service name if one was specified
      if (aRef.getServiceName() != null)
      {
         String serviceName = getQNameStr(aRef.getServiceName(), qnameToPrefixMap);
         Element svcName = AeXmlUtil.addElementNS(er, IAeBPELConstants.WSA_NAMESPACE_URI, "wsa:ServiceName", serviceName); //$NON-NLS-1$
         
         if (aRef.getServicePort() != null)
            svcName.setAttribute("PortName", aRef.getServicePort()); //$NON-NLS-1$
      }
      
      // Add reference properties if any were defined
      if (!aRef.getReferenceProperties().isEmpty()) 
      {
         Element props = AeXmlUtil.addElementNS(er, IAeBPELConstants.WSA_NAMESPACE_URI, "wsa:ReferenceProperties", null); //$NON-NLS-1$
         for(Iterator iter= aRef.getReferenceProperties().iterator(); iter.hasNext(); ) 
         {
               Element refElem = (Element)iter.next();
               String prefix = (String) qnameToPrefixMap.get(refElem.getNamespaceURI());
               if (prefix == null)
               {
                  prefix = refElem.getPrefix();
                  qnameToPrefixMap.put(refElem.getNamespaceURI(), prefix);
               }
               
               refElem.setPrefix(prefix);
                              
               props.appendChild(doc.importNode(refElem, true));
         }      
                
      }
      
      
      // Add policies if any were defined
      for(Iterator iter= aRef.getPolicies().iterator(); iter.hasNext(); )
         er.appendChild(doc.importNode((Element)iter.next(), true));

      // Add extensibility elements if any
      for (Iterator iter = aRef.getExtensibilityElements(); iter.hasNext(); )
         er.appendChild(doc.importNode((Element)iter.next(), true));

      // add all of the namespaces we've discovered      
      for (Iterator iter = qnameToPrefixMap.entrySet().iterator(); iter.hasNext();)
      {
         Map.Entry entry = (Entry) iter.next();
         String namespace = (String) entry.getKey();
         String prefix = (String) entry.getValue();
         
         er.setAttributeNS(IAeConstants.W3C_XMLNS, "xmlns:"+prefix, namespace); //$NON-NLS-1$
      }
     
      return doc;
   }

   /**
    * Converts a QName to a String using the prefix already found in the map or 
    * it'll generate a new prefix and add it to the map for this QName
    * @param aName
    * @param qnameToPrefixMap
    */
   private String getQNameStr(QName aName, Map qnameToPrefixMap)
   {
      String prefix = (String) qnameToPrefixMap.get(aName.getNamespaceURI());
      if (prefix == null)
      {
         prefix = "ns" + (qnameToPrefixMap.size() + 1); //$NON-NLS-1$
         qnameToPrefixMap.put(aName.getNamespaceURI(), prefix);
      }
      return prefix + ":" + aName.getLocalPart(); //$NON-NLS-1$
   }
}
