//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/expr/AeAbstractExpressionRunner.java,v 1.2 2005/08/04 20:01:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.expr;

import java.text.MessageFormat;
import java.text.ParseException;
import java.util.Date;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.impl.AeBpelException;
import org.activebpel.rt.bpel.impl.AeFaultFactory;
import org.activebpel.rt.util.AeSchemaDate;
import org.activebpel.rt.util.AeSchemaDateTime;
import org.activebpel.rt.util.AeSchemaDuration;
/*import org.activebpel.rt.xml.schema.AeSchemaDate;
import org.activebpel.rt.xml.schema.AeSchemaDateTime;
import org.activebpel.rt.xml.schema.AeSchemaDuration;
*/
/*import org.activebpel.rt.util.AeSchemaDate;
import org.activebpel.rt.util.AeSchemaDateTime;
import org.activebpel.rt.util.AeSchemaDuration;
*/
/**
 * This class defines some common functionality for all of our expression runners.
 */
public abstract class AeAbstractExpressionRunner implements IAeExpressionRunner
{
   /**
    * @see org.activebpel.rt.bpel.impl.expr.IAeExpressionRunner#executeExpression(org.activebpel.rt.bpel.impl.expr.IAeExpressionRunnerContext, java.lang.String)
    */
   public Object executeExpression(IAeExpressionRunnerContext aContext, String aExpression)
         throws AeException
   {
      Object nativeRval = doExecuteExpression(aExpression, aContext);
      return doConvertToJavaType(nativeRval, aExpression, aContext);
   }

   /**
    * This method is called to do the actual execution of the expression.  The return value is whatever
    * is returned from the 3rd party engine being use for a specific language (saxon, rhino, etc).
    * 
    * @param aExpression
    * @param aContext
    */
   protected abstract Object doExecuteExpression(String aExpression, IAeExpressionRunnerContext aContext)
         throws AeException;

   /**
    * This method is called to convert a native return value (which will be language and 3rd party
    * engine dependent) to an appropriate Java type.
    * 
    * @param aNativeRval
    * @param aExpression
    * @param aContext
    * @throws AeException
    */
   protected abstract Object doConvertToJavaType(Object aNativeRval, String aExpression,
         IAeExpressionRunnerContext aContext) throws AeException;

   /**
    * Converts a return value from <code>executeExpression</code> into a Java Boolean object.
    * This method can be overridden by language specific classes in order to be more strict
    * about the return type.
    * 
    * @param aNativeRval
    * @param aExpression
    * @param aContext
    * @throws AeException
    */
   protected Boolean doConvertToBoolean(Object aNativeRval, String aExpression,
         IAeExpressionRunnerContext aContext) throws AeException
   {
      if (aNativeRval instanceof Boolean)
      {
         return (Boolean) aNativeRval;
      }
      else if (aNativeRval instanceof String)
      {
         String str = (String) aNativeRval;
         if ("true".equalsIgnoreCase(str) || "false".equalsIgnoreCase(str)) //$NON-NLS-1$ //$NON-NLS-2$
         {
            return new Boolean("true".equalsIgnoreCase(str)); //$NON-NLS-1$
         }
      }
      else if (aNativeRval instanceof Integer)
      {
         int ival = ((Integer) aNativeRval).intValue();
         return new Boolean(ival > 0);
      }

      String msg = MessageFormat.format(
            AeMessages.getString("AeAbstractExpressionRunner.FAILED_TO_CONVERT_EXPRESSION_RVAL_ERROR"), //$NON-NLS-1$
            new Object [] { aExpression, aNativeRval } );
      throw new AeBpelException(msg, AeFaultFactory.INVALID_TRANS_CONDITION);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.expr.IAeExpressionRunner#executeBooleanExpression(org.activebpel.rt.bpel.impl.expr.IAeExpressionRunnerContext,
    *      java.lang.String)
    */
   public Boolean executeBooleanExpression(IAeExpressionRunnerContext aContext, String aExpression)
         throws AeException
   {
      Object obj = executeExpression(aContext, aExpression);
      return doConvertToBoolean(obj, aExpression, aContext);
   }

   /**
    * Converts a return value from <code>executeExpression</code> into a Java Date object.
    * This method can be overridden by language specific classes in order to be more strict
    * about the return type.
    * 
    * @param aNativeRval
    * @param aExpression
    * @param aContext
    * @throws AeException
    */
   protected Date doConvertToDeadline(Object aNativeRval, String aExpression,
         IAeExpressionRunnerContext aContext) throws AeException
   {
      if (aNativeRval instanceof Date)
         return (Date) aNativeRval;

      String deadline = aNativeRval.toString();
      try
      {
         AeSchemaDateTime dt = null;
         // Note - the deadline can be either xsd:date or xsd:dateTime according to the BPEL spec.
         if (deadline.indexOf("T") != -1) //$NON-NLS-1$
            dt = new AeSchemaDateTime(deadline);
         else
            dt = new AeSchemaDate(deadline);
         return dt.toDate();
      }
      catch (ParseException ex)
      {
         String msg = MessageFormat.format(AeMessages.getString("AeAbstractExpressionRunner.ERROR_PARSING_DEADLINE_ERROR"),  //$NON-NLS-1$
               new Object[] { deadline, ex.getMessage() });
         throw new AeBpelException(msg, AeFaultFactory.XPATH_DATE_PARSE_ERROR, ex);
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.expr.IAeExpressionRunner#executeDeadlineExpression(org.activebpel.rt.bpel.impl.expr.IAeExpressionRunnerContext, java.lang.String)
    */
   public Date executeDeadlineExpression(IAeExpressionRunnerContext aContext, String aExpression)
         throws AeException
   {
      Object deadlineObj = executeExpression(aContext, aExpression);
      return doConvertToDeadline(deadlineObj, aExpression, aContext);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.expr.IAeExpressionRunner#executeDurationExpression(org.activebpel.rt.bpel.impl.expr.IAeExpressionRunnerContext, java.lang.String)
    */
   public Date executeDurationExpression(IAeExpressionRunnerContext aContext, String aExpression)
         throws AeException
   {
      Object durationObj = executeExpression(aContext, aExpression);
      if (durationObj instanceof Date)
         return (Date) durationObj;

      String duration = durationObj.toString();
      try
      {
         return new AeSchemaDuration(duration).toDeadline();
      }
      catch (ParseException pe)
      {
         throw new AeException(pe);
      }
   }

}
