//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/function/AeGetVariablePropertyFunction.java,v 1.1 2005/06/08 12:50:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.function;

import java.util.List;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.IAeVariable;
import org.activebpel.rt.bpel.def.AeVariableDef;
import org.activebpel.rt.bpel.function.AeFunctionCallException;
import org.activebpel.rt.bpel.function.IAeFunctionExecutionContext;
import org.activebpel.rt.bpel.impl.AeBpelException;
import org.activebpel.rt.bpel.impl.AeFaultFactory;
import org.activebpel.rt.bpel.impl.AeNamespaceResolver;
import org.activebpel.rt.bpel.impl.expr.xpath.AeXPathFunctionContext;
import org.activebpel.rt.bpel.impl.expr.xpath.AeXPathNamespaceContext;
import org.activebpel.rt.bpel.xpath.AeXPathHelper;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;
import org.w3c.dom.Node;

/**
 * Class representing the XPath function used by expression evaluator to handle 
 * the BPEL getVariableProperty function call.
 */
public class AeGetVariablePropertyFunction extends AeAbstractBpelFunction
{
   // error message constants
   private static final String INVALID_ARGS              = getExternalizedString("AeAbstractBpelObject.ERROR_33"); //$NON-NLS-1$
   private static final String ERROR_WITH_PROPERTY_ALIAS = getExternalizedString("AeAbstractBpelObject.ERROR_34"); //$NON-NLS-1$
   private static final String MULT_NODES_SELECTED       = getExternalizedString("AeAbstractBpelObject.ERROR_35"); //$NON-NLS-1$
   private static final String ERROR_MSG                 = getExternalizedString("AeAbstractBpelObject.ERROR_36"); //$NON-NLS-1$
   
   /**
    * Constructor.
    */
   public AeGetVariablePropertyFunction()
   {
   }

   /**
    * @see org.activebpel.rt.bpel.function.IAeFunction#call(org.activebpel.rt.bpel.function.IAeFunctionExecutionContext, java.util.List)
    */
   public Object call(IAeFunctionExecutionContext aContext, List aArgs) throws AeFunctionCallException
   {
      Object result = null;
      
      if (aArgs.size() != 2)
      {
         throw new AeFunctionCallException( INVALID_ARGS );
      }

      // Get variable from arg list and make sure we have data to process
      IAeVariable variable = aContext.getAbstractBpelObject().getVariable(aArgs.get(0).toString());
      if (variable != null && variable.hasMessageData())
      {
         // Get variable def so that we may get the msg type and property
         AeVariableDef def = variable.getDefinition();
         if (def.getMessageType() != null)
         {
            QName msgType = def.getMessageType();
            QName msgProp = aContext.getAbstractBpelObject().parseQName(aArgs.get(1).toString());
            
            if (msgType != null && msgProp != null)
            {
               // Get the property alias given the msg type and property
               try
               {
                  IAePropertyAlias propAlias = getMessageProperty(aContext.getAbstractBpelObject(), msgType, msgProp);
                  if (propAlias != null)
                  {
                     // Get the data for the alias message part
                     result = variable.getMessageData().getData(propAlias.getPart());
                     if (result != null && ! AeUtil.isNullOrEmpty(propAlias.getQuery()))
                     {
                        if (result instanceof Node)
                        {
                           // Execute selection query provided by alias to get desired node
                           // Note: The namespace context changes here to have the
                           //       property alias resolve any prefixes.
                           result = AeXPathHelper.executeXPathExpression(
                                 propAlias.getQuery(), result,
                                 new AeXPathFunctionContext(aContext), 
                                 new AeXPathNamespaceContext(new AeNamespaceResolver(propAlias), (Node) result) );
                        }
                     }
                  }
               }
               catch (AeBusinessProcessException e)
               {
                  throw new AeFunctionCallException( ERROR_WITH_PROPERTY_ALIAS, e);
               }
                                  
            }               
         }
      }
      
      // Make sure we have selected only one node, otherwise create the appropriate fault
      if (result instanceof List && ((List)result).size() != 1)
      {
         Exception e = new AeBpelException(MULT_NODES_SELECTED, AeFaultFactory.SELECTION_FAILURE);            
         throw new AeFunctionCallException(ERROR_MSG, e);
      }         

      return result;
   }
}
