//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/list/AeWsdlDetail.java,v 1.3 2006/02/10 20:06:5
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.list;

import org.activebpel.rt.wsdl.def.AeBPELExtendedWSDLDef;

/**
 * Essentially a struct that wraps the detail information
 * for a single wsdl deployment.  Information includes the
 * location hint, the wsdl target namespace, the actual wsdl
 * (as a string), and an array of <code>AeWsdlPlanDetail</code>
 * objects that represent the process plans that utilize this
 * wsdl.
 */
public class AeWsdlDetail
{
    /** The location hint. */
    protected String mLocation;
    /** The wsdl target namespace. */
    protected String mNamespace;
    /** The wsdl xml as a string. */
    protected String mWsdlText;
    /** Array of plans that point to this wsdl. */
    protected AeWsdlPlanDetail[] mWsdlPlanDetails;
    /** The wsdl def object */
    protected AeBPELExtendedWSDLDef mDef;
    
    /**
     * Constructor.
     * @param aLocation
     * @param aDef
     * @param aDeployments
     */
    public AeWsdlDetail( String aLocation, AeBPELExtendedWSDLDef aDef, AeWsdlPlanDetail[] aPlanListing )
    {
        mLocation = aLocation;
        mNamespace = aDef.getTargetNamespace();
        mDef = aDef;
        mWsdlPlanDetails = aPlanListing;
    }
    
    /**
     * @return Returns the location.
     */
    public String getLocation()
    {
        return mLocation;
    }
    
    /**
     * @return Returns the namespace.
     */
    public String getNamespace()
    {
        return mNamespace;
    }
    
    /**
     * @return Returns the def.
     */
    public AeBPELExtendedWSDLDef getDef()
    {
       return mDef;
    }

    /**
     * @return Returns the wsdl xml as a string.
     */
    public String getWsdlText()
    {
       if( mWsdlText == null )
          mWsdlText = getDef().toString();
       
       return mWsdlText;
    }
    
    /**
     * @return Returns the array of <code>AeWsdlPlanDetail</code> objects.
     */
    public AeWsdlPlanDetail[] getWsdlPlanDetails()
    {
        return mWsdlPlanDetails;
    }
}
