// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/queue/AeAbstractQueuedObject.java,v 1.2 2005/07/12 00:16:3
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.queue;

import org.activebpel.rt.util.AeUtil;

import javax.xml.namespace.QName;

/**
 * Base class for objects that get queued in the engine. 
 */
abstract public class AeAbstractQueuedObject
{
   /** Port type of message queue entry */
   private QName mPortType;

   /** Operation of message queue entry */
   private String mOperation;

   /** name of partner Link of message queue entry */
   private String mPartnerLinkName;

   /**
    * Full constructor for queue object
    */
   public AeAbstractQueuedObject(String aPartnerLinkName, QName aPortType, String aOperation)
   {
      setPartnerLinkName(aPartnerLinkName);
      setPortType(aPortType);
      setOperation(aOperation);
   }
   
   /**
    * Default Constructor for JavaBean compliance. 
    */
   public AeAbstractQueuedObject()
   {
   }

   /**
    * @return The business operation in the queue.
    */
   public String getOperation()
   {
      return mOperation;
   }

   /**
    * @return The name of the partnerlink in the queue.
    */
   public String getPartnerLinkName()
   {
      return mPartnerLinkName;
   }

   /**
    * @return QName The portType of the operation in the queue.
    */
   public QName getPortType()
   {
      return mPortType;
   }

   /**
    * Sets the operation.
    */
   public void setOperation(String aString)
   {
      mOperation = aString;
   }

   /**
    * Sets the partner link name.
    */
   public void setPartnerLinkName(String aLinkName)
   {
      mPartnerLinkName = aLinkName;
   }

   /**
    * Sets the portType.
    */
   public void setPortType(QName aQName)
   {
      mPortType = aQName;
   }

   /**
    * Overides the base to check for specific equality of queue entries.
    * @see java.lang.Object#equals(java.lang.Object)
    */
   public boolean equals(Object aObject)
   {
      if(aObject instanceof AeAbstractQueuedObject)
      {
         AeAbstractQueuedObject other = (AeAbstractQueuedObject)aObject;
         
         return AeUtil.compareObjects(getPartnerLinkName(), other.getPartnerLinkName()) &&
          AeUtil.compareObjects(getOperation(), other.getOperation()) && 
          AeUtil.compareObjects(getPortType(), other.getPortType());
      }
      
      return super.equals(aObject);
   }

   /**
    * @see java.lang.Object#hashCode()
    */
   public int hashCode()
   {
      return hashCode(getPartnerLinkName()) ^ hashCode(getOperation()) ^ hashCode(getPortType());
   }

   /**
    * Returns <code>aObject.hashCode()</code> or <code>0</code> if
    * <code>aObject</code> is <code>null</code>.
    *
    * @param aObject
    */
   protected int hashCode(Object aObject)
   {
      return (aObject == null) ? 0 : aObject.hashCode();
   }
}
