// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/queue/AeInboundReceive.java,v 1.12 2006/01/03 19:55:0
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.queue;

import commonj.timers.Timer;

import java.util.Date;
import java.util.Map;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.impl.IAeProcessPlan;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.wsio.receive.IAeMessageContext;

/**
 * Models the data received from a partner.   
 */
public class AeInboundReceive extends AeCorrelatedReceive
{
   /** The process plan for the inbound receive. */
   private IAeProcessPlan mProcessPlan;
   /** Message data of message queue entry */
   private IAeMessageData mMessageData;
   /** id of the reply that is waiting to get queued, only set in the case of unmatched receives */
   private String mResponseReceiverId;
   /** id given by the queue manager, currently only used for unmatched receives */
   private String mQueueId;
   /** The context that for the inbound message */
   private IAeMessageContext mContext;
   /** The date when this inbound receive should expire (if it hasn't been dispatched yet). */
   private Date mTimeoutDate;
   /** The Timer that has been created to timeout this inbound receive at the above timeout date. */
   private Timer mTimeoutTimer;

   /**
    * Creates an inbound receive.
    */
   public AeInboundReceive(Map aCorrelation, IAeProcessPlan aProcPlan, IAeMessageData aData,
         IAeMessageContext aContext)
   {
      super(aContext.getPartnerLinkName(), aProcPlan.getProcessDef().getQName(), aProcPlan.getMyRolePortType(aContext.getPartnerLinkName()), aContext.getOperation(), aCorrelation);

      mContext = aContext;
      setProcessPlan(aProcPlan);
      setMessageData(aData);
      setTimeoutDate(null);
      setTimeoutTimer(null);
   }
   
   /**
    * Getter for the message's context.
    */
   public IAeMessageContext getContext()
   {
      return mContext;
   }
   
   /**
    * Getter for the message data
    */
   public IAeMessageData getMessageData()
   {
      return mMessageData;
   }

   /**
    * Getter for the queue id, a unique id assigned to this queue object.
    */
   public String getQueueId()
   {
      return mQueueId;
   }

   /**
    * Gettter for the response receiver id which identifies the object waiting to
    * get notified via a reply.
    */
   public String getResponseReceiverId()
   {
      return mResponseReceiverId;
   }

   /**
    * Setter for the message data.
    * @param aData
    */
   public void setMessageData(IAeMessageData aData)
   {
      mMessageData = aData;
   }

   /**
    * Setter for the queue id.
    * @param aString
    */
   public void setQueueId(String aString)
   {
      mQueueId = aString;
   }

   /**
    * Setter for the response receiver id.
    * @param aString
    */
   public void setResponseReceiverId(String aString)
   {
      mResponseReceiverId = aString;
   }

   /**
    * Getter for the process plan.
    */
   public IAeProcessPlan getProcessPlan()
   {
      return mProcessPlan;
   }

   /**
    * Setter for the process plan.
    */
   public void setProcessPlan(IAeProcessPlan aPlan)
   {
      mProcessPlan = aPlan;
   }

   /**
    * Convenience method that gets the process qname
    */
   public QName getProcessName()
   {
      return getProcessPlan().getProcessDef().getQName();
   }

   /**
    * @return Returns the timeoutDate.
    */
   public Date getTimeoutDate()
   {
      return mTimeoutDate;
   }

   /**
    * @param aTimeoutDate The timeoutDate to set.
    */
   public void setTimeoutDate(Date aTimeoutDate)
   {
      mTimeoutDate = aTimeoutDate;
   }

   /**
    * @return Returns the timeoutTimer.
    */
   public Timer getTimeoutTimer()
   {
      return mTimeoutTimer;
   }

   /**
    * @param aTimeoutTimer The timeoutTimer to set.
    */
   public void setTimeoutTimer(Timer aTimeoutTimer)
   {
      mTimeoutTimer = aTimeoutTimer;
   }
}
