// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/queue/AeInvoke.java,v 1.10 2005/10/28 21:07:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.queue;

import java.util.Map;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.IAeEndpointReference;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.IAePartnerLink;
import org.activebpel.rt.bpel.impl.AeDataConverter;
import org.activebpel.rt.bpel.impl.IAeMessageReceiver;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.xml.AeXMLParserBase;
import org.activebpel.wsio.IAeWebServiceMessageData;
import org.activebpel.wsio.invoke.IAeInvoke;
import org.w3c.dom.Document;

/**
 * Represents an in memory message queue object. 
 */
public class AeInvoke extends AeAbstractQueuedObject implements IAeInvoke
{
   /** Flag indicates if this invoke is a one-way invoke and not expecting a response */
   private boolean mOneWay;
   /** Input message data for invoke. */
   private IAeMessageData mInputMessageData;
   /** Output message data for invoke. */
   private IAeMessageData mOutputMessageData;
   /** Fault for invoke. */
   private IAeFault mFault;
   /** Message receiver location path */
   private String mMessageReceiverPath;
   /** location path for the activity that is waiting for the invoke's response */
   private String mLocationPath;
   /** The partner endpoint reference */
   private IAeEndpointReference mPartnerReference;
   /** The my endpoint reference */
   private IAeEndpointReference mMyReference;   
   /** Receives the response for the invoke */   
   private IAeMessageReceiver mResponseReceiver;
   /** process name */
   private QName mProcessName;
   /** Process id is needed to differentiate between queued objects */
   private long mProcessId;
   /** The custom invoker uri. */
   private String mInvokeHandler;
   /** Business process properties. */
   private Map mProcessProperties;
   
   
   /** Create entry for queue receive */   
   public AeInvoke(long aProcessId, QName aProcessName, IAePartnerLink aPartnerLink, 
                               QName aPortType,
                               String aOperation, 
                               IAeMessageData aInputMessageData,
                               IAeMessageReceiver aMessageReceiver,
                               Map aProcessProperties )
   {
      super(aPartnerLink.getName(), aPortType, aOperation);
      setProcessId(aProcessId);
      mProcessName = aProcessName;      
      mPartnerReference = aPartnerLink.getPartnerReference();
      mMyReference = aPartnerLink.getMyReference();
      mInputMessageData = aInputMessageData;
      mMessageReceiverPath = aMessageReceiver.getLocationPath();
      mLocationPath = aMessageReceiver.getLocationPath();
      mProcessProperties = aProcessProperties;
   }
   
   /**
    * Getter for the process name for this invoke.
    */
   public QName getProcessName()
   {
      return mProcessName;
   }
   
   
   /** Create entry for invoke response received */   
   public AeInvoke(long aProcessId, String aLocationPath, 
                               IAeMessageData aOutputMessageData)
   {
      setProcessId(aProcessId);
      mOutputMessageData = aOutputMessageData;
      mLocationPath = aLocationPath;
   }

   /** Create entry for invoke response received */   
   public AeInvoke(long aProcessId, String aLocationPath,
                               IAeFault aFault)
   {
      setProcessId(aProcessId);
      mFault = aFault;
      mLocationPath = aLocationPath;
   }
   
   /**
    * Returns true if the object is of the same type and has the same process id and location path. 
    * @see java.lang.Object#equals(java.lang.Object)
    */
   public boolean equals(Object aObject)
   {
      if (aObject instanceof AeInvoke)
      {
         AeInvoke other = (AeInvoke) aObject;
         return other.getProcessId() == getProcessId() && AeUtil.compareObjects(getLocationPath(), other.getLocationPath());
      }
      return false;
   }
   
   /**
    * Gets the location path for the message receiver.
    */
   public String getLocationPath()
   {
      return mLocationPath;
   }

   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#getInputMessageData()
    */
   public IAeWebServiceMessageData getInputMessageData()
   {
      return AeDataConverter.convert(mInputMessageData);
   }
   
   /**
    * @return String The message receiver location path (may be null if no message yet).
    */
   public String getMessageReceiverPath()
   {
      return mMessageReceiverPath;
   }

   /**
    * Sets the message data.
    */
   public void setInputMessageData(IAeMessageData aData)
   {
      mInputMessageData = aData;
   }

   /**
    * @return The fault associated with the invokes response, null if none.
    */
   public IAeFault getFault()
   {
      return mFault;
   }

   /**
    * @return The output message data associated with invokes response, null if none.
    */
   public IAeMessageData getOutputMessageData()
   {
      return mOutputMessageData;
   }

   /**
    * @param aFault The fault to be associated with the invokes response.
    */
   public void setFault(IAeFault aFault)
   {
      mFault = aFault;
   }

   /**
    * @param aData The output message to be associated with the invokes response.
    */
   public void setOutputMessageData(IAeMessageData aData)
   {
      mOutputMessageData = aData;
   }
   
   /**
    * Accessor for the parnter reference endpoint.
    */
   public IAeEndpointReference getPartnerReference()
   {
      return mPartnerReference;
   }

   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#getPartnerEndpointReferenceString()
    */
   public String getPartnerEndpointReferenceString()
   {
      Document eprDoc = getPartnerReference().toDocument();
      return AeXMLParserBase.documentToString(eprDoc, true);
   }

   /**
    * Accessor for my reference endpoint.
    */
   public IAeEndpointReference getMyReference()
   {
      return mMyReference;
   }

   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#getPartnerEndpointReferenceString()
    */
   public String getMyEndpointReferenceString()
   {
      Document eprDoc = getMyReference().toDocument();
      return AeXMLParserBase.documentToString(eprDoc, true);
   }
   
   /**
    * Returns true if this invoke is a one way invoke and not expecting a response
    */
   public boolean isOneWay()
   {
      return mOneWay;
   }

   /**
    * Setter for the one way flag
    * @param aB
    */
   public void setOneWay(boolean aB)
   {
      mOneWay = aB;
   }

   /**
    * Setter for the response receiver. The invocation layer is responsible for
    * installing its own response receiver which will dispatch the response from
    * the invocation to the engine's appropriate queue methods.
    * @param aResponseReceiver
    */
   public void setResponseReceiver(IAeMessageReceiver aResponseReceiver)
   {
      mResponseReceiver = aResponseReceiver;
   }

   /**
    * Getter for the response receiver. This will be null if the invocation layer
    * hasn't intalled it yet.
    */
   public IAeMessageReceiver getResponseReceiver()
   {
      return mResponseReceiver;
   }

   /**
    * Setter for the process id
    * 
    * @param processId
    */
   public void setProcessId(long processId)
   {
      mProcessId = processId;
   }

   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#getProcessId()
    */
   public long getProcessId()
   {
      return mProcessId;
   }

   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#getCustomInvokerUri()
    * @deprecated
    */
   public String getCustomInvokerUri()
   {
      return getInvokeHandler();
   }
   
   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#getInvokeHandler()
    */
   public String getInvokeHandler()
   {
      return mInvokeHandler;
   }
   
   /**
    * @see org.activebpel.wsio.invoke.IAeInvoke#setInvokeHandler(java.lang.String)
    */
   public void setInvokeHandler(String aCustomInvokerUri)
   {
      mInvokeHandler = aCustomInvokerUri;
   }
   
   /**
    * @return Returns the processProperties.
    */
   public Map getBusinessProcessProperties()
   {
      return mProcessProperties;
   }

   public int getLocationId() {
	   // TODO Auto-generated method stub
	   return 0;
   }

   public long getTransmissionId() {
	   // TODO Auto-generated method stub
	   return 0;
   }

   public void setTransmissionId(long arg0) {
	   // TODO Auto-generated method stub

   }
}
