// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/queue/AeReply.java,v 1.5 2005/10/17 19:44:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.queue;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.impl.IAeReplyReceiver;
import org.activebpel.rt.util.AeUtil;

/**
 * An in memory queue object for replies.
 */
public class AeReply extends AeAbstractQueuedObject
{
   /** The invoke that's waiting for a reply */
   private IAeReplyReceiver mReplyReceiver;

   /** Process id is needed to differentiate between queued objects */
   private long mProcessId;
   
   /** value of the message exchange which is used to match replies to receives */
   private String mMessageExchange;

   /**
    * Creates a queue object for incoming receive data.
    * @param aProcessId
    * @param aReplyReceiver
    * @param aReceiveObject
    * @param aMessageExchange
    */
   public AeReply(long aProcessId, IAeReplyReceiver aReplyReceiver, 
               AeInboundReceive aReceiveObject, String aMessageExchange)
   {
      super(aReceiveObject.getPartnerLinkName(), aReceiveObject.getPortType(),
            aReceiveObject.getOperation());
      setProcessId(aProcessId);
      mReplyReceiver = aReplyReceiver;
      setMessageExchange(aMessageExchange);
   }

   /**
    * Creates a reply.
    * @param aProcessId 
    * @param aPartnerLinkName
    * @param aPortType
    * @param aOperation
    */
   public AeReply(long aProcessId, String aPartnerLinkName, QName aPortType, 
                             String aOperation, String aMessageExchange)
   {
      super(aPartnerLinkName, aPortType, aOperation);
      setProcessId(aProcessId);
      setMessageExchange(aMessageExchange);
   }
   
   /**
    * @return Returns the messageExchange.
    */
   public String getMessageExchange()
   {
      return mMessageExchange;
   }

   /**
    * @param aMessageExchange The messageExchange to set.
    */
   public void setMessageExchange(String aMessageExchange)
   {
      mMessageExchange = AeUtil.getSafeString(aMessageExchange);
   }
   
   /**
    * Getter for the response receiver.
    */
   public IAeReplyReceiver getReplyReceiver()
   {
      return mReplyReceiver; 
   }

   public void setProcessId(long processId)
   {
      mProcessId = processId;
   }

   public long getProcessId()
   {
      return mProcessId;
   }
   
   /**
    * @see java.lang.Object#equals(java.lang.Object)
    */
   public boolean equals(Object aObject)
   {
      if (aObject instanceof AeReply)
      {
         AeReply other = (AeReply) aObject;
         return getProcessId() == other.getProcessId() && AeUtil.compareObjects(getMessageExchange(), other.getMessageExchange()) && super.equals(aObject);
      }
      return false;
   }

   /**
    * @see java.lang.Object#hashCode()
    */
   public int hashCode()
   {
      return new Long(getProcessId()).hashCode() ^ super.hashCode();
   }
}
