// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/storage/AeInboundReceiveDeserializer.java,v 1.4 2005/10/17 19:44:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.storage;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.AeWSDLDefHelper;
import org.activebpel.rt.bpel.impl.AeBusinessProcessPropertyIO;
import org.activebpel.rt.bpel.impl.AeEndpointReference;
import org.activebpel.rt.bpel.impl.AeMessageDataDeserializer;
import org.activebpel.rt.bpel.impl.AeTypeMapping;
import org.activebpel.rt.bpel.impl.IAeImplStateNames;
import org.activebpel.rt.bpel.impl.IAeProcessPlan;
import org.activebpel.rt.bpel.impl.queue.AeInboundReceive;
import org.activebpel.rt.bpel.xpath.AeXPathHelper;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeXmlUtil;
import org.activebpel.rt.wsdl.def.IAePropertyAlias;
import org.activebpel.wsio.IAeWebServiceEndpointReference;
import org.activebpel.wsio.receive.AeMessageContext;
import org.activebpel.wsio.receive.IAeMessageContext;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Deserializes an inbound receive from its serialization.
 */
public class AeInboundReceiveDeserializer implements IAeImplStateNames
{
   /** The inbound receive once deserialized. */
   private AeInboundReceive mInboundReceive;

   /** The inbound receive's serialization. */
   private Element mInboundReceiveElement;

   /** The process plan to use to deserialize the inbound receive. */
   private IAeProcessPlan mProcessPlan;

   /** The type mapping to use to deserialize simple types. */
   private AeTypeMapping mTypeMapping;

   /**
    * Creates the correlation map for the incoming receive.
    *
    * @param aPlan
    * @param aData
    * @param aContext
    */
   private Map createCorrelationMap(IAeProcessPlan aPlan, IAeMessageData aData, IAeMessageContext aContext) throws AeBusinessProcessException
   {
      String partnerLinkName = aContext.getPartnerLinkName();
      String operation = aContext.getOperation();
      Collection names = aPlan.getCorrelatedPropertyNames(partnerLinkName, operation);
      Map map = new HashMap();

      for (Iterator i = names.iterator(); i.hasNext(); )
      {
         QName name = (QName) i.next();
         IAePropertyAlias alias = aPlan.getProcessDef().getPropertyAlias(aData.getMessageType(), name);
         QName type = AeWSDLDefHelper.getProperty(aPlan, name).getTypeName();
         Object value = AeXPathHelper.evaluatePropertyToSimpleType(alias, aData, getTypeMapping(), type);

         map.put(name, value);
      }

      return map;
   }

   /**
    * Creates an instance of {@link
    * org.activebpel.rt.bpel.impl.queue.AeInboundReceive} from its
    * serialization.
    *
    * @param aInboundReceiveElement
    * @throws AeBusinessProcessException
    */
   protected AeInboundReceive createInboundReceive(Element aInboundReceiveElement) throws AeBusinessProcessException
   {
      // Deserialize the message context and message data from the input XML
      // element.
      Map childElementsMap = getChildElementsMap(aInboundReceiveElement);
      IAeMessageContext context = createMessageContext(getElement(childElementsMap, STATE_MESSAGECONTEXT));
      IAeMessageData data = createMessageData(getElement(childElementsMap, STATE_MESSAGEDATA));

      // Verify that caller provided process plan.
      IAeProcessPlan plan = getProcessPlan();
      if (plan == null)
      {
         throw new IllegalStateException(AeMessages.getString("AeInboundReceiveDeserializer.ERROR_0")); //$NON-NLS-1$
      }

      // Reconstruct correlation map.
      Map correlationMap = createCorrelationMap(plan, data, context);

      // Create inbound receive.
      AeInboundReceive result = new AeInboundReceive(correlationMap, plan, data, context);
      return result;
   }

   /**
    * Creates an instance of {@link
    * org.activebpel.wsio.IAeWebServiceEndpointReference} from its
    * serialization.
    * 
    * @param aEndpointReferenceElement
    */
   protected IAeWebServiceEndpointReference createEndpointReference(Element aEndpointReferenceElement) throws AeBusinessProcessException
   {
      AeEndpointReference result = null;

      if (aEndpointReferenceElement != null)
      {
         result = new AeEndpointReference();
         result.setReferenceData(AeXmlUtil.getFirstSubElement(aEndpointReferenceElement));
      }

      return result;
   }

   /**
    * Deserializes the inbound receive's message context from its serialization.
    * 
    * @param aMessageContextElement
    * @throws AeBusinessProcessException
    */
   protected IAeMessageContext createMessageContext(Element aMessageContextElement) throws AeBusinessProcessException
   {
      AeMessageContext result = null;

      if (aMessageContextElement != null)
      {
         Map map = getChildElementsMap(aMessageContextElement);
         IAeWebServiceEndpointReference embeddedEndpointReference = createEndpointReference(getElement(map, STATE_PARTNERENDPOINTREFERENCE));
         IAeWebServiceEndpointReference myEndpointReference = createEndpointReference(getElement(map, STATE_MYENDPOINTREFERENCE));
         
         result = new AeMessageContext();
         result.setPartnerEndpointReference(embeddedEndpointReference);
         result.setMyEndpointReference(myEndpointReference);

         // Deserialize attributes.
         result.setOperation(aMessageContextElement.getAttribute(STATE_OPERATION));
         result.setPartnerLinkName(aMessageContextElement.getAttribute(STATE_PLINK));
         result.setPrincipal(aMessageContextElement.getAttribute(STATE_PRINCIPAL));
         result.setProcessVersion(aMessageContextElement.getAttribute(STATE_VERSION));

         // Deserialize process properties.
         NodeList nodes = aMessageContextElement.getChildNodes();
         Map properties = result.getBusinessProcessProperties();

         for (int i = nodes.getLength(); --i >= 0; )
         {
            Node node = nodes.item(i);

            if ((node instanceof Element) && STATE_PROCESSPROPERTY.equals(node.getNodeName()))
            {
               AeBusinessProcessPropertyIO.extractBusinessProcessProperty((Element) node, properties);            
            }
         }
      }

      return result;
   }

   /**
    * Deserializes the inbound receive's message data from its serialization.
    * 
    * @param aMessageDataElement
    * @throws AeBusinessProcessException
    */
   protected IAeMessageData createMessageData(Element aMessageDataElement) throws AeBusinessProcessException
   {
      IAeMessageData result = null;

      if (aMessageDataElement != null)
      {
         AeMessageDataDeserializer deserializer = new AeMessageDataDeserializer();
         deserializer.setMessageDataElement(aMessageDataElement);

         result = deserializer.getMessageData();
      }

      return result;
   }

   /**
    * Deserializes a {@link javax.xml.namespace.QName} from the specified
    * element.
    * 
    * @param aQNameElement
    */
   protected QName createQName(Element aQNameElement)
   {
      QName result = null;

      if (aQNameElement != null)
      {
         String localPart = aQNameElement.getAttribute(STATE_NAME);
         String namespace = aQNameElement.getAttribute(STATE_NAMESPACEURI);

         result = new QName(namespace, localPart);
      }

      return result;
   }

   /**
    * Sets output variables.
    */
   protected void deserialize() throws AeBusinessProcessException
   {
      if (mInboundReceive == null)
      {
         Element root = getInboundReceiveElement();
         if (root == null)
         {
            throw new IllegalStateException(AeMessages.getString("AeInboundReceiveDeserializer.ERROR_1")); //$NON-NLS-1$
         }

         mInboundReceive = createInboundReceive(root);
      }
   }

   /**
    * Returns a {@link java.util.Map} from tag names to child elements.
    * 
    * @param aElement
    */
   protected Map getChildElementsMap(Element aElement)
   {
      NodeList nodes = aElement.getChildNodes();
      Map result = new HashMap(nodes.getLength());

      for (int i = 0; i < nodes.getLength(); ++i)
      {
         Node node = nodes.item(i);

         if (node instanceof Element)
         {
            result.put(node.getNodeName(), node);
         }
      }

      return result;
   }

   /**
    * Returns the child element for the specified tag name from a {@link
    * java.util.Map} of tag names to child elements.
    * 
    * @param aChildElementsMap
    * @param aTagName
    */
   protected Element getElement(Map aChildElementsMap, String aTagName)
   {
      return (Element) aChildElementsMap.get(aTagName);
   }

   /**
    * Returns the process plan to use to deserialize the inbound receive.
    */
   protected IAeProcessPlan getProcessPlan()
   {
      return mProcessPlan;
   }

   /**
    * Returns the inbound receive deserialized from the serialization that was
    * set with {@link #setInboundReceiveElement}.
    */
   public AeInboundReceive getInboundReceive() throws AeBusinessProcessException
   {
      deserialize();

      return mInboundReceive;
   }

   /**
    * Returns the inbound receive serialization to use.
    */
   protected Element getInboundReceiveElement()
   {
      return mInboundReceiveElement;
   }

   /**
    * Returns type mapping to use to deserialize simple types.
    */
   protected AeTypeMapping getTypeMapping()
   {
      return mTypeMapping;
   }

   /**
    * Resets all output variables.
    */
   protected void reset()
   {
      mInboundReceive = null;
   }

   /**
    * Sets the process plan to use to deserialize the inbound receive.
    *
    * @param aProcessPlan
    */
   public void setProcessPlan(IAeProcessPlan aProcessPlan)
   {
      reset();

      mProcessPlan = aProcessPlan;
   }

   /**
    * Sets the inbound receive serialization to use.
    *
    * @param aInboundReceiveElement
    */
   public void setInboundReceiveElement(Element aInboundReceiveElement)
   {
      reset();

      mInboundReceiveElement = aInboundReceiveElement;
   }

   /**
    * Sets the type mapping to use to deserialize simple types.
    *
    * @param aTypeMapping
    */
   public void setTypeMapping(AeTypeMapping aTypeMapping)
   {
      reset();

      mTypeMapping = aTypeMapping;
   }
}
