// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/storage/AeReplyDeserializer.java,v 1.3 2005/10/17 19:44:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.storage;

import javax.xml.namespace.QName;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.impl.IAeImplStateNames;
import org.activebpel.rt.bpel.impl.queue.AeReply;
import org.activebpel.rt.util.AeXmlUtil;
import org.w3c.dom.Element;

/**
 * Deserializes a reply from its serialization.
 */
public class AeReplyDeserializer implements IAeImplStateNames
{
   /** The reply once deserialized. */
   private AeReply mReply;

   /** The reply's serialization. */
   private Element mReplyElement;

   /**
    * Creates an instance of {@link org.activebpel.rt.bpel.impl.queue.AeReply} from its
    * serialization.
    * 
    * @param aReplyElement
    * @throws AeBusinessProcessException
    */
   protected AeReply createReply(Element aReplyElement) throws AeBusinessProcessException
   {
      long processId         = Long.parseLong(aReplyElement.getAttribute(STATE_PID));
      String operation       = aReplyElement.getAttribute(STATE_OPERATION);
      String partnerLinkName = aReplyElement.getAttribute(STATE_PLINK);
      String messageExchange = aReplyElement.getAttribute(STATE_MESSAGE_EXCHANGE);
      QName portType = null;

      Element portTypeElement = AeXmlUtil.getFirstSubElement(aReplyElement);
      if (portTypeElement != null)
      {
         String localPart = portTypeElement.getAttribute(STATE_NAME);
         String namespace = portTypeElement.getAttribute(STATE_NAMESPACEURI);

         portType = new QName(namespace, localPart);
      }

      return new AeReply(processId, partnerLinkName, portType, operation, messageExchange);
   }

   /**
    * Returns the reply deserialized from the serialization that was set with
    * {@link #setReplyElement}.
    */
   public AeReply getReply() throws AeBusinessProcessException
   {
      if (mReply == null)
      {
         if (getReplyElement() == null)
         {
            throw new IllegalStateException(AeMessages.getString("AeReplyDeserializer.ERROR_0")); //$NON-NLS-1$
         }

         mReply = createReply(getReplyElement());
      }

      return mReply;
   }

   /**
    * Returns the reply serialization to use.
    */
   protected Element getReplyElement()
   {
      return mReplyElement;
   }

   /**
    * Resets all output variables.
    */
   protected void reset()
   {
      mReply = null;
   }

   /**
    * Sets the reply serialization to use.
    *
    * @param aReplyElement
    */
   public void setReplyElement(Element aReplyElement)
   {
      reset();

      mReplyElement = aReplyElement;
   }
}
