//$Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/visitors/AeOutstandingRepliesVisitor.java,v 1.1 2006/02/10 17:02:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.visitors; 

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.impl.AeAbstractBpelObject;
import org.activebpel.rt.bpel.impl.AeBusinessProcess;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;

/**
 * Visits the process and all scopes to determine if there are any outstanding replies.
 * If there are any receives left dangling at the completion/termination of a process then
 * we should fault them to avoid them timing out.
 */
public class AeOutstandingRepliesVisitor extends AeImplTraversingVisitor
{
   /** tracks if we've found an outstanding reply */
   private boolean mFound;

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.AeImplTraversingVisitor#traverse(org.activebpel.rt.bpel.impl.AeAbstractBpelObject)
    */
   protected void traverse(AeAbstractBpelObject aImpl) throws AeBusinessProcessException
   {
      if (!isFound())
      {
         super.traverse(aImpl);
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.AeImplTraversingVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl)
    */
   public void visit(AeActivityScopeImpl aImpl) throws AeBusinessProcessException
   {
      checkForOutstandingReplies(aImpl);

      if (!isFound())
      {
         super.visit(aImpl);
      }
   }

   /**
    * Sets the found flag if the scope has outstanding replies 
    * @param aImpl
    */
   protected void checkForOutstandingReplies(AeActivityScopeImpl aImpl)
   {
      if (aImpl.hasOutstandingReplies())
         setFound(true);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.AeImplTraversingVisitor#visit(org.activebpel.rt.bpel.impl.AeBusinessProcess)
    */
   public void visit(AeBusinessProcess aImpl) throws AeBusinessProcessException
   {
      checkForOutstandingReplies(aImpl);
      
      if (!isFound())
      {
         super.visit(aImpl);
      }
   }
   
   /**
    * Returns true if the process contains outstanding replies
    */
   public boolean containsOutstandingReplies()
   {
      return isFound();
   }

   /**
    * @return Returns the found.
    */
   protected boolean isFound()
   {
      return mFound;
   }

   /**
    * @param aFound The found to set.
    */
   protected void setFound(boolean aFound)
   {
      mFound = aFound;
   }
}
 
