// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/visitors/AeRestoreImplStateVisitor.java,v 1.39 2006/02/14 22:39:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.visitors;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.activebpel.rt.AeException;
import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.AeMessages;
import org.activebpel.rt.bpel.IAeBusinessProcess;
import org.activebpel.rt.bpel.IAeEndpointReference;
import org.activebpel.rt.bpel.IAeFault;
import org.activebpel.rt.bpel.def.AePartnerLinkDef;
import org.activebpel.rt.bpel.def.AeProcessDef;
import org.activebpel.rt.bpel.def.activity.AeActivityForEachDef;
import org.activebpel.rt.bpel.impl.AeAbstractBpelObject;
import org.activebpel.rt.bpel.impl.AeBpelState;
import org.activebpel.rt.bpel.impl.AeBusinessProcess;
import org.activebpel.rt.bpel.impl.AeBusinessProcessPropertyIO;
import org.activebpel.rt.bpel.impl.AeMessageDataDeserializer;
import org.activebpel.rt.bpel.impl.AePartnerLink;
import org.activebpel.rt.bpel.impl.IAeImplStateNames;
import org.activebpel.rt.bpel.impl.activity.AeActivityCompensateImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityForEachParallelImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityInvokeImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityReceiveImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityWaitImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityWhileImpl;
import org.activebpel.rt.bpel.impl.activity.AeLoopActivity;
import org.activebpel.rt.bpel.impl.activity.support.AeBaseEvent;
import org.activebpel.rt.bpel.impl.activity.support.AeBaseHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCoordinationContainer;
import org.activebpel.rt.bpel.impl.activity.support.AeCoordinatorCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.activity.support.AeDefaultFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeImplicitCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeImplicitFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeLink;
import org.activebpel.rt.bpel.impl.activity.support.AeOnAlarm;
import org.activebpel.rt.bpel.impl.activity.support.AeOnMessage;
import org.activebpel.rt.bpel.impl.activity.support.AeParticipantCompensator;
import org.activebpel.rt.bpel.impl.activity.support.AeProcessCompensationCallbackWrapper;
import org.activebpel.rt.bpel.impl.activity.support.IAeCompensationCallback;
import org.activebpel.rt.bpel.impl.queue.AeInboundReceive;
import org.activebpel.rt.bpel.impl.storage.AeFaultDeserializer;
import org.activebpel.rt.bpel.impl.storage.AeInboundReceiveDeserializer;
import org.activebpel.rt.bpel.impl.storage.AeRestoreImplState;
import org.activebpel.rt.message.IAeMessageData;
import org.activebpel.rt.util.AeUtil;
import org.activebpel.rt.util.AeXmlUtil;
import org.activebpel.wsio.receive.AeMessageContext;
import org.activebpel.wsio.receive.IAeMessageContext;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Visits a tree of BPEL implementation objects to restore the state from an
 * instance of <code>AeRestoreImplState</code>.
 */
public class AeRestoreImplStateVisitor extends AeBaseRestoreVisitor
{
   /**
    * Maps compensation callback location paths to the compensation handlers
    * that need them.
    */
   private final Map mCompensationCallbackOwnersMap = new HashMap();

   /**
    * Constructor.
    *
    * @param aImplState The state object from which to restore state.
    */
   public AeRestoreImplStateVisitor(AeRestoreImplState aImplState)
   {
      super(aImplState);
   }

   /**
    * Creates the process's create message from its serialization, if the
    * process still has a create message.
    *
    * @param aImpl
    * @return AeInboundReceive
    * @throws AeBusinessProcessException
    */
   protected AeInboundReceive createCreateMessage(AeBusinessProcess aImpl) throws AeBusinessProcessException
   {
      Element implElement = getImplState().getElement(aImpl);
      String xpath = "./" + STATE_CREATEMESSAGE; //$NON-NLS-1$
      Element createMessageElement = (Element) selectOptionalNode(implElement, xpath, "Error restoring create message"); //$NON-NLS-1$

      return (createMessageElement == null) ? null : createCreateMessage(aImpl, createMessageElement);
   }

   /**
    * Creates the process's create message from its serialization.
    *
    * @param aImpl
    * @param aElement
    * @return AeInboundReceive
    * @throws AeBusinessProcessException
    */
   protected AeInboundReceive createCreateMessage(IAeBusinessProcess aImpl, Element aElement) throws AeBusinessProcessException
   {
      Element root = AeXmlUtil.getFirstSubElement(aElement);
      AeInboundReceive createMessage;

      if (root.getTagName() == IAeImplStateNames.STATE_MESSAGEDATA)
      {
         // This is the legacy case. We used to synthesize the inbound receive
         // from just the partner link, operation, and message data.
         String partnerLinkName = getAttribute(aElement, STATE_PLINK);
         String operation = getAttribute(aElement, STATE_OPERATION);

         // Deserialize the message data.
         AeMessageDataDeserializer deserializer = new AeMessageDataDeserializer();
         deserializer.setMessageDataElement(root);

         IAeMessageData messageData = deserializer.getMessageData();

         IAeMessageContext context = new AeMessageContext(partnerLinkName, operation);

         // Construct the create message with enough attributes to allow the
         // process to match a message receiver to the create message: partner
         // link, port type, and operation (see
         // AeMessageReceiver.matches(AeCorrelatedReceive aCorrelatedReceive)).
         createMessage = new AeInboundReceive(
            null, // correlation map
            aImpl.getProcessPlan(),
            messageData,
            context
         );
      }
      else
      {
         AeInboundReceiveDeserializer deserializer = new AeInboundReceiveDeserializer();
         deserializer.setInboundReceiveElement(root);
         deserializer.setProcessPlan(aImpl.getProcessPlan());
         deserializer.setTypeMapping(aImpl.getEngine().getTypeMapping());

         createMessage = deserializer.getInboundReceive();
      }

      return createMessage;
   }

   /**
    * Creates the fault object for the specified implementation
    * object, if the fault exists.
    *
    * @param aImpl
    * @return IAeFault
    * @throws AeBusinessProcessException
    */
   protected IAeFault createFault(AeAbstractBpelObject aImpl) throws AeBusinessProcessException
   {
      Element implElement = getImplState().getElement(aImpl);
      String xpath = "./" + STATE_FAULT; //$NON-NLS-1$
      Element faultElement = (Element) selectOptionalNode(implElement, xpath, "Error restoring fault"); //$NON-NLS-1$

      return (faultElement == null) ? null : createFault(faultElement);
   }

   /**
    * Creates an instance of <code>IAeFault</code> from its serialization.
    *
    * @param aElement
    * @return AeFault
    * @throws AeBusinessProcessException
    */
   protected IAeFault createFault(Element aElement) throws AeBusinessProcessException
   {
      int id = getAttributeInt(aElement, STATE_ID);
      IAeFault fault = (IAeFault) getObject(id);

      if (fault == null)
      {
         // Haven't seen this object id yet. Create the fault object.
         AeFaultDeserializer deserializer = new AeFaultDeserializer();
         deserializer.setFaultElement(aElement);
         deserializer.setProcess(getImplState().getProcess());

         fault = deserializer.getFault();

         // Save the fault object for future references to its object id.
         putIdObject(id, fault);
      }

      return fault;
   }

   /**
    * Restores an instance of <code>AeBaseEvent</code>.
    *
    * @param aBaseEvent
    * @throws AeBusinessProcessException
    */
   protected void restoreBaseEvent(AeBaseEvent aBaseEvent) throws AeBusinessProcessException
   {
      Element element = getImplState().getElement(aBaseEvent);
      boolean active = getAttributeBoolean(element, STATE_ACTIVE);
      boolean queued = getAttributeBoolean(element, STATE_QUEUED);

      aBaseEvent.setActive(active);
      aBaseEvent.setQueued(queued);
   }

   /**
    * Restores an endpoint reference.
    *
    * @param aReference The endpoint reference to restore.
    * @param aElement The document element for the endpoint reference's partner link.
    * @param aTag The tag for the element that contains the endpoint reference's data.
    * @throws AeBusinessProcessException
    */
   protected void restoreEndpointReference(IAeEndpointReference aReference, Element aElement, String aTag) throws AeBusinessProcessException
   {
      // Select the endpoint reference element from the specified partner role
      // element.
      String xpath = "./" + aTag + "/*"; // relative path from partnerLink element //$NON-NLS-1$ //$NON-NLS-2$
      Element element = (Element) selectSingleNode(aElement, xpath, "Error restoring endpoint reference"); //$NON-NLS-1$

      // Set the endpoint reference's data from the 1 matching element.
      aReference.setReferenceData(element);
   }

   /**
    * Restores a partner link.
    *
    * @param aProcess The business process implementation object that owns the partner link.
    * @param aPartnerLinkDef The definition object for the partner link to restore.
    * @param aPartnerLink The partner link to restore.
    */
   protected void restorePartnerLink(AeBusinessProcess aProcess, AePartnerLinkDef aPartnerLinkDef, AePartnerLink aPartnerLink) throws AeBusinessProcessException
   {
      Element element = getImplState().getElement(aPartnerLinkDef);

      // Actually, AeRestoreImplState.getElement will throw an exception
      // instead of returning null.
      if (element != null)
      {
         if (!AeUtil.isNullOrEmpty(aPartnerLink.getMyRole()))
         {
            IAeEndpointReference reference = aPartnerLink.getMyReference();
            restoreEndpointReference(reference, element, STATE_ROLE);
         }

         if (!AeUtil.isNullOrEmpty(aPartnerLink.getPartnerRole()))
         {
            IAeEndpointReference reference = aPartnerLink.getPartnerReference();
            restoreEndpointReference(reference, element, STATE_PROLE);
         }
      }
   }

   /**
    * Restores a process's partner links.
    *
    * @param aProcess The business process implementation object.
    */
   protected void restorePartnerLinks(AeBusinessProcess aProcess) throws AeBusinessProcessException
   {
      AeProcessDef processDef = (AeProcessDef) aProcess.getDefinition();

      for (Iterator i = processDef.getPartnerLinks(); i.hasNext(); )
      {
         AePartnerLinkDef partnerLinkDef = (AePartnerLinkDef) i.next();
         String name = partnerLinkDef.getName();
         AePartnerLink partnerLink = aProcess.getPartnerLink(name);

         restorePartnerLink(aProcess, partnerLinkDef, partnerLink);
      }
   }

   /**
    * Restore any business process properties.
    *
    * @param aProcess
    * @throws AeBusinessProcessException
    */
   protected void restoreBusinessProcessProperties( AeBusinessProcess aProcess ) throws AeBusinessProcessException
   {
      try
      {
         Collection extensionPropertyElements = getImplState().getBusinessProcessPropertiesElements();
         if( extensionPropertyElements != null )
         {
            Map properties = new HashMap();
            for( Iterator iter = extensionPropertyElements.iterator(); iter.hasNext(); )
            {
               Element propertyExtEl = (Element)iter.next();
               AeBusinessProcessPropertyIO.extractBusinessProcessProperty(propertyExtEl, properties );
            }

            aProcess.addBusinessProcessProperties( properties );
         }
      }
      catch( AeException ae )
      {
         // should only happen if there is some problem selecting the
         // extension property nodes
         throw new AeBusinessProcessException( ae.getLocalizedMessage(), ae );
      }
   }
   
   /**
    * Restore any faulting activity location paths.
    *
    * @param aProcess
    * @throws AeBusinessProcessException
    */
   protected void restoreFaultingActivityLocationPaths( AeBusinessProcess aProcess ) throws AeBusinessProcessException
   {
      try
      {
         Collection faultingActivityElements = getImplState().getFaultingActivityElements();
         if( faultingActivityElements != null )
         {
            for( Iterator iter = faultingActivityElements.iterator(); iter.hasNext(); )
            {
               Element faultingActivityEl = (Element)iter.next();
               String locationPath = faultingActivityEl.getAttribute( STATE_LOC );
               aProcess.getFaultingActivityLocationPaths().add( locationPath );
            }
         }
      }
      catch( AeException ae )
      {
         // should only happen if there is some problem selecting the
         // faulting activity nodes
         throw new AeBusinessProcessException( ae.getLocalizedMessage(), ae );
      }
   }

   /**
    * Restores a scope's correlation sets.
    *
    * @param aScopeElement The <code>Element</code> that owns the correlation set elements.
    * @param aScope The scope that owns the correlation sets.
    */
   protected void restoreScopeCorrelationSets(Element aScopeElement, AeActivityScopeImpl aScope) throws AeBusinessProcessException
   {
      String xpath = "./" + STATE_CORRSET; //$NON-NLS-1$
      List elements = selectNodes(aScopeElement, xpath, "Error restoring scope correlation sets"); //$NON-NLS-1$

      for (Iterator i = elements.iterator(); i.hasNext(); )
      {
         Element element = (Element) i.next();
         String name = getAttribute(element, STATE_NAME);
         AeCorrelationSet correlationSet = aScope.getCorrelationSet(name);

         if (correlationSet != null)
         {
            restoreCorrelationSet(element, correlationSet);
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.AeImplTraversingVisitor#traverse(org.activebpel.rt.bpel.impl.AeAbstractBpelObject)
    */
   protected void traverse(AeAbstractBpelObject aImpl) throws AeBusinessProcessException
   {
      Element element = getImplState().getElement(aImpl);
      
      if (aImpl.hasCustomLocationPath())
      {
         aImpl.setLocationId(getAttributeInt(element, STATE_LOCATIONID));
         AeBusinessProcess process = (AeBusinessProcess) aImpl.getProcess();
         process.addBpelObject(aImpl.getLocationPath(), aImpl);
      }

      AeBpelState state = AeBpelState.forName(getAttribute(element, STATE_STATE));
      if (state == null)
      {
         throw new AeBusinessProcessException(AeMessages.getString("AeRestoreImplStateVisitor.ERROR_29")); //$NON-NLS-1$
      }

      aImpl.setRawState(state);

      boolean terminating = getAttributeBoolean(element, STATE_TERMINATING);
      aImpl.setTerminating(terminating);

      // Restore fault.
      aImpl.setFault(createFault(aImpl));

      // Make sure business process can locate default and implicit objects.
      getImplState().getProcess().addBpelObject(aImpl.getLocationPath(), aImpl);

      boolean skipChildren = getAttributeBoolean(element, STATE_SKIPCHILDREN);
      if (!skipChildren)
      {
         super.traverse(aImpl);
      }
      else
      {
         if(aImpl.hasCustomLocationPath())
         {
            // visit the locations which are not saved in the document to ensure that their ids are correct
            aImpl.accept(new AeRegisterLocationVisitor((AeBusinessProcess) aImpl.getProcess()));
         }
         
         // if this is a deadpath then propagate state
         if (state == AeBpelState.DEAD_PATH)
         {
            // Propagate dead path state to the children that we're skipping.
            aImpl.setState(state);
         }
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityCompensateImpl)
    */
   public void visit(AeActivityCompensateImpl aImpl) throws AeBusinessProcessException
   {
      String nextIndex = getImplState().getAttribute(aImpl, STATE_NEXTINDEX);
      if (!AeUtil.isNullOrEmpty(nextIndex))
      {
         aImpl.setNextIndex(Integer.parseInt(nextIndex));
      }

      // We may have already traversed a compensation handler that needs to
      // call this compensate activity.
      AeCompensationHandler handler = (AeCompensationHandler) mCompensationCallbackOwnersMap.remove(aImpl.getLocationPath());
      if (handler != null)
      {
         handler.setCallback(aImpl);
      }

      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityReceiveImpl)
    */
   public void visit(AeActivityReceiveImpl aImpl) throws AeBusinessProcessException
   {
      boolean queued = "true".equals(getImplState().getAttribute(aImpl, STATE_QUEUED)); //$NON-NLS-1$
      aImpl.setQueued(queued);
      traverse(aImpl);
      if (aImpl.isQueued())
      {
         AeBusinessProcess process = (AeBusinessProcess) aImpl.getProcess();
         process.addReceiverKeyForConflictingReceives(aImpl);
      }
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityInvokeImpl)
    */
   public void visit(AeActivityInvokeImpl aImpl) throws AeBusinessProcessException
   {
      boolean queued = "true".equals(getImplState().getAttribute(aImpl, STATE_QUEUED)); //$NON-NLS-1$
      aImpl.setQueued(queued);
      String retries = getImplState().getAttribute(aImpl, STATE_RETRIES);
      if(AeUtil.notNullOrEmpty(retries))
         aImpl.setRetries(Integer.parseInt(retries));
      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl)
    */
   public void visit(AeActivityScopeImpl aImpl) throws AeBusinessProcessException
   {
      Element element = getImplState().getElement(aImpl);

      boolean hasImplicitCompensationHandler = "true".equals(getImplState().getAttribute(aImpl, STATE_HASIMPLICITCOMPENSATIONHANDLER)); //$NON-NLS-1$
      boolean hasImplicitFaultHandler        = "true".equals(getImplState().getAttribute(aImpl, STATE_HASIMPLICITFAULTHANDLER)); //$NON-NLS-1$
      boolean normalCompletion               = "true".equals(getImplState().getAttribute(aImpl, STATE_NORMALCOMPLETION)); //$NON-NLS-1$
      boolean hasCoordinations               = "true".equals(getImplState().getAttribute(aImpl, STATE_HASCOORDINATIONS)); //$NON-NLS-1$
      boolean snapshotRecorded               = "true".equals(getImplState().getAttribute(aImpl, STATE_SNAPSHOTRECORDED)); //$NON-NLS-1$
      int outstandingReplyCount              = getAttributeInt(element, STATE_OUTSTANDINGREPLIES);
      
      aImpl.setNormalCompletion(normalCompletion);
      aImpl.setSnapshotRecorded(snapshotRecorded);
      aImpl.setOutstandingReplyCount(outstandingReplyCount);
      
      if (hasImplicitCompensationHandler)
      {
         // Calling getCompensationHandler() will create the implicit
         // compensation handler if necessary. Create the implicit compensation
         // handler now, so that traverse() will see it.
         aImpl.getCompensationHandler();
      }

      if (hasImplicitFaultHandler)
      {
         // The implicit fault handler's fault will be restored when the fault
         // handler is traversed.
         aImpl.setFaultHandler(aImpl.createImplicitFaultHandler(null));
      }
      
      if (hasCoordinations)
      {
         // calling getCoordinationContainer creates the container.
         // this container will be visited by this traverser (via getChildrenForStateChange() )
         aImpl.getCoordinationContainer();
      }

      // Restore the scope's variables and correlation sets.
      restoreScopeVariables(element, aImpl);
      restoreScopeCorrelationSets(element, aImpl);

      // the comp info for this impl is restored by another visitor
      
      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityWaitImpl)
    */
   public void visit(AeActivityWaitImpl aImpl) throws AeBusinessProcessException
   {
      boolean queued = "true".equals(getImplState().getAttribute(aImpl, STATE_QUEUED)); //$NON-NLS-1$
      aImpl.setQueued(queued);

      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityWhileImpl)
    */
   public void visit(AeActivityWhileImpl aImpl) throws AeBusinessProcessException
   {
      boolean queued = "true".equals(getImplState().getAttribute(aImpl, STATE_QUEUED)); //$NON-NLS-1$
      aImpl.setQueued(queued);
      
      restoreLoopControl(aImpl);
      
      traverse(aImpl);
   }

   /**
    * Restores the loop control flag which tells us if the loop is terminating its
    * children because of a continue or break activity having executed w/in it.
    * @param aImpl
    */
   protected void restoreLoopControl(AeLoopActivity aImpl) throws AeBusinessProcessException
   {
      // the loop termination flag tells us if the loop is terminating its children
      // because of a continue or break activity.
      String loopFlag = getImplState().getAttribute(aImpl, STATE_LOOP_TERMINATION_REASON);
      aImpl.setEarlyTerminationReason(Integer.parseInt(loopFlag));
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl)
    */
   public void visit(AeActivityForEachImpl aImpl)
         throws AeBusinessProcessException
   {
      restoreForEachAttributes(aImpl);
      
      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.AeActivityForEachParallelImpl)
    */
   public void visit(AeActivityForEachParallelImpl aImpl)
         throws AeBusinessProcessException
   {
      restoreForEachAttributes(aImpl);
      Element element = getImplState().getElement(aImpl);
      int instanceValue = getAttributeInt(element, STATE_FOREACH_INSTANCE);
      aImpl.setInstanceValue(instanceValue);
      
      if (aImpl.getCounterValue() != -1)
      {
         // The process for restoring a parallel forEach's state involves visiting
         // the forEach's state element and then creating one child scope for each
         // of the iterations that had executed the last time the forEach was persisted.
         // The forEach maintains an instance value which is used to create unique
         // location paths for each of its child scopes. 
         // We need to restore the forEach's instance value to the value it was at
         // when the forEach executed and created its children. This way, we'll 
         // create child scopes with the same instance paths that they had the last
         // time they'd been in memory. 
         int startInstance = AeActivityForEachDef.getStartInstanceValue(instanceValue, aImpl.getStartValue(), aImpl.getFinalValue());
         int finalInstance = AeActivityForEachDef.getFinalInstanceValue(instanceValue, aImpl.getStartValue(), aImpl.getFinalValue());
         aImpl.createChildren(false, startInstance, finalInstance);
      }
      
      traverse(aImpl);

      // The forEach's element may contain child elements representing scopes that
      // had their state saved since they were in the middle of compensating when
      // the process was saved. The code below restores these scopes and their
      // children.
      String xpath = "./"+STATE_ACTY+"[@" + STATE_SCOPE_COMPENSATING + "='true']"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      List compensatingScopes = selectNodes(element, xpath, "Error restoring compensating scopes within forEach"); //$NON-NLS-1$
      for (Iterator iter = compensatingScopes.iterator(); iter.hasNext();)
      {
         Element scopeElement = (Element) iter.next();
         String scopeLocation = scopeElement.getAttribute(STATE_LOC);
         AeActivityScopeImpl scope = restoreScopeInstance(scopeLocation, false);
         visit(scope);
      }
   }
   
   /**
    * Reads the standard attributes for the forEach activity, whether its serial or parallel
    * @param aImpl
    * @throws AeBusinessProcessException
    */
   protected void restoreForEachAttributes(AeActivityForEachImpl aImpl) throws AeBusinessProcessException
   {
      restoreLoopControl(aImpl);
      
      Element element = getImplState().getElement(aImpl);
      int counterValue = getAttributeInt(element, STATE_FOREACH_COUNTER);
      int startValue = getAttributeInt(element, STATE_FOREACH_START);
      int finalValue = getAttributeInt(element, STATE_FOREACH_FINAL);
      int completionCondition = getAttributeInt(element, STATE_FOREACH_COMPLETIONCONDITION);
      int completionCount = getAttributeInt(element, STATE_FOREACH_COMPLETIONCOUNT);
      
      aImpl.setCounterValue(counterValue);
      aImpl.setStartValue(startValue);
      aImpl.setFinalValue(finalValue);
      aImpl.setCompletionCondition(completionCondition);
      aImpl.setCompletionCount(completionCount);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.AeBusinessProcess)
    */
   public void visit(AeBusinessProcess aProcess) throws AeBusinessProcessException
   {
      // Restore process fields.
      Element element = getImplState().getElement(aProcess);
      int processState = getAttributeInt(element, STATE_PROCESSSTATE);
      int processStateReason = getAttributeInt(element, STATE_PROCESSSTATEREASON);
      Date endDate = toDate(getAttribute(element, STATE_ENDDATEMILLIS));
      Date startDate = toDate(getAttribute(element, STATE_STARTDATEMILLIS));
      int maxLocationId = getAttributeInt(element, STATE_MAXLOCATIONID);
      
      int nextVariableId = getAttributeInt(element, STATE_NEXTVARIABLEID);
      if (nextVariableId == -1)
      {
         // This accounts for processes saved prior to the introduction of the 
         // variable id. We'll walk all of the variables in the state document 
         // and record the highest variable version number.
         String xpath = "//" + STATE_VAR + "/@" + STATE_VERSION; //$NON-NLS-1$ //$NON-NLS-2$
         List nodes = selectNodes(element.getOwnerDocument(), xpath, "Error selecting variable versions"); //$NON-NLS-1$
         nextVariableId = 0;
         for (Iterator iter = nodes.iterator(); iter.hasNext();)
         {
            Attr attr = (Attr) iter.next();
            nextVariableId = Math.max(nextVariableId, Integer.parseInt(attr.getNodeValue()));
         }
         // add one to move to a unique value
         nextVariableId += 1;
      }

      aProcess.setProcessState(processState);
      aProcess.setProcessStateReason(processStateReason);
      aProcess.setEndDate(endDate);
      aProcess.setStartDate(startDate);
      aProcess.setMaxLocationId(maxLocationId);
      aProcess.setNextVariableVersionNumber(nextVariableId);

      aProcess.setCreateMessage(createCreateMessage(aProcess));

      if(((AeProcessDef)aProcess.getDefinition()).containsSerializableScopes())
      {
         Node variableLockerData = getImplState().getVariableLockerData();
         aProcess.setVariableLockerData(variableLockerData);
      }

      // Restore partner links.
      restorePartnerLinks(aProcess);

      // Restore any business process properties
      restoreBusinessProcessProperties(aProcess);
      
      // Restore any faulting activity location paths
      restoreFaultingActivityLocationPaths(aProcess);

      // Now handle the process as a scope.
      visit((AeActivityScopeImpl) aProcess);
      
      // Scope visit needs to happen before we set the location paths for the process as they are dependent
      // upon impl objects created in the scope visitation.
      boolean suspended = isSuspended(processState);
      List locationPaths = getImplState().getExecutionQueuePaths();
      aProcess.setExecutionQueue(suspended, locationPaths);
   }
   
   /**
    * Returns true if the process is in a suspended state.
    * 
    * @param aProcessState
    */
   protected boolean isSuspended( int aProcessState )
   {
      return aProcessState == IAeBusinessProcess.PROCESS_SUSPENDED;
   }
   
   /** 
    * Restore the attributes common to fault and compensation handlers.
    * @param aImpl
    * @throws AeBusinessProcessException
    */
   protected void restoreBaseHandler(AeBaseHandler aImpl) throws AeBusinessProcessException
   {
      boolean hasCoordCompensator = "true".equals(getImplState().getAttribute(aImpl, STATE_HASCOORDCOMPENSATOR)); //$NON-NLS-1$
      if (hasCoordCompensator)
      {
         // creates the implicit compensate activity which will be 
         // returned as part of getChildrenForStateChange - hence
         // will be visited by this state restorer.
         aImpl.getCoordinatedActivityCompensator();
      }
   }   
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeCompensationHandler)
    */
   public void visit(AeCompensationHandler aImpl) throws AeBusinessProcessException
   {
      // Restore base
      restoreBaseHandler(aImpl);
      
      // Restore compensation callback.
      String callbackLocationPath = getImplState().getAttribute(aImpl, STATE_CALLBACK);
      if (!AeUtil.isNullOrEmpty(callbackLocationPath))
      {         
         // check to see if this is a callback for a coordinated compensator.
         // (instead of a normal compensate activity i.e. AeActivityCompensateImpl)
         boolean coordinated = "true".equals(getImplState().getAttribute(aImpl, STATE_CALLBACK_COORDINATED)); //$NON-NLS-1$
         String coordinationId = getImplState().getAttribute(aImpl, STATE_CALLBACK_COORD_ID);
         
         IAeCompensationCallback callback = null;
         if (coordinated)
         {
            // create a coordinated compensator.
            callback = new AeParticipantCompensator(coordinationId, getImplState().getProcess().getEngine() );

            // if it's the compensation handler for the process, then wrap the callback
            // to ensure that the process's state changes when the compensation is complete
            if (aImpl.getParent() == aImpl.getProcess())
            {
               callback = new AeProcessCompensationCallbackWrapper(callback);
            }
         }
         else
         {
            callback = (IAeCompensationCallback) getImplState().getProcess().findBpelObject(callbackLocationPath);
         }

         if (callback == null)
         {
            // If we couldn't find the callback object, then it's probably an
            // implicit compensate activity that hasn't been traversed yet. Put
            // this compensation handler into a map keyed by callback location,
            // so that visit(AeActivityCompensateImpl aImpl) can set the
            // callback object later.
            mCompensationCallbackOwnersMap.put(callbackLocationPath, aImpl);
         }
         else
         {
            aImpl.setCallback(callback);
         }
      }
      
      // the comp info for this impl is restored by another visitor

      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeImplicitCompensationHandler)
    */
   public void visit(AeImplicitCompensationHandler aImpl) throws AeBusinessProcessException
   {
      visit((AeCompensationHandler) aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeLink)
    */
   public void visit(AeLink aImpl) throws AeBusinessProcessException
   {
      String status = getImplState().getAttribute(aImpl, STATE_EVAL);

      if (STATE_UNKNOWN.equals(status))
      {
         aImpl.clearStatus();
      }
      else
      {
         aImpl.setRawStatus("true".equals(status)); //$NON-NLS-1$
      }

      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeOnAlarm)
    */
   public void visit(AeOnAlarm aImpl) throws AeBusinessProcessException
   {
      restoreBaseEvent(aImpl);

      traverse(aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeOnMessage)
    */
   public void visit(AeOnMessage aImpl) throws AeBusinessProcessException
   {
      restoreBaseEvent(aImpl);

      traverse(aImpl);
      
      if (aImpl.isQueued())
      {
         AeBusinessProcess process = (AeBusinessProcess) aImpl.getProcess();
         process.addReceiverKeyForConflictingReceives(aImpl);
      }
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeCoordinationContainer)
    */
   public void visit(AeCoordinationContainer aImpl) throws AeBusinessProcessException
   {
      // get count.
      int count = getAttributeInt(getImplState().getElement(aImpl), STATE_COORDINATION_COUNT);
      // get each coord id and state and restore it.
      for (int index = 0; index < count; index++)
      {  
         String idName = STATE_COORDINATION_ID + String.valueOf(index);
         String stateName = STATE_COORD_STATE + String.valueOf(index);
         String id = getImplState().getAttribute(aImpl, idName);
         String state = getImplState().getAttribute(aImpl, stateName);
         aImpl.restoreCoordination(id, state);
      }
      traverse(aImpl);
   }

   /** 
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeCoordinatorCompensationHandler)
    */
   public void visit(AeCoordinatorCompensationHandler aImpl) throws AeBusinessProcessException
   {
      visit((AeCompensationHandler) aImpl);
   }
   
   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeDefaultFaultHandler)
    */
   public void visit(AeDefaultFaultHandler aImpl) throws AeBusinessProcessException
   {
      // restore base handler information
      restoreBaseHandler( (AeBaseHandler) aImpl);
      traverse(aImpl);      
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeFaultHandler)
    */
   public void visit(AeFaultHandler aImpl) throws AeBusinessProcessException
   {
      visit((AeDefaultFaultHandler) aImpl);
   }

   /**
    * @see org.activebpel.rt.bpel.impl.visitors.IAeImplVisitor#visit(org.activebpel.rt.bpel.impl.activity.support.AeImplicitFaultHandler)
    */
   public void visit(AeImplicitFaultHandler aImpl) throws AeBusinessProcessException
   {
      visit((AeDefaultFaultHandler) aImpl);
   }     
}
