// $Header: /Development/AEDevelopment/projects/org.activebpel.rt.bpel/src/org/activebpel/rt/bpel/impl/visitors/IAeImplVisitor.java,v 1.9 2005/10/28 21:07:2
/*
 * Copyright (c) 2004-2006 Active Endpoints, Inc.
 *
 * This program is licensed under the terms of the GNU General Public License
 * Version 2 (the "License") as published by the Free Software Foundation, and 
 * the ActiveBPEL Licensing Policies (the "Policies").  A copy of the License 
 * and the Policies were distributed with this program.  
 *
 * The License is available at:
 * http: *www.gnu.org/copyleft/gpl.html
 *
 * The Policies are available at:
 * http: *www.activebpel.org/licensing/index.html
 *
 * Unless required by applicable law or agreed to in writing, this program is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS
 * OF ANY KIND, either express or implied.  See the License and the Policies
 * for specific language governing the use of this program.
 */
package org.activebpel.rt.bpel.impl.visitors;

import org.activebpel.rt.bpel.AeBusinessProcessException;
import org.activebpel.rt.bpel.impl.AeBusinessProcess;
import org.activebpel.rt.bpel.impl.activity.AeActivityAssignImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityBreakImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityCompensateImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityContinueImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityEmptyImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityFlowImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityForEachImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityForEachParallelImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityInvokeImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityPickImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityReceiveImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityReplyImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityScopeImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivitySequenceImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivitySuspendImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivitySwitchImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityTerminateImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityThrowImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityWaitImpl;
import org.activebpel.rt.bpel.impl.activity.AeActivityWhileImpl;
import org.activebpel.rt.bpel.impl.activity.support.AeCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCoordinationContainer;
import org.activebpel.rt.bpel.impl.activity.support.AeCoordinatorCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeCorrelationSet;
import org.activebpel.rt.bpel.impl.activity.support.AeDefaultFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeEventHandlersContainer;
import org.activebpel.rt.bpel.impl.activity.support.AeFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeImplicitCompensationHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeImplicitFaultHandler;
import org.activebpel.rt.bpel.impl.activity.support.AeLink;
import org.activebpel.rt.bpel.impl.activity.support.AeOnAlarm;
import org.activebpel.rt.bpel.impl.activity.support.AeOnMessage;
import org.activebpel.rt.bpel.impl.activity.support.AeSwitchCase;
import org.activebpel.rt.bpel.impl.activity.support.AeSwitchOtherwise;


/**
 * Visitor interface specification for BPEL Implementation classes.
 */
public interface IAeImplVisitor
{
   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityAssignImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityCompensateImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityEmptyImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityFlowImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityInvokeImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityPickImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityReceiveImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityReplyImpl aImpl) throws AeBusinessProcessException;
   
   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivitySuspendImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityScopeImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeBusinessProcess aImpl) throws AeBusinessProcessException;
   
   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivitySequenceImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivitySwitchImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityTerminateImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityThrowImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityWaitImpl aImpl) throws AeBusinessProcessException;
   
   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityForEachImpl aImpl) throws AeBusinessProcessException;
   
   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityForEachParallelImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityWhileImpl aImpl) throws AeBusinessProcessException;
   
   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeActivityContinueImpl aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    * @throws AeBusinessProcessException
    */
   public void visit(AeActivityBreakImpl aImpl) throws AeBusinessProcessException;

   /////////  Activity Support

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeOnAlarm aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeOnMessage aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeCompensationHandler aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeImplicitCompensationHandler aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeDefaultFaultHandler aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeFaultHandler aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeImplicitFaultHandler aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeSwitchOtherwise aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeSwitchCase aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeCorrelationSet aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeEventHandlersContainer aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */
   public void visit(AeLink aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */   
   public void visit(AeCoordinationContainer aImpl) throws AeBusinessProcessException;

   /**
    * Visits the specified implementation object
    * @param aImpl
    */   
   public void visit(AeCoordinatorCompensationHandler aImpl) throws AeBusinessProcessException;

   
}
